#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import getopt
import os
import shutil
import sys
import tempfile

import debinterface
from netaddr import *
from systemd import journal

stream = journal.stream(identifier='apply_networking')
INTERFACES_FILE = "/etc/network/interfaces"


def set_common_options(options=None, adapter=None, ipv=4):
    if options['vlan'] != '0':
        pos = options['name'].find(".%s" % options['vlan'])
        if pos != -1:
            adapter.setUnknown("vlan-raw-device", options['name'][0:pos])

    if ipv == 4:
        if options['metric'] != '':
            adapter.setUnknown("metric", options['metric'])
        #adapter.setUnknown("autoconf", "0")

    if options['preup'] != '':
        adapter.setPreUp(options['preup'])

    if options['predown'] != '':
        adapter.setPreDown(options['predown'])

    if options['up'] != '':
        adapter.setUp(options['up'])

    if options['down'] != '':
        adapter.setDown(options['down'])

    if options['postup'] != '':
        adapter.setUnknown("post-up", options['postup'])

    if options['postdown'] != '':
        adapter.setPostDown(options['postdown'])


#
def set_ipv6(workingFileName, options=None):
    write_iface = False
    with open(workingFileName, 'w') as f:
        pass

    interfaces = debinterface.Interfaces(interfaces_path=workingFileName, backup_path='/tmp/xxx')
    interfaces.removeAdapterByName(options['name'])
    adapter = interfaces.addAdapter(options['name'])

    if adapter == None:
        print('Fail to create adapter', file=stream)
        print_help()
        return

    # If bootproto defined
    if options['source'] != '':
        if options['source'] == 'dhcp6' or options['source'] == 'dhcp46':  # Source is DHCP6/DHCP46
            adapter.setAddressSource('dhcp')  # accepts only dhcp, not dncp6, dhcp4, or dhcp46
            adapter.setAddrFam('inet6')
            adapter.setUnknown("autoconf", "1")

            if options['onboot']:
                adapter.setAuto(options['onboot'])

            if options['gateway6'] != '':
                adapter.setGateway(options['gateway6'])
            else:
                adapter.setUnknown("accept_ra", "2")

            if options['dns6'] != '':
                dnses = options['dns6'].split(',')
                dnsopts = ''.join(map(lambda dns_srv: ' '+ dns_srv, dnses))                
                adapter.setDnsNameservers(dnses)
                adapter.setUp('resolvectl -6 dns {} {}'.format((options['name'], dnsopts)))

            set_common_options(options, adapter, 6)
            write_iface = True

        elif options['source'] == 'static' and options['address6'] != '':  # Source is static (IPv6)
            adapter.setAddressSource(options['source'])
            adapter.setAddrFam('inet6')
            adapter.setAddress(options['address6'])
            adapter.setUnknown("autoconf", "1")

            if options['netmask6'] != '':
                adapter.setNetmask(options['netmask6'])

            if options['onboot']:
                adapter.setAuto(options['onboot'])

            if options['gateway6'] != '':
                adapter.setGateway(options['gateway6'])

            if options['dns6'] != '':
                dnses = options['dns6'].split(',')
                dnsopts = ''.join(map(lambda dns_srv: ' '+ dns_srv, dnses))                
                adapter.setDnsNameservers(dnses)
                adapter.setUp('resolvectl -6 dns {} {}'.format(options['name'], dnsopts))

            set_common_options(options, adapter)
            write_iface = True

        elif options['source'] == 'manual':  # Source is manual (IPv6)
            adapter.setAddressSource(options['source'])
            adapter.setAddrFam('inet6')
            if options['address6'] != '':
                adapter.setAddress(options['address6'])
                adapter.setUnknown("autoconf", "1")
            #else:
            #    adapter.setUnknown("autoconf", "0")

            if options['onboot']:
                adapter.setAuto(options['onboot'])

            set_common_options(options, adapter, 6)
            write_iface = True

    if write_iface:
        interfaces.writeInterfaces()

    return write_iface


#
def set_ipv4(workingFileName, options=None):
    write_iface = False
    with open(workingFileName, 'w') as f:
        pass

    interfaces = debinterface.Interfaces(interfaces_path=workingFileName, backup_path='/tmp/xxx')
    interfaces.removeAdapterByName(options['name'])
    adapter = interfaces.addAdapter(options['name'])

    if adapter == None:
        print('Fail to create adapter', file=stream)
        print_help()
        return

    # If bootproto defined
    if options['source'] != '':
        if options['source'] == 'dhcp' or options['source'] == 'dhcp46':  # Source is DHCP/DHCP46
            adapter.setAddressSource('dhcp')  # accepts only dhcp, not dncp6, dhcp4, or dhcp46
            if options['family'] != '':
                adapter.setAddrFam(options['family'])
            else:
                adapter.setAddrFam('inet')

            if options['onboot']:
                adapter.setAuto(options['onboot'])

            if options['gateway'] != '':
                adapter.setGateway(options['gateway'])

            if options['dns'] != '':
                dnses = options['dns'].split(',')
                dnsopts = ''.join(map(lambda dns_srv: ' '+ dns_srv, dnses))
                adapter.setDnsNameservers(dnses)
                adapter.setUp('resolvectl -4 {} {}'.format(options['name'], dnsopts))

            set_common_options(options, adapter)
            write_iface = True

        elif options['source'] == 'static' and options['address'] != '':  # Source is static (IPv4)
            adapter.setAddressSource(options['source'])
            adapter.setAddrFam('inet')
            adapter.setAddress(options['address'])

            if options['netmask'] != '':
                adapter.setNetmask(options['netmask'])

            if options['onboot']:
                adapter.setAuto(options['onboot'])

            if options['gateway'] != '':
                adapter.setGateway(options['gateway'])

            if options['dns'] != '':
                dnses = options['dns'].split(',')
                dnsopts = ''.join(map(lambda dns_srv: ' '+ dns_srv, dnses))
                adapter.setDnsNameservers(dnses)
                adapter.setUp('resolvectl -4 dns {} {}'.format(options['name'], dnsopts))

            set_common_options(options, adapter)
            write_iface = True

        elif options['source'] == 'manual':  # Source is manual (IPv4)
            adapter.setAddressSource(options['source'])
            adapter.setAddrFam('inet')
            if options['address'] != '':
                adapter.setAddress(options['address'])

            if options['onboot']:
                adapter.setAuto(options['onboot'])

            set_common_options(options, adapter)
            write_iface = True

    if write_iface:
        interfaces.writeInterfaces()

    return write_iface


def apply_net_config(options=None):
    if options['source'] != '':
        # Is bootproto is DHCP or DHCP46 clear IPv4 address and mask fields
        if options['source'] == 'dhcp' or options['source'] == "dhcp46":
            # Clear static addresses
            options['address'] = ''
            options['netmask'] = ''

        # Is bootproto is DHCP6 or DHCP46 clear IPv6 address and mask fields
        if options['source'] == 'dhcp6' or options['source'] == "dhcp46":
            # Clear static addresses
            options['address6'] = ''
            options['netmask6'] = ''

    workingFileName_IPv4 = '/tmp/ifcfg-%sv4' % options['name']
    set_ipv4(workingFileName_IPv4, options)

    data_IPv4 = None
    if os.path.exists(workingFileName_IPv4):
        fin_IPv4 = open(workingFileName_IPv4, "r")
        data_IPv4 = fin_IPv4.read()
        fin_IPv4.close()
        os.remove(workingFileName_IPv4)

    workingFileName_IPv6 = '/tmp/ifcfg-%sv6' % options['name']
    set_ipv6(workingFileName_IPv6, options)

    data_IPv6 = None
    if os.path.exists(workingFileName_IPv6):
        fin_IPv6 = open(workingFileName_IPv6, "r")
        data_IPv6 = fin_IPv6.read()
        fin_IPv6.close()

    # If new data is generated update the old cfg-xxx file
    if data_IPv4 != None or data_IPv6 != None:
        delete_net_config(options)

        fullFileName = '/etc/network/interfaces.d/ifcfg-%s' % options['name']
        if options['persist']:
            fullFileName = '/etc/network/interfaces.d/persistcfg-%s' % options['name']

        fout = open(fullFileName, "w")
        if data_IPv4 != None:
            fout.write(data_IPv4)
        if data_IPv6 != None:
            fout.write(data_IPv6)

        fout.close()

        update_interfaces_file()


def delete_net_config(options=None, bNeedUpdateInterfacesFile=False):
    if 'deleteall' in options['name']:
        map(os.unlink, (os.path.join('/etc/network/interfaces.d', f) for f in os.listdir('/etc/network/interfaces.d') if
                        f.startswith('ifcfg-')))
        with open(INTERFACES_FILE, 'w') as f:
            f.truncate()

    # Parse the /etc/network/interfaces.d/ifcfg-xxxx file
    fullFileName = '/etc/network/interfaces.d/ifcfg-%s' % options['name']
    if os.path.exists(fullFileName):
        os.remove(fullFileName)

    if True == bNeedUpdateInterfacesFile:
        update_interfaces_file()


def print_help():
    print('apply_networking.py <OPTIONS>')
    print('-h This message')
    print('-n IFNAME - interface name (mandatory)')
    print('-a IPv4ADDRESS - IPv4 address of the interface (do not use with -f)')
    print('-b BOOTPROTO - Boot protocol')
    print('-d IPv4ADDRESS - DNS server for IPv4 addresses')
    print('--ipv6-dns IPv6ADDRESS - DNS server for IPv6 addresses')
    print('-e inet|inet6 - address family')
    print('-g IPv4ADDRESS - Gateway for IPv4 addresses')
    print('--ipv6-gateway IPv6ADDRESS - Gateway for IPv6 addresses')
    print('-f IPv6ADDRESS - IPv6 address of the interface (do not use with -a)')
    print('-m NETMASK - IPv4 netmask')
    print('--ipv6_netmask NETMASK - IPv6 netmask')
    print('-c METRIC - metric of interface (integer)')
    print('-o <yes|no> - Bring up interface on boot')
    print('-i <cm> - If the device is either CM10 or CM100')
    print('-j UP - quoted string of up command')
    print('-k DOWN - quoted string of down command')
    print('-p PREUP - quoted string of pre-up command')
    print('-q POSTUP - quoted string of post-up command')
    print('-r PREDOWN - quoted string of pre-down command')
    print('-s POSTDOWN - quoted string of post-down command')
    print('-v VLANID - vlan')
    print('-z IFNAME - interface name (mandatory). Remove the interface')
    print('--ovs-master OVS - OVS master for that port')
    print('--ovs-ports PORTS - List of ports that should be connected to ovs bridge')
    print('--ipv4-prefix IPv4_address/mask - IPv4 prefix of the interface')
    print('--ipv6-prefix IPv6_address/mask - IPv6 prefix of the interface')
    print('--static - keep the interface on ZTP')


#
def update_interfaces_file():
    try:
        temp_file = tempfile.NamedTemporaryFile(mode='w+', prefix="update_interfaces_", delete=False)
        temp_file_path = temp_file.name
        temp_file.write('auto lo\n')
        temp_file.write('iface lo inet loopback\n\n')
        for data in sorted(os.listdir('/etc/network/interfaces.d')):
            if 'ifcfg-' in data or 'persistcfg-' in data:
                temp_file.write('source /etc/network/interfaces.d/%s\n' % data)

        temp_file.close()
        shutil.move(temp_file_path, INTERFACES_FILE)
    except Exception as e:
        print('Fail to update interfaces file : {}'.format(e), file=stream)


#
def main(argv):
    ifname = ''
    bootproto = ''
    address = ''
    netmask = ''
    gateway = ''
    gateway6 = ''
    onboot = False
    ifcm = False
    dnsnameservers = ''
    dnsnameservers6 = ''
    vlan = '0'
    delete = 'no'
    up = ''
    down = ''
    preup = ''
    postup = ''
    predown = ''
    postdown = ''
    metric = '100'
    ovs_master = ''
    ovs_ports = ''
    ipv4prefix = ''
    ipv6prefix = ''
    address6 = ''
    netmask6 = ''
    family = ''
    persist = False

    try:
        opts, args = getopt.getopt(argv, "ha:b:c:d:e:f:g:i:j:k:m:n:o:p:q:r:s:v:z:",
                                   ['ovs-master=', 'ovs-ports=', 'ipv6-dns=', 'ipv6-gateway=', 'ipv4-prefix=',
                                    'ipv6-prefix=', 'ipv6-netmask=', 'static'])
    except getopt.GetoptError:
        print_help()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print_help()
            sys.exit()
        elif opt in ("-n"):
            ifname = arg
        elif opt in ("-b"):
            bootproto = arg
        elif opt in ("-a"):
            address = arg
        elif opt in ("-m"):
            netmask = arg
        elif opt in ("--ipv6-netmask"):
            netmask6 = arg
        elif opt in ("-g"):
            if IPNetwork(arg).version == 4:
                gateway = arg
            else:
                print("ERROR: The argument of '-g' option is not an IPv4 address!")
                sys.exit(1)
        elif opt in "--ipv6-gateway":
            if IPNetwork(arg).version == 6:
                gateway6 = arg
            else:
                print("ERROR: The argument of '--ipv6-gateway' option is not an IPv6 address!")
                sys.exit(1)
        elif opt in ("-o"):
            if arg == 'yes':
                onboot = True
        elif opt in ("-i"):
            if arg == 'cm':
                ifcm = True
        elif opt in ("-d"):
            for server in arg.split(","):
                if not IPNetwork(server).version == 4:
                    print("ERROR: The argument of '-d' option is not an IPv4 address!")
                    sys.exit(1)

                dnsnameservers = arg
        elif opt in ("--ipv6-dns"):
            for server in arg.split(","):
                if not IPNetwork(server).version == 6:
                    print("ERROR: The argument of '--ipv6-dns' option is not an IPv6 address!")
                    sys.exit(1)

                dnsnameservers6 = arg
        elif opt in ("-e"):
            family = arg
        elif opt in ("-f"):
            address6 = arg
        elif opt in ("-v"):
            vlan = arg
        elif opt in '-c':
            metric = arg
        elif opt in '-z':
            ifname = arg.strip()
            delete = 'yes'
        elif opt in '-j':
            up = arg
        elif opt in '-k':
            down = arg
        elif opt in '-p':
            preup = arg
        elif opt in '-q':
            postup = arg
        elif opt in ("-r"):
            predown = arg
        elif opt in ("-s"):
            postdown = arg
        elif opt == "--ovs-master":
            ovs_master = arg
        elif opt == "--ovs-ports":
            ovs_ports = arg
        elif opt == "--ipv4-prefix":
            ip = IPNetwork(arg)
            address = str(ip.ip)
            netmask = str(ip.netmask)
        elif opt == "--ipv6-prefix":
            ip = IPNetwork(arg)
            address6 = str(ip.ip)
            netmask6 = str(ip.prefixlen)
        elif opt == "--static":
            persist = True

    options = {
        'name': ifname,
        'gateway': gateway,
        'gateway6': gateway6,
        'source': bootproto,
        'netmask': netmask,
        'netmask6': netmask6,
        'address': address,
        'address6': address6,
        'onboot': onboot,
        'is_cloud_metro': ifcm,
        'dns': dnsnameservers,
        'dns6': dnsnameservers6,
        'vlan': vlan,
        'up': up,
        'down': down,
        'preup': preup,
        'postup': postup,
        'predown': predown,
        'postdown': postdown,
        'metric': metric,
        'ovs_master': ovs_master,
        'ovs_ports': ovs_ports,
        'family': family,
        'persist': persist,
    }
    if delete == 'no':
        apply_net_config(options)
    else:
        delete_net_config(options, bNeedUpdateInterfacesFile=True)

    return


if __name__ == "__main__":
    main(sys.argv[1:])

