#! /usr/bin/env python3

import sysrepo_interfaces.sysrepo_interfaces3 as sr
from pyroute2 import IPDB
import vppapi_if.vppapi_if as vpp_api
import time
import traceback
import ipaddress
import os

APP_NAME = 'vpp_apply_networking'
LTE_OFFSET = 100
NAT_OFFSET = 200
MGMT_OFFSET = 300
WIFI_OFFSET = 400

if_cfg = {}
vpp_if = None


def set_ip(ifname, ip, prefix):
    ipch = ipaddress.ip_address(ip)
    if ipch.is_loopback or ipch.is_link_local:
        return

    vppcfg = if_cfg.get(ifname, None)
    if vppcfg is None:
        return


    print('set_ip -> {} - {}/{}'.format(ifname, ip, prefix))


    if vppcfg.get('bvi', None):
        idx = vppcfg.get('bvi')
        ret = vpp_if.add_interface_address(idx=idx, addr='{}/{}'.format(ip, prefix))
        if ret:
            print(" Error adding interface {} address {}/{} - {} ".format(ifname, ip, prefix, ret))
        else:
            if 'yes' == os.getenv('RTCTL', ''):
                ret = vpp_if.routectl_set_ip(ip)
            else:
                ret = vpp_if.drctl_set_ip(vppcfg.get('host_idx'), ip)
            if ret:
                print(" Error setting router address {} - {} ".format(ip, ret))
            if ipch.version == 6:
                idx = vppcfg.get('bvi')
                vpp_if.ip6_ra_config(idx, suppress=1)


def add_ip_route(ip_prefix, gw, metric, ifidx, is_multipath):
    global vpp_if

    print('add_ip_route({}, {}, {} if {})'.format(ip_prefix, gw, metric, ifidx))

    metr = metric if metric <= 255 else 0
    for retry in range(3):
        try:
            ret = vpp_if.add_ip_route(ip_prefix, next_hop_address=gw, metric=metr, next_hop_sw_if_index=ifidx,
                                      is_multipath=is_multipath)
            if ret:
                print(
                    'Add ip route fail ({}, gw {}, metric {} if {}) ret={}'.format(ip_prefix, gw, metric, ifidx, ret))
            return
        except Exception:
            if retry != 2:
                time.sleep(0.5)

    print(traceback.format_exc())
    print('add_ip_route generate exception')


def get_system_name(name, vlan=None):
    ifname = name

    if name.find("data") > -1:
        ifname = "br-{}".format(name)

    if 'lte' in name:
        ifname = "br-{}".format(name)

    if 'wifi' in name:
        ifname = "br-{}".format(name)

    if vlan is not None and vlan != 0:
        ifname = "{}.{}".format(ifname, vlan)
    return ifname


# def get_vpp_name(name, mode):
def get_vpp_name(ifcfg):
    if_type = ifcfg.get('type', None)

    if not if_type:
        print('No type for name {}'.format(ifcfg.get('name', None)))
        return None

    if if_type == 'lte':
        tap_id = int(ifcfg.get('name').replace('lte', ''))
        return 'tap{}'.format(LTE_OFFSET + tap_id)
    elif if_type == 'ethernet':
        if 'data' in ifcfg.get('name', ''):
            if 'virtio' == ifcfg.get('mode', ''):
                return 'host-{}'.format(ifcfg.get('name', 'UNK'))
            elif 'dpdk' == ifcfg.get('mode', ''):
                return ifcfg.get('name', 'UNK')

            return 'Unk-vpp-if-data'
        elif 'mgmt0' == ifcfg.get('name', ''):
            mgmt_id = int(ifcfg.get('name').replace('mgmt', ''))
            return 'tap{}'.format(mgmt_id + MGMT_OFFSET)
        return 'Unk-vpp-if'

    elif if_type == 'nat':
        nat_id = int(ifcfg.get('name').replace('nat', ''))
        return 'tap{}'.format(nat_id + NAT_OFFSET)

    elif if_type == 'wifi':
        if "vap" in ifcfg.get('name'):
            tap_id = int(ifcfg.get('name').split("_")[0].replace('wifi','')) + int(ifcfg.get('name').split("_")[1].replace("vap",""))
        else:
            tap_id = int(ifcfg.get('name').replace('wifi', ''))
        
        return 'tap{}'.format(WIFI_OFFSET + tap_id)

    elif if_type == 'openvpn':
        return None

    elif if_type == 'ipsecvpn':
        return None

    elif if_type == 'vxlan':
        return None

    elif if_type == 'loopback':
        return None

    print('UnkType-{}'.format(ifcfg.get('type', '?!?!')))
    return None

def get_vpp_hostif_name(ifcfg):
    if_type = ifcfg.get('type', None)

    if not if_type:
        print('No type for name {}'.format(ifcfg.get('name', None)))
        return None

    if if_type == 'lte':
        tap_id = int(ifcfg.get('name').replace('lte',''))
        return 'tap{}'.format(LTE_OFFSET+tap_id)
    elif if_type == 'ethernet':
        if 'data' in ifcfg.get('name', ''):
            tap_id = int(ifcfg.get('name').replace('data',''))
            return 'tap{}'.format(tap_id)

            return 'Unk-vpp-if-data'
        elif 'mgmt0' == ifcfg.get('name', ''):
            mgmt_id = int(ifcfg.get('name').replace('mgmt',''))
            return 'tap{}'.format(mgmt_id+MGMT_OFFSET)
        return 'Unk-vpp-if'

    elif if_type == 'nat':
        nat_id = int(ifcfg.get('name').replace('nat',''))
        return 'tap{}'.format(nat_id+NAT_OFFSET)

    elif if_type == 'wifi':
        tap_id = int(ifcfg.get('name').replace('wifi',''))
        return 'tap{}'.format(WIFI_OFFSET+tap_id)

    elif if_type == 'openvpn':
        return None

    elif if_type == 'ipsecvpn':
        return None

    elif if_type == 'vxlan':
        return None

    elif if_type == 'loopback':
        return None

    return None

def load_ucpe_config(vpp_if):
    try:
        srcl = sr.SysrepoInterfaces(app_name=APP_NAME, priority=2)
        ifcfg = srcl.get_interfaces(startup=True)

        for key, cfg in ifcfg.items():
            vpp_name = get_vpp_name(cfg)
            if vpp_name:
                vpp_if_cfg = vpp_if.get_if_by_name(vpp_name)
                if vpp_if_cfg:
                    vpp_params = dict()
                    vpp_params['interface'] = vpp_name
                    vpp_params['idx'] = vpp_if_cfg.get('sw_if_index')
                    bridge = vpp_if.get_bridge_domains(vpp_params['idx'])
                    if bridge:
                        bvi = next(bridge)['bvi_sw_if_index']
                        if 0xffffffff != int(bvi):
                            vpp_params['bvi'] = bvi
                        else:
                            vpp_params['bvi'] = None
                    hostifname = get_vpp_hostif_name(cfg)
                    vpp_params['host_idx'] = vpp_if.get_ifidx_by_name(hostifname)
                    if_cfg[get_system_name(cfg.get('name', 'UNK'), vlan=cfg.get('vlan', None))] = vpp_params
    except:
        print('load_ucpe_config')

def load_initial_routes():
    ipdb = IPDB()
    try:
        for key, intf in ipdb.interfaces.items():
            if key != intf['ifname']:
                continue;
            for (ip, prefixlen) in intf['ipaddr']:
                set_ip(intf['ifname'], '{}'.format(ip), '{}'.format(prefixlen))

        for rt in ipdb.routes:
            if rt['type'] == 1 and (rt['family'] == 2 or rt['family'] == 10):
                if 'dst' not in rt:
                    dst = "default"
                else:
                    dst = rt['dst']
                orig_dst = dst
                if "default" == dst:
                    if rt['family'] == 2:
                        dst = "0.0.0.0/0"
                    else:
                        dst = "::/0"

                if 'priority' in rt:
                    metric = rt['priority']
                else:
                    metric = None

                is_worse = False
                if metric:
                    ipdb_temp2 = IPDB()
                    dst_routes = ipdb_temp2.routes
                    for route in dst_routes:
                        if route.get('dst', 'default') == orig_dst and route.get('family') == rt[
                            'family'] and route.get('priority', 10000) < metric:
                            is_worse = True
                    ipdb_temp2.release()

                # is this route with worse metric than other, skip
                if is_worse:
                    continue

                if 'oif' in rt:
                    oif = rt['oif']
                    interface = ipdb.interfaces[oif]
                    oifidx = None
                    if if_cfg.get(interface.ifname, None) and if_cfg.get(interface.ifname).get('bvi', None):
                        oifidx = if_cfg.get(interface.ifname).get('bvi')
                else:
                    oif = None
                    oifidx = None

                if len(rt['multipath']) > 0:
                    for mp in rt['multipath']:
                        if 'oif' in mp:
                            oif = mp['oif']
                            interface = ipdb.interfaces[oif]
                            oifidx = None
                            if if_cfg.get(interface.ifname, None) and if_cfg.get(interface.ifname).get('bvi', None):
                                oifidx = if_cfg.get(interface.ifname).get('bvi')
                        else:
                            oif = None
                            oifidx = None
                        if 'gateway' in mp:
                            gw = mp['gateway']
                        else:
                            gw = None
                        if gw:
                            add_ip_route(dst, gw, metric, oifidx, True)
                elif 'gateway' in rt:
                    gw = rt['gateway']
                    if gw:
                        add_ip_route(dst, gw, metric, oifidx, True)
    except:
        pass


def main():
    global vpp_if

    try:
        vpp_if = vpp_api.VppApiInterface(APP_NAME)
    except:
        print('Failed VPP API Init')
        exit(1)
    load_ucpe_config(vpp_if)
    load_initial_routes()


if __name__ == '__main__':
    main()
