#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import syslog
from enum import Enum
from tveapi.common import utils

from sysrepo_interfaces import sysrepo_interfaces3 as api
from sysrepo_wg_concentrator import sysrepo_wg_concentrator as wg_api

class ValidateErrors(Enum):
    OK = 0
    NOT_FOUND = -1
    IP_DUP_FOUND = -2
    METRIC_DUP_FOUND = -3
    NO_MGMNT = -4
    NOVALUE_IN_SYSREPO = -5
    
def getErrorAsText(error, optional_text=""):
    errorText = None

    if error == ValidateErrors.OK:
        errorText = ""
    elif error == ValidateErrors.NOT_FOUND:
        errorText = "Validation Error: Interface '%s' does not exist!" % optional_text
    elif error == ValidateErrors.IP_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' with the same IP found!" % optional_text
    elif error == ValidateErrors.METRIC_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' has the same type bootproto, metric [and set gateway, if present]!" % optional_text
    elif error == ValidateErrors.NO_MGMNT:
        errorText = "Validation Error: At least one management should be present!"
    elif error == ValidateErrors.NOVALUE_IN_SYSREPO:
        errorText = "Validation Error: Could not retrieve interface data!"

    return errorText

def validate_interface_by_type(ifname, iftype):
    sri = api.SysrepoInterfaces(app_name="set interfaces")
    interface = next((intf for intf in sri.get_interfaces().keys() if intf == ifname), None)

    if interface in sri.get_interfaces().keys():
        return
    else:
        sri.set_interface(ifname)
        sri.set_interface_param(ifname, 'type', iftype)

def del_wireguard_conc_int(name):
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.del_wireguard_concentrator(name)
    concentrator_api.commit()

def set_wireguard_conc_int(name, param, value):
    if not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return

    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")

    validate_interface_by_type(name, 'wireguard')

    if param == "concentrator-port":
        if int(value) in range(1, 65536):
            concentrator_api.set_wireguard_concentrator_param(name, param, int(value))
        else:
            error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)
            return
    else:
        concentrator_api.set_wireguard_concentrator_param(name, param, value)

    concentrator_api.commit()
    
def set_concentrator_params(type, name, param, value):
    if not type or not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.set_wireguard_concentrator_param(name, param, value)
    concentrator_api.commit()

def set_external_peers(name, endpoint, param, value):
    if not endpoint or not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return
        
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.set_wireguard_concentrator_external_peers(name, endpoint, param, value)
    concentrator_api.commit()

def del_external_peers(name, endpoint):
    if not endpoint:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.del_wireguard_concentrator_external_peers(name, endpoint)
    concentrator_api.commit()
    
def set_static_clients(name, hostname, param, value):
    if not hostname or not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    code, stdout, stderr = utils.execute_shell_command(
            f"/usr/bin/sudo python3 /scripts/set_wireguard_concentrator.py static {name} {hostname} {value}")
    
def del_static_clients(name, hostname):
    if not hostname:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    code, stdout, stderr = utils.execute_shell_command(
            f"/usr/bin/sudo python3 /scripts/set_wireguard_concentrator.py static {name} {hostname} delete")
    
    
if __name__ == "__main__":
    argc = len(sys.argv)

    if 3 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)
        
    if sys.argv[1] == "set_wireguard_conc_int":
        if sys.argv[3] == "delete":
            print(sys.argv[2])
            del_wireguard_conc_int(sys.argv[2])
        else:
            print(sys.argv[2], sys.argv[3], sys.argv[4])
            set_wireguard_conc_int(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_ip_ranges":
        print(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
        set_concentrator_params(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_domain_name":
        print(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
        set_concentrator_params(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_static_routes":
        print(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
        set_concentrator_params(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_external_peers":
        if sys.argv[4] == "delete":
            print(sys.argv[2], sys.argv[3])
            del_external_peers(sys.argv[2], sys.argv[3])
        else:
            print(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
            # set_external_peers(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    elif sys.argv[1] == "set_static_clients":
        if sys.argv[4] == 'delete':
            print(sys.argv[2], sys.argv[3])
            del_static_clients(sys.argv[2], sys.argv[3])
        else:
            print(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
            set_static_clients(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    else:
        syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)
