#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import ast
import copy
import string
import subprocess
import sys
import syslog
from enum import Enum

from sysrepo_interfaces import sysrepo_interfaces3 as api


BIND_INTERFACE_FILE = '/kubernetes/bind-interface-file'


class ValidateErrors(Enum):
    OK = 0
    NOT_FOUND = -1
    IP_DUP_FOUND = -2
    METRIC_DUP_FOUND = -3
    NO_MGMNT = -4
    NOVALUE_IN_SYSREPO = -5
    BAD_WEP_PASSWORD = -6
    BAD_CHANNEL_IN_BAND = -7
    BAND_24_NOT_SUPPORTED = -8
    BAND_5_NOT_SUPPORTED = -9


def getErrorAsText(error, optional_text=""):
    errorText = None

    if error == ValidateErrors.OK:
        errorText = ""
    elif error == ValidateErrors.NOT_FOUND:
        errorText = "Validation Error: Interface '%s' does not exist!" % optional_text
    elif error == ValidateErrors.IP_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' with the same IP found!" % optional_text
    elif error == ValidateErrors.METRIC_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' has the same type bootproto, metric [and set gateway, if present]!" % optional_text
    elif error == ValidateErrors.NO_MGMNT:
        errorText = "Validation Error: At least one management should be present!"
    elif error == ValidateErrors.NOVALUE_IN_SYSREPO:
        errorText = "Validation Error: Could not retrieve interface data!"
    elif error == ValidateErrors.BAD_WEP_PASSWORD:
        errorText = "Validation Error: WEP password length should be 5/13/16 or 10/26/32 !"
    elif error == ValidateErrors.BAD_CHANNEL_IN_BAND:
        errorText = "Validation Error: Band 2.4GHz channel 1..14, band 5GHz channel >= 32 !"
    elif error == ValidateErrors.BAND_24_NOT_SUPPORTED:
        errorText = "Validation Error: Band 2.4GHz not supported !"
    elif error == ValidateErrors.BAND_5_NOT_SUPPORTED:
        errorText = "Validation Error: Band 5GHz not supported !"

    return errorText


# Get value from sysrepo, but the idea here is to decrease complexity of rule's logic all None values to be
# represent as empty strings
def getSysrepoValue(iface_data, valueName=""):
    if valueName is not None and valueName != "":
        value = iface_data.get(valueName)
        if value is not None:
            return value
    return ""


def validate_eth(instance, name, bootproto, gateway, management, ipv4, ipv6, metric):
    ifaces_data = instance.get_interfaces()

    ret_val = ""  # Emulated previous rule -> Ok

    # Merged values from function parameters and sysrepo
    curr_bootproto = bootproto
    curr_gateway = gateway
    curr_metric = metric

    # Compare subnet addresses in binary
    if gateway != '""' and ipv4 != '""':
        netmask = int(ipv4.split('/')[1])
        gateway_list = gateway.split('.')

        for i in range(len(gateway_list)):
            curr_oct = gateway_list[i]
            gateway_list[i] = '{0:08b}'.format(int(curr_oct))

        bin_gateway = ''.join(gateway_list)
        bin_gateway = bin_gateway[0: netmask]
        for i in range(netmask, 31):
            bin_gateway = bin_gateway + '0'

        ip = ipv4.split('/')[0]
        ip_list = ip.split('.')

        for i in range(len(ip_list)):
            curr_oct = ip_list[i]
            ip_list[i] = '{0:08b}'.format(int(curr_oct))

        bin_ip = ''.join(ip_list)
        bin_ip = bin_ip[0: netmask]
        for i in range(netmask, 31):
            bin_ip = bin_ip + '0'

        if bin_ip != bin_gateway:
            ret_val = "Gateway should be in same subnet as IPv4"
            return ret_val

    # ============ Rule - Presence ============
    if ret_val == "":  # Check result from previous rule
        ret_val = getErrorAsText(ValidateErrors.NOT_FOUND, name)  # prepare for not found error
        for iface_data in ifaces_data.values():
            try:
                if name == getSysrepoValue(iface_data, "name") \
                   and "ethernet" == getSysrepoValue(iface_data, "type"):
                    ret_val = getErrorAsText(ValidateErrors.OK)

                    # Use the rule for loading of missing data values for that interface name:
                    if curr_bootproto == '""':
                        curr_bootproto = getSysrepoValue(iface_data, "bootproto")

                    if curr_gateway == '""':
                        curr_gateway = getSysrepoValue(iface_data, "gateway")

                    if curr_metric == '""':
                        curr_metric = getSysrepoValue(iface_data, "metric")

                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # ============ Rule - Management presence ============
    if ret_val == "":  # Check result from previous rule
        # There is disabling of management. Check how many management interfaces are present
        if management != '""' and 0 == int(management):

            mgmnt_count = 0
            for iface_data in ifaces_data.values():
                if True == getSysrepoValue(iface_data, "management") \
                   and "ethernet" == getSysrepoValue(iface_data, "type"):
                    mgmnt_count = mgmnt_count + 1

            # If there is one, check the request for disabling for which one is
            if mgmnt_count == 1:
                for iface_data in ifaces_data.values():
                    try:
                        if name == getSysrepoValue(iface_data, "name") \
                           and getSysrepoValue(iface_data, "management") == True:
                            ret_val = getErrorAsText(ValidateErrors.NO_MGMNT)
                            break
                    except:
                        ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                        break

    # ============ Rule - IP Duplication ============
    if ret_val == "":  # Check result from previous rule
        if ipv4 != '""' or ipv6 != '""':
            for iface_data in ifaces_data.values():
                try:
                    if name != getSysrepoValue(iface_data, "name") \
                       and (ipv4 == getSysrepoValue(iface_data, "ipv4-prefix") \
                            or ipv6 == getSysrepoValue(iface_data, "ipv6-prefix")):
                        ret_val = getErrorAsText(ValidateErrors.IP_DUP_FOUND, getSysrepoValue(iface_data, "name"))
                        break
                except:
                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                    break

    # ============ Rule - Metric Duplication ============
    if ret_val == "":  # Check result from previous rule
        # Search for duplicated metrics in follow cases:
        # 1. if bootproto == dhcp -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        # 2. if bootproto == none and present gateway -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        if (curr_bootproto == "dhcp") or (curr_bootproto == "none" and curr_gateway != ""):
            for iface_data in ifaces_data.values():
                try:
                    # Check other interfaces for conflicts
                    if name != getSysrepoValue(iface_data, "name"):
                        if getSysrepoValue(iface_data, "name") == 'outband':
                            continue  # CloudMetro, historically, ignore outband if
                        if (getSysrepoValue(iface_data, "bootproto") == "dhcp" \
                            and int(curr_metric) == int(getSysrepoValue(iface_data, "metric"))) or \
                            (getSysrepoValue(iface_data, "bootproto") == "none" \
                             and getSysrepoValue(iface_data, "gateway") != "" \
                             and int(curr_metric) == int(getSysrepoValue(iface_data, "metric"))):
                            ret_val = getErrorAsText(ValidateErrors.METRIC_DUP_FOUND, getSysrepoValue(iface_data, "name"))
                            break
                except:
                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                    break

    return ret_val


def validate_eth_node(instance, name, **additional_parameters):
    ifaces_data = instance.get_interfaces()

    ret_val = ""  # Emulated previous rule -> Ok

    # Merged values from function parameters and sysrepo
    current_ipv4 = additional_parameters.get("ipv4", "")
    current_ipv6 = additional_parameters.get("ipv6", "")
    current_bootproto = additional_parameters.get("bootproto", "")
    current_gateway = additional_parameters.get("gateway", "")
    current_management = additional_parameters.get("management", "")
    current_metric = additional_parameters.get("metric", "")

    # ============ Rule - Presence ============
    if ret_val == "":  # Check result from previous rule
        ret_val = getErrorAsText(ValidateErrors.NOT_FOUND, name)  # prepare for not found error
        for iface_data in ifaces_data.values():
            try:
                if name == getSysrepoValue(iface_data, "name") and "ethernet" == getSysrepoValue(iface_data, "type"):
                    ret_val = getErrorAsText(ValidateErrors.OK)

                    # Use the rule for loading of missing data values for that interface name:
                    if current_bootproto == "":
                        current_bootproto = getSysrepoValue(iface_data, "bootproto")
                    if current_metric == "":
                        current_metric = getSysrepoValue(iface_data, "metric")

                    # IPv4 address and the gateway are loaded (if not empty) for comparison/validation
                    if current_gateway == "":
                        current_gateway = getSysrepoValue(iface_data, "gateway")
                    if current_ipv4 == "":
                        current_ipv4 = getSysrepoValue(iface_data, "ipv4-prefix")
                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # Compare subnet addresses in binary
    if current_gateway != '' and current_ipv4 != '' and (current_bootproto == "none" or current_bootproto == "dhcp6"):
        netmask = int(current_ipv4.split('/')[1])
        gateway_list = current_gateway.split('.')

        for i in range(len(gateway_list)):
            curr_oct = gateway_list[i]
            gateway_list[i] = '{0:08b}'.format(int(curr_oct))

        bin_gateway = ''.join(gateway_list)
        bin_gateway = bin_gateway[0: netmask]
        for i in range(netmask, 31):
            bin_gateway = bin_gateway + '0'

        ip = current_ipv4.split('/')[0]
        ip_list = ip.split('.')

        for i in range(len(ip_list)):
            curr_oct = ip_list[i]
            ip_list[i] = '{0:08b}'.format(int(curr_oct))

        bin_ip = ''.join(ip_list)
        bin_ip = bin_ip[0: netmask]
        for i in range(netmask, 31):
            bin_ip = bin_ip + '0'

        if bin_ip != bin_gateway:
            if additional_parameters.get("ipv4", "") != "":
                print(f"Make sure to change the gateway.\nThe current gateway ({current_gateway}) is not in the same subnet as the IPv4 address.")
            elif additional_parameters.get("gateway", "") != "":
                print(f"Make sure to change the IPv4 address.\nThe current IPv4 address ({current_ipv4}) is not in the same subnet as the gateway.")

    # ============ Rule - Management presence ============
    if ret_val == "":  # Check result from previous rule
        # If there is disabling of management -> check how many management interfaces are present
        # At least one interface needs to be present management
        if current_management == "0":
            mgmnt_count = 0
            for iface_data in ifaces_data.values():
                if True == getSysrepoValue(iface_data, "management"):
                    if_type = getSysrepoValue(iface_data, "type")
                    if if_type.find('ethernet') > -1 or if_type.find('lte') > -1:
                        mgmnt_count = mgmnt_count + 1

            # If there is one, check the request for disabling for which one is
            if mgmnt_count == 1:
                for iface_data in ifaces_data.values():
                    try:
                        if name == getSysrepoValue(iface_data, "name") \
                           and getSysrepoValue(iface_data, "management") == True:
                            ret_val = getErrorAsText(ValidateErrors.NO_MGMNT)  # prepare for not present management error
                            break
                    except:
                        ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                        break

    # ============ Rule - IP Duplication ============
    if ret_val == "":  # Check result from previous rule
        # If the user is trying to set IPv4 address, but the interface is in DHCP or DHCP46 mode, and error will occur
        # current_ipv4/current_ipv6 are not used here, because if the user does not enter a new IPv4/v6 address, there is no need for IP duplication verification
        if additional_parameters.get("ipv4", "") != "":
            if current_bootproto == "dhcp6" or current_bootproto == "none":
                for iface_data in ifaces_data.values():
                    try:
                        if name != getSysrepoValue(iface_data, "name") and (current_ipv4 == getSysrepoValue(iface_data, "ipv4-prefix")):
                            ret_val = getErrorAsText(ValidateErrors.IP_DUP_FOUND, getSysrepoValue(iface_data, "name"))
                            break
                    except:
                        ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                        break
            else:
                ret_val = f"Cannot change IP! The interface is in {current_bootproto} mode.\nYou can change the mode via the bootproto command."

        # If the user is trying to set IPv6 address, but the interface is in DHCP6 or DHCP46 mode, and error will occur
        if additional_parameters.get("ipv6", "") != "":
            if current_bootproto == "dhcp" or current_bootproto == "none":
                for iface_data in ifaces_data.values():
                    try:
                        if name != getSysrepoValue(iface_data, "name") and current_ipv6 == getSysrepoValue(iface_data, "ipv6-prefix"):
                            ret_val = getErrorAsText(ValidateErrors.IP_DUP_FOUND, getSysrepoValue(iface_data, "name"))
                            break
                    except:
                        ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                        break
            else:
                ret_val = f"Cannot change IP! The interface is in {current_bootproto} mode.\nYou can change the mode via the bootproto command."

    # ============ Rule - Metric Duplication ============
    if ret_val == "":  # Check result from previous rule
        # Search for duplicated metrics in follow cases:
        # 1. if bootproto == dhcp -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        # 2. if bootproto == none and present gateway -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        if (current_bootproto == "none" and current_gateway != "") or (current_bootproto != "none"):
            for iface_data in ifaces_data.values():
                try:
                    # Check other interfaces for conflicts
                    if name != getSysrepoValue(iface_data, "name"):
                        if ((getSysrepoValue(iface_data, "bootproto") == "dhcp") and int(current_metric) == int(getSysrepoValue(iface_data, "metric"))) or \
                           (getSysrepoValue(iface_data, "bootproto") == "none" and getSysrepoValue(iface_data, "gateway") != "" and int(current_metric) == int(getSysrepoValue(iface_data, "metric"))):
                            ret_val = getErrorAsText(ValidateErrors.METRIC_DUP_FOUND, getSysrepoValue(iface_data, "name"))

                            # Printing all interfaces with their metric values, so  the user can set the correct metric for the current interface
                            duplicate_name = getSysrepoValue(iface_data, "name")
                            print("Used metrics / gateways:")
                            for iface in ifaces_data.values():
                                try:
                                    selected = "      "
                                    if (getSysrepoValue(iface, "metric") != "" \
                                        and getSysrepoValue(iface, "name") != name \
                                        and (getSysrepoValue(iface, "bootproto") == "none" and getSysrepoValue(iface, "gateway") != "" \
                                             or getSysrepoValue(iface, "bootproto") != "none")):
                                        if (getSysrepoValue(iface, "name") == duplicate_name):
                                            selected = "ERR-> "
                                        if (getSysrepoValue(iface, "gateway") == ""):
                                            print(selected, "Interface '", getSysrepoValue(iface, "name"), "' metric ", getSysrepoValue(iface, "metric"), sep="")
                                        else:
                                            print(selected, "Interface '", getSysrepoValue(iface, "name"), "' metric ", getSysrepoValue(iface, "metric"), " gateway ", getSysrepoValue(iface, "gateway"), sep="")
                                except:
                                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                                    break
                            break
                except:
                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                    break

    return ret_val


def validate_lte(instance, name, enable, metric):
    ifaces_data = instance.get_interfaces()

    ret_val = ""  # Emulated previous rule -> Ok

    # Merged values from function parameters and sysrepo
    curr_bootproto = '""'
    curr_enable = enable
    curr_metric = metric

    # ============ Rule - Presence ============
    if ret_val == "":  # Check result from previous rule
        ret_val = getErrorAsText(ValidateErrors.NOT_FOUND, name)  # prepare for not found error
        for iface_data in ifaces_data.values():
            try:
                if name == iface_data.get("name") and "lte" == iface_data.get("type"):
                    ret_val = getErrorAsText(ValidateErrors.OK)

                    # Use the rule for loading of missing data values for that interface name:
                    if curr_bootproto == '""':
                        curr_bootproto = getSysrepoValue(iface_data, "bootproto")

                    if curr_enable == '""':
                        curr_enable = getSysrepoValue(iface_data, "enable")

                    if curr_metric == '""':
                        curr_metric = getSysrepoValue(iface_data, "metric")

                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # ============ Rule - Metric Duplication ============

    if ret_val == "":  # Check result from previous rule
        # Search for duplicated metrics in follow cases:
        # 1. if bootproto == dhcp -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        if curr_bootproto == "dhcp":
            for iface_data in ifaces_data.values():
                try:
                    # Check other interfaces for conflicts
                    if name != getSysrepoValue(iface_data, "name"):
                        if (getSysrepoValue(iface_data, "bootproto") == "dhcp" \
                            and int(curr_metric) == int(getSysrepoValue(iface_data, "metric"))) \
                            or (getSysrepoValue(iface_data, "bootproto") == "none" \
                                and getSysrepoValue(iface_data, "gateway") != "" \
                                and int(curr_metric) == int(getSysrepoValue(iface_data, "metric"))):
                            ret_val = getErrorAsText(ValidateErrors.METRIC_DUP_FOUND,
                                                     getSysrepoValue(iface_data, "name"))
                            break
                except:
                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                    break

    return ret_val


def validate_lte_node(instance, name, **additional_parameters):
    ifaces_data = instance.get_interfaces()

    ret_val = ""  # Emulated previous rule -> Ok

    # Merged values from function parameters and sysrepo
    current_metric = additional_parameters.get("metric", "")
    trace_host = additional_parameters.get("trace_host", "")

    # ============ Rule - Presence ============
    if ret_val == "":  # Check result from previous rule
        ret_val = getErrorAsText(ValidateErrors.NOT_FOUND, name)  # prepare for not found error
        for iface_data in ifaces_data.values():
            try:
                if name == iface_data.get("name") and "lte" == iface_data.get("type"):
                    ret_val = getErrorAsText(ValidateErrors.OK)

                    # Use the rule for loading of missing data values for that interface name:
                    current_bootproto = getSysrepoValue(iface_data, "bootproto")
                    current_enable = getSysrepoValue(iface_data, "enable")
                    if current_metric == "":
                        current_metric = getSysrepoValue(iface_data, "metric")
                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # Check for enable status (for needed parameters)
    if ret_val == "":
        for iface_data in ifaces_data.values():
            try:
                if trace_host != '' and trace_host != 'none':
                    if current_enable is not True:
                        ret_val = "Cannot configure trace-host, LTE interface not enabled."
                        break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # ============ Rule - Metric Duplication ============
    if ret_val == "":  # Check result from previous rule
        # Search for duplicated metrics in follow cases:
        # 1. if bootproto == dhcp -> check metric of other interfaces
        if current_bootproto == "dhcp":
            for iface_data in ifaces_data.values():
                try:
                    # Check other interfaces for conflicts
                    if name != getSysrepoValue(iface_data, "name"):
                        if (getSysrepoValue(iface_data, "bootproto") == "dhcp" and int(current_metric) == int(getSysrepoValue(iface_data, "metric"))) or \
                           (getSysrepoValue(iface_data, "bootproto") == "none" and getSysrepoValue(iface_data, "gateway") != "" and int(current_metric) == int(getSysrepoValue(iface_data, "metric"))):
                            ret_val = getErrorAsText(ValidateErrors.METRIC_DUP_FOUND, getSysrepoValue(iface_data, "name"))

                            # Printing all interfaces with their metric values, so  the user can set the correct metric for the current interface
                            duplicate_name = getSysrepoValue(iface_data, "name")
                            print("Used metrics / gateways:")
                            for iface in ifaces_data.values():
                                try:
                                    selected = "      "
                                    if (getSysrepoValue(iface, "metric") != "" \
                                        and getSysrepoValue(iface, "name") != name \
                                        and (getSysrepoValue(iface, "bootproto") == "none" and getSysrepoValue(iface, "gateway") != "" \
                                             or getSysrepoValue(iface, "bootproto") != "none")):
                                        if (getSysrepoValue(iface, "name") == duplicate_name):
                                            selected = "ERR-> "
                                        if (getSysrepoValue(iface, "gateway") == ""):
                                            print(selected, "Interface '", getSysrepoValue(iface, "name"), "' metric ", getSysrepoValue(iface, "metric"), sep="")
                                        else:
                                            print(selected, "Interface '", getSysrepoValue(iface, "name"), "' metric ", getSysrepoValue(iface, "metric"), " gateway ", getSysrepoValue(iface, "gateway"), sep="")
                                except:
                                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                                    break

                            break
                except:
                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                    break

    return ret_val


def validate_wifi(instance, name, enable, metric, param, value):
    ifaces_data = instance.get_interfaces()

    ret_val = ""  # Emulated previous rule -> Ok

    # Merged values from function parameters and sysrepo
    curr_bootproto = '""'
    curr_enable = enable
    curr_metric = metric

    # ============ Rule - Presence ============
    if ret_val == "":  # Check result from previous rule
        ret_val = getErrorAsText(ValidateErrors.NOT_FOUND, name)  # prepare for not found error
        for iface_data in ifaces_data.values():
            try:
                if name == iface_data.get("name") and "wifi" == iface_data.get("type"):
                    ret_val = getErrorAsText(ValidateErrors.OK)

                    # Use the rule for loading of missing data values for that interface name:
                    if curr_bootproto == '""':
                        curr_bootproto = getSysrepoValue(iface_data, "bootproto")

                    if curr_enable == '""' or curr_enable == '':
                        curr_enable = getSysrepoValue(iface_data, "enable")

                    if curr_metric == '""' or curr_metric == '':
                        curr_metric = getSysrepoValue(iface_data, "metric")

                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # Check for enable status (for needed parameters)
    if ret_val == "":
        for iface_data in ifaces_data.values():
            try:
                if param == 'trace-host' and value != 'none':
                    if curr_enable is not True:
                        ret_val = "Cannot configure trace-host, Wi-Fi interface not enabled."
                        break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # ============ Rule - Metric Duplication ============

    if ret_val == "":  # Check result from previous rule
        # Search for duplicated metrics in follow cases:
        # 1. if bootproto == dhcp -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        if curr_bootproto in ['dhcp', 'dhcp46', 'dhcp6']:
            for iface_data in ifaces_data.values():
                try:
                    # Check other interfaces for conflicts
                    if name != getSysrepoValue(iface_data, "name"):
                        if (getSysrepoValue(iface_data, "bootproto") in ['dhcp', 'dhcp46', 'dhcp6'] \
                            and int(curr_metric) == int(getSysrepoValue(iface_data, "metric"))) \
                            or (getSysrepoValue(iface_data, "bootproto") == "none" \
                                and getSysrepoValue(iface_data, "gateway") != "" \
                                and int(curr_metric) == int(getSysrepoValue(iface_data, "metric"))):
                            ret_val = getErrorAsText(ValidateErrors.METRIC_DUP_FOUND,
                                                     getSysrepoValue(iface_data, "name"))
                            break
                except:
                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                    break

    if ret_val == "":  # Check result from previous rule
        keymgmt = None
        passphrase = None
        if 'passphrase' == param:
            keymgmt = ifaces_data['wifi0']['security']['key-mgmt']
            passphrase = value
        elif 'key-mgmt' == param:
            keymgmt = value
            passphrase = ifaces_data.get('wifi0').get('security').get('passphrase', '')

        if keymgmt and passphrase and 'WEP' == keymgmt:
            is_passphrase_in_hex = all(c in string.hexdigits for c in passphrase)
            passphrase_len = len(passphrase)
            if True == is_passphrase_in_hex and passphrase_len not in (10, 26, 32):
                print("WEP passphrase length {} != 10/26/32 !".format(passphrase_len))
                ret_val = getErrorAsText(ValidateErrors.BAD_WEP_PASSWORD)
            elif False == is_passphrase_in_hex and passphrase_len not in (5, 13, 16):
                print("WEP passphrase length {} != 5/13/16 !".format(passphrase_len))
                ret_val = getErrorAsText(ValidateErrors.BAD_WEP_PASSWORD)

    if ret_val == "":  # Check result from previous rule
        band = None
        channel = None
        wifi_enable = None
        wifi_mode = None
        if 'band' == param:
            channel = int(ifaces_data['wifi0']['radio-settings']['channel'])
            band = value
            wifi_enable = int(ifaces_data['wifi0']['enable'])
            wifi_mode = ifaces_data['wifi0']['wifi-mode']
        elif 'channel' == param:
            channel = int(value)
            band = ifaces_data['wifi0']['radio-settings']['band']
            wifi_enable = int(ifaces_data['wifi0']['enable'])
            wifi_mode = ifaces_data['wifi0']['wifi-mode']
        elif 'enable' == param:
            channel = int(ifaces_data['wifi0']['radio-settings']['channel'])
            band = ifaces_data['wifi0']['radio-settings']['band']
            wifi_enable = int(value)
            wifi_mode = ifaces_data['wifi0']['wifi-mode']
        elif 'wifi-mode' == param:
            channel = int(ifaces_data['wifi0']['radio-settings']['channel'])
            band = ifaces_data['wifi0']['radio-settings']['band']
            wifi_enable = int(ifaces_data['wifi0']['enable'])
            wifi_mode = value

        if wifi_mode and wifi_mode in ('AP', 'ap'):
            fiveg_channels = (32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56,
                              58, 60, 62, 64, 68, 96, 100, 102, 104, 106, 108, 110,
                              112, 114, 116, 118, 120, 122, 124, 126, 128,
                              132, 134, 136, 138, 140, 142, 144, 149,
                              151, 153, 155, 157, 159, 161, 165, 169, 173)
            try:
                hw_channels = subprocess.run(['/usr/bin/sudo', 'iw', 'phy', 'phy0', 'channels'],
                                             stdout=subprocess.PIPE, stderr=subprocess.DEVNULL,
                                             encoding='UTF-8')
            except subprocess.CalledProcessError as cpe:
                print('ERROR {}'.format(cpe))
                pass
            except OSError:
                print('Missing /usr/bin/sudo')
                pass

            available_2g_channels = list()
            available_5g_channels = list()
            if 0 == hw_channels.returncode:
                for ch in tuple(range(1, 15)):
                    if '[{}] \n'.format(ch) in hw_channels.stdout:
                        available_2g_channels.append(ch)

                for ch in fiveg_channels:
                    if '[{}] \n'.format(ch) in hw_channels.stdout:
                        available_5g_channels.append(ch)

            if wifi_enable and band and channel \
               and 1 == wifi_enable and '2.4GHz' == band \
               and channel not in available_2g_channels:
                ret_val = getErrorAsText(ValidateErrors.BAD_CHANNEL_IN_BAND)
            elif wifi_enable and band and channel \
                 and 1 == wifi_enable and '5GHz' == band \
                 and channel not in available_5g_channels:
                ret_val = getErrorAsText(ValidateErrors.BAD_CHANNEL_IN_BAND)
            elif '2.4GHz' == band and not available_2g_channels:
                ret_val = getErrorAsText(ValidateErrors.BAND_24_NOT_SUPPORTED)
            elif '5GHz' == band and not available_5g_channels:
                ret_val = getErrorAsText(ValidateErrors.BAND_5_NOT_SUPPORTED)

    return ret_val


def validate_vpn(instance, name):
    ifaces_data = instance.get_interfaces()

    for iface_data in ifaces_data.values():
        try:
            if name == iface_data.get("name") and "openvpn" == iface_data.get("type"):
                return ""
                break
        except:
            ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
            break

    return getErrorAsText(ValidateErrors.NOT_FOUND, name)


def validate_nat_node(name):
    nat_ifs = ['nat0', 'nat1', 'nat2', 'nat3', 'nat4']
    if name not in nat_ifs:
        return ("Only interfaces from nat0 to nat4 are supported.")
    return ""


def validate_loopback(instance, name, **additional_parameters):
    ifaces_data = instance.get_interfaces()

    ret_val = ""  # Emulated previous rule -> Ok

    # Merged values from function parameters and sysrepo
    current_ipv4 = additional_parameters.get("ipv4", "")
    current_ipv6 = additional_parameters.get("ipv6", "")

    # ============ Rule - IP Duplication ============
    # if ret_val == "":  # Check result from previous rule
    if current_ipv4 != "":
        for iface_data in ifaces_data.values():
            try:
                if name != getSysrepoValue(iface_data, "name") and (current_ipv4 == getSysrepoValue(iface_data, "ipv4-prefix")):
                    ret_val = getErrorAsText(ValidateErrors.IP_DUP_FOUND, getSysrepoValue(iface_data, "name"))
                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break
    if current_ipv6 != "":
        for iface_data in ifaces_data.values():
            try:
                if name != getSysrepoValue(iface_data, "name") and (current_ipv6 == getSysrepoValue(iface_data, "ipv6-prefix")):
                    ret_val = getErrorAsText(ValidateErrors.IP_DUP_FOUND, getSysrepoValue(iface_data, "name"))
                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    return ret_val


def validate_interface_by_type(sri, ifname, iftype):
    interface = next((intf for intf in sri.get_interfaces().keys() if intf == ifname), None)

    if interface in sri.get_interfaces().keys():
        return
    else:
        sri.set_interface(ifname)
        sri.set_interface_param(ifname, 'type', iftype)


def check_bind_interface():
    try:
        with open(BIND_INTERFACE_FILE, 'r') as file:
            content = file.read()
            return content
    except FileNotFoundError:
        return "File not found."
    except Exception as e:
        return f"An error occurred: {e}"


def check_if_concentrator_or_peer_params(name, param):
    interfaces = api.SysrepoInterfaces(app_name="get wireguard params")
    peer_params = interfaces.get_wireguard_peer_all_params(name)
    conc_params = interfaces.get_wireguard_concentrator_params(name)


    if param == 'peers' and peer_params is not None:
        return True
    elif param == 'concentrator' and conc_params is not None:
        return True
    else:
        return False

def set_eth(name, new_bootproto, new_ip, new_mode, new_management, new_vlan, new_gateway, new_dns, new_ipv6, new_metric, new_trace, new_linkstateprop, new_ippass, new_dns6, new_gateway6):
    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = validate_eth(interfaces, name, new_bootproto, new_gateway, new_management, new_ip, new_ipv6, new_metric)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "ethernet")

        if new_management != '""':
            if int(new_management) == 1:
                interfaces.set_interface_param(name, "management", True)
            else:
                interfaces.set_interface_param(name, "management", False)
        if new_ip != '""' and new_ip != '0.0.0.0/0':
            interfaces.set_interface_param(name, "ipv4-prefix", new_ip)
        else:
            interfaces.unset_interface_param(name, "ipv4-prefix")
        if new_ipv6 != '""' and new_ipv6 != '0000::/0':
            interfaces.set_interface_param(name, "ipv6-prefix", new_ipv6)
        else:
            interfaces.unset_interface_param(name, "ipv6-prefix")
        if new_metric != '""':
            interfaces.set_interface_param(name, "metric", new_metric)
        if new_trace != '""' and new_trace != 'none':
            interfaces.set_interface_param(name, "trace-host", new_trace)
        else:
            interfaces.unset_interface_param(name, "trace-host")
        if new_bootproto != '""':
            interfaces.set_interface_param(name, "bootproto", new_bootproto)
        if new_mode != '""':
            interfaces.set_interface_param(name, "mode", new_mode)
        if new_dns != '""':
            interfaces.set_interface_param(name, "dns", new_dns)
        else:
            interfaces.unset_interface_param(name, "dns")
        if new_gateway != '""' and new_gateway != '0.0.0.0':
            interfaces.set_interface_param(name, "gateway", new_gateway)
        else:
            interfaces.unset_interface_param(name, "gateway")
        if new_vlan != '""' and new_vlan != '0':
            interfaces.set_interface_param(name, "vlan", new_vlan)
        else:
            interfaces.unset_interface_param(name, "vlan")

        if new_linkstateprop != '""':
            if int(new_linkstateprop) == 1:
                interfaces.set_interface_param(name, "link-state-propagation", True)
            else:
                interfaces.set_interface_param(name, "link-state-propagation", False)
        elif 'mgmt' not in name:
            interfaces.set_interface_param(name, "link-state-propagation", True)

        if new_ippass != '""':
            if 'mgmt' in name:
                print("IP Passthrough feature is not supported for this interface")
                return
            if int(new_ippass) == 1:
                interfaces.set_ippass_param(name, True)
            else:
                interfaces.set_ippass_param(name, False)

        if new_dns6 != '""':
            interfaces.set_interface_param(name, "dns6", new_dns6)
        else:
            interfaces.unset_interface_param(name, "dns6")

        if new_gateway6 != '""' and new_gateway6 != '::':
            interfaces.set_interface_param(name, "gateway6", new_gateway6)
        else:
            interfaces.unset_interface_param(name, "gateway6")

        interfaces.commit()

    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_lte(name, new_enable, new_apn, new_user, new_pass, new_management, new_metric, new_trace, new_ippass):
    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = validate_lte(interfaces, name, new_enable, new_metric)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "lte")

        if new_enable != '""':
            if int(new_enable) == 1:
                interfaces.set_interface_param(name, "enable", True)
            else:
                interfaces.set_interface_param(name, "enable", False)
        if new_apn != '""':
            interfaces.set_interface_param(name, "apn", new_apn)
        if new_user != '""':
            interfaces.set_interface_param(name, "user", new_user)
        if new_pass != '""':
            interfaces.set_interface_param(name, "pass", new_pass)
        if new_management != '""':
            if int(new_management) == 1:
                interfaces.set_interface_param(name, "management", True)
            else:
                interfaces.set_interface_param(name, "management", False)
        if new_metric != '""':
            interfaces.set_interface_param(name, "metric", new_metric)
        if new_trace != '""':
            interfaces.set_interface_param(name, "trace-host", new_trace)

        if new_ippass != '""':
            if int(new_ippass) == 1:
                interfaces.set_ippass_param(name, True)
            else:
                interfaces.set_ippass_param(name, False)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def unset_lte(name, param):
    if not name or not param:
        print("Error applying lte configuration")
        return
    interfaces = api.SysrepoInterfaces(app_name="unset lte")
    interfaces.set_interface(name)
    if param != "":
        interfaces.set_interface_param(name, param, "")

    interfaces.commit()


def set_vpn(name, new_enable, new_management, new_type, new_ovpn_endpoint, new_ovpn_port):
    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = validate_vpn(interfaces, name)

    if error_message == "":

        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "openvpn")

        if new_enable != '""':
            if int(new_enable) == 1:
                interfaces.set_interface_param(name, "enable", True)
            else:
                interfaces.set_interface_param(name, "enable", False)
        if new_management != '""':
            if int(new_management) == 1:
                interfaces.set_interface_param(name, "management", True)
            else:
                interfaces.set_interface_param(name, "management", False)
        if new_ovpn_endpoint != '""':
            interfaces.set_concentrator_endpoint(name, new_ovpn_endpoint)
        if new_ovpn_port != '""':
            if int(new_ovpn_port) in range(1, 65536):
                interfaces.set_concentrator_port(name, new_ovpn_port)
            else:
                error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
                syslog.syslog(syslog.LOG_ERR, error_message)
                print(error_message)
                return

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_ipsec(name, new_enable, new_management, new_remote_gateway, new_local_id, new_remote_id,
              new_ike_version, new_ike_mode, new_protocol_mode, new_ike_phase_1_rounds, new_ike_phase_1_id,
              new_ike_phase_1_encryption, new_ike_phase_1_authentication, new_ike_phase_1_dh_group,
              new_ike_phase_2_local_ts, new_ike_phase_2_remote_ts, new_ike_phase_2_id, new_ike_phase_2_encryption,
              new_ike_phase_2_authentication, new_ike_phase_2_dh_group):
    ike_phase_1_rounds = ast.literal_eval(new_ike_phase_1_rounds.replace('=', ':'))
    ike_phase_1_id = new_ike_phase_1_id.split(',')
    ike_phase_1_encryption = new_ike_phase_1_encryption.split(',')
    ike_phase_1_authentication = new_ike_phase_1_authentication.split(',')
    ike_phase_1_dh_group = new_ike_phase_1_dh_group.split(',')
    ike_phase_2_id = new_ike_phase_2_id.split(',')
    ike_phase_2_encryption = new_ike_phase_2_encryption.split(',')
    ike_phase_2_authentication = new_ike_phase_2_authentication.split(',')
    ike_phase_2_dh_group = new_ike_phase_2_dh_group.split(',')

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    interfaces.set_interface(name)
    interfaces.set_interface_param(name, "type", "ipsecvpn")

    if new_enable != '""':
        if int(new_enable) == 1:
            interfaces.set_interface_param(name, "enable", True)
        else:
            interfaces.set_interface_param(name, "enable", False)
    if new_management != '""':
        if int(new_management) == 1:
            interfaces.set_interface_param(name, "management", True)
        else:
            interfaces.set_interface_param(name, "management", False)
    if new_remote_gateway != '""' and new_remote_gateway != '0.0.0.0':
        interfaces.set_interface_param(name, "remote-gw", new_remote_gateway)
    if new_local_id != '""':
        interfaces.set_interface_param(name, "local-id", new_local_id)
    if new_remote_id != '""':
        interfaces.set_interface_param(name, "remote-id", new_remote_id)
    if new_ike_version != '""':
        interfaces.set_interface_param(name, "ike-version", new_ike_version)
        if int(new_ike_version) == 1:
            if new_ike_mode != '""':
                interfaces.set_interface_param(name, "ike-mode", new_ike_mode)
        else:
            interfaces.unset_interface_param(name, "ike-mode")
    if new_protocol_mode != '""':
        interfaces.set_interface_param(name, "protocol-mode", new_protocol_mode)
    if new_ike_phase_1_rounds != '""':
        if ike_phase_1_rounds is not None:
            for key, value in ike_phase_1_rounds.items():
                if key == "local":
                    for ike_phase_1_round in ike_phase_1_rounds[key]:
                        for local_key, local_value in ike_phase_1_round.items():
                            if local_key == 'authentication-method' and local_value == 'pubkeys':
                                interfaces.unset_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "cert", ike_phase_1_round['cert'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "key", ike_phase_1_round['key'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "key-pass", ike_phase_1_round['key-pass'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "ca-cert", ike_phase_1_round['ca-cert'])
                            elif local_key == 'authentication-method' and local_value == 'pre-shared-keys':
                                interfaces.unset_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "secret", ike_phase_1_round['pre-shared-keys'])
                            elif local_key == 'authentication-method' and local_value == 'xauth':
                                if int(interfaces.get_interface_param(name, "ike-version")) != 1:
                                    print("Xauth is only available in IKE v1!")
                                    return
                                interfaces.unset_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "username", ike_phase_1_round['xauth-username'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "password", ike_phase_1_round['xauth-password'])
                            elif local_key == 'authentication-method' and local_value == 'eap':
                                if int(interfaces.get_interface_param(name, "ike-version")) != 2:
                                    print("EAP is only available in IKE v2!")
                                    return
                                interfaces.unset_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round(name, int(ike_phase_1_round['id']))
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "username", ike_phase_1_round['eap-username'])
                                interfaces.set_ike1_local_round_param(name, int(ike_phase_1_round['id']), "password", ike_phase_1_round['eap-password'])
                if key == "remote":
                    for remote_key, remote_value in ike_phase_1_rounds[key].items():
                        if remote_key == 'authentication-method' and remote_value == 'pubkeys':
                            interfaces.unset_ike1_remote_round(name, int(ike_phase_1_rounds[key]['id']))
                            interfaces.set_ike1_remote_round(name, int(ike_phase_1_rounds[key]['id']))
                            interfaces.set_ike1_remote_round_param(name, int(ike_phase_1_rounds[key]['id']), "authentication-method", ike_phase_1_rounds[key]['authentication-method'])
                            interfaces.set_ike1_remote_round_param(name, int(ike_phase_1_rounds[key]['id']), "cert", ike_phase_1_rounds[key]['cert'])
                            interfaces.set_ike1_remote_round_param(name, int(ike_phase_1_rounds[key]['id']), "key", ike_phase_1_rounds[key]['key'])
                            interfaces.set_ike1_remote_round_param(name, int(ike_phase_1_rounds[key]['id']), "key-pass", ike_phase_1_rounds[key]['key-pass'])
                            interfaces.set_ike1_remote_round_param(name, int(ike_phase_1_rounds[key]['id']), "ca-cert", ike_phase_1_rounds[key]['ca-cert'])
                        elif remote_key == 'authentication-method' and remote_value == 'pre-shared-keys':
                            interfaces.unset_ike1_remote_round(name, int(ike_phase_1_rounds[key]['id']))
                            interfaces.set_ike1_remote_round(name, int(ike_phase_1_rounds[key]['id']))
                            interfaces.set_ike1_remote_round_param(name, int(ike_phase_1_rounds[key]['id']), "authentication-method", ike_phase_1_rounds[key]['authentication-method'])
                            interfaces.set_ike1_remote_round_param(name, int(ike_phase_1_rounds[key]['id']), "secret", ike_phase_1_rounds[key]['pre-shared-keys'])
    if new_ike_phase_1_id != '""' and new_ike_phase_1_id.replace(",", "") != "":
        if ike_phase_1_id is not None:
            interfaces.unset_ike1_proposals(name)
            for ids in ike_phase_1_id:
                if ids != "":
                    interfaces.set_ike1_proposals(name, int(ids))
                    interfaces.set_ike1_proposals_param(name, int(ids), "encryption-algorithm", ike_phase_1_encryption[int(ids)-1])
                    interfaces.set_ike1_proposals_param(name, int(ids), "authentication-algorithm", ike_phase_1_authentication[int(ids)-1])
    if new_ike_phase_1_dh_group != '""':
        if ike_phase_1_dh_group is not None:
            interfaces.unset_ike1_dh_groups(name)
            if ike_phase_1_dh_group != 'none':
                for dh_group in ike_phase_1_dh_group:
                    interfaces.set_ike1_dh_groups(name, dh_group)
    if new_ike_phase_2_local_ts != '""':
        if new_ike_phase_2_local_ts == 'none':
            interfaces.unset_ike2_local_ts(name)
        else:
            interfaces.set_ike2_local_ts(name, new_ike_phase_2_local_ts)
    if new_ike_phase_2_remote_ts != '""':
        if new_ike_phase_2_remote_ts == 'none':
            interfaces.unset_ike2_remote_ts(name)
        else:
            interfaces.set_ike2_remote_ts(name, new_ike_phase_2_remote_ts)
    if new_ike_phase_2_id != '""' and new_ike_phase_2_id.replace(",", "") != "":
        if ike_phase_2_id is not None:
            interfaces.unset_ike2_proposals(name)
            for ids in ike_phase_2_id:
                if ids != "":
                    interfaces.set_ike2_proposals(name, int(ids))
                    interfaces.set_ike2_proposals_param(name, int(ids), "encryption-algorithm", ike_phase_2_encryption[int(ids) - 1])
                    interfaces.set_ike2_proposals_param(name, int(ids), "authentication-algorithm", ike_phase_2_authentication[int(ids) - 1])
    if new_ike_phase_2_dh_group != '""':
        if ike_phase_2_dh_group is not None:
            interfaces.unset_ike2_dh_groups(name)
            if ike_phase_2_dh_group != 'none':
                for dh_group in ike_phase_2_dh_group:
                    interfaces.set_ike2_dh_groups(name, dh_group)

    interfaces.commit()


def set_ipsecvpn_proposals(phase, name, new_id, new_encr, new_auth):
    ike_id = new_id.split(',')
    encr = new_encr.split(',')
    auth = new_auth.split(',')

    if ike_id is None or encr is None or auth is None:
        return

    interfaces = api.SysrepoInterfaces(app_name="set ipsecvpn")
    interfaces.set_interface(name)

    for ids in ike_id:
        if ids != "":
            interfaces.set_proposal(phase, name, int(ids))
            interfaces.set_proposal_param(phase, name, int(ids), "encryption-algorithm", encr[int(ids) - 1])
            interfaces.set_proposal_param(phase, name, int(ids), "authentication-algorithm", auth[int(ids) - 1])

    interfaces.commit()


def set_ipsecvpn_dh_groups(phase, name, new_groups):
    groups = new_groups.split(',')
    if groups is None:
        return

    groups.sort(key=int)

    interfaces = api.SysrepoInterfaces(app_name="set ipsecvpn")
    interfaces.set_interface(name)

    interface = interfaces.session.get_data("/ucpe:config/interfaces:interface[name='{}']".format(name))
    groups_indexed = []
    if interface and 'config' in interface and 'interface' in interface['config']:
        for i in interface['config']['interface']:
            if phase in i and 'dh-groups' in i[phase]:
                groups_indexed = i[phase]['dh-groups']

    for dh_group in groups_indexed:
        if dh_group['group'] not in groups:
            if 'ike-phase-1' in phase:
                interfaces.unset_ike1_dh_group(name, dh_group['group'])
            else:
                interfaces.unset_ike2_dh_group(name, dh_group['group'])

    for dh_group in groups:
        interfaces.set_dh_groups(phase, name, dh_group)

    interfaces.commit()


def set_ipsecvpn_rounds(ipsecvpn_type, name, new_rounds):
    rounds = ast.literal_eval(new_rounds.replace('=', ':'))
    interfaces = api.SysrepoInterfaces(app_name="set ipsecvpn")
    interfaces.set_interface(name)

    for ike_phase_1_round in rounds[ipsecvpn_type]:
        if not ike_phase_1_round.get('authentication-method', ""):
            continue
        if ike_phase_1_round['authentication-method'] == 'pubkeys':
            interfaces.set_ike1_round(name, ipsecvpn_type, int(ike_phase_1_round['id']))
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "ca-cert", ike_phase_1_round['ca-cert'])
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "cert", ike_phase_1_round['cert'])
            if ike_phase_1_round.get('key', ""):
                interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "key", ike_phase_1_round['key'])
            if ike_phase_1_round.get('key-pass', ""):
                interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "key-pass", ike_phase_1_round['key-pass'])
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'secret')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'username')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'password')

        elif ike_phase_1_round['authentication-method'] == 'pre-shared-keys':
            interfaces.set_ike1_round(name, ipsecvpn_type, int(ike_phase_1_round['id']))
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "secret", ike_phase_1_round['pre-shared-keys'])
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'cert')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'key')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'key-pass')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'ca-cert')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'username')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'password')

        elif ike_phase_1_round['authentication-method'] == 'xauth':
            if int(interfaces.get_interface_param(name, "ike-version")) != 1:
                print("Xauth is only available in IKE v1!")
                return
            interfaces.set_ike1_round(name, ipsecvpn_type, int(ike_phase_1_round['id']))
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "username", ike_phase_1_round['xauth-username'])
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "password", ike_phase_1_round['xauth-password'])
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'secret')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'cert')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'key')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'key-pass')
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'ca-cert')

        elif ike_phase_1_round['authentication-method'] == 'eap':
            if int(interfaces.get_interface_param(name, "ike-version")) != 2:
                print("EAP is only available in IKE v2!")
                return
            interfaces.set_ike1_round(name, ipsecvpn_type, int(ike_phase_1_round['id']))
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "authentication-method", ike_phase_1_round['authentication-method'])
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "ca-cert", ike_phase_1_round['ca-cert'])
            interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "cert", ike_phase_1_round['cert'])
            interfaces.unset_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), 'secret')
            if ipsecvpn_type == 'local':
                if ike_phase_1_round.get('key', ""):
                    interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "key", ike_phase_1_round['key'])
                if ike_phase_1_round.get('key-pass', ""):
                    interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "key-pass", ike_phase_1_round['key-pass'])

                interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "username", ike_phase_1_round['eap-username'])
                interfaces.set_ike1_round_param(name, ipsecvpn_type, int(ike_phase_1_round['id']), "password", ike_phase_1_round['eap-password'])

    interfaces.commit()


def set_ipsecvpn_ike_version(name, version, mode):
    interfaces = api.SysrepoInterfaces(app_name="set ipsecvpn")
    interfaces.set_interface(name)
    interfaces.set_interface_param(name, "ike-version", version)
    if int(version) == 1 and mode:
        interfaces.set_interface_param(name, "ike-mode", mode)
    else:
        interfaces.unset_interface_param(name, "ike-mode")

    interfaces.commit()


def set_ipsecvpn(param, name, value):
    if not param or not value:
        print("Error applying ipsecvpn configuration")
        return

    interfaces = api.SysrepoInterfaces(app_name="set ipsecvpn")
    interfaces.set_interface(name)
    interfaces.set_interface_param(name, "type", "ipsecvpn")

    if param == 'enable' or param == 'management':
        interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
    elif param == 'local-ts':
        if value == 'none':
            interfaces.unset_ike2_local_ts(name)
        else:
            interfaces.set_ike2_local_ts(name, value)
    elif param == 'remote-ts':
        if value == 'none':
            interfaces.unset_ike2_remote_ts(name)
        else:
            interfaces.set_ike2_remote_ts(name, value)
    else:
        if value == 'none':
            interfaces.unset_interface_param(name, param)
        else:
            interfaces.set_interface_param(name, param, value)

    interfaces.commit()


def unset_ipsecvpn(param, ipsecvpn_type, name, value):
    if not param or not value:
        print("Error applying ipsecvpn configuration")
        return
    interfaces = api.SysrepoInterfaces(app_name="set ipsecvpn")
    interfaces.set_interface(name)
    if param == 'proposal':
        interfaces.unset_proposal(ipsecvpn_type, name, int(value))
    elif param == 'rounds':
        interfaces.unset_ike1_round(name, ipsecvpn_type, int(value))

    interfaces.commit()


def set_wifi(name, param, value):
    if not param or not value:
        print("Error applying Wi-Fi configuration")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")
    enable = ""
    if param == "enable":
        enable = value

    metric = ""
    if param == "metric":
        metric = value

    error_message = validate_wifi(interfaces, name, enable, metric, param, value)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "wifi")

        if param == "enable" and value != '""':
            if int(value) == 1:
                interfaces.set_interface_param(name, param, True)
            else:
                interfaces.set_interface_param(name, param, False)
        elif param == "management" and value != '""':
            if int(value) == 1:
                interfaces.set_interface_param(name, param, True)
            else:
                interfaces.set_interface_param(name, param, False)
        elif param == "wifi-mode" and value != '""':
            if "ap" in value:
                interfaces.set_interface_param(name, param, "AP")
            else:
                interfaces.set_interface_param(name, param, value)
        elif param == "ipv4-prefix":
            if value != '""' and value != '0.0.0.0/0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "ipv6-prefix":
            if value != '""' and value != '0000::/0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "gateway":
            if value != '""' and value != '0.0.0.0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "gateway6":
            if value != '""' and value != '::':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "dns":
            if value != '""' and value != '0.0.0.0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "dns6":
            if value != '""' and value != '::':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "trace-host":
            if value != '""' and value != 'none':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "ip-passthrough" and value != '""':
            if int(value) == 1:
                interfaces.set_interface_param(name, param, True)
            else:
                interfaces.set_interface_param(name, param, False)
        elif param == "nat" and value != '""':
            if int(value) == 1:
                interfaces.set_interface_param(name, param, True)
            else:
                interfaces.set_interface_param(name, param, False)
        elif (param == "band" or param == "channel" or param == "transmit-power" or param == "country") and value != '""':
            interfaces.set_interface_wifi_radio_param(name, param, value)
        elif (param == "key-mgmt" or param == "passphrase" or param == "max-clients" or param == "mac-acl") and value != '""':
            interfaces.set_interface_wifi_securty_param(name, param, value)
        elif (param == "hidden" or param == "isolation") and value != '""':
            if int(value) == 1:
                interfaces.set_interface_wifi_securty_param(name, param, True)
            else:
                interfaces.set_interface_wifi_securty_param(name, param, False)
        elif param == "wan-list" and value != '""':
            wans = None
            wans = value.split(',')
            if wans is not None:
                wan_copy = copy.copy(wans)
                ifaces_data = interfaces.get_interfaces()
                for iface_data in ifaces_data.values():
                    try:
                        ifname = iface_data.get("name")
                        if ifname in wan_copy:
                            wan_copy.remove(ifname)
                    except:
                        continue
                if len(wan_copy) > 0 and 'none' not in wan_copy:
                    print("%% Interface \'%s\' is not valid WAN interface !!!" % wan_copy.pop())
                else:
                    interfaces.unset_wifi_wan_interfaces(name)
                    if 'none' not in wans:
                        for wan_name in wans:
                            interfaces.set_wifi_wan_interface(name, wan_name)
        elif param == "allow-mac" and value != '""':
            interfaces.set_mac_allow(name, value)
        elif param == "deny-mac" and value != '""':
            interfaces.set_mac_deny(name, value)
        elif value != '""':
            interfaces.set_interface_param(name, param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def unset_wifi(name, param, value):
    if not param or not value:
        print("Error applying Wi-Fi configuration")
        return
    interfaces = api.SysrepoInterfaces(app_name="unset wifi")
    interfaces.set_interface(name)
    if param != "":
        if param == "mac-allow":
            interfaces.unset_mac_allow(name, value)
        if param == "mac-deny":
            interfaces.unset_mac_deny(name, value)

    interfaces.commit()


def set_eth_node(name, param, value):
    if not param or not value:
        print("Error applying ethernet configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = ""

    if param == "bootproto":
        error_message = validate_eth_node(interfaces, name, bootproto=value)
    elif param == "gateway":
        error_message = validate_eth_node(interfaces, name, gateway=value)
    elif param == "ipv4-prefix":
        error_message = validate_eth_node(interfaces, name, ipv4=value)
    elif param == "ipv6-prefix":
        error_message = validate_eth_node(interfaces, name, ipv6=value)
    elif param == "management":
        error_message = validate_eth_node(interfaces, name, management=value)
    elif param == "metric":
        error_message = validate_eth_node(interfaces, name, metric=value)
    else:
        error_message = validate_eth_node(interfaces, name)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "ethernet")

        if param == "bootproto":
            # Deleting static values for ip address, dns, gateway
            if value == "dhcp":
                interfaces.unset_interface_param(name, "ipv4-prefix")
                interfaces.unset_interface_param(name, "gateway")
                interfaces.unset_interface_param(name, "dns")
            elif value == "dhcp6":
                interfaces.unset_interface_param(name, "ipv6-prefix")
                interfaces.unset_interface_param(name, "gateway6")
                interfaces.unset_interface_param(name, "dns6")
            elif value == "dhcp46":
                interfaces.unset_interface_param(name, "ipv4-prefix")
                interfaces.unset_interface_param(name, "gateway")
                interfaces.unset_interface_param(name, "dns")
                interfaces.unset_interface_param(name, "ipv6-prefix")
                interfaces.unset_interface_param(name, "gateway6")
                interfaces.unset_interface_param(name, "dns6")

            interfaces.set_interface_param(name, param, value)
        if param == "ipv4-prefix":
            if value != '0.0.0.0/0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "ipv6-prefix":
            if value != '0000::/0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "gateway":
            if value != '0.0.0.0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "gateway6":
            if value != '::':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "ip-passthrough":
            bind_interface = check_bind_interface()
            if bind_interface.startswith('br-data') or bind_interface.startswith('br-tun'):
                bind_interface = bind_interface.replace("br-", "").strip()
                if bind_interface != name:
                    interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
                else:
                    print("ip-passthrough cannot be set on Kubernetes bind-interface!")
                    return
        elif (param == "management" or param == "link-state-propagation"):
            interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
        elif param == "trace-host":
            if value != 'none':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == "vlan":
            if value != '0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)

        # Unsetting a parameter
        # value -> the parameter that needs to be unset
        elif param == "no":
            if value == "ipv4":
                interfaces.unset_interface_param(name, "ipv4-prefix")
            elif value == "ipv6":
                interfaces.unset_interface_param(name, "ipv6-prefix")
            else:
                interfaces.unset_interface_param(name, value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_interface_param(name, param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_static_ipv4_eth_node(name, newip, newgw):
    if not newip or not newgw:
        print("Error applying static ipv4 ethernet configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set_static_ipv4")
    error_message = validate_eth_node(interfaces, name, bootproto="none", gateway=newgw, ipv4=newip)
    if error_message != "":
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)
        
    error_message = validate_eth_node(interfaces, name)
    if error_message != "":
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "ethernet")
        interfaces.set_interface_param(name, "bootproto", "none")
        interfaces.set_interface_param(name, "ipv4-prefix", newip)
        interfaces.set_interface_param(name, "gateway", newgw)
        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)

        
def set_lte_node(name, param, value):
    if not param or not value:
        print("Error applying LTE configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = ""

    if param == "metric":
        error_message = validate_lte_node(interfaces, name, metric=value)
    elif param == "trace-host":
        error_message = validate_lte_node(interfaces, name, trace_host=value)
    else:
        error_message = validate_lte_node(interfaces, name)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "lte")

        # Setting boolean parameters
        if param == "enable" or param == "management" or param == "ip-passthrough":
            interfaces.set_interface_param(name, param, True if int(value) == 1 else False)

        # Unsetting a parameter
        # value -> the parameter that needs to be unset
        elif param == "no":
            if value == "password":
                interfaces.unset_interface_param(name, "pass")
            else:
                interfaces.unset_interface_param(name, value)
        elif param == "password":
            interfaces.set_interface_param(name, "pass", value)
        elif param == "trace-host":
            if value == "none":
                interfaces.unset_interface_param(name, param)
            else:
                interfaces.set_interface_param(name, param, value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_interface_param(name, param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_nat_node(name, param, value):
    if not param or not value:
        print("Error applying NAT configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = validate_nat_node(name)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "nat")

        # Setting boolean parameters
        if param == "enable" or param == "port-forwarding-all":
            interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
        elif param == "wan-list":
            wans = None
            wans = value.split(',')
            if wans is not None:
                wan_copy = copy.copy(wans)
                ifaces_data = interfaces.get_interfaces()
                for iface_data in ifaces_data.values():
                    try:
                        ifname = iface_data.get("name")
                        if ifname in wan_copy:
                            wan_copy.remove(ifname)
                    except:
                        continue
                if len(wan_copy) > 0 and 'none' not in wan_copy:
                    print("Interface \'%s\' is not valid WAN interface !!!" % wan_copy.pop())
                else:
                    nat_iface = interfaces.session.get_data("/ucpe:config/interfaces:interface[name='{}']".format(name))
                    wan_list = []
                    if nat_iface and 'config' in nat_iface and 'interface' in nat_iface['config']:
                        for i in nat_iface['config']['interface']:
                            if 'wan-interface' in i:
                                wan_list = i['wan-interface']
                    for wan_iface in wan_list:
                        if wan_iface not in wans:
                            interfaces.unset_nat_wan_interface(name, wan_iface)
                    if 'none' not in wans:
                        for wan_name in wans:
                            interfaces.set_nat_wan_interface(name, wan_name)

        # Unsetting a parameter
        # value -> the parameter that needs to be unset
        elif param == "no":
            if value == "wan-list":
                interfaces.unset_nat_wan_interfaces(name)
            else:
                interfaces.unset_interface_param(name, value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_interface_param(name, param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_vpn_node(name, param, value):
    if not param or not value:
        print("Error applying VPN configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = validate_vpn(interfaces, name)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "openvpn")

        # Setting boolean parameters
        if param == "enable" or param == "management":
            interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
        elif param == "endpoint":
            interfaces.set_concentrator_endpoint(name, value)
        elif param == "port":
            #if int(value) in range(1, 65536):
            interfaces.set_concentrator_port(name, value)
            #else:
            #    error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            #    syslog.syslog(syslog.LOG_ERR, error_message)
            #    print(error_message)
            #    return

        # Not used (there are no parameters for unsetting for a VPN interface)
        elif param == "no":
            interfaces.unset_interface_param(name, value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_interface_param(name, param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_loopback(name, param, value):
    if not param or not value:
        print("Error applying loopback configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    error_message = ""
    if param == "ipv4-prefix":
        error_message = validate_loopback(interfaces, name, ipv4=value)
    elif param == "ipv6-prefix":
        error_message = validate_loopback(interfaces, name, ipv6=value)

    if error_message == "":
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, "type", "loopback")

        # Setting boolean parameters
        if param == "enable":
            interfaces.set_interface_param(name, param, True if int(value) == 1 else False)

        # Unsetting a parameter
        elif param == "no":
            if value == "ipv4-address":
                interfaces.unset_interface_param(name, "ipv4-prefix")
            elif value == "ipv6-address":
                interfaces.unset_interface_param(name, "ipv6-prefix")
            else:
                interfaces.unset_interface_param(name, value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_interface_param(name, param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_pppoe(name, param, value):
    if not param or not value:
        print("Error applying PPPoE configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    validate_interface_by_type(interfaces, name, 'pppoe')

    if param == "no":
        interfaces.unset_interface_param(name, value)
    else:
        interfaces.set_interface_param(name, param, value)
        if interfaces.get_interface_param(name, 'metric') is None:
            interfaces.set_interface_param(name, 'metric', '130')
        if interfaces.get_interface_param(name, 'mtu') is None:
            interfaces.set_interface_param(name, 'mtu', '1492')

    interfaces.commit()


def del_loopback(name):
    interfaces = api.SysrepoInterfaces(app_name="set interfaces")
    interfaces.unset_interface(name)
    interfaces.commit()

def set_wireguard(name, param, value):
    if not param or not value:
        print("Error applying WireGuard configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    validate_interface_by_type(interfaces, name, 'wireguard')

    if param == "no":
        interfaces.unset_interface_param(name, value)
    elif param == 'local-port':
        if int(value) in range(1, 65536):
            interfaces.set_interface_param(name, param, int(value))
        else:
            error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)
            return
    else:
        interfaces.set_interface_param(name, param, value)
        if interfaces.get_interface_param(name, 'mtu') is None:
            interfaces.set_interface_param(name, 'mtu', '1420')

    interfaces.commit()

def del_wireguard_peer(name, endpoint):
    if not endpoint:
        print("Error applying wireguard peer configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="unset wireguard peer")
    interfaces.unset_wireguard_peer(name, endpoint)
    interfaces.commit()


def set_wireguard_peer(name, endpoint, param, value):
    if not param or (param != 'no' and value == "''"):
        print("Error applying wireguard peer configuration.")
        return

    if check_if_concentrator_or_peer_params(name, 'concentrator'):
        print('Failed to add peer! Wireguard Concentrator is set on the interface.')
        return

    interfaces = api.SysrepoInterfaces(app_name="set wireguard peer")
    interfaces.set_interface_param(name, "type", "wireguard")
    interfaces.set_wireguard_peer(name, endpoint)

    if param == "no":
        if value != "''":
            interfaces.unset_wireguard_peer_param(name, endpoint, value)
        else:
            interfaces.unset_wireguard_peer(name, endpoint)
    elif param == 'peer-port':
        if int(value) in range(1, 65536):
            interfaces.set_wireguard_peer_param(name, endpoint, param, int(value))
        else:
            error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)
            return

    # Setting all other parameters that do not require additional validation/configuration
    else:
        interfaces.set_wireguard_peer_param(name, endpoint, param, value)

    interfaces.commit()

def set_wireguard_concentrator(name, param, value):
    if not param or (param != 'no' and value == "''"):
        print("Error applying wireguard concentrator configuration.")
        return

    if check_if_concentrator_or_peer_params(name, 'peers'):
        print('Failed to set Wireguard Concentrator! There are peers on this interface.')
        return

    interfaces = api.SysrepoInterfaces(app_name="set wireguard concentrator")
    interfaces.set_interface_param(name, "type", "wireguard")

    if param == 'concentrator-port':
        if int(value) in range(1, 65536):
            interfaces.set_wireguard_concentrator_param(name, param, int(value))
        else:
            error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)
            return
    else:
        interfaces.set_wireguard_concentrator_param(name, param, value)

    interfaces.commit()


def del_wireguard_concentrator(name):
    interfaces = api.SysrepoInterfaces(app_name="delete wireguard concentrator")
    interfaces.del_wireguard_concentrator(name)
    interfaces.commit()


def set_trace_host_opts(name, param, value, probe=None):
    if not param or not value:
        print("Error applying trace host configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set trace host opts")

    error_message = ""
    # Validations should be added for interval, count, timeout, ..

    if error_message == "":
        interfaces.set_interface(name)

        # Setting boolean parameters
        if param == "trace-state-propagation":
            interfaces.set_container_param(name, 'tracehost', param, True if int(value) == 1 else False)
        elif param == "enable":
            interfaces.set_container_param(name, 'tracehost', param, True if int(value) == 1 else False)

        # Unsetting a parameter
        elif param == "no":
            if value != "":
                interfaces.unset_container_param(name, 'tracehost', value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_container_param(name, 'tracehost', param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)

def set_trace_probe_host_opts(name, probe, param, value):
    if not param or (param != 'no' and value == "''"):
        print("Error applying trace host probe configuration.")
        return

    interfaces = api.SysrepoInterfaces(app_name="set trace host opts")

    error_message = ""
    # Validations should be added for interval, count, timeout, ..

    if error_message == "":
        # interfaces.set_interface(name)

        interfaces.set_tracehost_probe(name, int(probe))
        if param == "no":
            if value != "''":
                interfaces.unset_tracehost_probe_param(name, int(probe), value)
            else:
                interfaces.unset_tracehost_probe(name, int(probe))
        elif param == 'min-upload' or param == 'min-download':
            interfaces.set_tracehost_probe_param(name, int(probe), param, format(round(float(value),3), '.3f'))

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_tracehost_probe_param(name, int(probe), param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)

if __name__ == "__main__":
    argc = len(sys.argv)

    if 3 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)
    if sys.argv[1] == "set_eth_int":
        set_eth(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6], sys.argv[7], sys.argv[8], sys.argv[9], sys.argv[10], sys.argv[11], sys.argv[12], sys.argv[13], sys.argv[14], sys.argv[15], sys.argv[16])
    elif sys.argv[1] == "set_eth_node_int":
        set_eth_node(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_static_ipv4_eth_node":
        set_static_ipv4_eth_node(sys.argv[2], sys.argv[3], sys.argv[4])        
    elif sys.argv[1] == "set_lte_int":
        if sys.argv[2] == "unset":
            unset_lte(sys.argv[3], sys.argv[4])
        else:
            set_lte(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6], sys.argv[7], sys.argv[8], sys.argv[9], sys.argv[10])
    elif sys.argv[1] == "set_lte_node_int":
        set_lte_node(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_nat_int":
        set_nat_node(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_vpn_int":
        set_vpn(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6], sys.argv[7])
    elif sys.argv[1] == "set_vpn_node_int":
        set_vpn_node(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_ipsec_int":
        set_ipsec(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6], sys.argv[7], sys.argv[8], sys.argv[9], sys.argv[10], sys.argv[11], sys.argv[12],
                  sys.argv[13], sys.argv[14], sys.argv[15], sys.argv[16], sys.argv[17], sys.argv[18], sys.argv[19], sys.argv[20], sys.argv[21])
    elif sys.argv[1] == "set_ipsecvpn_int":
        if sys.argv[2] == "rounds":
            set_ipsecvpn_rounds(sys.argv[3], sys.argv[4], sys.argv[5])
        elif sys.argv[2] == "proposals":
            set_ipsecvpn_proposals(sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6], sys.argv[7])
        elif sys.argv[2] == "dh-groups":
            set_ipsecvpn_dh_groups(sys.argv[3], sys.argv[4], sys.argv[5])
        elif sys.argv[2] == "ike-version":
            set_ipsecvpn_ike_version(sys.argv[3], sys.argv[4], sys.argv[5])
        elif sys.argv[2] == 'unset':
            unset_ipsecvpn(sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6])
        else:
            set_ipsecvpn(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_wifi_int":
        if sys.argv[2] == "unset":
            unset_wifi(sys.argv[3], sys.argv[4], sys.argv[5])
        else:
            set_wifi(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_loopback_int":
        if sys.argv[3] == "delete":
            del_loopback(sys.argv[2])
        else:
            set_loopback(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_pppoe_int":
        if sys.argv[3] == "delete":
            # this will do the job for pppoe deletion
            del_loopback(sys.argv[2])
        else:
            set_pppoe(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_trace_host_opts":
        set_trace_host_opts(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_trace_probe_host_opts":
        set_trace_probe_host_opts(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    elif sys.argv[1] == "set_wireguard_int":
        if sys.argv[3] == "delete":
            # this will do the job for wireguard deletion
            del_loopback(sys.argv[2])
        else:
            set_wireguard(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_wireguard_peer":
        if sys.argv[4] == "delete":
            del_wireguard_peer(sys.argv[2], sys.argv[3])
        else:
            print(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
            set_wireguard_peer(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    elif sys.argv[1] == "set_wireguard_concentrator":
        if sys.argv[3] == "delete":
            print(sys.argv[2], sys.argv[3])
            del_wireguard_concentrator(sys.argv[2])
        else:
            print(sys.argv[2], sys.argv[3], sys.argv[4])
            set_wireguard_concentrator(sys.argv[2], sys.argv[3], sys.argv[4])
    else:
        syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)
