#!/usr/bin/env python3
import json
import os
import shutil
import subprocess
import sys
import tarfile
import tempfile
import uuid

unique_prefix = uuid.uuid4().hex[:6].upper()
UCPE_STATUS_FILE = "/config/_hb/ucpe_status.json"


def vpn(key_type, dir_name, file_name):
    if key_type not in ('ca', 'cert', 'key'):
        raise Exception('Unknown vpn key type {} !'.format(key_type))

    unique_file_name = unique_prefix + file_name
    shutil.copy('{}/{}'.format(dir_name, file_name),
                '/config/etc/openvpn/{}'.format(unique_file_name))
    os.chmod('/config/etc/openvpn/{}'.format(unique_file_name), 0o600)

    try:
        subprocess.run(['/bin/sed', '-i', 's#^{} .*#{} "{}"#'.
                        format(key_type, key_type, unique_file_name),
                        '/config/etc/openvpn/client.conf'])
    except subprocess.CalledProcessError as e:
        raise Exception('Error vpn {}'.format(e))
    except OSError as e:
        raise Exception('Error vpn {}'.format(e))


def web(key_type, dir_name, file_name):
    if key_type not in ('SSLCertificateFile', 'SSLCertificateKeyFile', 'bashrc',
                        'keyfile', 'ssl_key_file', 'key_file', 'key', 'ssl_key',
                        'certfile', 'ssl_cert_file', 'cert_file', 'cert', 'ssl_cert'):
        raise Exception('Unknown ssl key type {} !'.format(key_type))

    unique_file_name = unique_prefix + file_name
    shutil.copy('{}/{}'.format(dir_name, file_name),
                '/etc/apache2/{}'.format(unique_file_name))
    os.chmod('/etc/apache2/{}'.format(unique_file_name), 0o644)

    try:
        if key_type == 'key' or key_type == 'cert':
            return
        elif key_type == 'ssl_key' or key_type == 'ssl_cert':
            subprocess.run(['/bin/sed', '-i', 's#{}\": \".*#{}\": \"/etc/apache2/{}\",#'.
                            format(key_type, key_type, unique_file_name),
                            '/etc/sslrl/sslrl.json'])
            return
        elif key_type == 'bashrc':
            subprocess.run(['/bin/sed', '-i', 's#function curl.*#function curl\(\) { /usr/bin/curl "\$@" --key /etc/apache2/%skey --cert /etc/apache2/%scrt;}#' % (unique_file_name[0:-3], unique_file_name[0:-3]), '/root/admin-openrc.sh'])
            return

        subprocess.run(['/bin/sed', '-i', 's#{} .*#{} /etc/apache2/{}#'.
                        format(key_type, key_type, unique_file_name),
                        '/etc/apache2/conf-available/splash.conf'])

        subprocess.run(['/bin/sed', '-i', 's#{} .*#{} /etc/apache2/{}#'.
                        format(key_type, key_type, unique_file_name),
                        '/etc/apache2/sites-available/tveapi-wsgi.conf'])
        #
        subprocess.run(['/bin/sed', '-i', 's#{}.*#{}=/etc/apache2/{}#'.
                        format(key_type, key_type, unique_file_name),
                        '/etc/tveapi/tveapi.conf'])

    except subprocess.CalledProcessError as e:
        raise Exception('Error web {}'.format(e))
    except OSError as e:
        raise Exception('Error web {}'.format(e))


def heartbeat(key_type, dir_name, file_name):
    if key_type not in ('ssl_certfile', 'ssl_keyfile', 'ssl_ca_certs'):
        raise Exception('Unknown heartbeat key type {} !'.format(key_type))

    unique_file_name = unique_prefix + file_name
    shutil.copy('{}/{}'.format(dir_name, file_name),
                '/scripts/{}'.format(unique_file_name))
    os.chmod('/scripts/{}'.format(unique_file_name), 0o600)

    try:
        subprocess.run(['/bin/sed', '-i', 's#{}=.*,#{}="/scripts/{}",#'.
                        format(key_type, key_type, unique_file_name),
                        '/scripts/hb_update.py'])
    except subprocess.CalledProcessError as e:
        raise Exception('Error heartbeat {}'.format(e))
    except OSError as e:
        raise Exception('Error heartbeat {}'.format(e))


if __name__ == '__main__':
    if os.path.exists(sys.argv[1]):
        with tempfile.TemporaryDirectory() as tar_temp_dir:
            with tarfile.open(sys.argv[1]) as archive:
                archive.extractall(tar_temp_dir)
                vpnca_start = [filename for filename in os.listdir(tar_temp_dir) if filename.startswith('vpnca')][0]
                serial = vpnca_start[5:-4]
                has_custom_cert = False

                if os.path.isfile('{}/vpnca{}.crt'.format(tar_temp_dir, serial)) \
                   and os.path.isfile('{}/vpn{}.crt'.format(tar_temp_dir, serial)) \
                   and os.path.isfile('{}/vpn{}.key'.format(tar_temp_dir, serial)):
                    vpn('ca', tar_temp_dir, 'vpnca{}.crt'.format(serial))
                    vpn('cert', tar_temp_dir, 'vpn{}.crt'.format(serial))
                    vpn('key', tar_temp_dir, 'vpn{}.key'.format(serial))
                    has_custom_cert = True

                if os.path.isfile('{}/web{}.crt'.format(tar_temp_dir, serial)) \
                   and os.path.isfile('{}/web{}.key'.format(tar_temp_dir, serial)):
                    web('SSLCertificateFile', tar_temp_dir, 'web{}.crt'.format(serial))
                    web('SSLCertificateKeyFile', tar_temp_dir, 'web{}.key'.format(serial))
                    web('ssl_cert_file', tar_temp_dir, 'web{}.crt'.format(serial))
                    web('certfile', tar_temp_dir, 'web{}.crt'.format(serial))
                    web('cert_file', tar_temp_dir, 'web{}.crt'.format(serial))
                    web('cert', tar_temp_dir, 'web{}.crt'.format(serial))
                    web('ssl_cert', tar_temp_dir, 'web{}.crt'.format(serial))
                    web('ssl_key_file', tar_temp_dir, 'web{}.key'.format(serial))
                    web('keyfile', tar_temp_dir, 'web{}.key'.format(serial))
                    web('key_file', tar_temp_dir, 'web{}.key'.format(serial))
                    web('key', tar_temp_dir, 'web{}.key'.format(serial))
                    web('ssl_key', tar_temp_dir, 'web{}.crt'.format(serial))
                    web('bashrc', tar_temp_dir, 'web{}.crt'.format(serial))
                    has_custom_cert = True

                if os.path.isfile('{}/hbca{}.crt'.format(tar_temp_dir, serial)) \
                   and os.path.isfile('{}/hb{}.crt'.format(tar_temp_dir, serial)) \
                   and os.path.isfile('{}/hb{}.key'.format(tar_temp_dir, serial)):
                    heartbeat('ssl_ca_certs', tar_temp_dir, 'hbca{}.crt'.format(serial))
                    heartbeat('ssl_certfile', tar_temp_dir, 'hb{}.crt'.format(serial))
                    heartbeat('ssl_keyfile', tar_temp_dir, 'hb{}.key'.format(serial))
                    has_custom_cert = True

                if has_custom_cert and os.path.isfile(UCPE_STATUS_FILE):
                    with open(UCPE_STATUS_FILE, "r+", encoding='utf-8') as json_file:
                        data = json.load(json_file)
                        data['ucpe:status']['certs'] = 'user'
                        json_file.seek(0)
                        json.dump(data, json_file)
                        json_file.truncate()
                        json_file.close()
    else:
        raise Exception('Error keys archive file ({}) does not exist'.format(sys.argv[1]))

    sys.exit(0)
