#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import syslog
import ipaddress
import time

from sysrepo_interfaces import sysrepo_interfaces3 as api_if
from sysrepo_vrrp import sysrepo_vrrp as api


INTERFACE_OLD = None
PRIORITY_OLD = None
VIPS_OLD = None


def verify_interface(ifname):

    if ifname.startswith('data'):
        pass
    else:
        print("Invalid interface, please set physical interface (data0..dataX)")
        sys.exit(1)

    interfaces = api_if.SysrepoInterfaces(app_name="get interfaces")
    interface = next((intf for intf in interfaces.get_interfaces().keys() if intf == ifname), None)

    if interface in interfaces.get_interfaces().keys():
        pass
    else:
        print(f"Interface {ifname} does not exist")
        syslog.syslog(syslog.LOG_ERR, f"Interface {ifname} does not exist")
        sys.exit(1)


def verify_priority(priority):
    priority = int(priority)
    if 1 <= priority <= 255:
        pass
    else:
        print("Invalid priority, please set priority value between (1-254)")
        sys.exit(1)


def verify_ips(ips):
    ip_addrs = ips.split(",")
    ipv4_addresses = []
    ipv6_addresses = []

    for ip in ip_addrs:
        try:
            # Attempt to create an IPv4Address object
            ipv4 = ipaddress.IPv4Address(ip)
            ipv4_addresses.append(str(ipv4))
        except ipaddress.AddressValueError:
            try:
                # Attempt to create an IPv6Address object
                ipv6 = ipaddress.IPv6Address(ip)
                ipv6_addresses.append(str(ipv6))
            except ipaddress.AddressValueError:
                # If it's neither IPv4 nor IPv6, raise an error
                print(f"{ip} is not a valid IP address")
                ipv4_addresses.clear()
                ipv6_addresses.clear()
                syslog.syslog(syslog.LOG_ERR, f"{ip} is not a valid IP address")
                sys.exit(1)

    # Check if both IPv4 and IPv6 addresses exist
    if ipv4_addresses and ipv6_addresses:
        print("IPv4 and IPv6 addresses cannot be mixed")
        ipv4_addresses.clear()
        ipv6_addresses.clear()
        syslog.syslog(syslog.LOG_ERR, "IPv4 and IPv6 addresses cannot be mixed")
        sys.exit(1)

    # Check if there are any IP addresses in the input string
    if not ipv4_addresses and not ipv6_addresses:
        print("No IP addresses found in input string")
        ipv4_addresses.clear()
        ipv6_addresses.clear()
        syslog.syslog(syslog.LOG_ERR, "No IP addresses found in input string")
        sys.exit(1)

    return ",".join(ipv4_addresses) or ",".join(ipv6_addresses)


def check_if_all_params_are_present(vr_id):
    sr = api.SysrepoVRRP(app_name=f"get VRRP instance ID: {vr_id}")
    vr_instance = sr.get_running_vr(vr_id)
    for vr_id, config in vr_instance.items():
        if 'interface' in config and 'vip' in config and 'peer' in config and 'priority' in config:
            return True


def verify_vip_peers_ip_version(vr_id):
    sr = api.SysrepoVRRP(app_name=f"get VRRP instance ID: {vr_id}")
    vr_instance = sr.get_running_vr(vr_id)
    try:
        for vr_id, config in vr_instance.items():
            vip = config['vip'].split(",")
            peer = config['peer'].split(",")

            # Check VIP address
            vip_versions = set()
            for address in vip:
                try:
                    address_version = ipaddress.ip_address(address).version
                    vip_versions.add(address_version)
                except ValueError:
                    print(f"Invalid VIP address: {address}")

            # Check Peer addresses
            peer_versions = set()
            for address in peer:
                try:
                    address_version = ipaddress.ip_address(address).version
                    peer_versions.add(address_version)
                except ValueError:
                    print(f"Invalid Peer address: {address}")

            # Check if both VIP and Peer are using the same IP version
            if len(vip_versions) == 1 and len(peer_versions) == 1 and vip_versions == peer_versions:
                return True
            else:
                return False
    except:
        print("VIP has not yet been set!")


def valid_ip_address(ips):
    ip_addr = ips.split(',')
    for IP in ip_addr:
        try:
            return "IPv4" if type(ipaddress.ip_address(IP)) is ipaddress.IPv4Address else "IPv6"
        except ValueError:
            return "Invalid IP address"


def set_vrrp_interface(vr_id, ifname):
    if not vr_id or not ifname:
        print("Error applying VRRP interface configuration.")
        return

    verify_interface(ifname)

    sr = api.SysrepoVRRP(app_name="set VRRP interface")

    if sr.get_virtual_router(vr_id):
        sr.set_virtual_router_param(vr_id, 'interface', ifname)
        sr.commit()
    else:
        sr.set_virtual_router(vr_id)
        sr.commit()
        sr.set_virtual_router_param(vr_id, 'interface', ifname)
        sr.commit()


def set_vrrp_priority(vr_id, priority):
    if not vr_id or not priority:
        print("Error applying VRRP priority configuration.")
        return

    verify_priority(priority)

    sr = api.SysrepoVRRP(app_name="set VRRP priority")

    if sr.get_virtual_router(vr_id):
        sr.set_virtual_router_param(vr_id, 'priority', priority)
        sr.commit()
    else:
        sr.set_virtual_router(vr_id)
        sr.commit()
        sr.set_virtual_router_param(vr_id, 'priority', priority)
        sr.commit()


def set_vrrp_vips(vr_id, vips):
    if not vr_id or not vips:
        print("Error applying VRRP virtual IPs configuration.")
        return

    vips_strings = verify_ips(vips)

    # only one VIP address could be used due to bug in VRRP plugin
    vips_list = vips_strings.split(",")
    if len(vips_list) > 1:
        print('Please set only one VIP address')
        return

    sr = api.SysrepoVRRP(app_name="set VRRP virtual IPs")

    # below code is needed due to bug in VPP, if vip ip is replaced with ipv6 where peer is ipv4
    # VPP converts the ipv6 to ipv4 and applies it in the VRRP instance
    # if this instance is later deleted results in VPP hang
    peer_conf = sr.get_virtual_router_param(vr_id, 'peer')
    enabled = sr.get_virtual_router_param(vr_id, 'enable')
    if peer_conf and enabled:
        peer_ip_version = valid_ip_address(peer_conf)
        vip_ip_version = valid_ip_address(vips)
        if peer_ip_version != vip_ip_version:
            print(f'You are trying to set {vip_ip_version} VIP address, where peer/s are set with {peer_ip_version}, please stop the instance first prior making changes!')
            return
        else:
            if sr.get_virtual_router(vr_id):
                sr.set_virtual_router_param(vr_id, 'vip', vips_strings)
                sr.commit()
            else:
                sr.set_virtual_router(vr_id)
                sr.commit()
                sr.set_virtual_router_param(vr_id, 'vip', vips_strings)
                sr.commit()
    else:
        if sr.get_virtual_router(vr_id):
            sr.set_virtual_router_param(vr_id, 'vip', vips_strings)
            sr.commit()
        else:
            sr.set_virtual_router(vr_id)
            sr.commit()
            sr.set_virtual_router_param(vr_id, 'vip', vips_strings)
            sr.commit()


def set_vrrp_peers(vr_id, peers):
    if not vr_id or not peers:
        print("Error applying VRRP peers configuration.")
        return
    
    peers_strings = verify_ips(peers)

    sr = api.SysrepoVRRP(app_name="set VRRP peers")

    # below code is needed due to bug in VPP, if peer ip is replaced with ipv6 where peer is ipv4
    # VPP converts the ipv6 to ipv4 and applies it in the VRRP instance
    # if this instance is later deleted results in VPP hang
    vip_conf = sr.get_virtual_router_param(vr_id, 'vip')
    enabled = sr.get_virtual_router_param(vr_id, 'enable')
    if vip_conf and enabled:
        peer_ip_version = valid_ip_address(peers)
        vip_ip_version = valid_ip_address(vip_conf)
        if vip_ip_version != peer_ip_version:
            print(f'You are trying to set {peer_ip_version} peer/s address/es, where VIP is set with {vip_ip_version}, please stop the instance first prior making changes!')
            return
        else:
            if sr.get_virtual_router(vr_id):
                sr.set_virtual_router_param(vr_id, 'peer', peers_strings)
                sr.commit()
            else:
                sr.set_virtual_router(vr_id)
                sr.commit()
                sr.set_virtual_router_param(vr_id, 'peer', peers_strings)
                sr.commit()
    else:
        if sr.get_virtual_router(vr_id):
            sr.set_virtual_router_param(vr_id, 'peer', peers_strings)
            sr.commit()
        else:
            sr.set_virtual_router(vr_id)
            sr.commit()
            sr.set_virtual_router_param(vr_id, 'peer', peers_strings)
            sr.commit()


def enable_vrrp(vr_id, param):
    if not vr_id or not param:
        print(f"Error enabling VRRP instance with ID: {vr_id}.")
        return

    if param == "true":
        sr = api.SysrepoVRRP(app_name=f"enable VRRP instance with id {vr_id}")
        if sr.get_virtual_router(vr_id):
            if check_if_all_params_are_present(vr_id):
                if verify_vip_peers_ip_version(vr_id):
                    sr.set_virtual_router_param(vr_id, 'enable', param)
                    sr.commit()
                else:
                    print("Please use either IPv4 or IPv6 for VIP and peer/s!")
            else:
                print(f"Please set all parameters for the current VR")
        else:
            if check_if_all_params_are_present(vr_id):
                if verify_vip_peers_ip_version(vr_id):
                    sr.set_virtual_router(vr_id)
                    sr.commit()
                    sr.set_virtual_router_param(vr_id, 'enable', param)
                    sr.commit()
                else:
                    print("Please use either IPv4 or IPv6 for VIP and peer/s!")
            else:
                print(f"Please set all parameters for the current VR")
    else:
        sr = api.SysrepoVRRP(app_name=f"disable VRRP instance with id {vr_id}")
        if sr.get_virtual_router(vr_id):
            sr.set_virtual_router_param(vr_id, 'enable', param)
            sr.commit()
        else:
            sr.set_virtual_router(vr_id)
            sr.commit()
            sr.set_virtual_router_param(vr_id, 'enable', param)
            sr.commit()


def get_vr_state(vr_id):
    if not vr_id:
        print(f"Error getting VR state.")
        return

    sr = api.SysrepoVRRP(app_name="get VR state")

    if check_if_all_params_are_present(vr_id):
        if sr.get_virtual_router_param(vr_id, 'enable'):
            # in case we have a race condition, when user requests VR status right after applying instance
            # and the instance is not yet applied, timer is added to wait for the instance to get applied
            # otherwise buffer leak could occur and vpp will hang
            time.sleep(3)
            vr_state = sr.get_vr_state(vr_id)
            if vr_state:
                print(f"The current state of this VR is: {vr_state}")
            else:
                print("No data")
        else:
            print("Current VR has not yet been enabled")
    else:
        print("Please set all parameters for the current VR")


def del_vrrp(vr_id):
    if not vr_id:
        print("Error deleting VRRP ID configuration.")
        return

    sr = api.SysrepoVRRP(app_name=f"del VRRP instance with ID:{vr_id}")
    if sr.get_virtual_router(vr_id):
        sr.unset_virtual_router(vr_id)
        sr.commit()
    else:
        sr.set_virtual_router(vr_id)
        sr.commit()
        sr.unset_virtual_router(vr_id)
        sr.commit()


if __name__ == "__main__":

    argc = len(sys.argv)

    if argc < 3:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)

    if sys.argv[1] == "set_vrrp_interface":
        set_vrrp_interface(int(sys.argv[2]), sys.argv[3])
    elif sys.argv[1] == "set_vrrp_priority":
        set_vrrp_priority(int(sys.argv[2]), sys.argv[3])
    elif sys.argv[1] == "set_vrrp_vips":
        set_vrrp_vips(int(sys.argv[2]), sys.argv[3])
    elif sys.argv[1] == "set_vrrp_peers":
        set_vrrp_peers(int(sys.argv[2]), sys.argv[3])
    elif sys.argv[1] == "enable_vrrp":
        enable_vrrp(int(sys.argv[2]), sys.argv[3])
    elif sys.argv[1] == "get_vrrp_status":
        get_vr_state(int(sys.argv[2]))
    elif sys.argv[1] == "del_vrrp":
        del_vrrp(int(sys.argv[2]))
    else:
        syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)

