#!/bin/bash
#set -x

source /scripts/ipv6_functions

GLANCE_CONFIG_FILE="/etc/glance/glance-api.conf"
TMPDIR=`mktemp -d`
TMPDIR_MYSQL="${TMPDIR}/mysql"
MYSQL_BACKUP_FILE="mysql_backup.tar.gz"
GLANCE_BACKUP_FILE="glance_backup.tar.gz"
TMPDIR_ETCD="${TMPDIR}/etcd"
ETCD_BACKUP_FILE="etcd_backup.tar.gz"
ETCD_DATADIR="/var/lib/etcd/default"
BACKUP_LOCATION="/backups"
MARIADB_CONFIG="/etc/mysql/mariadb.conf.d/50-server.cnf"

prep_directories()
{
    local destination=${1}
    local server=${2}
    local directory=${3}
    local username=${4}
    local password=${5}
    hostname=$(hostname)

    # IPv6 support
    if [[ $server == *":"* ]]; then
        _prepareForIPv6 ${server} server
    fi

    ftp_login="user $username $password"
    if [ -z $username ]; then
        ftp_login="user anonymous"
    fi

    is_remote=`grep -rn "/SAN/mysql" $MARIADB_CONFIG || true`
    is_attached_volume_mode=`grep -rn "/attached_volume/mysql" $MARIADB_CONFIG || true`

    if [[ $destination == "san_storage" ]]; then
        if [[ -z $is_remote ]]; then
            echo "Remote SAN storage is not available."
            exit 1
        fi
        # Get remote disk
        available_iscsi_disk=`lsblk -S | grep iscsi | awk '{print $1}'`
        if [[ $available_iscsi_disk ]]; then
            available_size_disk_orig=`df | grep ${available_iscsi_disk} | awk '{print $4+0}'`;
            available_size_disk=$(( $available_size_disk_orig - 1048576 ))
        fi
        mariadb_size=`du -s /SAN/mysql/ | awk '{print $1+0}'`
        glance_size=`du -s /SAN/glance/ | awk '{print $1+0}'`
    elif [[ $destination == "attached_volume_storage" ]]; then
        if [[ -z $is_attached_volume_mode ]]; then
            echo "Attached volume is not available."
            exit 1
        fi
        available_size_disk_orig=`df | awk '{ if ($6=="/attached_volume") print $4+0}'`
        available_size_disk=$(( available_size_disk_orig - 1048576 ))
        mariadb_size=`du -s /attached_volume/mysql/ | awk '{print $1+0}'`
        glance_size=`du -s /attached_volume/glance/ | awk '{print $1+0}'`
    else
        available_size_disk_orig=`df | awk '{ if ($6=="/") print $4+0}'`
        available_size_disk=$(( available_size_disk_orig - 1048576 ))
        mariadb_size=`du -s /var/lib/mysql/ | awk '{print $1+0}'`
        glance_size=`du -s /var/lib/glance/ | awk '{print $1+0}'`
    fi

    if [[ $available_size_disk -le 0 ]]; then
        available_size_disk=0
    fi
    db_size=$((${mariadb_size%.*}+${glance_size%.*}))
    local_space_req=$(( 2 * db_size ))

    if [[ $destination == "san_storage" ]] && [[ $available_size_disk -le $local_space_req ]]; then
            rdisk_mb=$(( available_size_disk_orig / 1024 ))
            rdisk_req=$(( local_space_req / 1024 ))
            echo "Not enough free space on remote SAN storage [${rdisk_mb}MB] to create the DB backup archive [${rdisk_req}MB]."
            exit 1
    elif [[ $destination == "attached_volume_storage" ]] && [[ $available_size_disk -le $local_space_req ]]; then
        sdisk_mb=$(( available_size_disk_orig / 1024 ))
        sdisk_req=$(( local_space_req / 1024 ))
        echo "Not enough free space on attached volume [${sdisk_mb}MB] to create the DB backup archive [${sdisk_req}MB]."
        exit 1
    else
        if [[ $available_size_disk -le $local_space_req ]]; then
            ldisk_mb=$(( available_size_disk_orig / 1024 ))
            ldisk_req=$(( local_space_req / 1024 ))
            echo "Not enough free space on local file system [${ldisk_mb}MB] to create the DB backup archive [${ldisk_req}MB]."
            exit 1
        fi
    fi

    if [[ "$destination" != *"_storage"* ]]; then
        if [ "$destination" == 'scp' ]; then
            result=`sshpass -p ${password} ssh -T -o StrictHostKeyChecking=no -o LogLevel=ERROR ${username}@${server} 2>&1 <<EOF
cd $directory 2>&1
ls -la |grep "db_backup_${hostname}"
df $directory |awk 'NR>1'
EOF
`
        elif [ "$destination" == 'ftp' ]; then
            result=`ftp -n ${server} 2>&1 <<EOF
$ftp_login
cd $directory
ls -la
bye
EOF
`
        fi
        res=$?
        if [[ $result == *"No such file or directory"* || $result == *"Failed to change directory"* ]]; then
            echo "Directory (${directory}) does not exist on the remote storage"
            exit 1
        elif [[ $result == *"Could not resolve hostname"* || $result == *"No route to host"* || $result == *"Name or service not known"* ]]; then
            echo "No access to ${server}"
            exit 1
        elif [[ $result == *"Permission denied, please try again"* ]]; then
            echo "SSH connection failed"
            exit 1
        elif [[ $result == *"Login incorrect."* ]]; then
            echo "Login failed. Please login with correct USERNAME and PASSWORD"
            exit 1
        elif [[ $result == *"This FTP server is anonymous only"* ]]; then
            echo "Login failed. This FTP server is anonymous only."
            exit 1
        fi
        if [[ "$destination" == 'scp' && "$res" == "255" ]] ; then
            echo "SSH connection failed"
            exit 1
        fi

        if [ "$destination" == 'ftp' ]; then
            if [ $username ]; then
                ssh_params="-p ${password} sftp -q -o StrictHostKeyChecking=no ${username}@${server}"
                free_space=`sshpass ${ssh_params} <<EOF|awk 'NR == 4{print $3+0}'
cd $directory
df
bye
EOF
`
                res=$?
                if [[ "$res" == 0 ]]; then
                    if [[ $db_size -ge $free_space ]]; then
                        echo "Not enough space on the remote storage to perform backup!"
                        exit 1
		    fi
                fi
            fi
            result=`echo $result | grep -o "db_backup_${hostname}[^ ]*"`
        fi
        readarray -t x <<<"$result"

        if [ "$destination" == 'scp' ]; then
            free_space=$(echo ${x[-1]} | awk '{print $4}')

            if [[ $db_size -ge $free_space ]]; then
                echo "Not enough space on the remote storage to perform backup!"
                exit 1
            fi

            unset 'x[${#x[@]}-1]'
        fi

        if [[ ${#x[@]} -ge 5 ]]; then
            if [ "$destination" == 'scp' ]; then
                del=$(echo ${x[0]} | awk '{print $9}')
                if [[ $del != "" ]]; then 
                    sshpass -p ${password} ssh -T -o StrictHostKeyChecking=no -o LogLevel=ERROR ${username}@${server} rm ${directory}/${del}
                fi
            else
                del=$(echo ${x[0]})
                ftp -n ${server} <<EOF
$ftp_login
cd $directory
delete ${del}
bye
EOF
            fi

        fi
    fi

    backup_location="$BACKUP_LOCATION"
    if [[ $destination == "san_storage" ]]; then
        backup_location="/SAN$BACKUP_LOCATION"
        mkdir -p /SAN$TMPDIR_MYSQL
        chmod 777 -R /SAN$TMPDIR
        mkdir -p /SAN$TMPDIR_ETCD
        if [[ ! -d "/SAN$BACKUP_LOCATION" ]]; then
            mkdir -p /SAN$BACKUP_LOCATION
            mkdir -p /SAN$BACKUP_LOCATION/mysql
            chmod 777 -R /SAN$BACKUP_LOCATION
        fi
        backup_mysql "/SAN$TMPDIR_MYSQL" "/SAN$BACKUP_LOCATION"
        backup_glance "/SAN$BACKUP_LOCATION"
        backup_etcd "/SAN$TMPDIR_ETCD" "/SAN$BACKUP_LOCATION"
    elif [[ $destination == "attached_volume_storage" ]]; then
        backup_location="/attached_volume$BACKUP_LOCATION"
        mkdir -p /attached_volume$TMPDIR_MYSQL
        chmod 777 -R /attached_volume$TMPDIR_MYSQL
        mkdir -p /attached_volume$TMPDIR_ETCD
        chmod 777 -R /attached_volume$TMPDIR_ETCD
        if [[ ! -d "/attached_volume$BACKUP_LOCATION" ]]; then
            mkdir -p /attached_volume$BACKUP_LOCATION
            mkdir -p /attached_volume$BACKUP_LOCATION/mysql
            chmod 777 -R /attached_volume$BACKUP_LOCATION
        fi
        backup_mysql "/attached_volume$TMPDIR_MYSQL" "/attached_volume$BACKUP_LOCATION"
        backup_glance "/attached_volume$BACKUP_LOCATION"
        backup_etcd "/attached_volume$TMPDIR_ETCD" "/attached_volume$BACKUP_LOCATION"

    else
        mkdir -p $TMPDIR_MYSQL
        mkdir -p $TMPDIR_ETCD

        chmod 777 -R $TMPDIR
        if [[ ! -d $BACKUP_LOCATION ]]; then
            mkdir $BACKUP_LOCATION
        fi
        backup_mysql "$TMPDIR_MYSQL" "$BACKUP_LOCATION"
        backup_glance "$BACKUP_LOCATION"
        backup_etcd "$TMPDIR_ETCD" "$BACKUP_LOCATION"

    fi

    cd $backup_location
    databases_backups=`ls | grep backup.tar.gz`
    # Backup version
    backup_name="db_backup_"$hostname"_`date +"%Y%m%d%H%M"`.tar.gz"

    if [[ "$destination" == *"_storage"* ]]; then
        oldest_backup=`ls -ltr | grep "db_backup_$hostname*" | awk 'NR==1{print $9}'`
        backups_count=`ls -ltr | grep "db_backup_$hostname*" | wc -l`

        if [[ $backups_count -ge 5 ]]; then
            rm $oldest_backup
        fi

        tar czf $backup_name $databases_backups
        echo "Successfully created backup of databases in '$backup_location' directory with '$backup_name' name."
    else
        tar czf $backup_name $databases_backups
        if [ "$destination" == 'scp' ]; then
            # IPv6 support
            if [[ $server == *":"* ]]; then
                _prepareForIPv6 ${server} server
            fi
            sshpass -p ${password} scp -o StrictHostKeyChecking=no ${backup_location}/${backup_name} ${username}@${server}:${directory}
        elif [ "$destination" == 'ftp' ]; then
            output=`ftp -n ${server} 2>&1 <<EOF
$ftp_login
cd $directory
lcd $backup_location
put $backup_name
bye
EOF
`
        fi
        rm -f $backup_location/$backup_name
        if [[ $output == *"netout: Broken pipe"* ]]; then
            echo "Failed to transfer backup '$backup_name' on remote host($server) via $destination."
        else
            echo "Successfully transferred backup '$backup_name' on remote host($server) via $destination."
        fi
    fi

    if [[ $backup_location == *"/SAN"* ]]; then
        rm -rf "/SAN$TMPDIR" && rm -rf /SAN$BACKUP_LOCATION/*backup.tar.gz
    elif [[ $backup_location == *"/attached_volume"* ]]; then
        rm -rf "/attached_volume$TMPDIR" && rm -rf /attached_volume$BACKUP_LOCATION/*backup.tar.gz
    else
        rm -rf "$TMPDIR" && rm -rf $BACKUP_LOCATION/*backup.tar.gz
    fi

}

backup_mysql ()
{
    #try to guess mysql passwd
    mysql -uroot --password='' -ANe"SELECT COUNT(1) FROM mysql.user where user='root'" &>/dev/null && true
    res=$?
    if [ "$res" == "0" ] ; then
        MARIABACKUPCMD="/usr/bin/mariabackup --user=root --port=3306 --host=localhost "
    else
        MARIABACKUPCMD="/usr/bin/mariabackup --user=root --password=root --port=3306 --host=localhost  "
    fi

    echo "Backing up mysql..."
    ${MARIABACKUPCMD} --backup --target-dir=${1}  &>/dev/null
    pushd ${1} >/dev/null
    tar czf ${2}/${MYSQL_BACKUP_FILE} .
    popd >/dev/null
    rm -rf $TMPDIR_MYSQL
}


backup_etcd ()
{
    systemctl stop etcd
    pushd ${1} >/dev/null
    cp -r ${ETCD_DATADIR}/* .
    tar czf ${2}/${ETCD_BACKUP_FILE} .
    popd >/dev/null
    rm -rf $TMPDIR_ETCD
    systemctl start etcd
}

backup_glance ()
{
    if [[ ! -f $GLANCE_CONFIG_FILE ]] ; then
      return
    fi

    GLANCE_DIR=$(cat $GLANCE_CONFIG_FILE  | grep ^filesystem_store_datadir | cut -f 2 -d "=" | head -1 | xargs)

    echo "Backing up glance..."
    pushd ${GLANCE_DIR} >/dev/null
    tar czf ${1}/${GLANCE_BACKUP_FILE} .
    popd >/dev/null
}

echo "Starting backup."
prep_directories $1 $2 $3 $4 $5 # destination, server, directory, username, password
exit 0
