#!/bin/bash
set -e

source /scripts/ipv6_functions

NFVTIME_PREINIT_SERVICE="/lib/systemd/system/nfvtime-preinit.service"
MARIADB_SERVICE="/lib/systemd/system/mariadb.service"
MARIADB_CONFIG="/etc/mysql/mariadb.conf.d/50-server.cnf"
GLANCE_SERVICE="/lib/systemd/system/glance-api.service"
GLANCE_CONFIG="/etc/glance/glance-api.conf"
SAN_CONFIG="/etc/iscsi/iscsid.conf"
SAN_INITIATOR_CONFIG="/etc/iscsi/initiatorname.iscsi"

# echo an error message before exiting
#trap 'cod=$?; if [ "$cod" != "0" ] ; then  echo "Failure.Last operation failed with exit code $cod."; fi' EXIT

function success_log () {
	echo -e "$1"
}

function err_log () {
	echo -e "Error in $1: $2\nChanging storage to local..."
	initial-setup-local "force"
}

function initial-setup-local () {
	local mode=${1}
	local is_attached_volume_mode=`grep -rn "/attached_volume/mysql" $MARIADB_CONFIG || true`

	if [[ $mode != "force" ]]; then
		if [[ $(systemctl is-active SAN.mount) == "active" ]] && [[ -d /SAN/mysql ]] && [[ -d /SAN/glance ]]; then
			echo -n "Do you want to copy databases from remote SAN storage to local (yes/no)? "
			read ANS
			if [[ "$ANS" == "YES" || "$ANS" == "yes" ]]; then
				sudo rsync -c -av -q /SAN/mysql/ /var/lib/mysql/ || echo "Failed rsync Mariadb remote directory to local"
				sudo rsync -c -av -q /SAN/glance/ /var/lib/glance/images/ || echo "Failed rsync Glance remote directory to local"
			fi
		elif [[ $is_attached_volume_mode ]]; then
			echo -n "Do you want to copy databases from attached volume to local storage (yes/no)? "
			read ANS
			if [[ "$ANS" == "YES" || "$ANS" == "yes" ]]; then
				sudo rsync -c -av -q /attached_volume/mysql/ /var/lib/mysql/ || echo "Failed rsync Mariadb attached volume directory to local"
				sudo rsync -c -av -q /attached_volume/glance/ /var/lib/glance/images/ || echo "Failed rsync Glance attached volume directory to local"
			fi
		fi
	fi

	# Set/restore configuration to local
	sed -i "s/^#After/After/" $NFVTIME_PREINIT_SERVICE
	sed -i "s/^#Wants/Wants/" $NFVTIME_PREINIT_SERVICE
	sed -i '/After=network-online.target iscsid.service SAN.mount/d' $MARIADB_SERVICE
	sed -i "s|datadir.*$|datadir         = /var/lib/mysql|" $MARIADB_CONFIG
	sed -i "s|filesystem_store_datadir.*$|filesystem_store_datadir = /var/lib/glance/images/|" $GLANCE_CONFIG
	sed -i '|/SAN.*$|d' /etc/fstab
	sed -i '|/attached_volume.*$|d' /etc/fstab
	sed -i '/RequiresMountsFor=.*$/d' $GLANCE_SERVICE
	#sed -i "s/iscsid.startup/#iscsid.startup/" $SAN_CONFIG

	#systemctl restart iscsid || err_log "${FUNCNAME[0]}" "Failed to stop iscsid.service"
	if [[ $(systemctl is-active SAN.mount) == "active" ]]; then
		umount -l /SAN > /dev/null 2>&1 || true
		rm -rf /SAN > /dev/null 2>&1 || true
	fi
	umount -l /attached_volume > /dev/null 2>&1 || true
	rm -rf /attached_volume > /dev/null 2>&1 || true

	# Logout from all active sessions
	iscsiadm -m session -u > /dev/null 2>&1 || true
	# Delete all active targets
	iscsiadm -m node --op=delete > /dev/null 2>&1 || true

	systemctl daemon-reload || true
	systemctl restart mariadb.service || echo "Failed to restart mariadb.service"
	systemctl restart glance-api.service || echo "Failed to restart glance-api.service"

	success_log "Successfully configured system to work with local storage."
	exit 0
}

function initial-setup-remote () {
	local server=${1}
	local port=${2}
	local initiator_name=${3}
	local target_name=${4}
	local username=${5}
	local password=${6}
	local timeout=10
	local retry=0

	# IPv6 support
	if [[ $server == *":"* ]]; then
		_prepareForIPv6 ${server} server
	fi

	if [[ $(systemctl is-active SAN.mount) == "active" ]]; then
		umount -l /SAN > /dev/null 2>&1 || true
		rm -rf /SAN > /dev/null 2>&1 || true
	fi
	# Logout from all active sessions
	iscsiadm -m session -u > /dev/null 2>&1 || true
	# Delete all active targets
	iscsiadm -m node --op=delete > /dev/null 2>&1 || true

	# Prepare initiator config
	success_log "Initial configuration of ISCSI is starting..."
	sed -i "s/InitiatorName=.*$/InitiatorName=${initiator_name}/" $SAN_INITIATOR_CONFIG
	sed -i -e "s/#node.session.auth.authmethod/node.session.auth.authmethod/" -e "s/node.session.auth.authmethod =.*$/node.session.auth.authmethod = None/" $SAN_CONFIG
	sed -i "s/^node.session.auth.username/#node.session.auth.username/" $SAN_CONFIG
	sed -i "s/^node.session.auth.password/#node.session.auth.password/" $SAN_CONFIG
	if [[ $username != "" ]] && [[ $password != "" ]]; then
		sed -i "/node.session.auth.username/c\node.session.auth.username = ${username}" $SAN_CONFIG
		sed -i "/node.session.auth.password/c\node.session.auth.password = ${password}" $SAN_CONFIG
		sed -i "s/node.session.auth.authmethod =.*$/node.session.auth.authmethod = CHAP/" $SAN_CONFIG
	fi
	sed -i "s/iscsid.startup.*$/iscsid.startup = \/sbin\/iscsid/" $SAN_CONFIG
	sed -i "s/node.startup.*$/node.startup = automatic/" $SAN_CONFIG

	# Connect to Target
	systemctl stop iscsid.socket || err_log "${FUNCNAME[0]}" "Failed to stop iscsid.socket"
	systemctl disable iscsid.socket > /dev/null 2>&1 || err_log "${FUNCNAME[0]}" "Failed to disable iscsid.socket"
	systemctl restart iscsid || err_log "${FUNCNAME[0]}" "Failed to restart iscsid.service"

	# Login to the target
	sleep $timeout
	# Discovery new target
	iscsiadm -m discovery -t sendtargets -p ${server}:${port} > /dev/null 2>&1 || true

	iscsiadm --mode node --target ${target_name} --portal ${server}:${port} --name=discovery.sendtargets.use_discoveryd --value=Yes > /dev/null 2>&1 || true
	iscsiadm --mode node --target ${target_name} --portal ${server}:${port} --name=discovery.sendtargets.discoveryd_poll_inval --value=30 > /dev/null 2>&1 || true
	if [[ $username != "" ]]; then
		iscsiadm --mode node --target ${target_name} --portal ${server}:${port} --name=node.session.auth.username --value=$username > /dev/null 2>&1 || true
	fi
	if [[ $password != "" ]]; then
		iscsiadm --mode node --target ${target_name} --portal ${server}:${port} --name=node.session.auth.password --value=$password > /dev/null 2>&1 || true
	fi

	# Login to new target
	iscsiadm -m node --login -T ${target_name} > /dev/null 2>&1 || err_log "${FUNCNAME[0]}" "Failed login to the target."

	# Automatically login to target on every boot 
	#iscsiadm -m node -T ${4} -p ${server}:${port} -o update -n node.startup -v automatic
	#iscsiadm -m node -T ${4} -p ${server}:${port} -o update -n node.conn[0].startup -v automatic

	# Retry mechanism for iscsi.service
	while true; do
		sleep $timeout
		if [[ $retry -gt 5 ]] && [[ $(systemctl is-active iscsi.service) == "inactive" ]]; then
			err_log "${FUNCNAME[0]}" "iscsi.service failed to start."
		elif [[ $(systemctl is-active iscsi.service) == "active" ]]; then
			break
		fi
		systemctl restart iscsi.service
		retry=$(($retry+1))
	done
	retry=0

	# Controller service startup fixes
	sed -i "s/^After=/#After=/" $NFVTIME_PREINIT_SERVICE
	sed -i "s/^Wants=/#Wants=/" $NFVTIME_PREINIT_SERVICE
	sed -i "s/Restart=.*$/Restart=on-failure/" $MARIADB_SERVICE

	#Improve startup and stop sequences
	grep -q '^After=' $MARIADB_SERVICE || sed -i '/\[Unit\]/a After=network-online.target iscsid.service SAN.mount' $MARIADB_SERVICE

	sed -i "s/use_lvmetad = 1/use_lvmetad = 0/" /etc/lvm/lvm.conf
	sed -i "s|locking_dir = \"/run/lock/lvm\"|locking_dir = \"/root/lock/lvm\"|" /etc/lvm/lvm.conf
	systemctl disable lvm2-lvmetad.service > /dev/null 2>&1 || true
	systemctl stop lvm2-lvmetad.service > /dev/null 2>&1 || true
	systemctl mask lvm2-lvmetad.service > /dev/null 2>&1 || true

	systemctl disable lvm2-lvmetad.socket > /dev/null 2>&1 || true
	systemctl stop lvm2-lvmetad.socket > /dev/null 2>&1 || true
	systemctl mask lvm2-lvmetad.socket > /dev/null 2>&1 || true

	systemctl disable lvm2-monitor.service > /dev/null 2>&1 || true
	systemctl stop lvm2-monitor.service > /dev/null 2>&1 || true
	systemctl mask lvm2-monitor.service > /dev/null 2>&1 || true

	systemctl disable lvm2-lvmpolld.socket > /dev/null 2>&1 || true
	systemctl stop lvm2-lvmpolld.socket > /dev/null 2>&1 || true
	systemctl mask lvm2-lvmpolld.socket > /dev/null 2>&1 || true

	success_log "Successfully configured ISCSI initiator."

	# Get remote disk
	available_iscsi_disk=`lsblk -S | grep iscsi | awk '{print $1}'`
	available_iscsi_device=`lsblk -l | grep ${available_iscsi_disk}1 | awk '{print $1}'`

	if [[ -z $available_iscsi_disk ]]; then
		err_log "${FUNCNAME[0]}" "Not free disk available to mount."
	elif [[ -z $available_iscsi_device ]]; then
		echo -n "Filesystem is missing for the new SAN storage. Create it (yes/no) "
		read ANS
		if [ "$ANS" != "YES" -a "$ANS" != "yes" ]; then
			success_log "Aborted."
			exit 0
		else
			parted /dev/$available_iscsi_disk -s mklabel gpt || err_log "${FUNCNAME[0]}" "Failed to create new disk label for /dev/${available_iscsi_disk}"
			parted -s -a optimal -- /dev/$available_iscsi_disk mkpart extended 0% 100% || err_log "${FUNCNAME[0]}" "Failed to partition /dev/${available_iscsi_disk}"
            i=1
            while [ ! -e "/dev/${available_iscsi_disk}1" ]; do
                sleep 1;
                if [ $i == 60 ]; then
                    break
                fi
                i=$(( i + 1 ))
            done
            sleep $timeout
            if [ ! -e "/dev/${available_iscsi_disk}1" ]; then
                    err_log "${FUNCNAME[0]}" "Failed to partition /dev/${available_iscsi_disk}. The file /dev/${available_iscsi_disk} does not exist and no size was specified."
            fi
            # Create a (new) file system
            yes | mkfs.ext4 -q /dev/${available_iscsi_disk}1
            success_log "Successfully partitioned ${available_iscsi_disk}"
		fi
	fi

	available_disk=`lsblk -S | grep iscsi | awk '{print $1 1}'`

	success_log "Database copying to the remote storage is starting..."
	if [[ ! -z `lsblk -l | grep $available_disk | awk '{ if ($7!="/SAN") print $1}'` ]]; then
		if [[ ! -d "/SAN" ]]; then
			mkdir /SAN
		fi
		umount -l /SAN > /dev/null 2>&1 || true
		# Clear data in local /SAN dir 
		#rm -rf /SAN/* || true

		output=`sudo /sbin/fsck -n /dev/${available_disk} 2> /dev/null || true`
		if [[ $output == *"WARNING: Filesystem still has errors"* ]]; then
			echo "WARNING: Filesystem has errors"
			echo -n "Do you want to repair /dev/${available_disk} (yes/no)? "
			read ANS
			if [ "$ANS" != "YES" -a "$ANS" != "yes" ]; then
				success_log "Aborted."
				exit 1
			else
				sudo /sbin/fsck -y /dev/${available_disk} > /dev/null 2>&1 && success_log "/dev/${available_disk} successfully repaired." || true
				sleep 5
			fi
		fi

		mount -t auto "/dev/$available_disk" "/SAN" > /dev/null 2>&1 || yes | mkfs.ext4 -q /dev/$available_disk > /dev/null 2>&1 && mount -t auto "/dev/$available_disk" "/SAN" > /dev/null 2>&1 || true
		UUID=`blkid | grep /dev/$available_disk | awk -F'["|"]' '{print $2}'`
		sed -i '/\/SAN.*$/d' /etc/fstab
		echo "UUID=${UUID} /SAN            ext4    _netdev,nofail,x-systemd.automount 0 0" >> /etc/fstab
		systemctl daemon-reload

		# Retry mechanism for SAN.mount
		while true; do
			sleep $timeout
			if [[ $retry -gt 5 ]] && [[ $(systemctl is-active SAN.mount) == "inactive" ]]; then
				err_log "${FUNCNAME[0]}" "SAN.mount failed to start."
			elif [[ $(systemctl is-active SAN.mount) == "active" ]]; then
				break
			fi
			systemctl restart SAN.mount
			retry=$(($retry+1))
		done
		retry=0
	fi

	if [[ ! -z `lsblk -l | grep $available_disk | awk '{ if ($7!="/SAN") print $1}'` ]]; then
		err_log "${FUNCNAME[0]}" "Remote SAN disk is not mounted."
	fi

	use_remote_db="false"
	if [[ -f "/SAN/owner" ]]; then
		owner_type=`cat /SAN/owner`
		if [[ "$owner_type" == "controller" ]] || [[ "$owner_type" == "manager" ]];then
			echo -n "Detected ${owner_type} DB on the remote storage. Do you want to connect to it (yes/no)? "
			read ANS
			if [[ "$ANS" == "no" ]]; then
				echo -n "Do you want to clear the remote storage file system (yes/no)? "
				read ANS
				if [[ "$ANS" == "yes" ]]; then
					rm -rf /SAN/* || true
				elif [[ "$ANS" == "no" ]]; then
					success_log "Aborted."
					exit 0
				fi
			elif [[ "$ANS" == "yes" ]]; then
				use_remote_db="true"
				sed -i "s/datadir.*$/datadir        = \/SAN\/mysql/" $MARIADB_CONFIG
				sed -i "s/filesystem_store_datadir.*$/filesystem_store_datadir = \/SAN\/glance/" $GLANCE_CONFIG
			fi
		fi
	fi

	if [[ "$use_remote_db" == "false" ]]; then
		systemctl daemon-reload || true
		#systemctl stop mariadb.service || err_log "${FUNCNAME[0]}" "Failed to stop mariadb.service"
		for i in {0..5}; do
			systemctl stop mariadb.service 2>/dev/null
			res=$?
			if [[ "$res" == "0" ]]; then
				break;
			fi
			if [ $i == 5 ]; then
				err_log "${FUNCNAME[0]}" "Failed to stop mariadb.service"
			fi
			sleep 1
		done
		systemctl stop glance-api.service || err_log "${FUNCNAME[0]}" "Failed to stop glance-api.service"
		# Initial copy of databases
		# Mariadb
		sed -i "s/datadir.*$/datadir        = \/SAN\/mysql/" $MARIADB_CONFIG
		if [[ ! -d "/SAN/mysql" ]]; then
			mkdir /SAN/mysql
		fi
		if [[ -d "/attached_volume/mysql" ]]; then
			success_log "Copying MySQL data from attached volume to remote storage."
			sudo rsync -c -av -q /attached_volume/mysql/ /SAN/mysql/ || err_log "${FUNCNAME[0]}" "Failed rsync Mariadb directory to new location"
		else
			success_log "Copying MySQL data from local to remote storage."
			sudo rsync -c -av -q /var/lib/mysql/ /SAN/mysql/ || err_log "${FUNCNAME[0]}" "Failed rsync Mariadb directory to new location"
		fi
		# Glance
		sed -i "s/filesystem_store_datadir.*$/filesystem_store_datadir = \/SAN\/glance/" $GLANCE_CONFIG
		if [[ ! -d "/SAN/glance" ]]; then
			mkdir /SAN/glance
		fi
		if [[ -d "/attached_volume/glance" ]]; then
			success_log "Copying Glance data from attached volume to remote storage."
			sudo rsync -c -av -q /attached_volume/glance/ /SAN/glance/ || err_log "${FUNCNAME[0]}" "Failed rsync Glance directory to new location"
		else
			success_log "Copying Glance data from local to remote storage."
			sudo rsync -c -av -q /var/lib/glance/images/ /SAN/glance/ || err_log "${FUNCNAME[0]}" "Failed rsync Glance directory to new location"
		fi
		echo "controller" > /SAN/owner
		success_log "Successfully copied databases to the SAN storage."
	fi

	systemctl restart mariadb.service || err_log "${FUNCNAME[0]}" "Failed to restart mariadb.service"
	systemctl restart glance-api.service || err_log "${FUNCNAME[0]}" "Failed to restart glance-api.service"

	umount -l /attached_volume > /dev/null 2>&1 || true
	rm -rf /attached_volume > /dev/null 2>&1 || true
	
	success_log "Successfully configured system to work with remote storage."
}

function initial-setup-attached-volume () {
	if [ -f /config/.hamode ]; then
		echo "Attached volume can not be configured, beacuse the machine is deployed as part of HA cluster!"
		exit 0
	fi
	local timeout=10
	# Get attached volume disk
	system_disk=`lsblk -l -o NAME,MOUNTPOINT | grep -w / | cut -f1 -d " " | sed 's/[0-9]//g'`
	echo "Available disks:"
	lsblk -dp -o NAME,SIZE,TYPE,TRAN | grep "disk" | grep -v $system_disk | awk '{if ($4!="iscsi") print $1 " - size " $2"; "}'
	echo -en "none\nChoose attached volume: "
	read ANS
	if [[ "$ANS" == "none" ]] || [[ "$ANS" == "" ]]; then
		success_log "Aborted."
		exit 0
	elif [[ -z `lsblk -dp | grep $ANS` ]]; then
		success_log "$ANS is not available disk."
		exit 0
	else
		disk=${ANS}
		available_attached_volume_device=`lsblk -p | grep ${disk}1 | awk '{print $1}'`
		if [[ -z $available_attached_volume_device ]]; then
			echo -n "Filesystem is missing for the attached volume. Create it (yes/no) "
			read ANS
			if [ "$ANS" != "YES" -a "$ANS" != "yes" ]; then
				success_log "Aborted."
				exit 0
			else
				parted $disk -s mklabel gpt || err_log "${FUNCNAME[0]}" "Failed to create new disk label for ${disk}"
				parted -s -a optimal -- $disk mkpart extended 0% 100% || err_log "${FUNCNAME[0]}" "Failed to partition ${disk}"
                i=1
                while [ ! -e "${disk}1" ]; do
                    sleep 1;
                    if [ $i == 60 ]; then
                        break
                    fi
                    i=$(( i + 1 ))
                done
                sleep $timeout
                if [ ! -e "${disk}1" ]; then
                        err_log "${FUNCNAME[0]}" "Failed to partition ${disk}1. The file ${disk}1 does not exist and no size was specified."
                fi
                # Create a (new) file system
                yes | mkfs.ext4 -q ${disk}1
                success_log "Successfully partitioned ${disk}"
			fi
		fi
		
		success_log "Database copying to the attached volume is starting..."
		if [[ ! -z `lsblk -p | grep ${disk}1 | awk '{ if ($7!="/attached_volume") print $1}'` ]]; then
			if [[ ! -d "/attached_volume" ]]; then
				mkdir /attached_volume
			fi
			umount -l /attached_volume > /dev/null 2>&1 || true

			output=`sudo /sbin/fsck -n ${disk}1 2> /dev/null || true`
			if [[ $output == *"WARNING: Filesystem still has errors"* ]]; then
				echo "WARNING: Filesystem has errors"
				echo -n "Do you want to repair ${disk}1 (yes/no)? "
				read ANS
				if [ "$ANS" != "YES" -a "$ANS" != "yes" ]; then
					success_log "Aborted."
					exit 1
				else
					sudo /sbin/fsck -y ${disk}1 > /dev/null 2>&1 && success_log "${disk}1 successfully repaired." || true
					sleep 5
				fi
			fi

			mount -t auto "${disk}1" "/attached_volume" > /dev/null 2>&1 || yes | mkfs.ext4 -q $disk > /dev/null 2>&1 && mount -t auto "${disk}1" "/attached_volume" > /dev/null 2>&1 || true
			UUID=`blkid | grep $disk | awk -F'["|"]' '{print $2}'`
			sed -i '/\/attached_volume.*$/d' /etc/fstab
			echo "UUID=${UUID} /attached_volume            ext4    defaults 0 0" >> /etc/fstab
			systemctl daemon-reload
		fi

		if [[ ! -z `lsblk -p | grep ${disk}1 | awk '{ if ($7!="/attached_volume") print $1}'` ]]; then
			err_log "${FUNCNAME[0]}" "Attached volume is not mounted."
		fi

		use_remote_db="false"
		if [[ -f "/attached_volume/owner" ]]; then
			owner_type=`cat /attached_volume/owner`
			if [[ "$owner_type" == "controller" ]] || [[ "$owner_type" == "manager" ]];then
				echo -n "Detected ${owner_type} DB on the attached volume. Do you want to connect to it (yes/no)? "
				read ANS
				if [[ "$ANS" == "no" ]]; then
					echo -n "Do you want to clear the attached volume file system (yes/no)? "
					read ANS
					if [[ "$ANS" == "yes" ]]; then
						rm -rf /attached_volume/* || true
					elif [[ "$ANS" == "no" ]]; then
						success_log "Aborted."
						exit 0
					fi
				elif [[ "$ANS" == "yes" ]]; then
					use_remote_db="true"
					sed -i "s/datadir.*$/datadir        = \/attached_volume\/mysql/" $MARIADB_CONFIG
					sed -i "s/filesystem_store_datadir.*$/filesystem_store_datadir = \/attached_volume\/glance/" $GLANCE_CONFIG
				fi
			fi
		fi

		if [[ "$use_remote_db" == "false" ]]; then
			systemctl daemon-reload || true
			#systemctl stop mariadb.service || err_log "${FUNCNAME[0]}" "Failed to stop mariadb.service"
			for i in {0..5}; do
				systemctl stop mariadb.service 2>/dev/null
				res=$?
				if [[ "$res" == "0" ]]; then
					break;
				fi
				if [ $i == 5 ]; then
					err_log "${FUNCNAME[0]}" "Failed to stop mariadb.service"
				fi
				sleep 1
			done
			systemctl stop glance-api.service || err_log "${FUNCNAME[0]}" "Failed to stop glance-api.service"
			# Initial copy of databases
			# Mariadb
			sed -i "s/datadir.*$/datadir        = \/attached_volume\/mysql/" $MARIADB_CONFIG
			if [[ ! -d "/attached_volume/mysql" ]]; then
				mkdir /attached_volume/mysql
			fi
			if [[ -d "/SAN/mysql" ]]; then
				success_log "Copying MySQL data from remote storage to attached volume."
				sudo rsync -c -av -q /SAN/mysql/ /attached_volume/mysql/ || err_log "${FUNCNAME[0]}" "Failed rsync Mariadb directory to new location"
			else
				success_log "Copying MySQL data from local to attached volume."
				sudo rsync -c -av -q /var/lib/mysql/ /attached_volume/mysql/ || err_log "${FUNCNAME[0]}" "Failed rsync Mariadb directory to new location"
			fi

			# Glance
			sed -i "s/filesystem_store_datadir.*$/filesystem_store_datadir = \/attached_volume\/glance/" $GLANCE_CONFIG
			if [[ ! -d "/attached_volume/glance" ]]; then
				mkdir /attached_volume/glance
			fi
			if [[ -d "/SAN/mysql" ]]; then
				success_log "Copying Glance data from remote storage to attached volume."
				sudo rsync -c -av -q /SAN/glance/ /attached_volume/glance/ || err_log "${FUNCNAME[0]}" "Failed rsync Glance directory to new location"
			else
				success_log "Copying Glance data from local to attached volume."
				sudo rsync -c -av -q /var/lib/glance/images/ /attached_volume/glance/ || err_log "${FUNCNAME[0]}" "Failed rsync Glance directory to new location"
			fi

			echo "controller" > /attached_volume/owner
			success_log "Successfully copied databases to the attached volume storage."
		fi

		# Disable SAN
		# Logout from all active iSCSI sessions
		iscsiadm -m session -u > /dev/null 2>&1 || true
		# Delete all active iSCSI targets
		iscsiadm -m node --op=delete > /dev/null 2>&1 || true
		# Remove SAN configuration
		sed -i "s/^#After/After/" $NFVTIME_PREINIT_SERVICE
		sed -i "s/^#Wants/Wants/" $NFVTIME_PREINIT_SERVICE
		sed -i '/After=network-online.target iscsid.service SAN.mount/d' $MARIADB_SERVICE
		sed -i '|/SAN.*$|d' /etc/fstab
		sed -i '/RequiresMountsFor=.*$/d' $GLANCE_SERVICE

		umount -l /SAN > /dev/null 2>&1 || true
		rm -rf /SAN > /dev/null 2>&1 || true

		systemctl daemon-reload || true
		systemctl restart mariadb.service || err_log "${FUNCNAME[0]}" "Failed to restart mariadb.service"
		systemctl restart glance-api.service || err_log "${FUNCNAME[0]}" "Failed to restart glance-api.service"
		
		success_log "Successfully configured system to work with attached volume."
	fi
}

function disk-checkup () {
	local storage="remote"

	if [ $storage == "local" ]; then
		available_disk=`lsblk -l | grep part | awk '{ if ($7=="/") print $1}'`
	elif [ $storage == "remote" ]; then
		available_disk=`lsblk -l | grep part | awk '{ if ($7!="/") print $1}'`
		if [ -z $available_disk ]; then
			success_log "Remote storage is not available."
			exit 0
		fi
	fi

	output=`sudo /sbin/fsck -n /dev/${available_disk} 2> /dev/null || true`

	if [[ $output == *"WARNING: Filesystem still has errors"* ]]; then
		echo "WARNING: Filesystem has errors"
		echo -n "Do you want to repair /dev/${available_disk} (yes/no)? "
		read ANS
		if [ "$ANS" != "YES" -a "$ANS" != "yes" ]; then
			success_log "Aborted."
			exit 0
		else
			systemctl stop mariadb.service || true
			systemctl stop glance-api.service || true
			umount -l /SAN > /dev/null 2>&1 || true

			sudo /sbin/fsck -y /dev/${available_disk} > /dev/null 2>&1 && success_log "/dev/${available_disk} successfully repaired." || true
			sleep 5
			systemctl restart SAN.mount || true

                        if [[ ! -d /SAN/mysql ]] || [[ ! -d /SAN/glance ]]; then
                            err_log "${FUNCNAME[0]}" "remote storage is missing critical files."
			fi

			systemctl restart mariadb.service || true
			systemctl restart glance-api.service || true
		fi
	elif [[ $output == *"Possibly non-existent device"* ]]; then
		success_log "/dev/${available_disk} is non-existent device."
		exit 0
	else
		success_log "/dev/${available_disk} is working properly."
		exit 0
	fi
}

function show-disk-setup () {
	is_remote=`grep -rn "/SAN/mysql" $MARIADB_CONFIG || true`
	is_attached_volume=`grep -rn "/attached_volume/mysql" $MARIADB_CONFIG || true`
	fs_info=`df -t ext4 -h`

	if [[ $is_remote ]]; then
		int_name=`cat $SAN_INITIATOR_CONFIG | grep InitiatorName= | cut -d "=" -f2`
		tgt_name=`iscsiadm -m node -o show | grep "node.name" | awk '{print $3}'`
		san_addr=`iscsiadm -m node -o show | grep "node.discovery_address" | awk '{print $3}'`
		san_port=`iscsiadm -m node -o show | grep "node.conn\[0\].port" | awk '{print $3}'`
		auth=`iscsiadm -m node -o show | grep "node.session.auth.authmethod" | awk '{print $3}'`
		success_log "System is configured to work with remote SAN storage.\n\niSCSI configuration:\nInitiator name:\t\t$int_name\nTarget name:\t\t$tgt_name\nSAN storage address:\t$san_addr\nSAN storage port:\t$san_port\nAuthentication method:\t$auth"
	elif [[ $is_attached_volume ]]; then
		success_log "System is configured to work with attached volume."
	else
		success_log "System is configured to work with local storage."
	fi
	success_log "\nFile system information:\n$fs_info"
}

case "$1" in
	initial-setup)
		if [ $2 == "san" ]; then
			initial-setup-remote $3 $4 $5 $6 $7 $8 #server, port, initiator_name, target_name, username, password
		elif [ $2 == "local" ]; then
			initial-setup-local
		elif [ $2 == "attached-volume" ]; then
			initial-setup-attached-volume
		fi
	;;
	disk-checkup)
		disk-checkup
	;;
	show-disk-setup)
		show-disk-setup
	;;
	*)
		echo "Unknown operation"
		exit 1
	;;
esac

exit 0
