#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import getopt
import netaddr
from sysrepo_interfaces import sysrepo_interfaces3 as api
import copy

def apply_nat_if_sysrepo( name, enable=None, ipv4=None, netmask=None, ipv6=None, netmask6=None, wan_names=None, p_fwd_all=None ):
    # print "Enable ", enable
    # print "IPv4  ", ipv4
    # print "Netmask ", netmask
    # print "WAN ", wan_names
    interfaces = api.SysrepoInterfaces(app_name="set interfaces")

    if wan_names is not None:
        wan_copy = copy.copy(wan_names)
        ifaces_data = interfaces.get_interfaces()
        for iface_data in ifaces_data.values():
            try:
                ifname = iface_data.get("name")
                if ifname in wan_copy:
                    wan_copy.remove(ifname)
            except:
                continue

        if len(wan_copy) > 0 and wan_copy != 'none':
            print ("%% Interface \'%s\' is not valid WAN interface !!!" % wan_copy.pop())
            return


    interfaces.set_interface(name)
    interfaces.set_interface_param(name, "type", "nat")

    if ipv4 is not None and netmask is not None:
        interfaces.set_interface_param(name, "ipv4-prefix", ipv4+'/'+netmask)

    if ipv6 is not None and netmask6 is not None:
        interfaces.set_interface_param(name, "ipv6-prefix", ipv6+'/'+netmask6)

    if enable is not None:
        interfaces.set_interface_param(name, "enable", enable)

    if wan_names is not None:
        interfaces.unset_nat_wan_interfaces(name)
        if wan_names != 'none':
            for wan_name in wan_names:
                interfaces.set_nat_wan_interface(name, wan_name)

    if p_fwd_all is not None:
        interfaces.set_interface_param(name, "port-forwarding-all", p_fwd_all)

    interfaces.commit()

def print_help( name ):
    #print '%s -i <tunnel interface name> -s <source IPv6> -d <destination IPv6> [-t <target IPv4 prefix>] -c <dir>' % name
    print ("Params:")
    print (" -p --pid <PID> \t PID of CLI session")
    print (" -n --name <NAME> \t NAT interface name")
    print (" -i --ipv4 <IPv4 prefix> \t IPv4 address of interface")
    print (" -6 --ipv6 <IPv6 prefix> \t IPv6 address of interface")
    print (" -e --enable <1,0> \t Enable/Disable interface")
    print (" -w --wan-if <NAME[,NAME]> \t List of WAN interfaces")
    print (" -h \t This help")

def main( name, argv ):
    pid = None
    if_name = None
    enable = None
    ipv4 = None
    ipv6 = None
    mask = None
    wan_names = None
    ipv4_addr_mask = None
    ipv6_addr_mask = None
    p_fwd_all = None

    try:
        opts, args = getopt.getopt(argv,"p:n:i:6:e:w:f:h",["pid=","name=", "ipv4=", "ipv6=", "enable=", "wan_if=", "port_fwd_all="])
    except getopt.GetoptError:
        print_help( name )
        sys.exit(2)

    for opt, arg in opts:
        #print "Opt %s arg \'%s\'" % (opt, arg)
        if opt == '-h':
            print_help( name )
            sys.exit(0)
        elif opt in ("-p", "--pid"):
            pid = arg
        elif opt in ("-n", "--name"):
            if_name = arg
        elif opt in ("-i", "--ipv4"):
            if arg:
                ipv4_addr_mask = arg
        elif opt in ("-6", "--ipv6"):
            if arg:
                ipv6_addr_mask = arg
        elif opt in ("-e", "--enable"):
            if arg:
                if '1' == arg:
                    enable=True
                else:
                    enable=False
        elif opt in ("-w", "--wan_if"):
            if arg:
                #print "Get list of interfaces"
                wan_names = arg.split( ',' )
        elif opt in ("-f", "--port_fwd_all"):
            if arg:
                if '1' == arg:
                    p_fwd_all=True
                else:
                    p_fwd_all=False

    nat_ifs = ['nat0', 'nat1', 'nat2', 'nat3', 'nat4']

    if if_name not in nat_ifs:
        print ("%% Only interfaces from nat0 to nat4 are supported !!!")
        return

    if ipv4_addr_mask:
        ip = netaddr.IPNetwork( ipv4_addr_mask )
        ipv4 = str(ip.ip)
        mask = str(ip.netmask)

    if ipv6_addr_mask:
        ip = netaddr.IPNetwork( ipv6_addr_mask )
        ipv6 = str(ip.ip)
        mask6 = str(ip.prefixlen)

    if wan_names is not None:
        for c_wan_name in wan_names:
            if c_wan_name == 'none':
                apply_nat_if_sysrepo( if_name, enable=enable, ipv4=ipv4, netmask=(ipv4_addr_mask.split("/")[1]) if ipv4_addr_mask else None, ipv6=ipv6, netmask6=(ipv6_addr_mask.split("/")[1]) if ipv6_addr_mask else None, wan_names=c_wan_name )
                return

    apply_nat_if_sysrepo( if_name, enable=enable, ipv4=ipv4, netmask=(ipv4_addr_mask.split("/")[1]) if ipv4_addr_mask else None, ipv6=ipv6, netmask6=(ipv6_addr_mask.split("/")[1]) if ipv6_addr_mask else None, wan_names=wan_names, p_fwd_all=p_fwd_all )

if __name__ == '__main__':
    main(sys.argv[0], sys.argv[1:])
