#!/bin/bash
shopt -s expand_aliases
set -e
#set -x

source /scripts/ipv6_functions

GLANCE_CONFIG_FILE="/etc/glance/glance-api.conf"
TMPDIR=`mktemp -d`
TMPDIR_MYSQL="${TMPDIR}/mysql"
TMPDIR_ETCD="${TMPDIR}/etcd"
MYSQL_BACKUP_FILE="mysql_backup.tar.gz"
ETCD_BACKUP_FILE="etcd_backup.tar.gz"
GLANCE_BACKUP_FILE="glance_backup.tar.gz"
BACKUP_LOCATION="/backups"
ETCD_DATADIR="/var/lib/etcd/default"


function success_log () {
    echo "$1"
}

function err_log () {
    echo "Error in $1: $2"
    exit 1
}

prep_directories ()
{
    backup_location=${1}
    if [[ $backup_location == *"/SAN"* ]]; then
        mkdir -p /SAN$TMPDIR_MYSQL
        mkdir -p /SAN$TMPDIR_ETCD
        chmod 777 -R /SAN$TMPDIR
        if [[ ! -d "$backup_location" ]]; then
            mkdir -p $backup_location
        fi
    elif [[ $backup_location == *"/attached_volume"* ]]; then
        mkdir -p /attached_volume$TMPDIR_MYSQL
        mkdir -p /attached_volume$TMPDIR_ETCD
        chmod 777 -R /attached_volume$TMPDIR
        if [[ ! -d "$backup_location" ]]; then
            mkdir -p $backup_location
        fi
    else
        mkdir -p $TMPDIR_MYSQL
        mkdir -p $TMPDIR_ETCD
        chmod 777 -R $TMPDIR
        if [[ ! -d $BACKUP_LOCATION ]]; then
            mkdir $BACKUP_LOCATION
        fi
    fi

}

prep_backup ()
{
    local storage=${1}
    local backup=${2}
    local backup_location=${3}
    local remote_directory=${4}
    local server=${5}
    local username=${6}
    local password=${7}

    # IPv6 support
    if [[ $server == *":"* ]]; then
        _prepareForIPv6 ${server} server
    fi

    ftp_login="user $username $password"
    if [ -z $username ]; then
        ftp_login="user anonymous"
    fi

    cd $backup_location

    available_iscsi_disk=`lsblk -S | grep iscsi | awk '{print $1}'`
    available_attached_volume_disk=`lsblk -l | grep attached_volume | awk '{print $1}'`
    if [[ $available_iscsi_disk ]]; then
        available_size_iscsi_disk=`df | grep ${available_iscsi_disk} | awk '{print $4+0}'`;
        available_size_iscsi_disk=$(( $available_size_iscsi_disk - 1048576 ))
    elif [[ $storage == "attached_volume_storage" ]]; then
        available_size_attached_volume_disk=`df | grep ${available_attached_volume_disk} | awk '{print $4+0}'`;
        available_size_attached_volume_disk=$(( $available_size_attached_volume_disk - 1048576 ))
    fi
    available_size_local_disk=`df | awk '{ if ($6=="/") print $4+0}'`
    available_size_local_disk=$(( available_size_local_disk - 1048576 ))

    if [[ $storage == "ftp" ]]; then
        archive_size=`ftp -n $server << EOF | grep $backup | awk '{print $5}'
$ftp_login
cd $remote_directory                                                          
ls -la
bye
EOF
`
        archive_size=$(( archive_size / 1024 ))
        local_space_req=$(( 2 * archive_size ))
        if [[ $available_size_iscsi_disk ]]; then
            if [[ $available_size_iscsi_disk -le $local_space_req ]]; then
                if [[ $available_size_local_disk -le $local_space_req ]]; then
                    ldisk_mb=$(( available_size_local_disk / 1024 ))
                    rdisk_mb=$(( available_size_iscsi_disk / 1024 ))
                    ldisk_req=$(( local_space_req / 1024 ))
                    echo "Not enough space (both local file system[${ldisk_mb}MB] and remote SAN storage[${rdisk_mb}MB]) to perform restore from the backup archive[required: ${ldisk_req}MB]"
                    exit 1
                fi
            fi
        elif [[ $available_size_attached_volume_disk ]]; then
            if [[ $available_size_attached_volume_disk -le $local_space_req ]]; then
                if [[ $available_size_local_disk -le $local_space_req ]]; then
                    ldisk_mb=$(( available_size_local_disk / 1024 ))
                    sdisk_mb=$(( available_size_attached_volume_disk / 1024 ))
                    ldisk_req=$(( local_space_req / 1024 ))
                    echo "Not enough space (both local file system[${ldisk_mb}MB] and attached volume [${sdisk_mb}MB]) to perform restore from the backup archive[required: ${ldisk_req}MB]"
                    exit 1
                fi
            fi
        else
            if [[ $available_size_local_disk -le $local_space_req ]]; then
                ldisk_mb=$(( available_size_local_disk / 1024 ))
                ldisk_req=$(( local_space_req / 1024 ))
                echo "Not enough space (local file system[${ldisk_mb}$MB]) to perform restore from the backup archive[required: ${ldisk_req}MB]"
                exit 1
            fi
        fi

        ftp -n $server > /dev/null 2<&1 << EOF
$ftp_login
cd $remote_directory
lcd $backup_location
get $backup
bye
EOF
    elif [[ $storage == "scp" ]]; then
        archive_size=`sshpass -p $password ssh -T -o StrictHostKeyChecking=no -o LogLevel=ERROR $username@$server <<EOF| grep $backup | awk '{print $5}'
cd $remote_directory
ls -la
exit
EOF
`
        archive_size=$(( archive_size / 1024 ))
        local_space_req=$(( 2 * archive_size ))
        if [[ $available_size_iscsi_disk ]]; then
            if [[ $available_size_iscsi_disk -le $local_space_req ]]; then
                if [[ $available_size_local_disk -le $local_space_req ]]; then
                    ldisk_mb=$(( available_size_local_disk / 1024 ))
                    rdisk_mb=$(( available_size_iscsi_disk / 1024 ))
                    ldisk_req=$(( local_space_req / 1024 ))
                    echo "Not enough space (both local file system[${ldisk_mb}MB] and remote SAN storage[${rdisk_mb}MB]) to perform restore from the backup archive[required: ${ldisk_req}MB]"
                    exit 1
                fi
            fi
        elif [[ $available_size_attached_volume_disk ]]; then
            if [[ $available_size_attached_volume_disk -le $local_space_req ]]; then
                if [[ $available_size_local_disk -le $local_space_req ]]; then
                    ldisk_mb=$(( available_size_local_disk / 1024 ))
                    sdisk_mb=$(( available_size_attached_volume_disk / 1024 ))
                    ldisk_req=$(( local_space_req / 1024 ))
                    echo "Not enough space (both local file system[${ldisk_mb}MB] and attached volume[${sdisk_mb}MB]) to perform restore from the backup archive[required: ${ldisk_req}MB]"
                    exit 1
                fi
            fi
        else
            if [[ $available_size_local_disk -le $local_space_req ]]; then
                ldisk_mb=$(( available_size_local_disk / 1024 ))
                ldisk_req=$(( local_space_req / 1024 ))
                echo "Not enough space (local file system[${ldisk_mb}$MB]) to perform restore from the backup archive[required: ${ldisk_req}MB]"
                exit 1
            fi
        fi
        sshpass -p $password scp -q -o StrictHostKeyChecking=no $username@$server:$remote_directory/$backup $backup_location > /dev/null 2<&1
    elif [[ $storage == "attached_volume_storage" ]] || [[ $storage == "local_storage" ]]; then
        archive_size=`ls -la $backup_location | grep $backup | awk '{print $5}'`
        archive_size=$(( archive_size / 1024 ))
        local_space_req=$(( 2 * archive_size ))

        if [[ $storage == "attached_volume_storage" ]] && [[ $available_size_attached_volume_disk -le $local_space_req ]]; then
            sdisk_mb=$(( available_size_attached_volume_disk / 1024 ))
            sdisk_req=$(( local_space_req / 1024 ))
            echo "Not enough space (attached volume file system[${sdisk_mb}$MB]) to perform restore from the backup archive[required: ${sdisk_req}MB]"
            exit 1
        elif [[ $storage == "local_storage" ]] && [[ $available_size_local_disk -le $local_space_req ]]; then
            ldisk_mb=$(( available_size_local_disk / 1024 ))
            ldisk_req=$(( local_space_req / 1024 ))
            echo "Not enough space (local file system[${ldisk_mb}$MB]) to perform restore from the backup archive[required: ${ldisk_req}MB]"
            exit 1
        fi
    fi

    if [[ ! -f $backup_location/$backup ]]; then
        echo "This backup is not available."
        exit 1
    fi

    tar xzf $backup_location/$backup || err_log "${FUNCNAME[0]}" "This backup is not available."
}

cleanup_directories ()
{
    local storage=${1}
    local backup=${2}
    local backup_location=${3}

    if [[ $backup_location == *"/SAN"* ]]; then
        rm -rf /SAN$TMPDIR
    elif [[ $backup_location == *"/attached_volume"* ]]; then
        rm -rf /attached_volume$TMPDIR
    else
        rm -rf $TMPDIR
    fi
    rm -rf $backup_location/*backup.tar.gz

    if [[ ! -z $backup ]] && [[ $storage != "local_storage" ]] && [[ $storage != "attached_volume_storage" ]]; then
        rm -rf $backup_location/$backup
    fi
}

restore_mysql ()
{
    backup_location=${1}

    #try to guess mysql passwd
    mysql -uroot --password='' -ANe"SELECT COUNT(1) FROM mysql.user where user='root'" &>/dev/null && true
    res=$?
    if [ "$res" == "0" ] ; then
        MARIABACKUPCMD="/usr/bin/mariabackup --user=root --port=3306 --host=localhost "
    else
        MARIABACKUPCMD="/usr/bin/mariabackup --user=root --password=root --port=3306 --host=localhost  "
    fi

    datadir=`cat /etc/mysql/mariadb.conf.d/50-server.cnf | grep datadir | awk '{print $3}'`

    echo "Restoring mysql..."
    if [[ $backup_location == *"/SAN"* ]]; then
        pushd /SAN${TMPDIR_MYSQL} >/dev/null
    elif [[ $backup_location == *"/attached_volume"* ]]; then
        pushd /attached_volume${TMPDIR_MYSQL} >/dev/null
    else
        pushd ${TMPDIR_MYSQL} >/dev/null
    fi
    tar xzf ${backup_location}/${MYSQL_BACKUP_FILE} .
    systemctl stop mariadb.service
    if [[ $backup_location == *"/SAN"* ]]; then
        $MARIABACKUPCMD --prepare --target-dir=/SAN${TMPDIR_MYSQL}   &>/dev/null
    elif [[ $backup_location == *"/attached_volume"* ]]; then
        $MARIABACKUPCMD --prepare --target-dir=/attached_volume${TMPDIR_MYSQL}   &>/dev/null
    else
        $MARIABACKUPCMD --prepare --target-dir=${TMPDIR_MYSQL}   &>/dev/null
    fi
    rm -rf $datadir/*
    if [[ $backup_location == *"/SAN"* ]]; then
        $MARIABACKUPCMD --copy-back --target-dir=/SAN${TMPDIR_MYSQL}  &>/dev/null
    elif [[ $backup_location == *"/attached_volume"* ]]; then
        $MARIABACKUPCMD --copy-back --target-dir=/attached_volume${TMPDIR_MYSQL}  &>/dev/null
    else
        $MARIABACKUPCMD --copy-back --target-dir=${TMPDIR_MYSQL}  &>/dev/null
    fi
    chown -R mysql:mysql $datadir/
    systemctl start mariadb.service
}


restore_etcd ()
{
    backup_location=${1}

    if [[ $backup_location == *"/SAN"* ]]; then
        pushd /SAN${TMPDIR_ETCD} >/dev/null
    elif [[ $backup_location == *"/attached_volume"* ]]; then
        pushd /attached_volume${TMPDIR_ETCD} >/dev/null
    else
        pushd ${TMPDIR_ETCD} >/dev/null
    fi
    tar xzf ${backup_location}/${ETCD_BACKUP_FILE}
    systemctl stop etcd
    rm -rf ${ETCD_DATADIR}/*
    cp -r . ${ETCD_DATADIR}
    chown -R etcd:etcd /var/lib/etcd
    systemctl start etcd
}



restore_glance ()
{
    backup_location=${1}

    if [[ ! -f $GLANCE_CONFIG_FILE ]] ; then
      return
    fi

    GLANCE_DIR=$(cat $GLANCE_CONFIG_FILE  | grep ^filesystem_store_datadir | cut -f 2 -d "=" | head -1 | xargs)

    systemctl stop glance-api
    echo "Restoring glance..."
    pushd ${GLANCE_DIR} >/dev/null

    tar xzf ${backup_location}/${GLANCE_BACKUP_FILE}

    popd >/dev/null
    systemctl start glance-api
}

restore_stop_services ()
{
    systemctl stop rabbitmq-server || true
    systemctl stop nova-api.service nova-conductor.service nova-scheduler.service nova-serialproxy.service nova-spiceproxy.service  || true
    systemctl stop neutron-server || true

}

restore_restart_services ()
{
    systemctl restart rabbitmq-server || true
    systemctl restart nova-api.service nova-conductor.service nova-scheduler.service nova-serialproxy.service nova-spiceproxy.service  || true
    systemctl restart neutron-server || true

    #restart nova-compute on every uCPE connected to the controller
    tmpfile=$(mktemp /tmp/temp-script.XXXXXX)
    cat > $tmpfile <<EOF
#!/bin/bash

source /scripts/basic_vars
source /scripts/basic_functions
source /root/admin-openrc.sh
source /scripts/ipv6_functions

sleep 10

ids=\$(nova --insecure hypervisor-list | awk 'NR>3 {print \$2}' | head -n -1)
if [[ \$ids ]]; then
    readarray -t x <<<"\$ids"
    for i in "\${x[@]}"; do
        ip=\$(nova --insecure hypervisor-show \$i | grep host_ip | awk '{print \$4}')
        if [[ \$ip == *":"* ]]; then
            _prepareForIPv6 \${ip} ip
        fi
        token=\$(curl -si --insecure -d '{"auth":{"identity":{"methods":["password"],"password":{"user":{"domain":{"name":"default"},"name":"'a1bfb873ca23417fce59461a940a114f'","password":"'fd22985d0f43a88626a0c6d29d8cab5c'"}}},"scope":{"project":{"domain":{"name":"default"},"name":"admin"}}}}' -H "Content-type: application/json" https://\$ip:35357/v3/auth/tokens | awk '/X-Subject-Token/ {print \$2}' | tr -d '\r')
        if [ -z \${token} ]; then
            continue
        fi
        curl -H "X-Auth-Token: \$token" -H "Content-type:application/json" -k --insecure -X PUT https://\$ip:7373/v1/restart_nova
    done
fi

rm -- "\$0"

EOF

    chmod a+x $tmpfile
    $tmpfile > /dev/null 2<&1 &

}

show_backups ()
{
    local storage=${1}
    local detailed=${2}
    local backup_location=${3}
    local server=${4}
    local username=${5}
    local password=${6}
    ftp_login="user $username $password"

    if [ -z $username ]; then
        ftp_login="user anonymous"
    fi

    if [[ $storage == "show_remote_ftp" ]]; then
        available_backups=`ftp -n $server <<EOF| grep "db_backup_$hostname" || true
$ftp_login
ls $backup_location
bye
EOF
`
    elif [[ $storage == "show_remote_scp" ]]; then
        available_backups=`sshpass -p $password ssh -T -o StrictHostKeyChecking=no -o LogLevel=ERROR $username@$server <<EOF| grep "db_backup_$hostname" || true
ls -la $backup_location 2>&1
exit
EOF
`
    else
        hostname=$(hostname)
        if [ -d $backup_location ]; then
            available_backups=`ls -la $backup_location | grep "db_backup_$hostname" || true`
        fi
    fi

    if [[ $available_backups ]]; then
        readarray -t x <<<"$available_backups"
        echo "Available backups:"
        delimiter_name=`echo "${x[@]}" | awk '{print " "$9}' | wc -m`
        delimiter="+"
        delimiter+=`printf '%.0s-' $(seq 1 $delimiter_name)`
        if [[ $detailed == "true" ]]; then
            delimiter+="+--------------+-----------+"
            echo -e $delimiter
            printf '| Name %*s | Created at   | Size (MB) |\n' "$(( $delimiter_name-7 ))"
            echo -e $delimiter
            for i in "${x[@]}"; do
                    echo $i | awk '{printf "| "$9 " | "$6" "$7" "$8" |"; printf " %-10.2f|\n", $5/1024/1024}'
            done
            echo -e $delimiter
        else
            delimiter+="+"
            echo -e $delimiter
            printf '| Name %*s |\n' "$(( $delimiter_name-7 ))"
            echo -e $delimiter
            for i in "${x[@]}"; do
                    echo $i | awk '{printf "| "$9 " |\n"}'
            done
            echo -e $delimiter
        fi
    else
        echo "No backups available."
    fi
    exit 0
}

hostname=$(hostname)
backup_location="$BACKUP_LOCATION"
if [[ $1 == "san_storage" ]] || [[ $1 == "show_san" ]]; then
    backup_location="/SAN$BACKUP_LOCATION"
elif [[ $1 == "attached_volume_storage" ]] || [[ $1 == "show_attached_volume" ]]; then
    backup_location="/attached_volume$BACKUP_LOCATION"
fi

if [[ $1 == "show_local" ]] || [[ $1 == "show_attached_volume" ]] || [[ $1 == "show_san" ]]; then
    show_backups $1 $2 ${backup_location}
elif [[ $1 == "show_remote_ftp" ]] || [[ $1 == "show_remote_scp" ]]; then
    show_backups $1 $2 $4 $3 $5 $6 # storage, detailed, directory, server, username, password
fi
echo "Starting restore."
prep_directories $backup_location
prep_backup $1 $2 ${backup_location} $3 $4 $5 $6 #storage, backup, backup_location, remote_directory, server, username, password
restore_stop_services
restore_mysql ${backup_location}
restore_glance ${backup_location}
restore_etcd ${backup_location}
restore_restart_services
cleanup_directories $1 $2 ${backup_location} #storage, backup, backup_location
echo "Done."
exit 0
