#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import syslog
from sysrepo_security import sysrepo_security as api
import ipaddress


def set_auth_local(security):
    security.set_authentication_auth_type("local")
    security.unset_authentication_tacacs_key()
    security.unset_authentication_tacacs_server()

    security.unset_authentication_radius_server()
    security.unset_authentication_radius_port()
    security.unset_authentication_radius_key()
    security.unset_authentication_radius_timeout()

    security.unset_authentication_backup_radius_server()
    security.unset_authentication_backup_radius_port()
    security.unset_authentication_backup_radius_key()
    security.unset_authentication_backup_radius_timeout()
    security.unset_fallback_authentication()


def set_auth_tacacs(security, new_ip=None, new_key=None):
    security.set_authentication_auth_type("tacacs")
    if new_ip is not None:
        security.set_authentication_tacacs_server(new_ip)

    if new_key is not None:
        security.set_authentication_tacacs_key(new_key)


def set_auth_radius(security, new_ip=None, new_port=None, new_key=None, new_timeout=None, new_fallback=None):
    security.set_authentication_auth_type("radius")
    if new_ip is not None:
        security.set_authentication_radius_server(new_ip)

    if new_port != '""' and new_port is not None:
        if int(new_port) in range(1, 65536):
            security.set_authentication_radius_port(new_port)
        else:
            error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)
            return

    if new_key is not None:
        security.set_authentication_radius_key(new_key)

    if new_timeout is not None:
        security.set_authentication_radius_timeout(new_timeout)

    if new_fallback is not None:
        if new_fallback == "1":
            new_fallback = "enabled"
        else:
            new_fallback = "disabled"
        security.set_fallback_authentication(new_fallback)


def set_auth_backup_radius(security, backup_ip=None, backup_port=None, backup_key=None, backup_timeout=None):
    security.set_authentication_auth_type("radius")
    if backup_ip is not None:
        security.set_authentication_backup_radius_server(backup_ip)

    if backup_port != '""' and backup_port is not None:
        if int(backup_port) in range(1, 65536):
            security.set_authentication_backup_radius_port(backup_port)
        else:
            error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)
            return

    if backup_key is not None:
        security.set_authentication_backup_radius_key(backup_key)

    if backup_timeout is not None:
        security.set_authentication_backup_radius_timeout(backup_timeout)


def get_old_radius_server_values():
    with open('/etc/pam_radius_auth.conf', 'r') as file:
        f = file.readlines()

        server_old = f[0].split(":")
        if len(server_old) > 5:
            server_params = server_old[-1:]
            server_old.pop()
            server_old = ':'.join(server_old)
            port_old = server_params[0].split(' ')[0]
            key_old = server_params[0].split(' ')[1]
            timeout_old = server_params[0].split(' ')[2].strip()
            fallback = check_if_fallback_to_local_is_enabled()
            return server_old, port_old, key_old, timeout_old, fallback
        else:
            server_old = f[0].split(':')[0].strip()
            port_old_split = f[0].split(':')[1]
            port_old = port_old_split.split(' ')[0]
            key_old = f[0].split(' ')[1]
            timeout_old = f[0].split(' ')[2].strip()
            fallback = check_if_fallback_to_local_is_enabled()
            return server_old, port_old, key_old, timeout_old, fallback


def get_old_backup_radius_server_values():
    with open('/etc/pam_radius_auth.conf', 'r') as file:
        f = file.readlines()

        server_backup_old = f[1].split(':')
        if len(server_backup_old) > 5:
            server_params = server_backup_old[-1:]
            server_backup_old.pop()
            server_backup_old = ':'.join(server_backup_old)
            port_backup_old = server_params[0].split(' ')[0]
            key_backup_old = server_params[0].split(' ')[1]
            timeout_backup_old = server_params[0].split(' ')[2].strip()
            return server_backup_old, port_backup_old, key_backup_old, timeout_backup_old
        else:
            server_backup_old = f[1].split(':')[0].strip()
            port_backup_old_split = f[1].split(':')[1]
            port_backup_old = port_backup_old_split.split(' ')[0]
            key_backup_old = f[1].split(' ')[1]
            timeout_backup_old = f[1].split(' ')[2].strip()
            return server_backup_old, port_backup_old, key_backup_old, timeout_backup_old


def check_if_fallback_to_local_is_enabled():
    with open('/etc/pam.d/login', 'r') as f:
        content = f.readline()
        data = content.split(" ")
        if "fallback\n" in data:
            return "1"
        else:
            return "0"


def validate_cli_source_list(source_list=None):
    if source_list == None:
        # source list not provided - command is executed only with enable/disable option
        return

    new_ips = source_list.split(',')
    for new_ip in new_ips:
        if new_ip != '""' and new_ip != '':
            try:
                test_ip = ipaddress.ip_interface(new_ip)
            except ValueError:
                print('Invalid IP Address / IP Network in access-source-ip-list - {}' .format(new_ip))
                sys.exit(0)


def set_cli_whitelist(security, enabled=None, source_list=None):
    addips = False
    validate_cli_source_list(source_list=source_list)

    if enabled is not None:
        if enabled == 'true' or enabled == '1':
            security.set_cli_whitelist_enabled(True)
            addips = True
        elif enabled == 'false' or enabled == '0':
            security.set_cli_whitelist_enabled(False)
    for source_ip_cfg in security.get_cli_whitelist_access_source_ip():
        security.unset_cli_whitelist_access_source_ip(source_ip_cfg)

    if addips and source_list is not None:
        new_ips = source_list.split(',')
        for new_ip in new_ips:
            if new_ip != '""' and new_ip != '':
                security.set_cli_whitelist_access_source_ip(new_ip)


def set_ssh_listen_port(security, listen_port):
    if listen_port is not None:
        try:
            port_number = int(listen_port)
            if 1024 <= port_number <= 65535:
                security.set_cli_ssh_server_listen_port(port_number)
            else:
                error_message = "Invalid port number. Port number must be in the range: 1024 to 65535."
                syslog.syslog(syslog.LOG_ERR, error_message)
                print(error_message)
        except ValueError:
            error_message = "Invalid port number. Port number must be an integer."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)


if __name__ == "__main__":

    security = api.SysrepoSecurity(app_name="Security-commands")

    if sys.argv[1] == "auth_local":
        set_auth_local(security)
    elif sys.argv[1] == "auth_tacacs":
        set_auth_tacacs(security, sys.argv[2], sys.argv[3])
    elif sys.argv[1] == "auth_radius":
        try:
            set_auth_radius(security, sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6])
            rad_values = get_old_backup_radius_server_values()
            set_auth_backup_radius(security, rad_values[0], rad_values[1], rad_values[2], rad_values[3])
        except IndexError:
            pass
    elif sys.argv[1] == "auth_backup_radius":
        try:
            rad_values = get_old_radius_server_values()
            set_auth_radius(security, rad_values[0], rad_values[1], rad_values[2], rad_values[3], rad_values[4])
            set_auth_backup_radius(security, sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
        except IndexError:
            pass
    if sys.argv[1] == "cli_whitelist":
        if "" == sys.argv[3]:
            set_cli_whitelist(security, sys.argv[2], None)
        else:
            set_cli_whitelist(security, sys.argv[2], sys.argv[3])
    if sys.argv[1] == "set_ssh_server":
        set_ssh_listen_port(security, sys.argv[2])

    security.commit()
