#!/bin/bash
# -*- coding: utf-8 -*-
source /scripts/ipv6_functions

TVE_IP=${*: -3:1}
_prepareForIPv6 ${TVE_IP} TVE_IP
SCRIPTS_PATH=${*: -2:1}

source ${SCRIPTS_PATH}/common-commands.sh

USER=$(whoami)
GROUP=$(groups $USER | cut -d' ' -f3)

function set_local_auth_type() {
  IFS='#' tokens=( $1 )
  local new_authtype=${tokens[0]}

  /scripts/cli/tvesh/security-commands.py auth_local
}

function set_tacacs() {
  IFS='#' tokens=( $1 )
  local new_ip=${tokens[0]}
  local new_key=${tokens[1]}

  /scripts/cli/tvesh/security-commands.py auth_tacacs ${new_ip} ${new_key}

}

function set_radius() {
  IFS='#' tokens=( $1 )
  local new_ip=${tokens[0]}
  local new_port=${tokens[1]}
  local new_key=${tokens[2]}
  local new_timeout=${tokens[3]}
  local new_fallback=${tokens[4]}

  /scripts/cli/tvesh/security-commands.py auth_radius ${new_ip} ${new_port} ${new_key} ${new_timeout} ${new_fallback}
}

function set_backup_radius() {
  IFS='#' tokens=( $1 )
  local backup_ip=${tokens[0]}
  local backup_port=${tokens[1]}
  local backup_key=${tokens[2]}
  local backup_timeout=${tokens[3]}

  /scripts/cli/tvesh/security-commands.py auth_backup_radius ${backup_ip} ${backup_port} ${backup_key} ${backup_timeout}
}

function add_user() {

  new_user=$1
  new_member=$2
  new_password=$3

  if [ "${GROUP}" != "admin" ] ; then
    echo "You are not authorized to perform this operation"
    exit 1
  fi

  if id -u "$new_user" >/dev/null 2>&1; then
    echo "User '$new_user' already exists!"
    exit 1
  fi

  user_length=${#new_user}
  if [ $user_length -lt 4 ] ; then
    echo "Username '$new_user' is too short. Username must be at least 4 characters long."
    echo "User '$new_user' was not created!"
    exit 1
  fi

  sudo /usr/sbin/useradd ${new_user} -s /scripts/cli/bin/clish -g ${new_member} -N -l --home /tmp 1>/dev/null 2>&1
  sudo passwd ${new_user} -d 1>/dev/null 2>&1
  echo -e "${new_password}\n${new_password}\n" | sudo passwd ${new_user} 1>/dev/null 2>&1

  if [ 0 == $? ]; then
    if [ ! -e "/config/added_users" ]; then sudo touch /config/added_users; fi
    echo "${new_user}|${new_member}" | sudo tee -a /config/added_users > /dev/null
  else
    sudo /usr/sbin/userdel -r ${new_user} 1>/dev/null 2>&1
    echo "Password was not accepted!"
    echo "Passwords must contain at least 1 lowercase letter, 1 uppercase letter, 1 digit, 1 special character and have a minimum length of 8 characters; password must not contain the username in any form. The password must not contain consecutive digits or letters, or well known words."
  fi
}
function del_user() {
  IFS='#' tokens=( $1 )
  local new_user=${tokens[0]}
  local new_member=${tokens[1]}

  if [ "${GROUP}" != "admin" ] ; then
    echo "You are not authorized to perform this operation!"
    exit 1
  fi

  if [ "${new_user}" == "admin" ] ; then
    echo "Admin user cannot be deleted!"
    exit 1
  fi

  if [ "${USER}" == "${new_user}" ] ; then
    echo "User cannot delete themselves!"
    exit 1
  fi

  if [ "${new_user}" == "a1bfb873ca23417fce59461a940a114f" ] ; then
    echo "User '${new_user}' does not exist!"
    exit 1
  fi
  
  if ! id -u "$new_user" >/dev/null 2>&1; then
    echo "User '$new_user' does not exist!"
    exit 1
  fi

  grep "${new_user}|${new_member}" /config/added_users >/dev/null 2>&1
  if [ 0 -ne $? ]; then
      echo "User '$new_user' is not member of group '$new_member'. Operation failed!"
      exit 1
  fi

  sudo /usr/sbin/userdel -r ${new_user} 1>/dev/null 2>&1
  
  case $? in
    0|12)
      sudo sed -i '/'"${new_user}"'|'"${new_member}"'/d' /config/added_users
      ;;
    8)
      echo "Deleting user failed! User '${new_user}' is currently logged in."
      ;;
    *)
      echo "Deleting user failed!"
      ;;
  esac
}

# Delete all added users on system restore configuration (BUG #5564)
function delete_all_users() {
  added_users=$(cat /config/added_users | tr "\n" " ")
  users_arr=($added_users)
  if [ ! -z ${users_arr} ]; then
    for usrname in "${users_arr[@]}";
    do
    user=`echo "$usrname" | cut -d \| -f 1`
    member_of=`echo "$usrname" | cut -d \| -f 2`
    del_user ${user}#${member_of}
    done
  fi
  sudo rm -f /config/added_users
  # Reboot system
  logger -p daemon.warn "Requesting system reload [system restore configuration]" || true
  sudo mkdir -p /config/_reboot/tve/now
  sudo systemctl stop sshd 1> /dev/null
  sudo systemctl stop ssh-root 1> /dev/null
}

function update_user_password() {

  new_user="$1"
  new_password="$2"

  if [ "${GROUP}" != "admin" ] ; then
    echo "You are not authorized to perform this operation"
    exit 1
  fi

  if [ "${new_user}" == "a1bfb873ca23417fce59461a940a114f" ] ; then
    echo "User '${new_user}' does not exist"
    exit 1
  fi

  # Fake password change to catch validation errors before deleting user password
  echo -e "${new_password}\n${new_password}\n" | sudo passwd ${new_user} 2>&1 | grep -v "No password supplied" > /tmp/message
  if [ -n "$(grep BAD /tmp/message)" ]; then
    cat /tmp/message
    rm -rf /tmp/message
    exit 0
  fi

  sudo passwd ${new_user} -d
  echo -e "${new_password}\n${new_password}\n" | sudo passwd ${new_user} 2>&1 | grep -v "No password supplied"

  if [ "admin" == ${new_user} ]; then
    sudo sed -i "s/^OS_PASSWORD=.*$/OS_PASSWORD=$new_password/g" /root/admin-openrc.env
  fi
}

function cli_whitelist() {
  IFS='#' tokens=( $1 )
  local enabled=${tokens[0]}
  local source_list=${tokens[1]}

  if [ 1 -eq $enabled ] && [ -z $source_list ]; then
    echo "access-source-ip-list is empty!"
  else
    /scripts/cli/tvesh/security-commands.py cli_whitelist "${enabled}" "${source_list}"
  fi
}

function set_ssh_server() {
    IFS='#' tokens=( $1 )
    local new_port=${tokens[0]}

    if [ -z "$new_port" ]; then
        echo "SSH port not specified!"
        exit 1
    fi

    /scripts/cli/tvesh/security-commands.py set_ssh_server "${new_port}"
}


case "$1" in
  set_local_auth_type)
    set_local_auth_type $2
    ;;
  set_tacacs)
    set_tacacs $2
    ;;
  set_radius)
    set_radius $2
    ;;
  set_backup_radius)
    set_backup_radius $2
    ;;
  add_user)
    add_user $2 $3 $4
    ;;
  del_user)
    del_user $2
    ;;
  delete_all_users)
    delete_all_users
    ;;
  update_user_password)
    update_user_password $2 $3
    ;;
  cli_whitelist)
    cli_whitelist $2
    ;;
  ssh_port)
    set_ssh_server $2
    ;;
  *)
    echo "Unknown operation"
    exit 1
    ;;
esac

exit 0
