#!/usr/bin/env python3

from sysrepo_interfaces import sysrepo_interfaces3 as sr
import sys
import subprocess
from pydbus import SystemBus


INDENT = 4 * ' '
PR_TYPE = {1: 'ICMP', 2: 'THROUGHPUT', 3: 'TCP', 4: 'HTTP', 5: 'HTTPS', 6: 'ARP', 7: 'DNS'}


def get_tracehost_monitor():
    try:
        bus = SystemBus()
        if bus is not None:
            return bus.get('net.telco.TracehostMonitor')
    except Exception:
        return None


def update_table_alignment(head, rows):
    for r_idx, row in enumerate(rows):
        update_required = False
        for idx, cell in enumerate(row):
            offset = len(head[idx]) - len(cell)
            if offset > 1:
                rows[r_idx][idx] = cell.rjust(len(cell) + int(offset/2))
                rows[r_idx][idx] = rows[r_idx][idx].ljust(len(rows[r_idx][idx]) + (len(head[idx]) - len(rows[r_idx][idx])))
            if offset < -1:
                head[idx] = head[idx].rjust(len(head[idx]) + int(abs(offset/2)))
                head[idx] = head[idx].ljust(len(head[idx]) + (len(cell) - len(head[idx])))
                update_required = True
            if offset == 1:
                rows[r_idx][idx] = cell.rjust(len(cell) + 1)
            if offset == -1:
                head[idx] = head[idx].rjust(len(head[idx]) + 1)
                update_required = True
            if update_required and idx == len(row) - 1 and r_idx > 0:
                update_table_alignment(head, rows[0:r_idx])


def beautify_table(head, rows):
    head_len = 0

    # Construct cell borders -> | cell | cell | cell |
    head[0] = '|' + head[0] + '|'
    head[-1] = '|' + head[-1] + '|'
    for idx, row in enumerate(rows):
        rows[idx][0] = '|' + rows[idx][0] + '|'
        rows[idx][-1] = '|' + rows[idx][-1] + '|'

    # Construct upper and bottom frame -> +------------------+
    for cell in head:
        head_len += len(cell)
    upp_bot_frame = '+' + head_len * '-' + '+'

    print(upp_bot_frame)
    [print(cell, end=' ') for cell in head]
    print('\n', end='')
    print(upp_bot_frame)
    for row in rows:
        [print(cell, end=' ') for cell in row]
        print('\n', end='')
    print(upp_bot_frame)


def get_tracehost_statistics(iface):
    HEAD = 'Trace-host statistics ' + iface

    sri = sr.SysrepoInterfaces(app_name='show-tracehost')
    th_monitor = get_tracehost_monitor()
    if th_monitor is None:
        sys.exit()

    iface = iface.replace('br-', '')
    th_statistics = th_monitor.get_statistics(iface)

    if th_statistics[0] == 0:

        probes = sri.get_tracehost_probes(iface)

        print(HEAD)
        print(f'Status: {"Up" if th_statistics[1][0] == 1 else "Down"}')
        print(f'Status up count: {th_statistics[1][1]}')
        print(f'Status down count: {th_statistics[1][2]}')
        for probe in th_statistics[1][3]:
            print(f'Probe ID {probe[0]}:')
            print(INDENT + f'Type: {PR_TYPE.get(probe[1])}')
            print(INDENT + 'Description: ' + probes.get(probe[0], {}).get('description', ''))
            print(INDENT + 'Sent probes: ' + str(probe[2]))
            print(INDENT + 'Failed probes: ' + str(probe[3]))


def get_tracehost_status():
    HEAD = ['Interface', 'Status', 'Metric']
    sri = sr.SysrepoInterfaces(app_name='show-tracehost')

    th_monitor = get_tracehost_monitor()
    if th_monitor is None:
        sys.exit()

    ifaces = sri.get_interfaces()

    n_th_status = [status for status in (list(th_monitor.get_status(iface)) for iface in ifaces)]
    for idx, status in enumerate(n_th_status):
        if status[0] == 0:
            n_th_status[idx][0] = 'br-' + list(ifaces.keys())[idx] if ifaces.get(list(ifaces.keys())[idx])["type"] != 'pppoe' else list(ifaces.keys())[idx]
            n_th_status[idx][1] = 'Up' if status[1] == 1 else 'Down'

    n_th_status = [status for status in n_th_status if status[0] != 1]
    routes = subprocess.check_output(['ip', 'route']).decode().splitlines()
    if not routes or not n_th_status:
        return
    for route in routes:
        if 'default' not in route:
            continue
        for idx, status in enumerate(n_th_status):
            if status[0] == route.split()[4]:
                n_th_status[idx].append(route.split()[-1])

    for idx, status in enumerate(n_th_status):
        if len(status) < 3:
            n_th_status[idx].append('')

    update_table_alignment(HEAD, n_th_status)
    beautify_table(HEAD, n_th_status)


if __name__ == "__main__":
    argc = len(sys.argv)
    if argc == 1:
        get_tracehost_status()
    if argc == 2:
        get_tracehost_statistics(sys.argv[1])

sys.exit(0)
