#!/usr/bin/env python3

import subprocess
import sys
import syslog
import vppapi_if.vppapi_if as vppif
import shlex

VPP_GET_MAC_TABLE_BY_BRIDGE_ID = 'sudo /usr/bin/vppctl show l2fib bd_id '
OVS_GET_BRIDGE_PORTS = 'sudo /usr/bin/ovs-vsctl list-ports '
OVS_GET_IFACE_PORT_NUMBER = 'ovs-vsctl list Interface {}'
GREP_PORT_NUMBER = '-w ofport'
OVS_GET_MAC_TABLE_BY_BRIDGE_NAME = 'sudo /usr/bin/ovs-appctl fdb/show '
INDENT = ' '
COLUMN_GAP = 7 * INDENT
FIRST_LINE_INDENT = 3 * INDENT


def send_command(cmd=None, use_popen=False, grep=None):
    if not cmd:
        return

    cmd = shlex.split(cmd)
    try:
        if use_popen:
            proc1 = subprocess.Popen(cmd, stdout=subprocess.PIPE)
            if grep:
                proc2 = subprocess.Popen(shlex.split('grep ' + grep), stdin=proc1.stdout,
                                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                proc1.stdout.close()  # Allow proc1 to receive a SIGPIPE if proc2 exits.
                output, err = proc2.communicate()
            else:
                output = proc1.communicate()
        else:
            output = subprocess.check_output(cmd, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        return
    return output.decode('utf-8').splitlines()


def get_network_mode():
    net_mode = subprocess.check_output(
        ['sysrepocfg', '-X', '-x', '/ucpe:config/system:system/network-mode', '-f', 'json', '-d',
         'startup']).decode("utf-8")
    if net_mode.find('"network-mode": "vpp"') > -1:
        return "vpp"

    return "ovs"


def get_interface_mode(iface_name):
    mode = subprocess.check_output(
        ['sysrepocfg', '-X', '-x', '/ucpe:config/interfaces:interface[name="{}"]/mode', '-f', 'json', '-d',
         'startup'.format(iface_name)]).decode("utf-8")
    if mode.find('"mode": "dpdk"') > -1:
        return "dpdk"

    return "virtio"


# def get_ppp_name(pppox, vpp):
#     host_if_index = vpp.get_pppox_conn_by_pppox_ifidx(vpp.get_ifidx_by_name(pppox))['host_if_index']
#     host_if_name = vpp.get_ifidx_by_name(host_if_index)
#     ppp_iface_number = int(host_if_name.replace('tap', '')) - 700
#     return 'ppp' + str(ppp_iface_number)


def get_vxlan_name(vxlan_tunnel, vpp):
    tap_idx = vpp.get_interface_bd(vpp.get_ifidx_by_name(vxlan_tunnel))['bd_id']
    tap_ifname = vpp.get_ifname_by_ifidx(tap_idx)
    host_if_name = vpp.get_tap_interface(tap_ifname)['host_if_name']
    return host_if_name.replace('br-', '')


def get_vxlan_tunnel_name(vxlan_name, vpp):
    tap = vpp.get_tap_interface_by_host_ifname('br-' + vxlan_name)
    if tap:
        bd = vpp.get_interface_bd(tap['sw_if_index'])
        bd_ifs = list(map(lambda x: x['sw_if_index'], bd['sw_if_details']))
        for iface in bd_ifs:
            if iface != tap['sw_if_index']:
                return vpp.get_ifname_by_ifidx(iface)


def get_ovs_mac_table(iface_name):
    bd_ports = send_command(OVS_GET_BRIDGE_PORTS + iface_name)
    if not bd_ports:
        return {}
    port_info = [send_command(OVS_GET_IFACE_PORT_NUMBER.format(p)
                              , use_popen=True, grep=GREP_PORT_NUMBER) for p in bd_ports]
    iface_to_port = {k: v[0].split(':')[1] for k, v in zip(bd_ports, port_info)}
    raw_table = send_command(OVS_GET_MAC_TABLE_BY_BRIDGE_NAME + iface_name)
    table = format_raw_table(raw_table, 'ovs', iface_to_port_map=iface_to_port)
    return table


def display_table(table):
    first_line = {"Mac-Address": "Interface-Name"}
    for key, val in first_line.items():
        print(FIRST_LINE_INDENT + key, end=COLUMN_GAP + FIRST_LINE_INDENT)
        print(val)
    print((len(FIRST_LINE_INDENT)*2 + len("Mac-Address") + len("Interface-Name") + len(COLUMN_GAP)) * '-')
    for key, val in table.items():
        name_gap_space = 0
        if len(val) < len(first_line['Mac-Address']):
            name_gap_space = (len(first_line['Mac-Address']) - len(val)) / 2
        print(key, end=COLUMN_GAP + abs(int(name_gap_space)) * ' ')
        print(val)


def format_raw_table(table, network_mode, vpp_obj=None, iface_to_port_map=None):
    if not table:
        return {}
    table.pop(0)
    formatted_table = {}
    if network_mode == 'vpp':
        if not vpp_obj:
            return
        table.pop()
        for row in table:
            key = row.split()[0]   # MAC Address
            val = row.split()[-1]  # Interface Name
            if val.find('loop') > -1 or val.find('tap') > -1:
                continue
            if val.find('vxlan_tunnel') > -1:
                val = get_vxlan_name(val, vpp_obj)
            val = val.replace('host-', '')
            formatted_table[key] = val
    if network_mode == 'ovs':
        for row in table:
            key = row.split()[-2]  # MAC Address
            val = row.split()[0]   # Port
            if val.find('LOCAL') > -1:
                continue
            formatted_table[key] = ''.join([iface if port.strip() == val else val for iface, port in iface_to_port_map.items()])

    return formatted_table


def get_vswitch_mac_table(vsw_name):
    # Not yet implemented in OVS
    table = {}
    network_mode = get_network_mode()
    if network_mode == 'vpp':
        vpp_if = vppif.VppApiInterface('mac-table-commands')
        bd_id = vpp_if.get_bd_id_by_tag(tag=vsw_name)
        if not bd_id:
            return {}
        print("bd_id {} ".format(bd_id))
        raw_table = send_command(VPP_GET_MAC_TABLE_BY_BRIDGE_ID + str(bd_id))
        table = format_raw_table(raw_table, network_mode, vpp_if)
        del vpp_if

    if network_mode == 'ovs':
        table = get_ovs_mac_table(vsw_name)

    return table


def get_interface_mac_table(iface_name):
    table = {}
    network_mode = get_network_mode()
    if network_mode == 'vpp':
        vpp_if = vppif.VppApiInterface('mac-table-commands')
        if iface_name.find('data') > -1:
            iface_name = 'host-' + iface_name if get_interface_mode(iface_name) == 'virtio' else iface_name
        iface_name = 'host-' + iface_name if iface_name.find('wifi') > -1 else iface_name
        if iface_name.find('vxlan') > -1:
            iface_name = get_vxlan_tunnel_name(iface_name, vpp_if)
        bd = vpp_if.get_interface_bd(vpp_if.get_ifidx_by_name(iface_name))
        if not bd:
            return {}
        bd_id = bd['bd_id']
        raw_table = send_command(VPP_GET_MAC_TABLE_BY_BRIDGE_ID + str(bd_id))
        table = format_raw_table(raw_table, network_mode, vpp_if)
        del vpp_if
    if network_mode == 'ovs':
        table = get_ovs_mac_table('br-' + iface_name)

    return table


if __name__ == "__main__":
    argc = len(sys.argv)
    if 3 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)

    if argc == 3:
        table = ''
        if sys.argv[1] == "vswitch":
            table = get_vswitch_mac_table(sys.argv[2])
        if sys.argv[1] == "interface":
            table = get_interface_mac_table(sys.argv[2])
        display_table(table)
    else:
        syslog.syslog(syslog.LOG_ERR, "Invalid number of arguments. Exit!")
        sys.exit(1)

    sys.exit(0)
