#!/usr/bin/env python3
import argparse
import logging
import logging.handlers
from sysrepo_interfaces import sysrepo_interfaces3 as sr

log = None

def set_log( name, level ):
    global log
    log = logging.getLogger(name=name)
    log.setLevel(level=level)

    syslog_hnd = logging.handlers.SysLogHandler(address = '/dev/log')
    formatter = logging.Formatter('%(name)s: %(levelname)s %(message)s')
    syslog_hnd.setFormatter(formatter)
    log.addHandler(syslog_hnd)

def delete_interface(sri, ifname):
    sri.unset_interface(ifname)
    sri.commit()

def check_interface(sri, ifname):
    interface = next((intf for intf in sri.get_interfaces().keys() if intf == ifname), None)

    if interface in sri.get_interfaces().keys():
        return
    else:
        # print('Create ')
        sri.set_interface(ifname)
        sri.set_interface_param(ifname, 'type', 'vxlan')

    
# Run as subscriber if called as a script
if __name__ == "__main__":
    set_log( 'vxlan-ifcfg', logging.DEBUG )
    parser = argparse.ArgumentParser()
    parser.add_argument("-n", "--name", required=True, type=str, help="VxLan interface name to manipulate.")
    parser.add_argument("-v", "--vni", type=int, help="VNI")
    parser.add_argument("-l", "--lvtep-ip", type=str, help="Local VTEP IP Address.")
    parser.add_argument("-i", "--lvtep-if", type=str, help="Local VTEP Interface.")
    parser.add_argument("-r", "--rvtep", type=str, help="Remote VTEP.")
    parser.add_argument("-R", "--remove", dest='remove', action='store_true', default=False )
    parser.add_argument("-e", "--enable", type=int, help="Enable/Disable")
    parser.add_argument('-d', dest='delete', action='store_true', default=False)
    parser.add_argument("-b", "--bootproto", type=str, help="Set bootproto none/dhcp/dhcp6/dhcp46")
    parser.add_argument("-t", "--type", type=str, help="Set tunnel type vxlan or vxlan-gpe")
    parser.add_argument("-m", "--management", type=str, help="Set interface management mode")
    parser.add_argument("-s", "--shg", type=str, help="Set SPlit Horizon Group")
    parser.add_argument("-c", "--mcast-ip", type=str, help="Set Multicast IP Address")
    parser.add_argument("--ip4", type=str, help="Set IPv4 prefix")
    parser.add_argument("--ip6", type=str, help="Set IPv6 prefix")
    parser.add_argument("-D", "--dns", type=str, help="Set IPv4 IP address for DNS")
    parser.add_argument("-N", "--no", type=str, help="Unset parameter")
    args = parser.parse_args()

    sri = sr.SysrepoInterfaces(app_name='vxlan-ifcfg')
    sri.set_active_datastore('startup')
    net_mode = sri.get_network_mode()
    sri.set_active_datastore('running')

    if net_mode != 'vpp':
        log.error('{} network mode does not support VxLAN.'.format(net_mode.upper()))
        exit(0)

    if args.delete and args.name:
        delete_interface(sri, args.name)
        exit(0)

    check_interface(sri, args.name)

    if args.vni:
        sri.set_interface_param(args.name, 'vni', args.vni)

    if args.lvtep_ip:
        log.debug('Set local vtep {} for intf {}'.format(args.rvtep, args.name))
        sri.unset_container_param(args.name, 'local', 'vtep-if')
        sri.set_container_param(args.name, 'local', 'vtep-addr', args.lvtep_ip)

    if args.lvtep_if:
        bootproto = sri.get_interface_param(args.lvtep_if, 'bootproto', '')
        iftype = sri.get_interface_param(args.lvtep_if, 'type', '')
        if (bootproto == str("none") and iftype == 'ethernet') or iftype == 'loopback':
            ipv4 = sri.get_interface_param(args.lvtep_if, 'ipv4-prefix', '')
            ipv6 = sri.get_interface_param(args.lvtep_if, 'ipv6-prefix', '')
            if ipv4 or ipv6:
                log.debug('Set local vtep {} for intf {}'.format(args.rvtep, args.name))
                sri.unset_container_param(args.name, 'local', 'vtep-addr')
                sri.set_container_param(args.name, 'local', 'vtep-if', args.lvtep_if)
            else:
                print("Static ip address must be set on {} interface first".format(args.lvtep_if))
        else:
            print("Bootproto must be set to static, first")



    if args.rvtep:
        log.debug('Set remote vtep {} for intf {}'.format(args.rvtep, args.name))
        rvteps = args.rvtep.split(',')
        for addr in rvteps:
            if args.remove == True:
                log.debug('Del remote vtep {} for intf {}'.format(addr, args.name))
                sri.del_vxlan_rvtep(args.name, addr)
            else:
                log.debug('Add remote vtep {} for intf {}'.format(addr, args.name))
                sri.add_vxlan_rvtep(args.name, addr)
        
        # sri.set_container_param(args.name, 'remote', 'vtep-addr', args.rvtep)

    if args.enable is not None:
        if int(args.enable) == 1:
            log.debug('Enable interface {}'.format(args.name))
            sri.set_interface_param(args.name, 'enable', True)
        else:
            log.debug('Disable interface {}'.format(args.name))
            sri.set_interface_param(args.name, 'enable', False)

    if args.management is not None:
        if int(args.management) == 1:
            log.debug('Enable management {}'.format(args.name))
            sri.set_interface_param(args.name, 'management', True)
        else:
            log.debug('Disable management {}'.format(args.name))
            sri.set_interface_param(args.name, 'management', False)

    if args.bootproto:
        log.debug('Set bootproto to {}'.format(args.bootproto))
        sri.set_interface_param(args.name, 'bootproto', args.bootproto)

    if args.ip4:
        log.debug('Set ip4 prefix to {}'.format(args.ip4))
        sri.set_interface_param(args.name, 'ipv4-prefix', args.ip4)

    if args.ip6:
        log.debug('Set ip6 prefix to {}'.format(args.ip6))
        sri.set_interface_param(args.name, 'ipv6-prefix', args.ip6)

    if args.type:
        log.debug('Set tunnel type to {}'.format(args.type))
        sri.set_interface_param(args.name, 'tunnel-type', args.type)

    if args.shg:
        log.debug('Set SHG to {}'.format(args.shg))
        sri.set_interface_param(args.name, 'shg', args.shg)

    if args.mcast_ip:
        if args.remove == True:
            log.debug('Remove remote Multicast vtep for intf {}'.format(args.name))
            sri.unset_container_param(args.name, 'remote', 'mcast-vtep-addr')
        else:
            log.debug('Set remote Multicast vtep {} for intf {}'.format(args.mcast_ip, args.name))
            sri.set_container_param(args.name, 'remote', 'mcast-vtep-addr', args.mcast_ip)

    if args.dns:
        log.debug("Set IPv4 IP address {} for DNS".format(args.dns))
        sri.set_interface_param(args.name, 'dns', args.dns)

    if args.no:
        log.debug("Unsetting {} parameter".format(args.no))
        if args.no == 'dns':
            sri.unset_interface_param(args.name, args.no)

    sri.commit()
    exit(0)
