#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import getopt
import logging
import os
import subprocess
import sys

from sysrepo_interfaces import sysrepo_interfaces3 as sr
from systemd import journal

TABLEID_OFFSET = 1
DEFAULT_METRIC = 100
DEFAULT_LTE_METRIC = 110
DEFAULT_WIFI_METRIC = 120

default_gw_stream = journal.stream(identifier='default_gw_set')


def print_help():
    print("Params:", file=default_gw_stream)
    print(" -g --gw <GATEWAY> \t Default gateway", file=default_gw_stream)
    print(" -i --ifname <NAME> \t Interface name", file=default_gw_stream)
    print(" -m --metric <METRIC> \t Route metric", file=default_gw_stream)
    print(" -a --add \t Add gatewaw, default is remove", file=default_gw_stream)
    print(" -h \t This help", file=default_gw_stream)


def wan_sys_name(ifs_map, wan):
    wan_sname = wan
    if_names = [ifname for ifname, ifs_cfg in ifs_map.items()]

    if wan not in if_names:
        return wan

    if_cfg = [ifs_cfg for ifname, ifs_cfg in ifs_map.items() if ifname == wan]

    if not if_cfg:
        return wan

    if_cfg = if_cfg[0]

    if 'data' in if_cfg['name']:
        wan_sname = 'br-{}'.format(if_cfg['name'])

    if 'vlan' in if_cfg.keys():
        wan_sname = '{name}.{tag}'.format(name=wan_sname, tag=if_cfg['vlan'])

    return wan_sname


def get_link_routes(ifname):
    link_rt = []
    iproutes = subprocess.check_output(['ip', 'route', 'list', 'exact', '0.0.0.0/0']).decode()

    print("Check connected for interface %s" % ifname, file=default_gw_stream)
    try:
        iproute = subprocess.check_output(['ip', 'route', 'list', 'dev', ifname, 'scope', 'link']).decode()
    except:
        iproute = ''

    rt_line = iproute.split(' ')
    rt_line = [x for x in rt_line if x != '']
    if len(rt_line):
        link_rt.append(rt_line[0])

    return link_rt


def get_interfaces():
    if_config = sr.SysrepoInterfaces(app_name="Default Route Set", log_level=logging.ERROR)
    return (if_config, if_config.get_interfaces())


def set_def_gw(ifname, gw, metric=0):
    ifs, if_map = get_interfaces()
    nat_ifs = [if_name for if_name, if_cfg in if_map.items() if 'nat' in if_name]

    for nat_name in nat_ifs:
        if ifs.get_interface_param(nat_name, 'enable') == False:
            continue

        wans = ifs.get_nat_wan_interfaces(nat_name)
        tableid = int(nat_name.replace('nat', '')) + TABLEID_OFFSET
        for wan, _ in wans.items():
            if wan_sys_name(if_map, wan) != ifname:
                continue
            print("Tableid=%d" % tableid, file=default_gw_stream)
            if gw is not None:
                try:
                    FNULL = open(os.devnull, 'w')
                    subprocess.check_call(
                        ['ip', 'route', 'add', 'default', 'via', str(gw), 'dev', str(ifname), 'metric', str(metric),
                         'table', str(tableid)], stdout=FNULL, stderr=FNULL)
                except subprocess.CalledProcessError as e:
                    print("Command \"%s\" failed with code %d" % (e.cmd, e.returncode), file=default_gw_stream)
                except OSError as e:
                    print("System error - \"%s\"" % (e.strerror), file=default_gw_stream)

            for connected in get_link_routes(ifname):
                try:
                    subprocess.check_call(
                        ['ip', 'route', 'add', str(connected), 'dev', str(ifname), 'table', str(tableid)])
                except subprocess.CalledProcessError:
                    pass

            # if we have load balancing, now add ECMP route with metric 1
            # this is demo only for NanoPI as a home router
            if os.path.isfile("/config/nat-lb"):
                os.system("ip route del default metric 1 table %d" % tableid)
                cmd='ip route add default proto static metric 1 table %d $(for metric in `seq 2 249` ; do  ip route list default table %d metric $metric ; done | sed \'s/default/nexthop/\'  | sed \'s/proto static/weight 1/\' | tr "\n" " ")' %  (tableid, tableid)
                os.system(cmd)



def del_def_gw(ifname, gw):
    ifs, if_map = get_interfaces()
    nat_ifs = [if_name for if_name, if_cfg in if_map.items() if 'nat' in if_name]

    for nat_name in nat_ifs:
        wans = ifs.get_nat_wan_interfaces(nat_name)
        tableid = int(nat_name.replace('nat', '')) + TABLEID_OFFSET
        for wan, wan_cfg in wans.items():
            if wan_sys_name(if_map, wan) != ifname:
                continue

            # if we have load balancing, now add ECMP route with metric 1
            # this is demo only for NanoPI as a home router
            if os.path.isfile("/config/nat-lb"):
                os.system("ip route del default metric 1 table %d" % tableid)
            try:
                FNULL = open(os.devnull, 'w')
                if gw is None:
                    subprocess.check_call(['ip', 'route', 'del', 'default', 'dev', ifname, 'table', str(tableid)],
                                          stdout=FNULL, stderr=FNULL)
                else:
                    subprocess.check_call(
                        ['ip', 'route', 'del', 'default', 'via', gw, 'dev', ifname, 'table', str(tableid)],
                        stdout=FNULL, stderr=FNULL)

            except subprocess.CalledProcessError as e:
                print("Command \"%s\" failed with code %d" % (e.cmd, e.returncode), file=default_gw_stream)
            except OSError as e:
                print("System error - \"%s\"" % e.strerror, file=default_gw_stream)



def main(name, argv):
    gw = None
    if_name = None
    metric = None
    remove = True

    try:
        opts, args = getopt.getopt(argv, "g:i:m:ha", ["gw=", "ifname=", "metric=", "add"])
    except getopt.GetoptError:
        print_help()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print_help()
            sys.exit(0)
        elif opt in ("-g", "--gw"):
            if '' != arg:
                gw = arg
        elif opt in ("-i", "--ifname"):
            if '' != arg and False == arg.startswith('--'):
                if_name = arg
        elif opt in ("-m", "--metric"):
            if '' != arg:
                metric = int(arg)
        elif opt in ("-a", "--add"):
            remove = False
        else:
            print("Unknown parameter !!!", file=default_gw_stream)
            print_help()
            sys.exit(2)

    if metric is None:
        if if_name is not None and True == if_name.startswith('lte'):
            metric = DEFAULT_LTE_METRIC
        elif if_name is not None and True == if_name.startswith('wifi'):
            metric = DEFAULT_WIFI_METRIC
        else:
            metric = DEFAULT_METRIC

        # print("Metric is not provided. Will be used default value {}".format(metric), file=default_gw_stream)

    if DEFAULT_METRIC == metric and if_name is not None and True == if_name.startswith('lte'):
        metric = DEFAULT_LTE_METRIC
    elif DEFAULT_METRIC == metric and if_name is not None and True == if_name.startswith('wifi'):
        metric = DEFAULT_WIFI_METRIC

    if remove:
        if if_name is None:
            # print("Interface is not specified", file=default_gw_stream)
            sys.exit(1)

        print("DEL %s,gw %s" % (if_name, gw), file=default_gw_stream)
        del_def_gw(if_name, gw)
    else:
        if if_name is None:
            print("Error Set default gw - if " + str(if_name) + ",gw " + str(gw) + ",metric " + str(metric),
                  file=default_gw_stream)
            sys.exit(1)

        print("SET %s,gw %s,metric %s" % (if_name, gw, metric), file=default_gw_stream)
        set_def_gw(if_name, gw, metric)


if __name__ == "__main__":
    main(sys.argv[0], sys.argv[1:])
