#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import ipaddress
import json
import logging
import sys

from systemd.journal import JournalHandler

CLUSTER_FILE = '/tmp/cluster.bak.json'
AZURE_FILE = '/tmp/azure.bak.json'
AMAZON_FILE = '/tmp/amazon.bak.json'

log = logging.getLogger(__name__)
log_fmt = logging.Formatter('%(levelname)s %(message)s')
log_ch = JournalHandler(SYSLOG_IDENTIFIER='ha-add')
log_ch.setFormatter(log_fmt)
log.addHandler(log_ch)
log.setLevel(logging.DEBUG)


def main(_argv):
    argc = len(_argv)
    if argc == 1:
        try:
            with open(CLUSTER_FILE, 'r') as f:
                data = f.read()
        except FileNotFoundError:
            log.error('FileNotFound {}'.format(CLUSTER_FILE))
            sys.exit(2)

        try:
            cluster_cfg = json.loads(data)
        except json.JSONDecodeError:
            log.error('Corrupted JSON file')
            log.error(data)
            sys.exit(2)

        found = False
        for peer in cluster_cfg.get('peers', []):
            if _argv[0] == peer['ip']:
                found = True
                break

        if not found:
            if 'peers' not in cluster_cfg:
                cluster_cfg['peers'] = []

            cluster_cfg['peers'].append({'ip': '{}'.format(_argv[0])})

        if 'peers' in cluster_cfg:
            a = cluster_cfg.pop('peers')
            a = sorted(a, key=lambda i: int(ipaddress.ip_address(i['ip'])))
            cluster_cfg['peers'] = a

        print(json.dumps(cluster_cfg, indent=4, sort_keys=True))

    elif argc == 2:
        try:
            with open(AZURE_FILE, 'r') as f:
                data = f.read()
        except FileNotFoundError:
            log.error('FileNotFound {}'.format(AZURE_FILE))
            sys.exit(2)

        try:
            cluster_cfg = json.loads(data)
        except json.JSONDecodeError:
            log.error('Corrupted JSON file')
            log.error(data)
            sys.exit(2)

        if 'azure' not in cluster_cfg:
            cluster_cfg['azure'] = {}

        if sys.argv[1] == 'set_member':
            found = False
            for peer in cluster_cfg['azure'].get('ha_cluster', []):
                if sys.argv[2] == peer['vm_name']:
                    found = True
                    break

            if not found:
                elem = {'vm_name': sys.argv[2]}
                if 'ha_cluster' in cluster_cfg['azure']:
                    cluster_cfg['azure']['ha_cluster'].append(elem)
                else:
                    cluster_cfg['azure']['ha_cluster'] = [elem]

        if sys.argv[1] == 'set_floatip':
            cluster_cfg['azure']['floating_ip'] = sys.argv[2]

        if sys.argv[1] == 'del_member':
            if 'azure' in cluster_cfg and 'ha_cluster' in cluster_cfg['azure']:
                new_vms = []
                for vm in cluster_cfg['azure']['ha_cluster']:
                    if sys.argv[2] != vm['vm_name']:
                        new_vms.append(vm)

                cluster_cfg['azure']['ha_cluster'] = new_vms

        if sys.argv[1] == 'del_floatip':
            if 'azure' in cluster_cfg and 'floating_ip' in cluster_cfg['azure']:
                cluster_cfg['azure'].pop('floating_ip')

        print(json.dumps(cluster_cfg, indent=4, sort_keys=True))
    elif argc == 3:
        try:
            with open(AMAZON_FILE, 'r') as f:
                data = f.read()
        except FileNotFoundError:
            log.error('FileNotFound {}'.format(AMAZON_FILE))
            sys.exit(2)

        try:
            cluster_cfg = json.loads(data)
        except json.JSONDecodeError:
            log.error('Corrupted JSON file')
            log.error(data)
            sys.exit(2)

        if 'amazon' not in cluster_cfg:
            cluster_cfg['amazon'] = {}

        if sys.argv[2] == 'set_member':
            cluster_cfg['amazon']['peers'] = sorted(sys.argv[3].split(','))
        elif sys.argv[2] == 'set_allocationid':
            if 'network_interface_ids' in cluster_cfg['amazon']:
                del cluster_cfg['amazon']['network_interface_ids']

            cluster_cfg['amazon']['allocation_id'] = sys.argv[3]
        elif sys.argv[2] == 'set_accesskeyid':
            cluster_cfg['amazon']['access_key_id'] = sys.argv[3]
        elif sys.argv[2] == 'set_secretaccesskey':
            cluster_cfg['amazon']['secret_access_key'] = sys.argv[3]
        elif sys.argv[2] == 'set_region':
            cluster_cfg['amazon']['region'] = sys.argv[3]
        elif sys.argv[2] == 'set_network_interface_ids':
            if 'allocation_id' in cluster_cfg['amazon']:
                del cluster_cfg['amazon']['allocation_id']

            cluster_cfg['amazon']['network_interface_ids'] = sorted(sys.argv[3].split(','))
        else:
            log.error('Unknown argument')
            sys.exit(2)

        print(json.dumps(cluster_cfg, indent=4, sort_keys=True))

    else:
        log.error('Not enough arguments')
        sys.exit(2)


if __name__ == '__main__':
    main(sys.argv[1:])
