#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
import logging
import subprocess
import sys

import netifaces
from systemd.journal import JournalHandler

CLUSTER_FILE = '/config/cluster.json'


def set_logs(identifier, level=logging.DEBUG):
    log = logging.getLogger(__name__)
    log_fmt = logging.Formatter('%(levelname)s [%(funcName)s:%(lineno)d] %(message)s')
    log_ch = JournalHandler(SYSLOG_IDENTIFIER=identifier)
    log_ch.setFormatter(log_fmt)
    log.addHandler(log_ch)
    log.setLevel(level)
    return log


def get_cluster_cfg(log):
    try:
        with open(CLUSTER_FILE, 'r') as f:
            data = f.read()
    except FileNotFoundError:
        log.error('FileNotFound {}'.format(CLUSTER_FILE))
        sys.exit(2)

    try:
        cluster_cfg = json.loads(data)
    except json.JSONDecodeError:
        log.error('Corrupted JSON {}'.format(CLUSTER_FILE))
        sys.exit(2)

    return cluster_cfg


def get_data0(log):
    try:
        out = subprocess.run(['/usr/bin/sysrepocfg', '--datastore=running',
                              '--format=json', '--xpath',
                              '/ucpe:config/interfaces:interface[name=\'data0\']/ipv4-prefix',
                              '--export'],
                             stdout=subprocess.PIPE, encoding='UTF-8')
    except subprocess.CalledProcessError as cpe:
        log.error('fail getting data0 ipv4 {}'.format(cpe))
        sys.exit(2)
    except OSError:
        log.error('Missing /usr/bin/sysrepocfg')
        sys.exit(2)

    if 0 != out.returncode:
        log.error('Can not get data0 ip code {}'.format(out.returncode))
        log.error('{}'.format(out))
        try:
            second_data0 = netifaces.ifaddresses('br-data0')[netifaces.AF_INET]
        except Exception as e:
            log.error('Fail geting second_data0 {}'.format(e))
            sys.exit(2)

        if not second_data0 or len(second_data0) < 1:
            log.error('Unknown second_data0 {}'.format(second_data0))
            sys.exit(2)

        return second_data0[0]['addr'], second_data0[0]['netmask']

    data0_address = None
    data0_network = None
    data0_address_start = out.stdout.find('ipv4-prefix": "')
    if data0_address_start > -1:
        data0_address_end = out.stdout.find('/', data0_address_start)
        if data0_address_end > -1:
            data0_address = out.stdout[data0_address_start + 15:data0_address_end]
            data0_network_end = out.stdout.find('"', data0_address_end)
            if data0_network_end > -1:
                data0_network = out.stdout[data0_address_end + 1:data0_network_end]
            else:
                log.debug('data0_network_end {}'.format(data0_network_end))
        else:
            log.debug('data0_address_end {}'.format(data0_address_end))
    else:
        log.debug('data0_address_start {}'.format(data0_address_start))

    if not data0_address:
        log.error('Can not match data0 address')
        log.debug(out)
        sys.exit(2)

    if not data0_network:
        log.error('Can not match data0 network')
        log.debug(out)
        sys.exit(2)

    return data0_address, data0_network


def get_peer1_ip(log, cluster_config=None, data0_address=None):
    if cluster_config:
        cluster_cfg = cluster_config
    else:
        cluster_cfg = get_cluster_cfg(log)

    if data0_address:
        data0 = data0_address
    else:
        data0, _ = get_data0(log)

    peers = cluster_cfg['peers']
    if len(peers) != 3:
        log.error('Not enough peers')
        sys.exit(2)

    peer1, peer2, peer3 = peers
    if data0 != peer1['ip']:
        return peer1['ip']
    elif data0 != peer2['ip']:
        return peer2['ip']
    elif data0 != peer3['ip']:
        return peer3['ip']

    log.error('Should not be here')
    sys.exit(2)


def get_peer2_ip(log, cluster_config=None, data0_address=None):
    if cluster_config:
        cluster_cfg = cluster_config
    else:
        cluster_cfg = get_cluster_cfg(log)

    if data0_address:
        data0 = data0_address
    else:
        data0, _ = get_data0(log)

    peer1_ip = get_peer1_ip(log, cluster_cfg, data0_address=data0)
    peers = cluster_cfg['peers']
    if len(peers) != 3:
        log.error('Not enough peers')
        sys.exit(2)

    peer1, peer2, peer3 = peers
    if data0 != peer1['ip'] and peer1_ip != peer1['ip']:
        return peer1['ip']
    elif data0 != peer2['ip'] and peer1_ip != peer2['ip']:
        return peer2['ip']
    elif data0 != peer3['ip'] and peer1_ip != peer3['ip']:
        return peer3['ip']

    log.error('Should not be here')
    sys.exit(2)


def exec_get_status(cmd, log, **kwargs):
    try:
        out = subprocess.run(cmd,
                             stdout=subprocess.DEVNULL,
                             stderr=subprocess.DEVNULL,
                             **kwargs)
        log.debug('{} = {}'.format(cmd, out))
    except subprocess.CalledProcessError as cpe:
        log.error('exec failed {} {}'.format(cmd, cpe))
        return False
    except OSError as oserror:
        log.error('Missing {}'.format(oserror))
        return False

    if 0 != out.returncode:
        return False

    return True


def exec_get_result(cmd, log):
    try:
        out = subprocess.run(cmd,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE,
                             encoding='UTF-8')
    except subprocess.CalledProcessError as cpe:
        log.error('exec failed {} {}'.format(cmd, cpe))
        sys.exit(2)
    except OSError as oserror:
        log.error('Missing {}'.format(oserror))
        sys.exit(2)

    return out
