#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
import logging
import os
import re
import socket
import subprocess
import sys

import ha

CLUSTER_FILE = '/run/cluster.json'
TELCOSECRET = b'RIYljGjlpBKPfWkAEVAPTBIlTraKLjHGt6N74glD7kNZcRGDdW3lHjvfvdjdtixgj2i6DElrIreKZQIMZKwRFfr5rmtRX7bcRS69GnoIIIsWeRGkJrViPqVD9LQ2Zya2bhLTmXsnJI9iQpXCVG7dPY8UrAQjTTPNjlIaI8xjxh2XzGfGAKDqx4lA6iyYVz78Unor0UCKFasE4vYsH8mJogxWInQuXhzLqaEfDhgivpMe19S0bmvPK399Ihf2Bfeg\n'
HAPROXY_CONFIG = '/etc/haproxy/haproxy.cfg'


def update_peer_status(cluster, log, peer, data):
    try:
        peer_cfg = json.loads(data)
    except json.JSONDecodeError:
        log.error('Corrupted peer JSON file')
        log.error(data)
        sys.exit(2)

    if 'nodename' in peer_cfg:
        peer['node'] = peer_cfg['nodename']

    peer['role'] = peer_cfg['role']
    peer['status'] = peer_cfg['status']

    with open(CLUSTER_FILE, 'w') as f:
        print(json.dumps(cluster, indent=4, sort_keys=False), file=f)


def add_peer_to_haproxy(peerip, log):
    haproxyconfigfile = ''
    with open(HAPROXY_CONFIG, 'r') as haproxyconfigfile:
        haproxyconfig = haproxyconfigfile.read()

    if re.search('\s*{}\s*'.format(peerip), haproxyconfig):
        log.debug('Peer {} already in haproxy config'.format(peerip))
        return

    log.debug('Add {} to haproxy config'.format(peerip))
    with open(HAPROXY_CONFIG, 'a') as f:
        f.write('    server {} {}:6443 check\n'.format(peerip, peerip))

    out = ha.exec_get_status(['/bin/systemctl', 'restart',
                              'haproxy.service'], log)
    if not out:
        log.error('Fail restart haproxy.service')
    else:
        log.debug('keepalived.service restart OK')


def rm_peer_from_haproxy(peerip, log):
    log.debug('Remove {} to haproxy config'.format(peerip))
    with open(HAPROXY_CONFIG, 'r') as f:
        old_haproxy_config_lines = f.readlines()

    new_haproxy_config_lines = []
    for line in old_haproxy_config_lines:
        if re.search('\s*{}\s*'.format(peerip), line):
            log.debug('Peer {} found in haproxy config'.format(peerip))
            continue

        new_haproxy_config_lines.append(line)

    with open(HAPROXY_CONFIG, 'w') as f:
        for line in new_haproxy_config_lines:
            print(line, end='', file=f)

    out = ha.exec_get_status(['/bin/systemctl', 'restart',
                              'haproxy.service'], log)
    if not out:
        log.error('Fail restart haproxy.service')
    else:
        log.debug('keepalived.service restart OK')


def main(_argv):
    if os.path.exists('/tmp/debughabeat'):
        log = ha.set_logs('habeat')
    else:
        log = ha.set_logs('habeat', level=logging.ERROR)

    cluster_cfg = ha.get_cluster_cfg(log)
    data0, _ = ha.get_data0(log)

    for peer in cluster_cfg.get('peers', []):
        if data0 == peer['ip']:
            try:
                out = subprocess.run(['/scripts/healthcheck.sh'],
                                     stdout=subprocess.PIPE,
                                     input=TELCOSECRET)
            except subprocess.CalledProcessError as cpe:
                log.error('fail getting healthcheck {}'.format(cpe))
                continue
            except OSError:
                log.error('Missing /scripts/healthcheck.sh')
                continue

            if 0 != out.returncode:
                log.error('Can not get healtcheck')
                sys.exit(2)

            log.debug('local data received: {}'.format(out.stdout.decode()))
            update_peer_status(cluster_cfg, log, peer, out.stdout.decode())
        else:
            try:
                with socket.create_connection((peer['ip'], 7), timeout=30) as sock:
                    sock.sendall(TELCOSECRET)
                    sockdata = sock.recv(256)

            except ConnectionRefusedError:
                log.error('{} ConnectionRefusedError'.format(peer['ip']))
                update_peer_status(cluster_cfg, log, peer,
                                   '{\"address\":\"' + '{}'.format(peer['ip']) +
                                   '\",\"role\":\"FAULT\",\"status\":\"down\"}')
                rm_peer_from_haproxy(peer['ip'], log)
                continue
            except OSError as oserror:
                log.error('{} {}'.format(peer['ip'], oserror))
                update_peer_status(cluster_cfg, log, peer,
                                   '{\"address\":\"' + '{}'.format(peer['ip']) +
                                   '\",\"role\":\"FAULT\",\"status\":\"down\"}')
                rm_peer_from_haproxy(peer['ip'], log)
                continue

            log.debug('peer data received: {}'.format(sockdata.decode()))
            update_peer_status(cluster_cfg, log, peer, sockdata.decode())
            add_peer_to_haproxy(peer['ip'], log)


if __name__ == '__main__':
    main(sys.argv[1:])
