#!/bin/bash

source /scripts/basic_vars || true

old_ip=$(grep -o 'node-ip[^ ]*' /etc/systemd/system/kubelet.service.d/10-kubeadm.conf | cut -d= -f2- | sed 's/[^a-zA-Z0-9:.]//g')
new_ip=$1
reason=$2
hostname=$(cat /etc/hostname)
TEMPLATES_PATH=/kubernetes/templates
CENTRAL_BOX=/config/central-box

LOG () { echo "$1" | systemd-cat -t k8s-ip-change -p "${2-info}" ; }
GET_PODS () { sudo kubectl get pods -A | grep "$1" ; }

LOG "Initiating handle_k8s_ip_change script"

if [ -z $old_ip ]; then
	LOG "No IP found in 10-kubeadm.conf"
        flags=$(grep -m1 "Environment" /etc/systemd/system/kubelet.service.d/10-kubeadm.conf | sed 's/.$//' | sed -n 's/Environment="KUBELET_KUBECONFIG_ARGS//p')
        flags_with_ip="$flags --node-ip=$new_ip"
        sed -i "s|$flags|$flags_with_ip|g" /etc/systemd/system/kubelet.service.d/10-kubeadm.conf
        exit 0
fi

[[ $old_ip == $new_ip ]] && exit 0

LOG "Setting --node-ip flag with the new IP '$new_ip'"
sed -i "s/$old_ip/$new_ip/" /etc/systemd/system/kubelet.service.d/10-kubeadm.conf

LOG "Replacing K8S IP address from $old_ip to $new_ip"

[ -f $K8S_DISABLED_FILE ] && [ ! -f $CENTRAL_BOX ] && exit 0

systemctl stop kube-proxy.service

cp -r /etc/kubernetes/ ./kubernetes-backup

cd /etc/kubernetes
find . -type f | xargs sed -i "s/$old_ip/$new_ip/"


# vppctl cnat translation del ... seems to not work
cnat_t_idx=$(vppctl show cnat translation | grep -w "10.96.0.1;443" | awk '{print $1}' | tr -d '[]')
LOG "Deleting cnat translation idx $cnat_t_idx"
vppctl cnat translation del "$cnat_t_idx"

# Wait until apiserver starts with new IP
until sudo ps aux | grep kube-apiserver | grep $new_ip
do
	LOG "Waiting on reinitialization of kube-apiserver..."
	sleep 2
done

rm /etc/kubernetes/pki/apiserver.crt /etc/kubernetes/pki/apiserver.key /etc/kubernetes/pki/etcd/peer.crt /etc/kubernetes/pki/etcd/peer.key /etc/kubernetes/pki/etcd/server.crt /etc/kubernetes/pki/etcd/server.key

# Update containerd insecure registries
function update_docker_config {
  cat << EOF > /tmp/shscript.sh
#!/bin/bash
CONTAINERD_REGISTRY_PATH=/etc/containerd/certs.d
hostname=$(cat /etc/hostname)
insecure_registries=("${ip_addr}:5001" "${hostname}:5001")
rm -rf \$CONTAINERD_REGISTRY_PATH/*
for registry in \${insecure_registries[@]}; do
    if [[ -d \$CONTAINERD_REGISTRY_PATH ]]; then
        mkdir \$CONTAINERD_REGISTRY_PATH/\$registry
        cat > \$CONTAINERD_REGISTRY_PATH/\$registry/hosts.toml <<EF
server = "http://\$registry"

[host."http://\$registry"]
  insecure_skip_verify = true
EF
    fi
done

EOF

  chmod 755 /tmp/shscript.sh
  /tmp/shscript.sh
  rm /tmp/shscript.sh
}
if [ -f /config/k8s-single-machine -o -f /config/central-box ]; then
    if [[ "$reason" == "BOUND"  || "$reason" == "BOUND6" ]]; then
        update_docker_config
    fi
    if [[ "$reason" == "REBOOT" || "$reason" == "REBOOT6" ]]; then
        update_docker_config
    fi
    if [[ "$reason" == "RENEW" || "$reason" == "RENEW6" ]]; then
        update_docker_config
    fi
    if [ -z "$reason" ]; then
        LOG "Reason is empty maybe static ip was set"
        update_docker_config
    fi
fi

sudo systemctl daemon-reload
sudo systemctl restart kubelet
sudo systemctl restart docker

sudo cp /etc/kubernetes/admin.conf /root/.kube/config
sudo chown $(id -u):$(id -g) /root/.kube/config

LOG "Regenerating certificates"
kubeadm init phase certs apiserver
kubeadm init phase certs etcd-peer
kubeadm init phase certs etcd-server


cd /etc/kubernetes
timeout=0
until sudo kubectl cluster-info | grep https://$new_ip:6443
do
        LOG "Waiting on certificates"
        ((++timeout)) && [[ $(( timeout % 120 )) == 0 ]] && timeout=0
        if [ $timeout -eq 0 ]; then
              LOG "Regenerating certificates"
              rm /etc/kubernetes/pki/apiserver.crt /etc/kubernetes/pki/apiserver.key /etc/kubernetes/pki/etcd/peer.crt /etc/kubernetes/pki/etcd/peer.key /etc/kubernetes/pki/etcd/server.crt /etc/kubernetes/pki/etcd/server.key
              kubeadm init phase certs apiserver
              kubeadm init phase certs etcd-peer
              kubeadm init phase certs etcd-server
              find . -type f | xargs sed -i "s/$old_ip/$new_ip/"
              sudo systemctl restart kubelet
        fi
        sleep 1
done

cd /etc/kubernetes/manifests/ && rm kube-controller-manager.yaml kube-scheduler.yaml

while GET_PODS kube-controller-manager || GET_PODS kube-scheduler
do
	LOG "Waiting on kube-controller-manager and kube-scheduler deletion..."
	sleep 3
done

LOG "Recreating kube-controller-manager and kube-scheduler manifests"
kubeadm init phase control-plane controller-manager
kubeadm init phase control-plane scheduler
systemctl restart kubelet

cd /var/tmp/

sudo kubectl get cm kube-proxy -n kube-system -o json > kube-proxy-template.json
sed -i "s/$old_ip/$new_ip/" kube-proxy-template.json
sudo kubectl patch configmap/kube-proxy -n kube-system --patch-file kube-proxy-template.json

sudo kubectl get cm cluster-info -n kube-public -o json > cluster-info-template.json
sed -i "s/$old_ip/$new_ip/" cluster-info-template.json
sudo kubectl patch configmap/cluster-info -n kube-public --patch-file cluster-info-template.json

chain=$(iptables -S -t nat | grep "default/kubernetes:https -> " | awk '{print $NF}')
if [ ! -z $chain ]; then
	mark_rid=$(iptables -L $chain --line-numbers -n -t nat | grep $old_ip | grep KUBE-MARK-MASQ | awk '{print $1}')
	dnat_rid=$(iptables -L $chain --line-numbers -n -t nat | grep to:$old_ip:6443 | grep DNAT | awk '{print $1}')
	LOG "Replacing iptables rules with ids $mark_rid and $dnat_rid in $chain chain"
	[ ! -z $mark_rid ] && iptables -t nat  -R $chain $mark_rid -s $new_ip/32 -m comment --comment "default/kubernetes:https" -j KUBE-MARK-MASQ
	[ ! -z $dnat_rid ] && iptables -t nat  -R $chain $dnat_rid -p tcp -m comment --comment "default/kubernetes:https" -m tcp -j DNAT --to $new_ip:6443
fi

kube_proxy=$(sudo kubectl get pods -A | grep kube-proxy | awk '{ print $2 }')

while true
do
  pod=$(sudo kubectl get pod -n kube-system ${kube_proxy} -o wide)
	ip_updated=$(grep ${new_ip} <<< "$pod")
	is_running=$(grep Running <<< "$pod")
	if [[ ! -z "${ip_updated}" && ! -z "${is_running}" ]]; then
	       LOG "Kube-proxy recreated with new ip"
	       break
	fi
	LOG "Waiting on kube-proxy deletion"
done

while true
do
	success=$(sudo kubectl rollout restart ds kube-proxy -n kube-system)
	[[ ! -z ${success} ]] && break
done
timeout=0
while true
do
	kube_proxy=$(sudo kubectl get pods -A | grep kube-proxy | awk '{ print $2 }')
	status=$(sudo kubectl get pod -n kube-system ${kube_proxy} -o jsonpath="{.status.phase}")
	if [[ "${status}" == "Pending" ]]; then
		kube_proxy=$(sudo kubectl get pods -A | grep kube-proxy | awk '{ print $2 }')
		while true
		do
			status=$(sudo kubectl get pod -n kube-system ${kube_proxy} -o jsonpath="{.status.phase}")
			if [[ "${status}" == "Running" ]]; then
				break 2
			fi
		done
	fi
	((++timeout)) && [[ $(( timeout % 100 )) == 0 ]] && LOG "Timeout waiting for kube-proxy ..." && break
	LOG "Waiting on kube-proxy rollout restart"
done

sleep 10
LOG "Restarting vpp container..."
vpp_manager=$(crictl ps | grep vpp | grep -v agent | awk '{ print $1 }')
sudo crictl rm -f "${vpp_manager}" || LOG "Failed to delete vpp container, reboot may be required"

sleep 10

LOG "Restarting agent container..."
agent=$(crictl ps | grep agent | awk '{ print $1 }')
sudo crictl rm -f "${agent}" || LOG "Failed to delete agent container, reboot may be required"

LOG "Setting $new_ip in kube-proxy service file"
sed -i "s/$old_ip/$new_ip/" /lib/systemd/system/kube-proxy.service
LOG "Restarting kube-proxy.service..."
systemctl daemon-reload
systemctl restart kube-proxy.service

# (vstojkov) Need to add controller IP and hostname in DNS of every pod in cluster, because
# CDI failed to pull image from controller's registry if user using hostname in image url
# More about this fix you can read at this article: https://coredns.io/plugins/hosts/
sed -i '10s/.*/            '${new_ip}' '${hostname}'/' "/kubernetes/templates/coredns.yaml"
kubectl apply -f "${TEMPLATES_PATH}/coredns.yaml"
