#!/usr/bin/env python3
import fcntl
import ipaddress
import json
import logging
import os
import socket
import struct
import subprocess
from re import MULTILINE
from re import search
from time import monotonic

import netifaces
import redis
from sysrepo_interfaces import sysrepo_interfaces3 as iface_api
from sysrepo_system import sysrepo_system as system_api
from systemd import journal

serial_cfg_file = '/config/.serial-number'
central_cbox_file = '/config/central-box'
UCPE_STATUS_FILE = '/config/_hb/ucpe_status.json'
ztp_auth_key = '/config/_hb/ztp_auth.key'
UCPE_STATUS_FILE_TEMPLATE = "{\"ucpe:status\": {\"status\": \"unknown\", \"hb-status\": \"unknown\", \"nsStatus\": \"unknown\", \"ztp-status\": \"unknown\", \"certs\": \"default\", \"license-id\": \"unknown\", \"apps\": \"false\", \"k8s\": \"false\", \"gpu\": \"false\",\"cluster\":{}, \"appslist\":{}, \"hostname\":\"\", \"auth_port\": \"unknown\"}}"
cbox = True if (os.path.isfile(central_cbox_file)) else False
HWTYPE_FILE = '/config/.hwtype'
NANOPIR2S = 'nanopir2s'
NANOPIR4S = 'nanopir4s'
IS_NANOPI = False
IS_NATIVE = None
HAMODE_FILE = '/config/.hamode'
CLUSTER_FILE = '/run/cluster.json'
APPS_LIST_FILE = '/tmp/application_list_heartbeat.json'
hwtype = ''
REDISPORT = 30378
NATIVE_ENABLED_FILE = '/config/native-enabled'
NATIVE_DISABLED_FILE = '/config/native-disabled'
ztp_protocol_version = 2
hb_update_stream = journal.stream(identifier='hb_update')
redis_ucpe_stream_name = 'ucpe_stream'
redis_controller_stream_name = 'controller_stream'
update_lock_file = '/var/run/nfvt-up.upgrading'
K8S_ENABLED_FILE='/config/k8s-enabled'
GPU_DRIVERS = '/config/nvidia-cuda'


def get_vcpe_manager_addr():
    mng = None
    system = system_api.SysrepoSystem(app_name="HB system", log_level=logging.ERROR)
    c_srvs = system.get_config_servers()
    for srv in c_srvs:
        mng = srv

        if mng is not None and mng != '':
            return mng

    try:
        # sysrepocfg works even if sysrepod is down
        sysrepo = subprocess.run(['/usr/bin/sysrepocfg', '--datastore=startup',
                                  '--format=json',
                                  '--xpath=/ucpe:config/system:system/config-server//*',
                                  '--export'],
                                 stdout=subprocess.PIPE, encoding='UTF-8')
    except subprocess.CalledProcessError as cpe:
        print('ERROR {}'.format(cpe), file=hb_update_stream)
        return ''
    except OSError:
        print('Missing /usr/bin/sysrepocfg', file=hb_update_stream)
        return ''

    if 0 != sysrepo.returncode:
        print('FAIL get config from sysrepo !', file=hb_update_stream)
        return ''

    data = json.loads(sysrepo.stdout)
    for srv in data['ucpe:config']['system:system']['config-server']:
        mng = srv['server']
        if mng is not None and mng != '':
            return mng
    return ""


def get_system_object(system_object):
    try:
        # sysrepocfg works even if sysrepod is down
        sysrepo = subprocess.run(['/usr/bin/sysrepocfg', '--datastore=running',
                                  '--format=json',
                                  '--xpath=/ucpe:config/system:system/%s' % system_object,
                                  '--export'],
                                 stdout=subprocess.PIPE, encoding='UTF-8')
    except:
        return ''

    if 0 != sysrepo.returncode:
        return ''

    try:
        data = json.loads(sysrepo.stdout)
        value = data['ucpe:config']['system:system'][system_object]
        if value is not None and value != '':
            return value
    except:
        return ""
    return ""


def get_project():
    return get_system_object("project")


def get_customer():
    return get_system_object("customer")


def get_service():
    return get_system_object("service")


def get_site():
    return get_system_object("site")


def get_operator():
    return get_system_object("operator")


def get_hb_base_mgmt_if():
    return {"name": "", "ipv4-address": "", "ipv6-address": "", "status": ""}


def get_hb_request():
    return {"auth": "unknown",
            "status": "unknown",
            "hb-status": "unknown",
            "ztp-status": "unknown",
            "certs": "default",
            "license-id": "unknown",
            "management": [],
            "version": "unknown",
            "apps": "false",
            "gpu": "false",
            "project": "",
            "customer": "",
            "service": "",
            "site": "",
            "operator": "",
            "hwtype": ""}


def update_status_file(key, value):
    if not (os.path.isfile(UCPE_STATUS_FILE)):
        ns_status_f = open(UCPE_STATUS_FILE, 'w')
        data = json.loads(UCPE_STATUS_FILE_TEMPLATE)
        data['ucpe:status'][key] = str(value)
        json.dump(data, ns_status_f)
        ns_status_f.close()
        os.chmod(UCPE_STATUS_FILE, 0o777)
    else:
        with open(UCPE_STATUS_FILE, "r+", encoding='utf-8') as json_file:
            data = json.load(json_file)

            data['ucpe:status'][key] = str(value)

            json_file.seek(0)
            json.dump(data, json_file)
            json_file.truncate()
            json_file.close()


def get_status_file_value(key):
    data = None
    if not (os.path.isfile(UCPE_STATUS_FILE)):
        data = json.loads(UCPE_STATUS_FILE_TEMPLATE)
    else:
        with open(UCPE_STATUS_FILE, "r", encoding='utf-8') as json_file:
            data = json.load(json_file)
            json_file.close()

    if data and key in data['ucpe:status']:
        return data['ucpe:status'][key]

    return "unknown"


def get_hb_status():
    status = get_status_file_value("hb-status")
    return status


def update_hb_status(status):
    # if we ever contacted, that is our state, no matter what
    if get_hb_status() == 'contacted':
        return

    update_status_file("hb-status", status)


def fill_sys_hwtype(hb_req):
    global hwtype
    hb_req['hwtype'] = hwtype
    hb_req['project'] = get_project()
    hb_req['operator'] = get_operator()
    hb_req['service'] = get_service()
    hb_req['site'] = get_site()
    hb_req['customer'] = get_customer()


def fill_sys_stat(hb_req):

    if cbox:
        os.system("systemctl reset-failed cloud-final.service 2>/dev/null 1>/dev/null || true")

    try:
        status = subprocess.check_output(['systemctl', 'is-system-running'])
        hb_req['status'] = status.strip()
    except subprocess.CalledProcessError as e:
        hb_req['status'] = e.output.strip()
    except OSError:
        print("systemctl failed", file=hb_update_stream)

    if os.path.isfile("/run/.reboot_in_progress"):
        hb_req['status'] = "rebooting"

    if 'running' in str(hb_req['status']):
        status = subprocess.check_output(['/scripts/basic_functions_call', 'check_lock_file', '-1'])
        if 'SYSTEM BUSY' in str(status):
            hb_req['status'] = 'busy'

        p1 = subprocess.Popen(['ps', '-C', 'post_homecalling.sh', '--no-headers'], stdout=subprocess.PIPE)
        p2 = subprocess.Popen(["wc", "-l"], stdin=p1.stdout, stdout=subprocess.PIPE)
        p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
        post_hm_running = p2.communicate()[0].strip().decode('utf-8')
        post_hm_running = int(post_hm_running)
        if post_hm_running:
            hb_req['status'] = 'busy'

    if 'running' in str(hb_req['status']):
        if (os.path.isfile('/run/post_hm_lock_file')) or (os.path.isfile('/run/sr_openstack_lock_file')):
            hb_req['status'] = 'busy'

    if type(hb_req['status']) is bytes:
        hb_req['status'] = hb_req['status'].decode().replace('\'', '')

    if 'running' in str(hb_req['status']):
        if os.path.isfile(update_lock_file):
            hb_req['status'] = 'upgrading'

    if ('running' in str(hb_req['status'])) and (os.path.isfile('/run/ztp-failed-flag')):
        # in case ZTP on boot failed, fallback - try homecalling
        os.system("rm -f /run/ztp-failed-flag")
        os.system("systemctl restart --no-block homecalling.service  2>/dev/null 1>/dev/null")
    elif ('running' in str(hb_req['status'])) and (os.path.isfile('/run/homecalling-failed')) and not cbox:
        # we are in running state, but homecalling is failing, lets try to see if we have to restart it.
        try:
            homecalling_status = subprocess.check_output(['systemctl', 'is-active', 'homecalling.service'])
            homecalling_status = homecalling_status.decode().strip()
        except:
            homecalling_status = "activating"

        if homecalling_status == "active":
            try:
                homecalling_vars = subprocess.check_output(['systemctl', 'show', 'homecalling.service'])
                homecalling_vars = homecalling_vars.decode()
                last_time = search(r"^ExecMainExitTimestampMonotonic=(.*)$", homecalling_vars, MULTILINE).group(1)
                last_time = int(int(last_time) / 1000000)
                sleep_time = open("/run/homecalling-failed", "r").readline().strip()
                sleep_time = int(sleep_time)
                time_now = int(monotonic())
                if sleep_time > 0 and (last_time + sleep_time) < time_now:
                    os.system("systemctl restart --no-block homecalling.service  2>/dev/null 1>/dev/null")
            except Exception as e:
                print('Exception {}'.format(e), file=hb_update_stream)

    if not cbox and not IS_NANOPI:
        # if we are not central-cbox,
        if 'running' in str(hb_req['status']):
            if 'unknown' in str(hb_req['license-id']):
                hb_req['status'] = 'unlicensed'

    if cbox:
        try:
            status = subprocess.check_output(['sudo', 'python3', '/scripts/isNodeReady.py', '\"\"', 'system-pods'])
            if not eval(status):
                hb_req['status'] = "degraded"
        except Exception as e:
            print("Checking status of system pods failed", file=hb_update_stream)

    update_status_file('status', hb_req['status'])


def fill_hb_stat(hb_req):
    if cbox:
        return

    hb_req['hb-status'] = get_hb_status()

    if hb_req['hb-status'] == 'not-contacted':
        p1 = subprocess.Popen(['ps', '-C', 'post_homecalling.sh', '--no-headers'], stdout=subprocess.PIPE)
        p2 = subprocess.Popen(["wc", "-l"], stdin=p1.stdout, stdout=subprocess.PIPE)
        p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
        post_hm_running = p2.communicate()[0].strip().decode('utf-8')
        post_hm_running = int(post_hm_running)
        if post_hm_running:
            hb_req['hb-status'] = 'contacted'


def fill_ztp_stat(hb_req):
    hb_req['ztp-status'] = get_status_file_value('ztp-status')


def fill_auth(hb_req):
    if cbox:  # if we are central-cbox, we'll use hostname for auth, as all controllers will be with same serial
        hb_req['auth'] = socket.gethostname()
        hb_req['hostname'] = hb_req['auth']
    else:
        hb_req['auth'] = open(serial_cfg_file, "r").readline().strip()
        hb_req['hostname'] = socket.gethostname()


def hb_get_interface_vtag(interface):
    vlan = interface.get('vlan', 0)

    if 'int' == type(vlan).__name__:
        return vlan

    if 0 == len(vlan):
        return 0

    vtag = int(vlan)

    return vtag


def fill_mgmt_ifs(hb_req):
    mng = None
    hb_req['management'] = []

    ifaces = iface_api.SysrepoInterfaces(app_name="HB interfaces", log_level=logging.ERROR)
    c_ifs = ifaces.get_interfaces()

    for interface in c_ifs:
        if 'management' in c_ifs[interface] and c_ifs[interface]['management']:
            mgmt_req_if = get_hb_base_mgmt_if()
            mgmt_req_if['name'] = c_ifs[interface]['name']

            if 'data' in c_ifs[interface]['name'] \
               or 'wifi' in c_ifs[interface]['name'] \
               or 'vxlan' in c_ifs[interface]['name']:
                sys_if = "br-%s" % c_ifs[interface]['name']
            else:
                sys_if = c_ifs[interface]['name']

            vlan = hb_get_interface_vtag(c_ifs[interface])
            if ('inband' in sys_if) or (os.path.isfile('/run/vpp.pid')):
                vlan = 0  # no vlan suffix for inband interfaces

            sys_if = sys_if if vlan == 0 else '%s.%d' % (sys_if, vlan)

            if os.path.isfile('/config/.hamode') and sys_if == 'br-data0':
                mgmt_req_if['ipv4-address'] = get_virtual_address(str(sys_if))
            else:
                mgmt_req_if['ipv4-address'] = get_ip_address(str(sys_if))

            mgmt_req_if['ipv6-address'] = get_ipv6_address(str(sys_if))
            mgmt_req_if['status'] = get_if_status(str(sys_if))
            hb_req['management'].append(mgmt_req_if)


def fill_version(hb_req):
    version_f = subprocess.check_output(['dpkg-query', "--showformat='${Version}'", '--show', 'nfvtimeos'])
    versionstr = version_f.decode().replace('\'', '')
    hb_req['version'] = (versionstr.strip())


def fill_certs(hb_req):
    hb_req['certs'] = 'default'
    try:
        is_user = subprocess.run(['/bin/grep', 'cert\ \"vcpe.crt\"',
                                  '/config/etc/openvpn/client.conf'],
                                 stdout=subprocess.PIPE, encoding='UTF-8')
    except subprocess.CalledProcessError as cpe:
        print('ERROR {}'.format(cpe), file=hb_update_stream)
        return
    except OSError:
        print('Missing /bin/grep', file=hb_update_stream)
        return
    if is_user.returncode:
        hb_req['certs'] = 'user'
    else:
        hb_req['certs'] = 'default'


def fill_license(hb_req):
    hb_req['license-id'] = 'unknown'
    if (os.path.isfile('/config/license/serial')):
        with open('/config/license/serial', "r", encoding='utf-8') as f:
            hb_req['license-id'] = f.read().strip()
    update_status_file('license-id', hb_req['license-id'])


def get_ip_address(ifname):
    SIOCGIFADDR = 0x8915
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        ifreq = fcntl.ioctl(
            s.fileno(),
            SIOCGIFADDR,  # SIOCGIFADDR
            struct.pack('256s', bytes(ifname[:15], 'utf-8')))
    except IOError as e:
        return ''
    return socket.inet_ntoa(ifreq[20:24])


def get_ipv6_address(ifname):
    try:
        addresses6 = netifaces.ifaddresses(ifname)[netifaces.AF_INET6]
    except Exception as e:
        return ''

    for addr in addresses6:
        if ifname in addr['addr']:
            addr_str = str.split(addr['addr'], '%')[0]
        else:
            addr_str = addr['addr']

        ipv6 = ipaddress.IPv6Address(str(addr_str))
        if ipv6.is_link_local is False and (ipv6.is_global is True or ipv6.is_private is True):
            return str(ipv6)
    return ''


def get_virtual_address(ifname):
    try:
        addresses_virtual = netifaces.ifaddresses(ifname)[netifaces.AF_INET]
    except Exception as e:
        print('Exception {}'.format(e), file=hb_update_stream)
        return ''

    # print('get_virtual_address len {}'.format(len(addresses_virtual)), file=hb_update_stream)
    if len(addresses_virtual) == 1:
        return addresses_virtual[0]['addr']
    elif len(addresses_virtual) == 2:
        return addresses_virtual[1]['addr']
    return ''


def get_if_status(ifname):
    SIOCGIFFLAGS = 0x8913
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        ifreq = fcntl.ioctl(
            s.fileno(),
            SIOCGIFFLAGS,
            struct.pack('256s', ifname[:15].encode()))

        if 0x01 & socket.ntohs((ifreq[16]) << 8 | (ifreq[17])):
            status = 'up'
        else:
            status = 'down'
    except Exception as e:
        status = 'down'

    return status


def call_server(hb_req):
    url = get_vcpe_manager_addr()
    headers = {'Content-Type': 'application/json', 'Token': ""}
    if os.path.isfile(ztp_auth_key):
        headers['Token'] = open(ztp_auth_key, 'r').readline().strip()
        if headers['Token'] != "":
            hb_req['Token'] = headers['Token']
    redis_body = json.dumps(hb_req)
    redis_srv = None
    try:
        redis_srv = redis.Redis(host=url, password='fd22985d0f43a88626a0c6d29d8cab5c',
                                ssl=True, socket_timeout=30, socket_keepalive=True, socket_connect_timeout=30,
                                ssl_keyfile='/scripts/key.pem',
                                ssl_certfile='/scripts/cert.pem',
                                ssl_ca_certs='/scripts/redis_ca.pem',
                                client_name=hb_req['auth'],
                                port=REDISPORT)
        if cbox:
            redis_srv.set('controller:status:{}'.format(hb_req['auth']), redis_body, ex=60)
        else:
            redis_srv.set('edge:status:{}'.format(hb_req['auth']), redis_body, ex=60)
        redis_srv.close()
        os.system("touch /run/manager-contacted")
    except Exception as e:
        if redis_srv:
                redis_srv.close()
        print('call_server {}:{} {}'.format(url, REDISPORT, e), file=hb_update_stream)
        return (False, {})
    # print('{}'.format(r), file=hb_update_stream)
    json_reply = {}
    stat = True
    return (stat, json_reply)


def fill_nanopi():
    global hwtype
    data = None
    if os.path.isfile(HWTYPE_FILE):
        with open(HWTYPE_FILE, 'r', encoding='utf-8') as f:
            data = f.read().strip()
            if data:
                hwtype = data

    if data and (data == NANOPIR2S or data == NANOPIR4S):
        return True

    return False


def fill_native():
    if os.path.isfile(NATIVE_ENABLED_FILE):
        return True
    elif os.path.isfile(NATIVE_DISABLED_FILE):
        return False

    return None


def fill_apps(hb_req):
    hb_req['apps'] = 'false'
    if IS_NATIVE and IS_NATIVE is True:
        hb_req['apps'] = 'true'

    update_status_file('apps', hb_req['apps'])
    return


def fill_k8s(hb_req):
    hb_req['k8s'] = 'false'
    if os.path.isfile(K8S_ENABLED_FILE) or os.path.isfile(central_cbox_file):
        hb_req['k8s'] = 'true'

    update_status_file('k8s', hb_req['apps'])
    return


def fill_auth_port(hb_req):
    hb_req['auth_port'] = 7374

    update_status_file('auth_port', hb_req['auth_port'])
    return


def fill_cluster(hb_req):
    hb_req['cluster'] = {}
    if os.path.isfile(HAMODE_FILE) \
       and os.path.isfile(CLUSTER_FILE):
        data = None
        with open(CLUSTER_FILE, 'r') as json_file:
            data = json.load(json_file)

        if data:
            hb_req['cluster'] = data

    update_status_file('cluster', hb_req['cluster'])
    return


def fill_apps_list(hb_req):
    hb_req['appslist'] = {}
    if not IS_NATIVE or IS_NATIVE is False:
        update_status_file('appslist', hb_req['appslist'])
        return

    try:
        if os.path.isfile(APPS_LIST_FILE):
            data = None
            with open(APPS_LIST_FILE, 'r') as json_file:
                data = json.load(json_file)

            if data:
                hb_req['appslist'] = data
    except:
        hb_req['appslist'] = {}

    update_status_file('appslist', hb_req['appslist'])
    try:
        out = subprocess.run(['/usr/bin/sudo', 'rm', '-f',
                              APPS_LIST_FILE],
                             stdout=subprocess.DEVNULL,
                             stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError as cpe:
        print('ERROR {}'.format(cpe), file=hb_update_stream)
        return
    except OSError:
        print('Missing /usr/bin/sudo', file=hb_update_stream)
        return

    if 0 != out.returncode:
        print('Fail rm old {}'.format(APPS_LIST_FILE),
              file=hb_update_stream)

    try:
        out = subprocess.run(['/usr/bin/sudo',
                              '/scripts/application_list.sh',
                              'all', 'heartbeat'],
                             stdout=subprocess.DEVNULL,
                             stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError as cpe:
        print('ERROR {}'.format(cpe), file=hb_update_stream)
        return
    except OSError:
        print('Missing /usr/bin/sudo', file=hb_update_stream)
        return

    if 0 != out.returncode:
        print('Fail calling /scripts/application_list.sh',
              file=hb_update_stream)

    return


def fill_gpu(hb_req):
    hb_req['gpu'] = 'false'
    if os.path.isfile(GPU_DRIVERS):
        hb_req['gpu'] = 'true'

    update_status_file('gpu', hb_req['gpu'])
    return


def main():
    if not (os.path.isfile(ztp_auth_key)):
        ztp_auth_f = open(ztp_auth_key, 'w')
        ztp_auth_f.close()
        os.chmod(ztp_auth_key, 0o777)

    global IS_NANOPI
    global IS_NATIVE
    IS_NANOPI = fill_nanopi()
    IS_NATIVE = fill_native()
    hb_req = get_hb_request()
    fill_license(hb_req)
    fill_sys_stat(hb_req)
    fill_hb_stat(hb_req)
    fill_ztp_stat(hb_req)
    fill_auth(hb_req)
    fill_mgmt_ifs(hb_req)
    fill_version(hb_req)
    fill_certs(hb_req)
    fill_apps(hb_req)
    fill_k8s(hb_req)
    fill_cluster(hb_req)
    fill_apps_list(hb_req)
    fill_sys_hwtype(hb_req)
    fill_auth_port(hb_req)
    fill_gpu(hb_req)

    # print("Send status - \"%s\"" % str(hb_req), file=hb_update_stream)

    (result, manager) = call_server(hb_req)

    if not result:
        print("record fail {}".format(hb_req), file=hb_update_stream)
        update_hb_status("not-contacted")
        return

    update_hb_status("contacted")


if __name__ == '__main__':
    main()
