#!/bin/bash
# -*- coding: utf-8 -*-
# coding: utf-8
# vim: tabstop=2 expandtab shiftwidth=2 softtabstop=2
# vim: set fileencoding=utf-8
set -o nounset
set -o pipefail
# set -o errexit

# Values passed in from keepalived
STATE=${1}

AMAZON_FILE=/config/amazon.json
CLUSTER_FILE=/config/cluster.json

if [ ! -f ${AMAZON_FILE} ]; then
  exit 0
fi
exec 1> >(systemd-cat -p info -t "keepalived_amazon") 2>&1
set -o xtrace

script_name=$(basename ${BASH_SOURCE[0]})
for pid in $(pidof -x ${script_name}); do
  if [ ${pid} -ne $$ ]; then
    echo "Kill other ${pid} keepalived_amazon.sh my ${$} instances"
    kill -9 ${pid}
  fi
done

export AWS_ACCESS_KEY_ID=$(/usr/bin/jq '.amazon.access_key_id | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')
export AWS_SECRET_ACCESS_KEY=$(/usr/bin/jq '.amazon.secret_access_key | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')
export AWS_DEFAULT_REGION=$(/usr/bin/jq '.amazon.region | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')
export ALLOCATION_ID=$(/usr/bin/jq '.amazon.allocation_id | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')
export VIRTUAL_ADDRESS=$(/usr/bin/jq '.virtual_address | select( . != null )' ${CLUSTER_FILE} | tr -d '" \n')

AWS_SLEEP=5
AWS_MAX_TIMEOUT=300

function retry_execute {
  cmd=${1}
  retry=0
  while true; do
    ${cmd}
    if [ "$?" != "0" ]; then
      echo "fail ${cmd}, retry..."
      sleep ${AWS_SLEEP}
      retry=$((retry+${AWS_SLEEP}))
      if [ ${retry} -ge ${AWS_MAX_TIMEOUT} ]; then
        echo "retry failed, will try switchover"
        touch /tmp/amazon_timeout
        exit 1
      fi
    else
      break
    fi
  done
} # function retry_execute

retry_execute "host -4 -W 1 ec2.${AWS_DEFAULT_REGION}.amazonaws.com"

if [ -n "${ALLOCATION_ID}" ]; then
  export HA_NODE_1=$(/usr/bin/jq '.amazon.peers[0] | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')
  export HA_NODE_2=$(/usr/bin/jq '.amazon.peers[1] | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')

  # Determine the internal or private dns name of the instance
  LOCAL_INTERNAL_DNS_NAME=$(wget -q -O - http://169.254.169.254/latest/meta-data/local-hostname)
  LOCAL_INTERNAL_DNS_NAME=$(echo ${LOCAL_INTERNAL_DNS_NAME} | sed -e 's/% //g')

  # OTHER_INSTANCE_DNS_NAME is the other node than this one, default assignment is HA_NODE_1
  OTHER_INSTANCE_DNS_NAME=${HA_NODE_1}

  # Use LOCAL_INTERNAL_DNS_NAME to determine which node this is and to set the Other Node name
  if [ "${HA_NODE_1}" = "${LOCAL_INTERNAL_DNS_NAME}" ]; then
    OTHER_INSTANCE_DNS_NAME=${HA_NODE_2}
  fi

  # Get the local instance ID
  INSTANCE_ID=$(wget -q -O - http://169.254.169.254/latest/meta-data/instance-id)
  INSTANCE_ID=$(echo $INSTANCE_ID | sed -e 's/% //g')

  # Get the instance ID of the other node
  OTHER_INSTANCE_ID=$(/usr/local/bin/aws ec2 describe-instances --filter "Name=private-dns-name,Values=${OTHER_INSTANCE_DNS_NAME}" | /usr/bin/python3 -c 'import json,sys;obj=json.load(sys.stdin); print obj["Reservations"][0]["Instances"][0]["InstanceId"]')

  # Get the ASSOCIATION_ID of the ElasticIP to the Instance
  ASSOCIATION_ID=$(/usr/local/bin/aws ec2 describe-addresses --allocation-id ${ALLOCATION_ID} | /usr/bin/python3 -c 'import json,sys;obj=json.load(sys.stdin); print obj["Addresses"][0]["AssociationId"]')

  # Get the INSTANCE_ID of the system the ElasticIP is associated with
  EIP_INSTANCE=$(/usr/local/bin/aws ec2 describe-addresses --allocation-id $ALLOCATION_ID | /usr/bin/python3 -c 'import json,sys;obj=json.load(sys.stdin); print obj["Addresses"][0]["InstanceId"]')
else
  export NETWORK_INTERFACE_ID_1=$(/usr/bin/jq '.amazon.network_interface_ids[0] | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')
  export NETWORK_INTERFACE_ID_2=$(/usr/bin/jq '.amazon.network_interface_ids[1] | select( . != null )' ${AMAZON_FILE} | tr -d '" \n')
  export LOCAL_PRIMARY_PRIVATE_IP=$(/usr/bin/sysrepocfg -X -x /ucpe:config/interfaces:interface[name=\'data0\']/ipv4-prefix -d running -f json | grep "ipv4-prefix" |  cut -d ':' -f 2 | tr -d ' "' | cut -d '/' -f 1)

  LOCAL_NETWORK_INTERFACE_ID=$(/usr/local/bin/aws ec2 describe-network-interfaces | /usr/bin/jq ".NetworkInterfaces[] | select (.PrivateIpAddress==\"${LOCAL_PRIMARY_PRIVATE_IP}\") | .NetworkInterfaceId" | tr -d '\n"')
  if [ "${NETWORK_INTERFACE_ID_1}" = "${LOCAL_NETWORK_INTERFACE_ID}" ]; then
    OTHER_NETWORK_INTERFACE_ID=${NETWORK_INTERFACE_ID_2}
  else
    OTHER_NETWORK_INTERFACE_ID=${NETWORK_INTERFACE_ID_1}
  fi
fi

case ${STATE} in
  "MASTER")
    rm -rf /config/_amazon/master/*
    if [ -n "${ALLOCATION_ID}" ]; then
      if [ -n "${ASSOCIATION_ID}" ]; then
        retry_execute "/usr/local/bin/aws ec2 disassociate-address --association-id ${ASSOCIATION_ID}"
      fi
      retry_execute "/usr/local/bin/aws ec2 associate-address --allocation-id ${ALLOCATION_ID} --instance-id ${INSTANCE_ID}"
      logger -t keepalived_amazon "MASTER Transfer from ${OTHER_INSTANCE_ID} to ${INSTANCE_ID}"
    elif [ -n "${NETWORK_INTERFACE_ID_1}" ]; then
      if [ -n "${OTHER_NETWORK_INTERFACE_ID}" ]; then
        /usr/local/bin/aws ec2 unassign-private-ip-addresses --network-interface-id ${OTHER_NETWORK_INTERFACE_ID} --private-ip-addresses ${VIRTUAL_ADDRESS}
      fi
      retry_execute "/usr/local/bin/aws ec2 assign-private-ip-addresses --network-interface-id ${LOCAL_NETWORK_INTERFACE_ID} --private-ip-addresses ${VIRTUAL_ADDRESS}"
    fi
    exit 0
    ;;
  "BACKUP"|"FAULT")
    rm -rf /config/_amazon/backup/*
    if [ -n "${ALLOCATION_ID}" ]; then
      if [ "${INSTANCE_ID}" = "${EIP_INSTANCE}" ]; then
        if [ -n "${ASSOCIATION_ID}" ]; then
          retry_execute "/usr/local/bin/aws ec2 disassociate-address --association-id ${ASSOCIATION_ID}"
        fi
        retry_execute "/usr/local/bin/aws ec2 associate-address --allocation-id ${ALLOCATION_ID} --instance-id ${OTHER_INSTANCE_ID}"
        logger -t keepalived_amazon "BACKUP Transfer from ${INSTANCE_ID} to ${OTHER_INSTANCE_ID}"
      fi
    elif [ -n "${NETWORK_INTERFACE_ID_1}" ]; then
      /usr/local/bin/aws ec2 unassign-private-ip-addresses --network-interface-id ${LOCAL_NETWORK_INTERFACE_ID} --private-ip-addresses ${VIRTUAL_ADDRESS}
    fi
    exit 0
    ;;
  *)
    logger -t keepalived_amazon "Unknown state: ${STATE}"
    exit 1
    ;;
esac
