#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import os
import re
import shutil
import signal
import tempfile
import time

import ha

log = ha.set_logs('keepalivedstate', level=logging.DEBUG)


def is_amazon():
    if os.path.exists('/sys/hypervisor/uuid'):
        with open('/sys/hypervisor/uuid', 'r') as f:
            line = f.read().strip()

        if line:
            is_ec2 = re.match('ec2', line, re.IGNORECASE)
            if is_ec2:
                return True

    elif os.path.exists('/sys/devices/virtual/dmi/id/product_uuid'):
        with open('/sys/devices/virtual/dmi/id/product_uuid', 'r') as f:
            line = f.read().strip()

        if line:
            is_ec2 = re.match('ec2', line, re.IGNORECASE)
            if is_ec2:
                return True

    return False


def is_azure():
    if os.path.exists('/sys/devices/virtual/dmi/id/sys_vendor'):
        with open('/sys/devices/virtual/dmi/id/sys_vendor', 'r') as f:
            line = f.read().strip()

        if line:
            is_microsoft = re.match('Microsoft', line)
            if is_microsoft:
                return True

    return False


def make_me_master():
    log.debug('begin')
    ha.exec_get_status(['/bin/systemctl', 'start', '--no-block',
                        'haproxy.service', 'habeat.timer',
                        'pingManager.service', 'sr-currentconfig.service'],
                       log)
    ha.exec_get_status(['/bin/systemctl', 'stop', '--no-block',
                        'healthcheck.service'], log)
    ha.exec_get_status(['/bin/systemctl', 'start', '--no-block',
                        'collectd.service'], log)
    if is_azure():
        log.debug('is_azure {}'.format(is_azure()))
        ha.exec_get_status(['/bin/systemctl', 'start', '--no-block',
                            'az-healthcheck.service'], log)
        ha.exec_get_status(['/bin/systemctl', 'enable', '--no-block',
                            'az-healthcheck.service'], log)
        tempfile.NamedTemporaryFile(dir='/config/_azure/', prefix='azure_handle.')
    elif is_amazon():
        log.debug('is_amazon {}'.format(is_amazon()))
        tempfile.NamedTemporaryFile(dir='/config/_amazon/master/', prefix='amazon_handle.')

    log.debug('Wait 85seconds for VPN to up')
    time.sleep(85)
    ha.exec_get_status(['/bin/systemctl', 'start', '--no-block',
                        'openvpn@client.service'], log)
    log.debug('end')


def make_me_backup():
    log.debug('begin')
    ha.exec_get_status(['/bin/systemctl', 'stop', '--no-block',
                        'collectd.service', 'habeat.timer',
                        'haproxy.service', 'pingManager.service',
                        'sr-currentconfig.service'], log)
    if os.path.exists('/run/openvpn/client.pid'):
        log.debug('/run/openvpn/client.pid exists')
        vpnid = None
        with open('/run/openvpn/client.pid', 'r') as f:
            vpnpid = f.read().strip()

        log.debug('vpnid {}'.format(vpnid))
        ha.exec_get_status(['/bin/systemctl', 'stop',
                            'openvpn@client.service'], log)
        ha.exec_get_status(['/bin/systemctl', 'reset-failed',
                            'openvpn@client.service'], log)

        if vpnpid:
            ha.exec_get_status(['/usr/bin/kill', '-9',
                                '{}'.format(vpnpid)], log)
            if os.path.isfile('/run/openvpn/client.pid'):
                os.remove('/run/openvpn/client.pid')

    ha.exec_get_status(['/bin/systemctl', 'start',
                        '--no-block', 'healthcheck.service'], log)
    ha.exec_get_status(['/usr/bin/killall', 'collectd'], log)
    ha.exec_get_status(['/bin/systemctl', 'reset-failed',
                        'haproxy.service', 'pingManager.service', 'sr-currentconfig.service',
                        'collectd.service', 'habeat.service'], log)
    if is_azure():
        log.debug('is_azure {}'.format(is_azure()))
        ha.exec_get_status(['/usr/bin/pkill', '-9', 'keepalived_azure.sh'], log)
        try:
            os.remove('/tmp/azure_timeout')
        except OSError:
            pass

        ha.exec_get_status(['/bin/systemctl', 'stop',
                            '--no-block', 'az-healthcheck.service'], log)
        ha.exec_get_status(['/bin/systemctl', 'disable',
                            '--no-block', 'az-healthcheck.service'], log)
    elif is_amazon():
        log.debug('is_amazon {}'.format(is_amazon()))
        ha.exec_get_status(['/usr/bin/pkill', '-9', 'keepalived_amazon.sh'], log)
        try:
            os.remove('/tmp/amazon_timeout')
        except OSError:
            pass

        tempfile.NamedTemporaryFile(dir='/config/_amazon/backup/', prefix='amazon_handle.')

    log.debug('end')


def signal_handler(signum, _frame):
    if signum != signal.SIGHUP:
        return

    log.debug('begin')
    state = None
    if os.path.exists('/config/cluster_role'):
        with open('/config/cluster_role', 'r') as f:
            state = f.read().strip()

    if state not in ['MASTER', 'BACKUP', 'FAULT']:
        log.debug('Unknown state {}'.format(state))
        return

    try:
        os.remove('/run/cluster.json')
    except OSError:
        pass

    shutil.copyfile('/config/cluster.json', '/run/cluster.json')
    if state in ['MASTER']:
        make_me_master()
    else:
        make_me_backup()

    log.debug('end')


def main():
    with open('/run/keepalivedstate.pid', 'w') as f:
        f.write('{}'.format(os.getpid()))

    signal.signal(signal.SIGHUP, signal_handler)
    while True:
        signal.pause()


if __name__ == '__main__':
    main()
