#! /usr/bin/env python3
import os
import sys
import json
import logging
import logging.handlers
import re
import vppapi_if.vppapi_if as vppif
from pyroute2 import netns, IPRoute
from ipaddress import IPv4Network, IPv4Address

log = None
DHCPD_OFFSET = 800


def set_log(name, level):
    global log
    log = logging.getLogger(name=name)
    log.setLevel(level=level)

    syslog_hnd = logging.handlers.SysLogHandler(address='/dev/log')
    formatter = logging.Formatter('%(name)s: %(levelname)s %(message)s')
    syslog_hnd.setFormatter(formatter)
    log.addHandler(syslog_hnd)


def create_ns(ifname):
    nsname = '{}_dhcpd'.format(ifname)
    if nsname in netns.listnetns():
        log.debug('Netns already exists')
        return
    netns.create(nsname)


def delete_ns(ifname):
    nsname = '{}_dhcpd'.format(ifname)
    if nsname not in netns.listnetns():
        log.debug('Netns does not exist')
        return
    netns.remove(nsname)


def clear_dhcpd_cfg(ifname, pid_file, leases_file, conf_file):
    if os.path.exists(pid_file):
        try:
            pid = open(pid_file, 'rb').readline()
            os.kill(int(pid), 9)
            os.system('rm -f {}'.format(pid_file))
        except:
            log.debug('There is no {}-dhcpd process'.format(ifname))

    if os.path.exists(leases_file):
        log.debug('Delete {}'.format(leases_file))
        os.system('rm -rf {}*'.format(leases_file))

    if os.path.exists(conf_file):
        log.debug('Delete {}'.format(conf_file))
        os.system('rm -f {}'.format(conf_file))


def dhcpd_status(vpp_if, ifname):
    conf_file = '/var/{}-dhcpd.conf'.format(ifname)
    leases_file = '/var/lib/dhcp/ns{}.leases'.format(ifname)
    nsname = '{}_dhcpd'.format(ifname)
    pid_file = '/var/run/{}-dhcpd.pid'.format(ifname)

    tap_id = int(re.sub('[a-z-]', '', ifname))
    bridge_id = vpp_if.search_vppif_by_hostifname(ifname)[1]
    if bridge_id == None:
        sys.exit("Unable to find interface with name: {}".format(ifname))
    tap_id = vpp_if.get_ifidx_by_name(
        'tap{}'.format(tap_id+bridge_id+DHCPD_OFFSET))
    nsname = '{}_dhcpd'.format(ifname)

    if nsname not in netns.listnetns():
        sys.exit('Failed to create namespace {}!'.format(nsname))
    elif tap_id == None:
        sys.exit('Failed to create tap interface in VPP!')
    elif not os.path.exists(conf_file) or not os.path.exists(leases_file):
        sys.exit('Failed to create configuration for DHCP!')
    elif not os.path.exists(pid_file) or os.stat(pid_file).st_size == 0:
        sys.exit('Failed to start {}-dhcpd process!'.format(ifname))


def dhcpd_cfg(vpp_if, ifname, config, status):
    conf_file = '/var/{}-dhcpd.conf'.format(ifname)
    leases_file = '/var/lib/dhcp/ns{}.leases'.format(ifname)
    nsifname = 'ns{}'.format(ifname)
    nsname = '{}_dhcpd'.format(ifname)
    pid_file = '/var/run/{}-dhcpd.pid'.format(ifname)

    log.debug('status is {}'.format(status))
    if status == 'IN_CREATE':
        log.debug('IN CREATE')
        create_dhcpd_if(vpp_if, ifname)
        os.system('/usr/bin/touch {}'.format(leases_file))
        os.system('/usr/bin/touch {}'.format(pid_file))
        return 0
    elif status == 'IN_CLOSE_WRITE':
        log.debug('IN_CLOSE_WRITE')
        try:
            pid = open(pid_file, 'rb').readline()
            if len(pid) > 0:
                log.debug('Stop and restart dhcpd before new configuration')
                os.kill(int(pid), 9)
                os.system('rm -f {}'.format(pid_file))
        except Exception as e:
            log.debug('Failed to stop and restart dhcpd before new configuration')
            pass
    elif status == 'IN_DELETE':
        log.debug('IN_DELETE')
        clear_dhcpd_cfg(ifname, pid_file, leases_file, conf_file)
        delete_dhcpd_if(vpp_if, ifname)
        delete_ns(ifname)
        return 0

    # if status in ['IN_CREATE', 'IN_CLOSE_WRITE', 'IN_MODIFY']:
    subnet = config['dhcp']['subnet']
    gateway = config['dhcp']['gateway']
    range = config['dhcp']['range']
    ldns = config['dhcp']['dns']
    domain = config['dhcp']['domain']
    static_config = config['dhcp']['staticConfigurations']
    cfg = ''
    if ldns != '':
        cfg += 'option domain-name-servers {};\n'.format(ldns)
    if domain != '':
        cfg += 'option domain-name \"{}\";\n'.format(domain)
    if gateway != '':
        cfg += 'option routers {};\n'.format(gateway)
    if subnet != '':
        cfg += 'subnet {} netmask {} {{\n'.format(subnet.split(
            '/')[0], (IPv4Network(subnet).with_netmask).split('/')[1])
    if range != '':
        cfg += '    range {};\n}}\n'.format(range.replace("-", " "))
    if static_config != None and static_config != []:
        for host in static_config:
            cfg += 'host {} {{\n'.format(host['host'])
            cfg += '    hardware ethernet {};\n'.format(
                host['hardwareEthernetAddress'])
            if host['fixedIpAddress'] != '':
                cfg += '    fixed-address {};\n}}\n'.format(
                    host['fixedIpAddress'])
            else:
                cfg += '}\n'
    cfg += 'lease-file-name "/var/lib/dhcp/{}.leases";\n'.format(nsifname)
    cfg += 'deny declines;\ndeny duplicates;\none-lease-per-client true;\nignore-client-uids true;\nauthoritative;'\
        '\ndeny unknown-clients;\nping-check false;'

    with open(conf_file, "w") as cfg_file:
        cfg_file.write(cfg)
    try:
        # (NFV-17459) Run dhcp server in namespace if the main interface IP is not equal to 'gateway' IP
        ipr = IPRoute()
        int_addr = ipr.get_addr(label=ifname)
        if gateway != '' and (len(int_addr) >= 1 and IPv4Address(gateway) == IPv4Address(int_addr[0].get_attr('IFA_LOCAL'))):
            log.debug('Executing dhcpd -cf {} -pf {}'.format(conf_file, pid_file))
            os.system(
                'dhcpd -cf {} -pf {}'.format(conf_file, pid_file))
        else:
            os.system('ip netns exec {} ifconfig {} {} netmask {}'.format(
                nsname, nsifname, gateway, (IPv4Network(subnet).with_netmask).split('/')[1]))
            log.debug('Run {}-dhcpd server on {}'.format(ifname, nsifname))
            os.system(
                'ip netns exec {} dhcpd -cf {} -pf {}'.format(nsname, conf_file, pid_file))
    except:
        sys.exit('Could not run {}-dhcpd server'.format(ifname))


def create_dhcpd_if(vpp_if, ifname):
    if not ifname:
        log.error('Interface name is not set!')
    create_ns(ifname)
    tap_id = int(re.sub('[a-z-]', '', ifname))

    # macarr =  [0xfe, random.getrandbits(8),
    #    random.getrandbits(8), random.getrandbits(8),
    #    random.getrandbits(8), random.getrandbits(8)]
    macarr = [0xfe, 0xfe, 0xfe, 0xfe, 0xfe, (tap_id + 150)]
    usedmac = ':'.join(["%02x" % x for x in macarr])
    nsname = '{}_dhcpd'.format(ifname)
    log.debug('DHCPD NS name {}'.format(nsname))
    log.debug('DHCPD ID {}'.format(tap_id))
    bridge_id = vpp_if.search_vppif_by_hostifname(ifname)[1]
    if bridge_id == None:
        sys.exit("Unable to find interface with name: {}".format(ifname))
    (ret, tapidx) = vpp_if.create_tap_interface('ns{}' .format(ifname), id=tap_id+bridge_id+DHCPD_OFFSET,
                                                host_namespace_set=True,
                                                host_namespace=nsname,
                                                host_mac_addr=usedmac,
                                                mac_addr=usedmac,
                                                host_mac_addr_set=True)
    if 0 != ret:
        sys.exit("Failed to create tap interface! Ret: {}".format(ret))

    # Create bridge-domain
    vpp_if.add_to_bridge(bridge_id, tapidx)
    vpp_if.ifup(tapidx)
    log.debug('DHCPD interface with id {} created in NS {}' .format(
        tapidx, nsname))


def delete_dhcpd_if(vpp_if, ifname):
    if not ifname:
        log.debug('DHCPD Interface does not exists!')
        return

    tap_id = int(re.sub('[a-z-]', '', ifname))
    bridge_id = vpp_if.search_vppif_by_hostifname(ifname)[1]
    if bridge_id == None:
        sys.exit("Unable to find interface with name: {}".format(ifname))
    tap_id = vpp_if.get_ifidx_by_name(
        'tap{}'.format(tap_id+bridge_id+DHCPD_OFFSET))
    if tap_id != None:
        vpp_if.ifdown(tap_id)
        vpp_if.delete_tap_interface(tap_id)
        log.debug('DHCPD interface with id {} deleted' .format(tap_id))


if __name__ == "__main__":

    set_log('kubernetes-dhcp-cfg', logging.DEBUG)

    if len(sys.argv) < 3:
        sys.exit('Missing config params!')

    config_file = sys.argv[1] + '/' + sys.argv[2]

    vpp_if = vppif.VppApiInterface('kubernetes-dhcp')

    try:
        if sys.argv[3] == 'CHECK_STATUS':
            dhcpd_status(vpp_if, sys.argv[2])
        elif sys.argv[3] == 'IN_CREATE':
            dhcpd_cfg(vpp_if, sys.argv[2], {}, sys.argv[3])
        else:
            with open(config_file) as json_file:
                config = json.load(json_file)
            dhcpd_cfg(vpp_if, sys.argv[2], config, sys.argv[3])
        del(vpp_if)
    except SystemExit as e:
        del(vpp_if)
        sys.exit('Kubernetes DHCP Error: {}'.format(e))
    except Exception as e:
        dhcpd_cfg(vpp_if, sys.argv[2], '', 'IN_DELETE')
        del(vpp_if)
        sys.exit('Kubernetes DHCP Error: {}'.format(e))

    sys.exit(0)
