#! /usr/bin/env python3
import os
import re
import subprocess
import sys
from platform import machine

from systemd import journal
from tveif import if_mgmt
from tveif import if_nat
from tveif import if_lte

import neutron_agent as na

OVS_DIR = "/usr/share/openvswitch"
DB_SOCK = "/run/openvswitch/db.sock"
OVS_CFG_DB = "/var/lib/openvswitch/conf.db"
dpdk_kernel_mods = ["uio", "igb_uio"]
# LIGHT_DPDK_CPU=None
bootSystemMACs = dict()
ovsStream = journal.stream(identifier='ovs-config')
sysrepo_file = "/config/_sysrepo"
vnf_management_if = "mgmt0"
LTEIF_OFPORT = 9
txqueuelen_default = '20000'
localif_def_queuelen = '4096'


def get_hw_type():
    if not os.path.isfile('/config/.hwtype'):
        return 'intel'

    with open('/config/.hwtype', 'r') as hw_type_file:
        return hw_type_file.readline().splitlines()[0]


hw_type = get_hw_type().lower()


def is_arm_arch():
    if 'aarch64' in machine():
        return True
    return False


class if_nat_(if_nat.if_nat):
    def __init__(self, if_cfg=None):
        self.name = None
        super(if_nat.if_nat, self).__init__(self.name, if_cfg)

    def write_ifcfg(self, fname=None):
        pass


def ovs_clean_db(db_file):
    if not os.path.isfile(db_file):
        print('File \'%s\' do not exist' % db_file, file=ovsStream)
        return

    os.remove(db_file)


def getBootSystemMACs():
    ifNames = [f for f in os.listdir('/sys/class/net') if
               os.path.isfile(os.path.join('/sys/class/net', f)) or os.path.islink(os.path.join('/sys/class/net', f))]
    for name in ifNames:
        if not os.path.isfile('/sys/class/net/' + name + '/address'):
            continue
        with open('/sys/class/net/' + name + '/address', 'r') as f1:
            mac = (f1.readline())[0:17]
            bootSystemMACs.update({name: mac})
            if name.startswith('data'):
                bootSystemMACs.update({'br-' + name: mac})

    # for n,k in bootSystemMACs.iteritems():
    #   print('%s->%s'%(n,k),file=ovsStream)


def getmac(interface):
    if interface in bootSystemMACs.keys():
        return bootSystemMACs[interface]
    else:
        mac = ""
        if True == os.path.isfile('/sys/class/net/' + interface + '/address'):
            with open('/sys/class/net/' + interface + '/address', 'r') as f1:
                mac = (f1.readline())[0:17]

        if not mac:
            print('Fail. Error. No MAC for %s.Quit !!!' % interface, file=ovsStream)
            return "00:00:00:00:00:00"
        else:
            print('%s:%s' % (interface, mac))
            return mac


def ovs_get_pci_if(ifname):
    net_rules_path = "/etc/udev/rules.d/70-persistent-net.rules"
    if not os.path.isfile(net_rules_path):
        return 'none'
    with open(net_rules_path, "r") as f:
        lines = f.readlines()

    for line in lines:
        if 0 == len(line.strip()):
            continue

        line = line.replace('==', '=')
        line = line.replace(' ', '')
        rules = line.strip().split(',')
        rules_dict = dict()
        for rule in rules:
            (key, val) = tuple(rule.split('='))
            rules_dict[key] = val.replace('\"', '')

        if 'net' == rules_dict.get('SUBSYSTEM', 'none') and ifname == rules_dict.get('NAME', 'none'):
            return rules_dict.get('KERNELS', 'none')

    return None


def ovs_attach_dpdk_if(pci_id):
    try:
        subprocess.check_call(['/sbin/dpdk-devbind', '-b', 'igb_uio', pci_id])
    except subprocess.CalledProcessError as e:
        print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
    except OSError as e:
        print('System error - \'%s\'' % e.strerror, file=ovsStream)


def ovs_dpdk_unbind_interfaces():
    output = subprocess.check_output(['/sbin/dpdk-devbind', '-s']).decode()
    empty_ln = 0
    start = False
    dpdk_if = 0
    for ln in output.splitlines():
        # syslog.syslog(syslog.LOG_DEBUG,'LN -> %s'%ln)
        if 0 == len(ln):
            empty_ln += 1

            if empty_ln < 2:
                continue
            else:
                break

        if 'DPDK-compatible' in str(ln):
            start = True
            continue

        if '======' in str(ln) or not start:
            continue

        fields = ln.split(' ')

        for token in fields:
            if 'unused' in token:
                legacy_drv_str = token.split('=')
                subprocess.check_call(['/sbin/dpdk-devbind', '-b', '%s' % legacy_drv_str[1], fields[0]])
                dpdk_if += 1


def ovs_get_interface_vtag(interface):
    vlan = interface.get('vlan', 0)

    if 'int' == type(vlan).__name__:
        return vlan

    if 0 == len(vlan): return 0

    vtag = int(vlan)
    return vtag


def dpdk_load_modules(load):
    if is_arm_arch() and hw_type != 'hr330a':
        return
    for mods in dpdk_kernel_mods:
        if True == load:
            subprocess.check_call(['/sbin/modprobe', '%s' % mods])
        else:
            subprocess.check_call(['/sbin/rmmod', '%s' % mods])


def create_bridge(name, datapath_type='system'):
    try:
        subprocess.check_call(['/usr/bin/ovs-vsctl', '--may-exist', 'add-br', name, '--', 'set', 'bridge', name,
                               'datapath_type=%s' % datapath_type])
    except subprocess.CalledProcessError as e:
        print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
    except OSError as e:
        print('System error - \'%s\'' % e.strerror, file=ovsStream)

    subprocess.check_call(['/usr/bin/ovs-vsctl', 'set', 'bridge', name, 'datapath_type=%s' % datapath_type])

    subprocess.check_call(['ifconfig', '%s' % name, 'txqueuelen', txqueuelen_default])

    subprocess.check_call(['/usr/bin/ovs-vsctl', 'set', 'Interface', name, 'options:n_rxq_desc=%s' % localif_def_queuelen, 'options:n_txq_desc=%s' % localif_def_queuelen])


def ovs_clean_entire_networking():
    try:
        subprocess.check_call(['/scripts/apply_networking.py', '-z', 'deleteall'])
    except subprocess.CalledProcessError as e:
        print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
    except OSError as e:
        print('System error - \'%s\'' % e.strerror, file=ovsStream)


def sr_ovs_net_create_bridge_interfaces(jconf):
    for interface in jconf:
        if not 'data' in interface:
            continue

        bootproto = jconf[interface].get('bootproto', 'manual')
        if bootproto == 'none':
            bootproto = 'manual'
            if ('ipv4-prefix' in jconf[interface] and jconf[interface]['ipv4-prefix']) or (
                    'ipv6-prefix' in jconf[interface] and jconf[interface]['ipv6-prefix']):
                bootproto = 'static'
            else:
                bootproto = 'manual'

        print('%s bootproto is %s' % (interface, bootproto), file=ovsStream)

        vlan = ovs_get_interface_vtag(jconf[interface])
        if 0 != vlan:
            print('%s in VLAN %d' % (interface, vlan), file=ovsStream)
        else:
            print('%s not in VLAN' % (interface), file=ovsStream)

        br_if_name = "br-%s%s" % (interface, '' if 0 == vlan else '.%d' % vlan)

        createBridgeIfString = [
            '/scripts/apply_networking.py',
            "-n%s" % str(br_if_name),
            "--ovs-master", "ovs",
            "--ovs-ports", str(interface)
        ]

        if 'ipv4-prefix' in jconf[interface] and jconf[interface].get('ipv4-prefix'):
            createBridgeIfString += [str("--ipv4-prefix"), str("%s" % jconf[interface].get('ipv4-prefix'))]
            if bootproto == 'none':
                bootproto = 'static'

        if 'gateway' in jconf[interface] and jconf[interface].get('gateway'):
            createBridgeIfString += [str("-g%s" % jconf[interface].get('gateway'))]

        if 'dns' in jconf[interface] and jconf[interface].get('dns'):
            createBridgeIfString += [str("-d%s" % jconf[interface].get('dns'))]

        if 'bootproto' in jconf[interface] and 'dhcp' == jconf[interface].get('bootproto'):
            createBridgeIfString += ["-bdhcp"]
        else:
            createBridgeIfString += [str('-b%s' % bootproto)]

        if 'ipv6-prefix' in jconf[interface] and jconf[interface].get('ipv6-prefix'):
            createBridgeIfString += [str("--ipv6-prefix=%s" % jconf[interface].get('ipv6-prefix'))]
            createBridgeIfString += ['-p"echo 0 > /proc/sys/net/ipv6/conf/%s/accept_dad"' % str(br_if_name)]

        if vlan:
            createBridgeIfString += ["-v%s" % vlan]

        createBridgePortString = [
            '/scripts/apply_networking.py',
            "-n%s" % str(interface),
            '-bmanual',
            "--ovs-master", str(br_if_name)
        ]

        try:
            subprocess.check_call(createBridgeIfString)
            subprocess.check_call(createBridgePortString)
        except subprocess.CalledProcessError as e:
            print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
        except OSError as e:
            print('System error - \'%s\'' % e.strerror, file=ovsStream)


def sr_ovs_clean_cfg(interfaces):
    for iface in interfaces:
        if 'data' in iface:
            try:
                subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', 'br-%s' % iface])
            except:
                pass

        if 'nat' in iface:
            try:
                subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', iface])
            except:
                pass

        if 'mgmt' in iface:
            try:
                subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', iface])
            except:
                pass

        if 'lte' in iface:
            try:
                subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', 'br-{}'.format(iface)])
            except:
                pass

        if 'wifi' in iface:
            try:
                subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', 'br-{}'.format(iface)])
            except:
                pass

    try:
        subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', 'br-int-system'])
        subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', 'br-int-netdev'])
        subprocess.check_call(['/usr/bin/ovs-vsctl', '--if-exists', 'del-br', 'br-chain'])
    except:
        pass


def setup_ethernet_bridges(ifname, ifcfg, dpdk_rxq):
    if_mode = ifcfg.get('mode', None)
    if if_mode is None:
        print('ERROR interface {} is ethernet, but do not have mode configured'.format(ifname), file=ovsStream)
        return

    if 'inband' in ifname:
        print('Don\'t setup ethernet bridge for \'{}\' interface'.format(ifname), file=ovsStream)
        return

    if 'sr-iov' == if_mode:
        if (not os.path.exists("/sys/class/net/%s/device/sriov_totalvfs" % ifname)):
            print("SR-IOV doesn't seem to be supported on interface %s!" % ifname)
        else:
            try:
                vf_count = int(subprocess.check_output("cat /sys/class/net/%s/device/sriov_totalvfs || true" % ifname,
                                                       shell=True).decode())
            except:
                vf_count = 0

            if vf_count <= 0 or vf_count > 8:
                vf_count = 8

            subprocess.check_output("echo %d > /sys/class/net/%s/device/sriov_numvfs || true" % (vf_count, ifname),
                                    shell=True)

    if_mac = getmac(ifname)
    if 'virtio' == if_mode or 'sr-iov' == if_mode:
        print('Interface \'%s\' is in VIRTIO mode' % ifname, file=ovsStream)
        create_bridge('br-%s' % ifname)
        try:
            subprocess.check_call(['/usr/bin/ovs-vsctl', 'add-port', 'br-%s' % ifname, ifname])
        except Exception as e:
            print(e, file=ovsStream)
    elif 'dpdk' == if_mode:
        print('Interface \'%s\' is in DPDK mode' % ifname, file=ovsStream)
        create_bridge('br-%s' % ifname, 'netdev')
        pci_id = ovs_get_pci_if(ifname)
        if pci_id and 'none' != pci_id:
            try:
                if hw_type == "hr330a":
                    # For Ampere HR330A we need to manually bind interface to DPDK
                    # before creating DPDK port in OVS. Otherwise hot-plug will fail
                    # and leave bridge unusable
                    subprocess.check_call(['/sbin/dpdk-devbind', '--bind=igb_uio', pci_id])

                subprocess.check_call(
                    ['/usr/bin/ovs-vsctl', 'add-port', 'br-%s' % ifname, ifname, '--', 'set', 'Interface', ifname,
                     'type=dpdk', 'options:dpdk-devargs=%s' % pci_id, 'options:n_rxq=%d' % dpdk_rxq])
            except Exception as e:
                print(e, file=ovsStream)
        elif (not pci_id or pci_id == 'none') and is_arm_arch():
            # On ARM there is always custom implementation of DPDK. Also, each
            # device handles port binding in a different way, so we need if
            # clause per each OVS-DPDK implementation
            interface_id = ifname[-1]
            if hw_type == "cybertan8040":
                subprocess.check_call(
                    ['/usr/bin/ovs-vsctl', 'add-port',
                     'br-%s' % ifname,
                     "dpdk-p%s" % interface_id,
                     '--', 'set', 'Interface',
                     "dpdk-p%s" % interface_id,
                     'type=dpdk',
                     'options:dpdk-devargs=eth_mvpp2%s,iface=data%s' %
                     (interface_id, interface_id), 'options:n_rxq=%d' % dpdk_rxq])
            elif hw_type in ("hk6010", "hk6010-8g", "hk6010-16g"):
                # Unfortunately here we need translation of interface into FM MAC
                # as those are not in order
                fm_mac_map = [3, 4, 6, 5, 10, 1]
                ifnumber = int(ifname[-1])
                fm_mac_interface = fm_mac_map[ifnumber]
                subprocess.check_call(
                    ['/usr/bin/ovs-vsctl', 'add-port',
                     'br-%s' % ifname,
                     'data%s' % ifnumber,
                     '--', 'set', 'Interface',
                     'data%s' % ifnumber,
                     'type=dpdk',
                     'options:dpdk-devargs=fm1-mac%s' % fm_mac_interface])
            elif hw_type in ("nsa3640", "ten64"):
                # For sake of mix mode in interface types additional
                # map ifname<->dpni is required. This map is created and saved
                # during execution of openvswitch-switch service for NSA3640 and TEN64
                ifnumber = int(ifname[-1])
                dpni = ""
                with open("/config/port_map.cfg") as file:
                    for line in file:
                        if ifname in line:
                            dpni = line.split(":")[1].strip('\n')
                if hw_type == "ten64":  #ten64 - has ports XG0 and XG1 swapped!
                    if ifname == "data8":
                        dpni = "dpni.9"
                    if ifname == "data9":
                        dpni = "dpni.8"
                if not dpni:
                    raise IOError("Map for interface {} not found!".format(ifname))
                subprocess.check_call(
                    ['/usr/bin/ovs-vsctl', 'add-port',
                     'br-%s' % ifname,
                     'data%s' % ifnumber,
                     '--', 'set', 'Interface',
                     'data%s' % ifnumber,
                     'type=dpdk',
                     'options:dpdk-devargs=%s' % dpni,
                     'options:n_rxq=%d' % dpdk_rxq])

    if not hw_type in ("hk6010", "hk6010-8g", "hk6010-16g"):
        subprocess.check_call(['/usr/bin/ovs-vsctl', 'set', 'bridge', 'br-%s' % ifname, 'other-config:hwaddr=%s' % if_mac])

    subprocess.check_call(['/usr/bin/ovs-ofctl', 'add-flow', 'br-%s' % ifname, 'table=0,action=NORMAL'])


def setup_lte_bridges(ifname):
    create_bridge('br-{}'.format(ifname))
    if os.path.exists('/sys/class/net/' + ifname):
        if if_lte.lte_check_raw_if(ifname):
            if_mac = if_lte.lte_get_raw_mac(ifname)  # '00:00:00:00:a1:00'
        else:
            if_mac = getmac(ifname)
        try:
            subprocess.check_call(
                ['/usr/bin/ovs-vsctl', 'set', 'bridge', 'br-%s' % ifname, 'other-config:hwaddr=%s' % if_mac])
            subprocess.check_call(['/usr/bin/ovs-vsctl', 'add-port', 'br-%s' % ifname, ifname])
            subprocess.check_call(['/usr/bin/ovs-vsctl', '--', 'set', 'Interface', ifname, 'ofport_request=%d' % LTEIF_OFPORT])
            subprocess.check_call(['/usr/bin/ovs-ofctl', 'add-flow', 'br-%s' % ifname, 'table=0,action=NORMAL'])
        except Exception as e:
            print(e, file=ovsStream)


def setup_wifi_bridges(ifname):
    create_bridge('br-{}'.format(ifname))
    if os.path.exists('/sys/class/net/' + ifname):
        if_mac = getmac(ifname)
        try:
            subprocess.check_call(
                ['/usr/bin/ovs-vsctl', 'set', 'bridge', 'br-%s' % ifname, 'other-config:hwaddr=%s' % if_mac])
            subprocess.check_call(['/usr/bin/ovs-vsctl', 'add-port', 'br-%s' % ifname, ifname])
            subprocess.check_call(['/usr/bin/ovs-ofctl', 'add-flow', 'br-%s' % ifname, 'table=0,action=NORMAL'])
        except Exception as e:
            print(e, file=ovsStream)


def sr_ovs_setup_cfg(jconf, is_dpdk):
    # remove db if exist
    # ovs_clean_db(OVS_CFG_DB)

    chain_mode = 'virtio'
    dpdk_init = is_dpdk
    data_interfaces = jconf
    # light_dpdk=False
    # pmd_light_cpu='0'

    try:
        if dpdk_init:
            chain_mode = 'dpdk'
            dpdk_load_modules(True)
            for ifname in data_interfaces:
                if 'data' in ifname:
                    if_mode = data_interfaces[ifname].get('mode')
                    if 'dpdk' == if_mode:
                        pci_id = ovs_get_pci_if(ifname)
                        if pci_id and 'none' != pci_id:
                            ovs_attach_dpdk_if(pci_id)

        dpdk_rxq = 1

        if dpdk_init:
            print('Init OVS in DPDK mode', file=ovsStream)
            subprocess.check_call(
                ['/usr/bin/ovs-vsctl', '--no-wait', 'set', 'Open_vSwitch', '.', 'other_config:dpdk-init=true'])
            cpus = open('/proc/cpuinfo').read().count('processor\t:')
            # pmd_light_cpu='0'
            from sysrepo_systemresources import sysrepo_systemresources as sysres_api
            sysres = sysres_api.SysrepoSystemResources(app_name="ovs-config system-resources")
            dpdk_cpus = str(sysres.get_dpdk_reserved_cpus())

            os_cpus = str(sysres.get_host_reserved_cpus())
            os_cpus_list = os_cpus.split(",")
            os_cpu_map = 0
            for i in os_cpus_list:
                if int(i) < cpus:
                    os_cpu_map |= (1 << int(i))

            # light_dpdk=jconf['json']['tve']['boot']['system-resources'].get('light-dpdk')
            if not dpdk_cpus:
                if cpus > 1:
                    dpdk_cpus = "1"
                else:
                    dpdk_cpus = "0"

            dpdk_cpu_list = dpdk_cpus.split(",")
            dpdk_rxq = len(dpdk_cpu_list)

            # if light_dpdk:
            #   os_cpus=str(jconf['json']['tve']['boot']['system-resources'].get(
            #     'host-reserved-cpus'))
            #   os_cpus_list=os_cpus.split(",")
            #   pmd_light_cpu=os_cpus_list[0]
            #   dpdk_cpu_list[0]=pmd_light_cpu

            cpu_map = 0
            for i in dpdk_cpu_list:
                if int(i) < cpus:
                    cpu_map |= (1 << int(i))

            print('Setting DPDK CPU map to 0x%x' % cpu_map, file=ovsStream)
            subprocess.check_call(['/usr/bin/ovs-vsctl', '--no-wait', 'set', 'Open_vSwitch', '.',
                                   'other_config:pmd-cpu-mask=0x%x' % cpu_map])

            subprocess.check_call(['/usr/bin/ovs-vsctl', '--no-wait', 'set', 'Open_vSwitch', '.',
                                   'other_config:dpdk-lcore-mask=0x%x' % os_cpu_map])

            subprocess.check_call(
                ['/usr/bin/ovs-vsctl', '--no-wait', 'set', 'Open_vSwitch', '.', 'other_config:dpdk-socket-mem=1024'])
            subprocess.check_call(['/usr/bin/ovs-vsctl', '--no-wait', 'set', 'Open_vSwitch', '.',
                                   'other_config:dpdk-hugepage-dir=/dev/hugepages'])

        # ARM devices have a custom way of starting OVS and almost every time it is
        # a 1-shot. It means that OVS needs to be preconfigured before starting
        # ovs-vswitchd unit. DB server is started early during device
        # initialization, so that all above ovs-vsctl DB operations can be done.
        # Now at this point ovs-vswitchd can finally be started.
        if hw_type == "cybertan8040":
            subprocess.check_call(['/usr/local/share/openvswitch/scripts/ovs-ctl',
                                   '--no-ovsdb-server', 'start'])
        elif hw_type in ("hk6010", "hk6010-8g", "hk6010-16g", "nsa3640", "ten64", "nanopir2s", "NanoPI R2S", "nanopir4s", "NanoPI R4S"):
            if hw_type in ("nsa3640", "ten64", "nanopir2s", "nanopir4s"):
                # On dpaa2 based platforms when starting PMD thread of ovs-vswitchd,
                # it first scans for environment variables. If DPRC is not found, it
                # will not start ovs-vswitchd appropriately resulting in errors when
                # doing add-port for DPDK DPNIs. Therefore we need to set DPRC explicitly
                # so that we can bind bridge to DPDK thereafter.
                os.environ['DPRC'] = 'dprc.2'
            subprocess.check_call(['ovs-vswitchd',
                                    'unix:/usr/local/var/run/openvswitch/db.sock',
                                    '--pidfile', '--detach',
                                    '--log-file=/var/log/openvswitch/ovs-vswitchd.log'
                                    ])

        sr_ovs_clean_cfg(jconf)

        if hw_type not in ("nanopir2s", "NanoPI R2S", "nanopir4s", "NanoPI R4S"):
            create_bridge('br-int-system')
            create_bridge('br-int-netdev', 'netdev')
            if 'virtio' == chain_mode:
                print('Chain configured in VIRTIO mode', file=ovsStream)
                create_bridge('br-chain')
                na.set_agent_data_type_path('/etc/neutron/plugins/ml2/openvswitch_agent.ini.agent', 'system')
                na.set_agent_data_type_path('/etc/neutron/plugins/ml2/openvswitch_agent.ini', 'system')
            elif 'dpdk' == chain_mode:
                print('Chain configured in DPDK mode', file=ovsStream)
                create_bridge('br-chain', 'netdev')
                na.set_agent_data_type_path('/etc/neutron/plugins/ml2/openvswitch_agent.ini.agent', 'netdev')
                na.set_agent_data_type_path('/etc/neutron/plugins/ml2/openvswitch_agent.ini', 'netdev')

        for ifname in data_interfaces:
            if 'ethernet' == data_interfaces[ifname].get('type'):
                setup_ethernet_bridges(ifname, data_interfaces[ifname], dpdk_rxq)
            elif 'lte' == data_interfaces[ifname].get('type'):
                setup_lte_bridges(ifname)
            elif 'wifi' == data_interfaces[ifname].get('type'):
                setup_wifi_bridges(ifname)

        # if light_dpdk:
        #   subprocess.check_call(['/scripts/light_dpdk.sh',pmd_light_cpu])

        subprocess.check_call(['/usr/bin/ovs-appctl', 'vlog/set', 'ANY:file:off'])
        subprocess.check_call(['/usr/bin/ovs-appctl', 'vlog/set', 'ANY:syslog:err'])


    except subprocess.CalledProcessError as e:
        print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
    except OSError as e:
        print('System error - \'%s\'' % e.strerror, file=ovsStream)


"""
All dataX interfaces will have bridge
In Ubuntu:

auto lo
iface lo inet loopback

auto br0
iface br0 inet dhcp
        bridge_ports eth0
        bridge_stp off
        bridge_fd 0
        bridge_maxwait 0

"""


def sr_ovs_net_create_configured_interfaces(jconf):
    for interface in jconf:
        if "data" in interface or 'tun' in interface or 'nat' in interface:
            continue

        if_names = [f for f in os.listdir('/sys/class/net') if
                    os.path.isfile(os.path.join('/sys/class/net', f)) or os.path.islink(
                        os.path.join('/sys/class/net', f))]

        basename = re.sub(r'(.*)(\.[0-9]+)$', r'\1', interface)  # vlan tag interface - check for base name
        if not basename in if_names:
            continue

        print('Interface %s' % interface, file=ovsStream)

        vlan = ovs_get_interface_vtag(jconf[interface])

        createIfString = ['/scripts/apply_networking.py',
                          "-n%s%s" % (interface, '' if (0 == vlan or 'inband' in interface) else '.%d' % vlan)]

        onboot = 'yes' if jconf[interface].get('enable', True) and interface in if_names else 'no'
        onboot = 'no' if 'inband' in interface or 'outband' in interface else 'yes'
        createIfString += ["-o%s" % onboot]

        # In order to up inband and outband interfaces
        if 'inband' in interface or 'outband' in interface:
            createIfString += ["-icm"]

        if 'ipv4-prefix' in jconf[interface] and jconf[interface]['ipv4-prefix']:
            createIfString += [str("--ipv4-prefix"), str("%s" % jconf[interface].get('ipv4-prefix'))]
        if 'gateway' in jconf[interface] and jconf[interface]['gateway']:
            createIfString += [str("-g%s" % jconf[interface]['gateway'])]
        if 'dns' in jconf[interface] and jconf[interface]['dns']:
            createIfString += [str("-d%s" % jconf[interface]['dns'])]
        if 'bootproto' in jconf[interface] and 'dhcp' in jconf[interface]['bootproto']:
            if 'dhcp' in jconf[interface]['bootproto']:
                createIfString += ["-bdhcp"]
            elif 'static' in jconf[interface]['bootproto']:
                createIfString += ["-bstatic"]
        if 'ipv6-prefix' in jconf[interface] and jconf[interface]['ipv6-prefix']:
            createIfString += [str("--ipv6-prefix"), str("%s" % jconf[interface]['ipv6-prefix'])]
        if vlan:
            createIfString += ["-v%s" % vlan]

        print(createIfString, file=ovsStream)
        try:
            subprocess.check_call(createIfString)
        except subprocess.CalledProcessError as e:
            print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
        except OSError as e:
            print('System error - \'%s\'' % e.strerror, file=ovsStream)


def sr_ovs_net_create_nat(json_cfg, is_dpdk):
    # ip_forward=0
    dpdk_init = is_dpdk
    for interface in json_cfg:
        if json_cfg[interface]['type'] == 'nat':
            nat_if = if_nat_(if_cfg=json_cfg[interface])
            nat_if.create(dpdk=dpdk_init)


def sr_ovs_net_create_mgmt_if(json_cfg, is_dpdk):
    dpdk_init = is_dpdk
    for interface in json_cfg:
        if json_cfg[interface]['name'] == vnf_management_if:
            mgmt_if = if_mgmt.if_mgmt(if_cfg=json_cfg[interface])
            mgmt_if.create(dpdk=dpdk_init)


def sr_ovs_net_init():
    getBootSystemMACs()
    interfaces = sr_cfg.get_interfaces(startup=True)

    # Before configuring OVS,remove pci-passthrough/sr-iov interfaces
    id_dpdk = False
    copyOfDict = dict(interfaces)

    for (key, value) in copyOfDict.items():
        if 'dpdk' in interfaces[key].get('mode', ''):
            id_dpdk = True

    # Init OVS regarding to choosed configuration
    sr_ovs_setup_cfg(interfaces, id_dpdk)

    # Remove old data and br-data configurations
    ovs_clean_entire_networking()

    # Create ifcfg-XXX scripts for the network interfaces
    # sr_ovs_net_create_bridge_interfaces(interfaces)
    # sr_ovs_net_create_configured_interfaces(interfaces)
    sr_ovs_net_create_nat(interfaces, id_dpdk)
    sr_ovs_net_create_mgmt_if(interfaces, id_dpdk)


def sr_ovs_net_stop():
    interfaces = sr_cfg.get_interfaces(startup=True)
    # delete bridges
    sr_ovs_clean_cfg(interfaces)

    try:
        subprocess.check_call(['/usr/share/openvswitch/scripts/ovs-ctl', 'stop'])
        ovs_dpdk_unbind_interfaces()
        # ovs_clean_db(OVS_CFG_DB)
    except subprocess.CalledProcessError as e:
        print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
    except OSError as e:
        print('System error - \'%s\'' % e.strerror, file=ovsStream)


if __name__ == "__main__":
    argc = len(sys.argv)

    # NFV-12516
    if is_arm_arch():
        txqueuelen_default = '1000'

    if 2 > argc:
        print('No enough arguments. Exit!', file=ovsStream)
        sys.exit(1)

    if sys.argv[1] == "start":
        print('Start configure interfaces', file=ovsStream)

        from sysrepo_interfaces import sysrepo_interfaces3 as interfaces_api

        sr_cfg = interfaces_api.SysrepoInterfaces(app_name="ovs-config interfaces")
        sr_ovs_net_init()

        if hw_type in ("nanopir2s", "NanoPI R2S", "nanopir4s", "NanoPI R4S"):
            os.system("ps -eLo tid,psr,comm | grep -e revalidator -e handler -e ovs -e pmd -e urc -e eal | cut -f2 -d' ' | tr -d ' ' | xargs -L1 taskset -pc 2")

        sys.exit(0)
    elif sys.argv[1] == "stop":
        print('Stop configure interfaces', file=ovsStream)
        from sysrepo_interfaces import sysrepo_interfaces3 as interfaces_api

        sr_cfg = interfaces_api.SysrepoInterfaces(app_name="ovs-config interfaces")
        sr_ovs_net_stop()
        sys.exit(0)
    else:
        print('Unknown argument \'%s\'' % sys.argv[1], file=ovsStream)
        sys.exit(1)

    sys.exit(0)
