#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import os
from subprocess import PIPE, run, check_output
import sys
import re
import logging, logging.handlers
import logging.handlers
import time
import shutil
import json
import requests
from urllib.parse import urlparse, parse_qs
import ftplib
import netifaces

log = None

TEMPORARY_DIR = '/tmp/'
SUPPORTED_IMAGE_FORMATS = ['qcow2', 'iso', 'raw', 'img', 'tar']
LOCK_FILE = TEMPORARY_DIR + 'registry_api.lock'
BIND_INTERFACE_FILE = '/kubernetes/bind-interface-file'
DOCKERFILE = '''
# Shared Volume
FROM scratch
ADD {} /disk/
'''
CLUSTER_FILE="/config/cluster.json"
HA_MODE_FILE="/config/.hamode"


def set_log(name, level):
    global log
    log = logging.getLogger(name=name)
    log.setLevel(level=level)

    syslog_hnd = logging.handlers.SysLogHandler(address='/dev/log')
    formatter = logging.Formatter('%(name)s: %(levelname)s %(message)s')
    syslog_hnd.setFormatter(formatter)
    log.addHandler(syslog_hnd)


def set_status(status, image):
    log.debug('Image \'{}\' is in \'{}\' state.'.format(image, status))
    image = re.sub('[/:]', '_', image)
    with open('/tmp/registry_api_' + image, 'w') as f:
        print(status, file=f, end='')


# Don't call registry api (lock it) with more than one operation.
# With this implementation, if we have bulk upload of images (2 or more in same time)
# every process of registry_api.py will wait previosly started process.
def locks_and_key():
    while True:
        if os.path.exists(LOCK_FILE):
            time.sleep(5)
        else:
            fd = os.open(LOCK_FILE, os.O_RDWR | os.O_CREAT)
            os.fdopen(fd, "w+").close()
            time.sleep(5)
            return


# Delete unnecessary files and Docker images
def delete_garbage(name, tag, on_error=False, filename=''):
    dockerfile = os.path.join(TEMPORARY_DIR, 'Dockerfile')
    repo_dir = '/var/lib/local-registry/docker/registry/v2/repositories/' + name
    file_path = os.path.join(TEMPORARY_DIR, filename.split('/')[-1])

    try:
        os.remove(file_path) if os.path.exists(file_path) and file_path != TEMPORARY_DIR else None
        os.remove(dockerfile) if os.path.exists(dockerfile) else None
        shutil.rmtree(repo_dir) if on_error and os.path.exists(repo_dir) else None
        os.remove(LOCK_FILE) if os.path.exists(LOCK_FILE) else None
    except Exception as e:
        # Delete lock file if we have problem
        os.remove(LOCK_FILE) if os.path.exists(LOCK_FILE) else None
        sys.exit(e)

def import_from_remote(name, tag, registry_url=None, username=None, password=None):
    image = name + ':' + tag
    registry_url = None if registry_url == "None" else registry_url
    username, password = (None, None) if username == "None" or password == "None" else (username, password)
    bind_interface = 'br-data0'
    try:        
        if registry_url:
            registry_url = registry_url+'/library' if '/' not in name else registry_url
            remote_image = f'{registry_url.rstrip("/")}/{name}:{tag}'
        else:
            remote_image = f'docker.io/library/{name}:{tag}' if '/' not in name else f'docker.io/{name}:{tag}'
        
        if os.path.isfile(BIND_INTERFACE_FILE) and os.stat(BIND_INTERFACE_FILE).st_size != 0:
            with open(BIND_INTERFACE_FILE) as f:
                bind_interface = f.read()

        ip = netifaces.ifaddresses(bind_interface)[netifaces.AF_INET][0]['addr']
        repo = ip + ':5001/' + image

        # 1. Pull image from remote repo
        set_status('pulling', image)

        if username and password:
            ret = run('ctr -n k8s.io images pull --platform linux/amd64 --user {}:{} {}'.format(username, password, remote_image), shell=True, stdout=PIPE, stderr=PIPE)
        else:
            ret = run('ctr -n k8s.io images pull --platform linux/amd64 {}'.format(remote_image), shell=True, stdout=PIPE, stderr=PIPE)           
        if ret.returncode != 0:
            sys.exit(ret.stderr)

        # # 2. Get the image tag
        # loaded_image = None
        # images = client.images()

        # if registry_url:
        #     for img in images:
        #         if img.get("RepoTags"):
        #             if f"{name}:{tag}" in img["RepoTags"] or f"{repo_without_scheme}:{tag}" in img["RepoTags"]:
        #                 loaded_image = img["Id"].split(':')[1][0:12]
        #                 break
        #         else:
        #             print("RepoTags not found for image:", img)

        # else:
        #     loaded_image = client.inspect_image(image)['Id'].split(':')[1][0:12]


        # if not loaded_image:
        #     raise Exception(f"Image {name}:{tag} not found")

        # # 3. Set the tag to remote if we have url
        # if registry_url:
        #     repo = getoutput(
        #         "cat /etc/hostname") + ':30501/' + name
        # client.tag(loaded_image, repo, tag=tag, force=True)

        # # 4. Push the image in local repo
        # set_status('pushing', image)
        # 2. Tag image and push it to registry
        set_status('pushing', image)
        ret = run('ctr -n k8s.io images push --plain-http --platform linux/amd64 --allow-non-distributable-blobs {} {}'.format(repo, remote_image), shell=True, stdout=PIPE, stderr=PIPE)
        if ret.returncode != 0:
           sys.exit(ret.stderr)
        # 3. Remove uploaded image from containerd
        ret = run('ctr -n k8s.io images delete {}'.format(remote_image), shell=True, stdout=PIPE, stderr=PIPE)
        if ret.returncode != 0:
           sys.exit(ret.stderr)
        set_status('ready', image)

    except Exception as e:
        log.error(e)
        set_status('error', image)
        sys.exit(e)


def upload(name, tag, filename):
    file_extension = filename.lower().split('.')[-1]
    
    parsed_source = urlparse(filename)
    query_string = parse_qs(parsed_source.query)
    image = name + ':' + tag
    
    if parsed_source.scheme in ['http', 'https', 'ftp']:
        # Handle case when a URL is provided
        source = filename
        filename = query_string.get('files')[0] if 'files' in query_string else os.path.basename(parsed_source.path)
        local_file = os.path.join(TEMPORARY_DIR, filename)
        set_status('downloading', image)
        if parsed_source.scheme in ['http', 'https']:
            response = requests.get(source, verify=False, timeout=60) 
            with open(local_file, 'wb') as f:
                f.write(response.content)
        elif parsed_source.scheme == 'ftp':
            ftp = ftplib.FTP()
            ftp.connect(parsed_source.hostname, parsed_source.port or 21)
            ftp.login(parsed_source.username or 'anonymous', parsed_source.password or '')
            ftp.cwd(os.path.dirname(parsed_source.path))

            with open(local_file, 'wb') as f:
                ftp.retrbinary('RETR ' + os.path.basename(parsed_source.path), f.write)

            ftp.quit()

        file_path = local_file
    else:
        file_path = os.path.join(TEMPORARY_DIR, filename)
    

    # TODO VS: Test/extend for tar.gz
    try:
        bind_interface = 'br-data0'
        if os.path.isfile(BIND_INTERFACE_FILE) and os.stat(BIND_INTERFACE_FILE).st_size != 0:
            with open(BIND_INTERFACE_FILE) as f:
                bind_interface = f.read()

        ip = netifaces.ifaddresses(bind_interface)[netifaces.AF_INET][0]['addr']
        if os.path.exists(CLUSTER_FILE) and os.path.exists(HA_MODE_FILE):
            with open(CLUSTER_FILE) as config_file:
                data = json.load(config_file)
            repo = data['virtual_address'] + ':5001/' + image
        repo = ip + ':5001/' + image
        if file_extension == 'tar':
            # 1. Import from .tar
            ret = run('ctr -n k8s.io image import --digests --skip-digest-for-named {}'.format(file_path), shell=True, stdout=PIPE, stderr=PIPE)
            if ret.returncode != 0:
                sys.exit(ret.stderr)
            loaded_image = ret.stdout.decode('utf8').split(' ')[1]
            # 2. Tag image and push it to registry (--platform linux/amd64 parameter is removed because amd64 is default)
            set_status('pushing', image)
            ret = run('ctr -n k8s.io images push --plain-http --allow-non-distributable-blobs {} {}'.format(repo, loaded_image), shell=True, stdout=PIPE, stderr=PIPE)
            if ret.returncode != 0:
                sys.exit(ret.stderr)
            # 3. Remove uploaded image from containerd
            ret = run('ctr -n k8s.io images delete {}'.format(loaded_image), shell=True, stdout=PIPE, stderr=PIPE)
            if ret.returncode != 0:
                sys.exit(ret.stderr)
        elif file_extension in SUPPORTED_IMAGE_FORMATS:
            # 1. Create Dockerfile file
            dockerfile = os.path.join(TEMPORARY_DIR, 'Dockerfile')
            open(dockerfile, 'w').write(
                DOCKERFILE.format(filename))
            # 2. Build and push image
            set_status('pushing', image)
            if 'buildkitd' not in run('kubectl buildkit version', shell=True, stdout=PIPE, stderr=PIPE).stdout.decode('utf8'):
                sys.exit("Buildkit POD is not in helty status!")
            ret = run('kubectl buildkit build -n local-registry --no-cache --insecure-skip-tls-verify -t {} -f /tmp/Dockerfile /tmp/.'.format(repo), shell=True, stdout=PIPE, stderr=PIPE)
            if ret.returncode != 0:
                sys.exit(ret.stderr)
            ret = run('ctr -n k8s.io images push --plain-http --platform linux/amd64 --allow-non-distributable-blobs {}'.format(repo), shell=True, stdout=PIPE, stderr=PIPE)
            run('kubectl buildkit prune', shell=True, stdout=PIPE, stderr=PIPE)
            if ret.returncode != 0:
                sys.exit(ret.stderr)
            # 3. Remove uploaded image from containerd
            ret = run('ctr -n k8s.io images delete {}'.format(repo), shell=True, stdout=PIPE, stderr=PIPE)
            if ret.returncode != 0:
                sys.exit(ret.stderr)
            # Remove images with <none> tag
            images = json.loads(check_output('crictl images -o json', shell=True))
            for img in images['images']:
                if img['repoTags'] == []:
                    run('ctr -n k8s.io images delete {}'.format(img['id']), shell=True)
        set_status('ready', image)
    except Exception as e:
        sys.exit(e)


if __name__ == "__main__":
    set_log('registry-api', logging.DEBUG)
    if len(sys.argv) < 3:
        sys.exit("Missing config params!")
    else:
        image = sys.argv[1] + ':' + sys.argv[2]
        locks_and_key()

        name = sys.argv[1]
        tag = sys.argv[2]
        registry_url = None
        username = None
        password = None

        try:
            # Create status file
            status_file = '/tmp/registry_api_{}'.format(re.sub('[/:]', '_', image))
            if not os.path.exists(status_file):
                fd = os.open(status_file, os.O_RDWR|os.O_CREAT)
                os.fdopen(fd, "w+").close()

            if sys.argv[3].lower().endswith(tuple(SUPPORTED_IMAGE_FORMATS)):
                upload(name, tag, sys.argv[3])
                delete_garbage(sys.argv[1], sys.argv[2], False, sys.argv[3])
            else:
                registry_url = sys.argv[3]

                if len(sys.argv) >= 5:
                    username = sys.argv[4]

                if len(sys.argv) >= 6:
                    password = sys.argv[5]

                if registry_url and username and password is not None:
                    import_from_remote(name, tag, registry_url, username, password)
                    delete_garbage(sys.argv[1], sys.argv[2], False)
                elif len(sys.argv) >= 3:
                    import_from_remote(name, tag, registry_url)
                    delete_garbage(sys.argv[1], sys.argv[2], False)
        except SystemExit as e:
            set_status('error', image)
            delete_garbage(sys.argv[1], sys.argv[2], True)
            sys.exit('Registry API Error: {}'.format(e))
        except Exception as e:
            set_status('error', image)
            delete_garbage(sys.argv[1], sys.argv[2], True)
            sys.exit('Registry API Error: {}'.format(e))
    
    sys.exit(0)
