#!/usr/bin/env python3
from sysrepo_kubernetes import sysrepo_kubernetes as k8s_api
from sysrepo_interfaces import sysrepo_interfaces3 as sr_if
import os
import sys
import json
import subprocess
import logging
from systemd.journal import JournalHandler

log = logging.getLogger(__name__)
log_fmt = logging.Formatter("%(levelname)s %(message)s")
log_ch = JournalHandler(SYSLOG_IDENTIFIER='sr-kubernetes-commands')
log_ch.setFormatter(log_fmt)
log.addHandler(log_ch)
log.setLevel(logging.DEBUG)

def join_controller(controller):
    sr = k8s_api.SysrepoKubernetes(app_name="KubernetesCommands")
    old_controller = sr.get_controller()
    if controller != None and controller == old_controller:
        print ("UCPe has already been joined to this controller!")
    elif controller != None and controller != "" and controller != old_controller:
        sr.set_controller(controller)
        sr.commit()

def reset_controller():
    sr = k8s_api.SysrepoKubernetes(app_name="KubernetesCommands")
    sr.delete_controller()
    sr.commit()

def leave_ha_controller(hamaster):
    log.debug('leave_ha_controller begin')
    sr = k8s_api.SysrepoKubernetes(app_name="KubernetesCommands", log_level=logging.DEBUG)
    ret1 = sr.cluster_reset()
    ret2 = sr.clean_worker_in_kube_cluster(hamaster)[0]
    sr.commit()
    if not ret1:
        log.error('Cluster reset fail {}', ret1)
        print('Cluster reset fail {}', ret1)
        log.debug('leave_ha_controller end reset fail')
        return False

    if not ret2:
        log.error('Clean worker from cluster fail {}', ret2)
        print('Clean worker from cluster fail {}', ret2)
        log.debug('leave_ha_controller end worker clean fail')
        return False

    log.debug('leave_ha_controller end good')
    return True

def clean_worker(hostname, workers):
    sr = k8s_api.SysrepoKubernetes(app_name="KubernetesCommands", log_level=logging.DEBUG)
    successful, results = sr.clean_worker_in_kube_cluster(hostname, workers=workers)
    if not successful:
        print("Failed to execute command")
        return
    results = json.loads(results)
    for worker in results:
        code = next(iter((results[worker])))
        msg = results[worker][code]
        print(f'Deleting {worker} {"failed" if int(code) else "succeeded"} with message:\n{msg}')

def k8sdisabled():
    log.debug('Called k8sdisabled')
    sr = k8s_api.SysrepoKubernetes(app_name='KubernetesCommands', log_level=logging.DEBUG)
    sr.cluster_reset()
    current_controller = sr.get_controller()
    if current_controller:
        log.debug('Called k8sdisabled {}'.format(current_controller))
        sr.clean_worker_in_kube_cluster(current_controller)
    else:
        log.debug('Called k8sdisabled telco-systems-k8s-controller')
        sr.clean_worker_in_kube_cluster('telco-systems-k8s-controller')

    sr.commit()
    log.debug('commit completed, call system_reboot')
    sr._system_reboot()
    log.debug('reboot...')


def set_local_controller(val):
    sr = k8s_api.SysrepoKubernetes(app_name="KubernetesCommands")
    sr.set_cbox_enabled(val)
    sr.commit()


# get status of worker pod on kube controller
def get_worker_status():
    try:
        if os.path.isfile('/config/central-box') or os.path.isfile('/config/k8s-single-machine'):
            hostname = os.uname()[1]
            output = subprocess.check_output(f"python3 /scripts/isNodeReady.py {hostname}", shell=True)
            status = output.decode().strip()
        else:
            sr = k8s_api.SysrepoKubernetes(app_name="KubernetesCommands")
            status = sr.get_worker_status()
        if str(status).strip() == 'True':
            print ("Status in kubernetes cluster is 'Ready'")
        else:
            print ("Status in kubernetes cluster is 'NotReady'")
    except Exception as e:
        print(f"Error getting worker status: {e}")


def bind_interface(interface):
    sr = k8s_api.SysrepoKubernetes(app_name="KubernetesCommands")
    srif = sr_if.SysrepoInterfaces(app_name="KubernetesBindIntfc")
    ippass = srif.get_interface_param(interface, 'ip-passthrough')
    if ippass == True:
        print(f"Cannot set bind-interface on {interface} with ip-passthrough enabled! Please disable ip-passthrough first!")
        return
    else:
        sr.set_bind_interface(interface)
        sr.commit()


if __name__ == "__main__":

    argc = len(sys.argv)

    if argc < 2:
        print("No enough arguments. Exit!")
        sys.exit(1)
    elif argc == 2 and sys.argv[1] == "status":
        get_worker_status()
    elif argc == 2 and sys.argv[1] == "reset":
        reset_controller()
    elif argc == 3 and sys.argv[1] == "reset":
        if not leave_ha_controller(sys.argv[2]):
            log.error('leave_ha_controller {} fail'.format(sys.argv[2]))
            sys.exit(1)
    elif argc == 4 and sys.argv[1] == 'clean-worker':
        clean_worker(sys.argv[2], sys.argv[3])
    elif argc == 3 and sys.argv[1] == "join":
        join_controller(sys.argv[2])
    elif argc == 2 and sys.argv[1] == "enable":
        set_local_controller(True)
    elif argc == 3 and sys.argv[1] == "bind-interface":
        bind_interface(sys.argv[2])
    elif argc == 2 and sys.argv[1] == "disable":
        set_local_controller(False)
    elif argc == 2 and sys.argv[1] == "k8sdisabled":
        k8sdisabled()
    else:
        print("Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    log.debug('Done')
    sys.exit(0)
