#!/usr/bin/env python3
import getopt
import sys
from sysrepo_api import sysrepo_api3 as api
from netaddr import *

def set_idle(sr, value):
    if value != None and value != "":
        sr.set_item("/ietf-netconf-server:netconf-server/listen/idle-timeout", value)
        sr.commit()

def validate_endpoints(config, endpoint, ip, port):
    if 'endpoint' in config:
        for endp in config['endpoint']:
            if endpoint == endp['name']:
                continue
            if endp['ssh']['tcp-server-parameters']['local-address'] == ip and str(endp['ssh']['tcp-server-parameters']['local-port']) == str(port):
                print("Endpoint \"%s\" has the same address:port pair (%s:%s)" % (endp['name'], endp['ssh']['tcp-server-parameters']['local-address'], endp['ssh']['tcp-server-parameters']['local-port']))
                return False
            if (str(IPNetwork(ip).ip) == '0.0.0.0' or str(IPNetwork(ip).ip) == '::') and \
                   (str(IPNetwork(endp['ssh']['tcp-server-parameters']['local-address']).ip) == '0.0.0.0' or str(IPNetwork(endp['ssh']['tcp-server-parameters']['local-address']).ip) == '::') and \
                   str(endp['ssh']['tcp-server-parameters']['local-port']) == str(port):
                print("Endpoint \"%s\" has address:port pair (%s:%s)" % (endp['name'], endp['ssh']['tcp-server-parameters']['local-address'], endp['ssh']['tcp-server-parameters']['local-port']))
                print("Only one endpoint with address \'0.0.0.0\' or \'::\' can exist on the same port simultaneously!")
                return False
    return True

def set_endpoint(sr, endpoint, ip, port, idle, maxp, pint):
    exists = False
    if endpoint is not None and endpoint != "":
        config = sr.get_config_module('ietf-netconf-server:netconf-server')['netconf-server']['listen']
        if not validate_endpoints(config, endpoint, ip, port):
            sys.exit(1)
        if 'endpoint' in config:
            for endp in config['endpoint']:
                if endpoint in endp['name']:
                    exists = True
                    break
        if not exists:
            sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']".format(endpoint), endpoint)
        if ip is not None and ip != "":
            sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/tcp-server-parameters/local-address".format(endpoint), ip)
        if port is not None and port != "":
            sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/tcp-server-parameters/local-port".format(endpoint), port)
        if idle is not None and idle != "":
            sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/tcp-server-parameters/keepalives/idle-time".format(endpoint), idle)
        if maxp is not None and maxp != "":
            sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/tcp-server-parameters/keepalives/max-probes".format(endpoint), maxp)
        if pint is not None and pint != "":
            sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/tcp-server-parameters/keepalives/probe-interval".format(endpoint), pint)
        sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/ssh-server-parameters/server-identity/host-key[name='default-key']/public-key/keystore-reference".format(endpoint), "genkey")
        sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/ssh-server-parameters/client-authentication/supported-authentication-methods/publickey".format(endpoint), None)
        sr.set_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']/ssh/ssh-server-parameters/client-authentication/supported-authentication-methods/passsword".format(endpoint), None)
        sr.commit()

def del_endpoint(sr, value):
    if value is not None and value != "":
        sr.delete_item("/ietf-netconf-server:netconf-server/listen/endpoint[name='{}']".format(value))
        sr.commit()

if __name__ == "__main__":
    argc = len(sys.argv)
    sr = api.SysrepoAPI(app_name="NetconfCommands", namespace="/ietf-netconf-server:netconf-server", module_name="ietf-netconf-server")

    if argc < 2:
        print("No enough arguments. Exit!")
        sys.exit(1)
    elif argc == 3 and sys.argv[1] == "set_idle_timeout":
        set_idle(sr, sys.argv[2])
    elif argc >= 3 and sys.argv[1] == "set_endpoint":
        argv = sys.argv[2:]
        endpoint = None
        ip = None
        port = None
        idle = None
        maxprobe = None
        probveinterval = None

        try:
            opts, args = getopt.getopt(argv, "e:i:p:l:m:n:", ["endpoint=", "ip=", "port=", "idle-time=", "max-probes=", "probe-interval="])
        except getopt.GetoptError:
            sys.exit(2)
        for opt, arg in opts:
            if opt in ("-e", "--endpoint"):
                endpoint = arg
            elif opt in ("-i", "--ip"):
                ip = arg
            elif opt in ("-p", "--port"):
                port = arg
            elif opt in ("-l", "--idle-time"):
                idle = arg
            elif opt in ("-m", "--max-probes"):
                maxprobe = arg
            elif opt in ("-n", "--probe-interval"):
                probeinterval = arg

        set_endpoint(sr, endpoint, ip, port, idle, maxprobe, probeinterval)

    elif argc == 3 and sys.argv[1] == "del_endpoint":
        del_endpoint(sr, sys.argv[2])

    else:
        print("Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)
