#! /usr/bin/env python3
import os
import io
import shutil
import subprocess
import sysrepo_interfaces.sysrepo_interfaces3 as sri
import sysrepo_systemresources.sysrepo_systemresources as srr
import vppapi_if.vppapi_if as vppif
import pyroute2 as prt
import time
import random
from tveif import if_lte
import ipaddress
from systemd import journal

USE_TAP = True
BR_PREFFIX = 'br-'
VETH_PREFFIX = 'v'
CHAIN_TAP_ID = 99
LTE_OFFSET = 100
NAT_OFFSET = 200
MGMT_OFFSET = 300
WIFI_OFFSET = 400
VXLAN_OFFSET = 500

ENABLE_RT_CTL = False
ENABLE_DRCTL = True
if_mac = dict()

vppztpStream = journal.stream(identifier='ztp-pre-vpp')

#workaround for kernel 5.4 - fallback to generic dpdk driver in case there is not loadable vfio-pci
has_vfio_drv = os.system("cat /lib/modules/$(uname -r)/modules.builtin | grep -q vfio-pci.ko") != 0
if has_vfio_drv:
    mods = ["vfio-pci"]
else:
    mods=["uio_pci_generic"]
base_cfg_name = '/opt/etc/nfvt-vpp.conf'
dpdk_cfg_name = '/opt/etc/nfvt-dpdk.conf'
cpu_cfg_name = '/opt/etc/nfvt-cpu.conf'
cfg_name = '/etc/vpp/startup.conf'

def get_pci_id(ifname):
    '''Returns PCI ID as bytes of interface with name \'ifname\' or None.'''
    pci_id = None
    try:
        out = subprocess.check_output(['udevadm', 'info', '-q', 'path', '-p', '/sys/class/net/{}'.format(ifname)])
        if out.split(b'/')[2].decode() != 'virtual':
            pci_id = out.split(b'/')[-3]
            if b'virtio' in pci_id:
                pci_id = out.split(b'/')[-4]
        return pci_id
    except:
        pass

    return pci_id


def generate_mac(oui):
    return("%s%02x:%02x:%02x" % (
        oui,
        random.randint(0, 255),
        random.randint(0, 255),
        random.randint(0, 255),
    ))

def validate_lte_dongles():
    lte1_mac=""
    lte2_mac=""
    if os.path.exists('/sys/class/net/lte1/address') and os.path.exists('/sys/class/net/lte2/address'):
        with open('/sys/class/net/lte1/address','r') as f:
            lte1_mac = f.readline()[0:17]
        with open('/sys/class/net/lte2/address','r') as f:
            lte2_mac = f.readline()[0:17]
        if lte1_mac == lte2_mac:
            retry_count = 0
            while True:
                retry_count += 1
                lte2_mac = generate_mac(oui = lte2_mac[0:9])
                if lte1_mac != lte2_mac:
                    subprocess.run(['ifconfig', 'lte2', 'hw', 'ether', lte2_mac])
                    break
                if retry_count > 5:
                    # Could not generate new MAC address for lte2 (external lte dongle)
                    break


def get_if_mac(ifname):
    if not os.path.exists('/sys/class/net/' + ifname + '/address'):
        return None

    f = open('/sys/class/net/' + ifname + '/address', 'r')
    mac = f.readline()[0:17]
    print(mac)
    return mac



def get_vpp_cfg(cfg_name):
    if cfg_name is None:
        return []
    cfg_f = open(cfg_name, 'r')
    return cfg_f.readlines()


def gen_vpp_cpu(cfg_f):
    host_lst = []
    dpdk_lst = []
    sysres = srr.SysrepoSystemResources(app_name='vpp-pre-ztp')
    sysres.set_active_datastore("startup")
    host_cpu_str = sysres.get_host_reserved_cpus()
    if host_cpu_str:
        if ' ' in host_cpu_str:
            host_lst = host_cpu_str.split(' ')
        else:
            host_lst = host_cpu_str.split(',')

        if host_lst:
            cfg_f.write('\tmain-core {}\n'.format(host_lst[0]))

    dpdk_cpu_str = sysres.get_dpdk_reserved_cpus()
    if dpdk_cpu_str and dpdk_cpu_str!="none":
        if ' ' in dpdk_cpu_str:
            dpdk_lst = dpdk_cpu_str.split(' ')
        else:
            dpdk_lst = dpdk_cpu_str.split(',')

        if dpdk_lst:
            #cfg_f.write('\tworkers {}\n'.format(len(dpdk_lst)))
            cfg_f.write('\tcorelist-workers {}\n'.format(','.join(dpdk_lst)))


def gen_vpp_cfg(intf_cfg):
    '''Copy contents of \'base_cfg_name\' to \'cfg_name\'. Add other specific configuration based on interface configuration to end new file.'''
    global has_vfio_drv
    dpdk_ifs = {}
    print('Base config \'{}\', new config \'{}\'.'.format(base_cfg_name, cfg_name), file=vppztpStream)
    if base_cfg_name and os.path.exists(base_cfg_name) and os.path.isfile(base_cfg_name):
        # Copy base file to new cfg_name
        print('Copy base config \'{}\' to \'{}\''.format(base_cfg_name, cfg_name), file=vppztpStream)
        shutil.copy2(base_cfg_name, cfg_name)

    for ifname, ifcfg in intf_cfg.items():
        if ifcfg.get('type', None) == 'ethernet' and ifcfg.get('mode', 'virtio') == 'dpdk':
            pci = get_pci_id(ifname)
            if pci:
                dpdk_ifs[ifname] = pci
            else:
                print('Inerface \'{}\' is not PCI device !'.format(ifname), file=vppztpStream)

    if base_cfg_name:
        mode_str = 'r+'
    else:
        mode_str = 'w+'

    cfg_f = open(cfg_name, mode_str)
    print('FD for \'{}\' is {}'.format(cfg_name, cfg_f.fileno()), file=vppztpStream)

    cfg_f.seek(0, io.SEEK_END)

    cpu_cfg = get_vpp_cfg(cpu_cfg_name)
    cfg_f.write('cpu { \n')
    if cpu_cfg:
        cfg_f.writelines(cpu_cfg)

    gen_vpp_cpu(cfg_f)
    cfg_f.write(' } \n')

    cfg_f.write('dpdk { \n')
    dpdk_cfg = get_vpp_cfg(dpdk_cfg_name)
    if dpdk_cfg:
        cfg_f.writelines(dpdk_cfg)
        cfg_f.write('\n')

    if len(dpdk_ifs):
        for ifname, pci in dpdk_ifs.items():
            cfg_f.write('dev {pci} {{ \n'.format(pci=pci.decode()))
            cfg_f.write('name {} \n'.format(ifname))
            cfg_f.write('}}\n'.format(pci=pci.decode()))
        #workaround for kernel 5.4
        if not has_vfio_drv:
            cfg_f.write('uio-driver uio_pci_generic \n')
    else:
        cfg_f.write('no-pci \n')

    cfg_f.write('} \n')

    cfg_f.flush()

def start_vpp():
    try:
        for mod in mods:
            subprocess.check_call(['/sbin/modprobe', mod])
        ret = subprocess.check_call(['/usr/bin/vpp', '-c', cfg_name])
    except:
        return False

    return True


def set_vpp_mgmt(vpp_if, ifcfg):
    ifname = ifcfg.get('name', None)
    print('Set VPP bridge for MGMT interface \'{}\''.format(ifname), file=vppztpStream)

    mgmt_id = int(ifname.replace('mgmt', ''))

    (ret, ifid) = vpp_if.create_tap_interface(ifname, id=mgmt_id + MGMT_OFFSET)
    print('Create TAP interface \'{}\' - {}'.format(ifname, ret), file=vppztpStream)

    # Create bridge-domain
    ret = vpp_if.create_bridge_domain(ifid)

    # Attach interface to bridge-domain
    vpp_if.add_to_bridge(ifid, ifid)
    vpp_if.ifup(ifid)

    return


def set_vpp_eth(vpp_if, ifcfg):
    if 'mgmt0' == ifcfg.get('name', ''):
        return set_vpp_mgmt(vpp_if, ifcfg)

    print('Set VPP bridge for ETH interface \'{}\''.format(ifcfg.get('name', 'Unknown')), file=vppztpStream)
    ifmode = ifcfg.get('mode', None)

    if ifmode is None:
        return

    ip4_address = None
    ip6_address = None
    ifname = ifcfg.get('name', None)
    ip4_address = ifcfg.get('ipv4-prefix', None)
    ip6_address = ifcfg.get('ipv6-prefix', None)
    if ifmode == 'virtio':
        if not os.path.exists('/sys/class/net/{}'.format(ifname)):
            print("{} does not exist on device.".format(ifname), file=vppztpStream)
            return
        # Create host-interface
        (retval, ifid) = vpp_if.create_host_interface(ifname)
        # set polling mode on rx
        retval = vpp_if.set_interface_rx_mode(idx=ifid, mode='polling')

        # try to set HW queue len in virtio mode
        os.system("/sbin/ethtool -G {} rx 512 || true".format(ifname))
        os.system("/sbin/ethtool -G {} tx 4096 || true".format(ifname))
        os.system("/sbin/ethtool --offload {} gso off tso off sg off gro off rx off tx off sg off lro off rxvlan off txvlan off rxhash off || true".format(ifname))
        os.system("/sbin/ethtool --offload {} ufo off || true".format(ifname))
        os.system("/sbin/ethtool --offload {} tx-gre-segmentation off || true".format(ifname))
        os.system("/sbin/ethtool --offload {} tx-gre-csum-segmentation off || true".format(ifname))
        os.system("/sbin/ethtool --offload {} tx-udp_tnl-segmentation off || true".format(ifname))
        os.system("/sbin/ethtool --offload {} tx-gso-partial off || true".format(ifname))
    elif ifmode == 'dpdk':
        ifid = vpp_if.get_ifidx_by_name(ifname)
        if ifid is None:
            print("{} does not exist on device.".format(ifname), file=vppztpStream)
            return
    elif 'sr-iov' == ifmode:
        # Create host-interface
        (retval, ifid) = vpp_if.create_host_interface(ifname)
        if (not os.path.exists("/sys/class/net/%s/device/sriov_totalvfs" % ifname)):
            print("SR-IOV doesn't seem to be supported on interface %s!" % ifname, file=vppztpStream)
        else:
            try:
                vf_count = int(subprocess.check_output("cat /sys/class/net/%s/device/sriov_totalvfs || true" % ifname,
                                                       shell=True).decode())
            except:
                vf_count = 0

            if vf_count <= 0 or vf_count > 8:
                vf_count = 8

            subprocess.check_output("echo %d > /sys/class/net/%s/device/sriov_numvfs || true" % (vf_count, ifname),
                                    shell=True)
    else:
        ifid = None

    if ifid is None:
        print('Failed to create VPP interface for \'{}\''.format(ifname), file=vppztpStream)
        return

    print('Interface \'{}\' ID={}'.format(ifname, ifid), file=vppztpStream)

    # Create bridge-domain with ID eq. to interface ID
    ret = vpp_if.create_bridge_domain(ifid)
    print('Create Bridge \'{}\' returns {}'.format(ifid, ret), file=vppztpStream)
    # Up interface
    print('Add interface ID {} to bridge-domain {}'.format(ifid, ifid), file=vppztpStream)
    vpp_if.add_to_bridge(ifid, ifid)
    vpp_if.ifup(ifid)

    ifid = set_vpp_vlan(vpp_if, ifcfg, ifid)

    # Create br-XXX and attach to bridge-domain -- set mac ???
    if USE_TAP:
        (retval, bridx) = vpp_if.create_tap_interface('{}{}'.format(BR_PREFFIX, ifname),
                                                      host_mac_addr=if_mac.get(ifname, None))
    else:
        with prt.IPRoute() as ipr:
            # create interface pair
            ipr.link('add', ifname='{}{}'.format(BR_PREFFIX, ifname), kind='veth',
                     peer='{}{}'.format(VETH_PREFFIX, ifname))
            index = ipr.link_lookup(ifname='{}{}'.format(BR_PREFFIX, ifname))[0]
            ipr.link('set', index=index, address=if_mac.get(ifname, None))
            (retval, bridx) = vpp_if.create_host_interface('{}{}'.format(VETH_PREFFIX, ifname))

    print('Created interface {}{} with ID {}'.format(BR_PREFFIX, ifname, bridx), file=vppztpStream)
    if 0 == retval:
        print('Add interface ID {} to bridge-domain {}'.format(bridx, ifid), file=vppztpStream)
        vpp_if.add_to_bridge(ifid, bridx)
    else:
        print('TAP NOT CREATED', file=vppztpStream)

    if ifmode == 'virtio':
        subprocess.check_call(['ip', 'link', 'set', ifname, 'promisc', 'on'])
        # try to set tx queue len in virtio mode
        os.system("/sbin/ifconfig {}{} txqueuelen 512 || true".format(BR_PREFFIX, ifname))
        os.system("/sbin/ethtool --offload {}{} gso off tso off sg off gro off rx off tx off sg off lro off rxvlan off txvlan off rxhash off || true".format(BR_PREFFIX, ifname))
        os.system("/sbin/ethtool --offload {}{} ufo off || true".format(BR_PREFFIX, ifname))
        os.system("/sbin/ethtool --offload {}{} tx-gre-segmentation off || true".format(BR_PREFFIX, ifname))
        os.system("/sbin/ethtool --offload {}{} tx-gre-csum-segmentation off || true".format(BR_PREFFIX, ifname))
        os.system("/sbin/ethtool --offload {}{} tx-udp_tnl-segmentation off || true".format(BR_PREFFIX, ifname))
        os.system("/sbin/ethtool --offload {}{} tx-gso-partial off || true".format(BR_PREFFIX, ifname))

    vpp_if.ifup(bridx)

    if ENABLE_RT_CTL:
        # Set Loopback BVI
        print('Loop interface', file=vppztpStream)
        (retval, loop_idx) = vpp_if.create_loopback_interface(mac_addr=if_mac.get(ifname, None))
        print('Loop interface create {} {}'.format(retval, loop_idx), file=vppztpStream)
        vpp_if.add_to_bridge(ifid, loop_idx, port_type=vppif.L2_API_PORT_TYPE_BVI)
        print('Loop interface 2', file=vppztpStream)
        vpp_if.set_interface_learning(bridx, disable=1)
        vpp_if.enable_routectl(bridx, loop_idx)
        vpp_if.routectl_set_mac(if_mac.get(ifname, None))
        vpp_if.ifup(loop_idx)

    if ENABLE_DRCTL:
        (retval, loop_idx) = vpp_if.create_loopback_interface(mac_addr=if_mac.get(ifname,None))
        print('Loop interface create {} {}'.format(retval, loop_idx), file=vppztpStream)
        vpp_if.add_to_bridge(ifid,loop_idx, port_type=vppif.L2_API_PORT_TYPE_BVI)
        vpp_if.set_interface_learning(bridx, disable=1)
        ret = vpp_if.enable_drctl(bridx, mac=if_mac.get(ifname,None))
        ret = vpp_if.drctl_set_arp(bridx, loop_idx)
        ret = vpp_if.drctl_set_dhcp(bridx, loop_idx)
        if ip4_address:
            ipv4_a = ipaddress.ip_interface(ip4_address)
            ret = vpp_if.add_interface_address(idx=loop_idx,addr=ip4_address)
            ret = vpp_if.drctl_set_ip( bridx, ipv4_a.ip.exploded )
            print('ip4 address {} added to interface {}'.format(ip4_address, ifname), file=vppztpStream)
        if ip6_address:
            ipv6_a = ipaddress.ip_interface(ip6_address)
            print('DI: adding ipv6 address: {} exploded {}' .format(ip6_address, ipv6_a.ip.exploded), file=vppztpStream)
            ret = vpp_if.add_interface_address(idx=loop_idx,addr=ip6_address)
            ret = vpp_if.drctl_set_ip( bridx, ipv6_a.ip.exploded )
            vpp_if.ip6_ra_config(loop_idx, suppress=1)
            print('ip6 address {} added to interface {}'.format(ip6_address, ifname), file=vppztpStream)
        vpp_if.ifup(loop_idx)


def set_vpp_lte(vpp_if, ifcfg):
    ifname = ifcfg.get('name', None)

    print('Set VPP bridge for LTE interface \'{}\''.format(ifname), file=vppztpStream)

    if ifname is None:
        return

    # get tap id
    tap_id = int(ifname.replace('lte', ''))
    print('Tap ID {}'.format(tap_id), file=vppztpStream)

    if USE_TAP:
        (ret, bridx) = vpp_if.create_tap_interface('{}{}'.format(BR_PREFFIX, ifname), id=tap_id + LTE_OFFSET,
                                                   host_mac_addr=if_mac.get(ifname, None))
    else:
        with prt.IPRoute() as ipr:
            # create interface pair
            ipr.link('add', ifname='{}{}'.format(BR_PREFFIX, ifname), kind='veth',
                     peer='{}{}'.format(VETH_PREFFIX, ifname))
            index = ipr.link_lookup(ifname='{}{}'.format(BR_PREFFIX, ifname))[0]
            ipr.link('set', index=index, address=if_mac.get(ifname, None))
            (ret, bridx) = vpp_if.create_host_interface('{}{}'.format(VETH_PREFFIX, ifname))

    if 0 == ret:
        vpp_if.ifup(bridx)

    ret = vpp_if.create_bridge_domain(bridx)
    vpp_if.add_to_bridge(bridx, bridx)

    if not os.path.exists('/sys/class/net/{}'.format(ifname)):
        print('Interface {}, do not exist. Skip add to bridge!'.format(ifname), file=vppztpStream)
        return

    (ret, ifid) = vpp_if.create_host_interface(ifname)
    if ret != 0:
        print('Failde to attach {} interface in VPP!', file=vppztpStream)
        return

    # set polling rx mode
    ret = vpp_if.set_interface_rx_mode(idx=ifid, mode='polling')

    # raw ip ?!?!
    if if_lte.lte_check_raw_if(ifname):
        print('LTE interface \'{}\'is in RAW IP mode !'.format(ifname), file=vppztpStream)
        vpp_if.enable_rawip(idx=ifid, dst_mac=if_mac.get(ifname, None))

    vpp_if.add_to_bridge(bridx, ifid)
    vpp_if.ifup(ifid)

    if ENABLE_RT_CTL:
        # Set Loopback BVI
        (retval, loop_idx) = vpp_if.create_loopback_interface(mac_addr=if_mac.get(ifname, None))
        vpp_if.add_to_bridge(bridx, loop_idx, port_type=vppif.L2_API_PORT_TYPE_BVI)
        vpp_if.set_interface_learning(bridx, disable=1)
        vpp_if.enable_routectl(bridx, loop_idx)
        vpp_if.routectl_set_mac(if_mac.get(ifname, None))
        vpp_if.ifup(loop_idx)

    if ENABLE_DRCTL:
        (retval, loop_idx) = vpp_if.create_loopback_interface(mac_addr=if_mac.get(ifname,None))
        vpp_if.add_to_bridge(bridx,loop_idx, port_type=vppif.L2_API_PORT_TYPE_BVI)
        vpp_if.set_interface_learning(bridx, disable=1)
        vpp_if.enable_drctl(bridx, mac=if_mac.get(ifname,None))
        vpp_if.drctl_set_arp(bridx, loop_idx)
        vpp_if.drctl_set_dhcp(bridx, loop_idx)
        vpp_if.ifup(loop_idx)


def set_vpp_nat(vpp_if, ifcfg):
    print('Set NAT bridge for interface \'{}\''.format(ifcfg.get('name', None)), file=vppztpStream)
    # Cretate interface
    ifname = ifcfg.get('name', None)
    print(ifname)
    nat_id = int(ifname.replace('nat', ''))

    (ret, ifid) = vpp_if.create_tap_interface(ifname, id=nat_id + NAT_OFFSET)
    print('Create TAP interface \'{}\' - {}'.format(ifname, ret), file=vppztpStream)

    # Create bridge-domain
    ret = vpp_if.create_bridge_domain(ifid)

    # Attach interface to bridge-domain
    vpp_if.add_to_bridge(ifid, ifid)
    vpp_if.ifup(ifid)


def set_vpp_wifi(vpp_if, ifcfg):
    ifname = ifcfg.get('name', None)

    print('Set VPP bridge for wifi interface \'{}\''.format(ifname), file=vppztpStream)

    if ifname is None:
        return

    # get tap ind
    if "vap" in ifname:
        return
    
    tap_id = int(ifname.replace('wifi', ''))
    print('Tap ID {}'.format(tap_id), file=vppztpStream)

    if USE_TAP:
        (ret, bridx) = vpp_if.create_tap_interface('{}{}'.format(BR_PREFFIX, ifname), id=tap_id + WIFI_OFFSET,
                                                   host_mac_addr=if_mac.get(ifname, None))
    else:
        with prt.IPRoute() as ipr:
            # create interface pair
            ipr.link('add', ifname='{}{}'.format(BR_PREFFIX, ifname), kind='veth',
                     peer='{}{}'.format(VETH_PREFFIX, ifname))
            index = ipr.link_lookup(ifname='{}{}'.format(BR_PREFFIX, ifname))[0]
            ipr.link('set', index=index, address=if_mac.get(ifname, None))
            (ret, bridx) = vpp_if.create_host_interface('{}{}'.format(VETH_PREFFIX, ifname))

    if 0 == ret:
        vpp_if.ifup(bridx)

    ret = vpp_if.create_bridge_domain(bridx)
    vpp_if.add_to_bridge(bridx, bridx)

    if not os.path.exists('/sys/class/net/{}'.format(ifname)):
        print('Interface {}, do not exist. Skip add to bridge!'.format(ifname), file=vppztpStream)
        return

    (ret, ifid) = vpp_if.create_host_interface(ifname)
    if ret != 0:
        print('Failde to attach {} interface in VPP!', file=vppztpStream)
        return

    # set polling rx mode
    ret = vpp_if.set_interface_rx_mode(idx=ifid, mode='polling')

    vpp_if.add_to_bridge(bridx, ifid)
    vpp_if.ifup(ifid)

    if ENABLE_RT_CTL:
        # Set loop as BVI
        (retval, loop_idx) = vpp_if.create_loopback_interface(mac_addr=if_mac.get(ifname, None))
        vpp_if.add_to_bridge(bridx, loop_idx, port_type=vppif.L2_API_PORT_TYPE_BVI)
        vpp_if.set_interface_learning(bridx, disable=1)
        vpp_if.enable_routectl(bridx, loop_idx)
        vpp_if.routectl_set_mac(if_mac.get(ifname, None))
        vpp_if.ifup(loop_idx)

    if ENABLE_DRCTL:
        # Set loop as BVI
        (retval, loop_idx) = vpp_if.create_loopback_interface(mac_addr=if_mac.get(ifname,None))
        vpp_if.add_to_bridge(bridx,loop_idx, port_type=vppif.L2_API_PORT_TYPE_BVI)
        vpp_if.set_interface_learning(bridx, disable=1)
        vpp_if.enable_drctl(bridx, mac=if_mac.get(ifname,None))
        vpp_if.drctl_set_arp(bridx, loop_idx)
        vpp_if.drctl_set_dhcp(bridx, loop_idx)
        vpp_if.ifup(loop_idx)


def set_vpp_bridges(intf_cfg):
    print('Configuring bridge-domains in VPP.', file=vppztpStream)

    tries = 5
    for i in range(tries):
        try:
            vpp_if = vppif.VppApiInterface('vpp-config')
        except:
            if i < tries - 1:
                time.sleep(8)
                continue
            else:
                raise
        break

    for _, ifcfg in intf_cfg.items():
        if ifcfg.get('type', None) == 'ethernet':
            set_vpp_eth(vpp_if, ifcfg)
        elif ifcfg.get('type', None) == 'lte':
            set_vpp_lte(vpp_if, ifcfg)
        elif ifcfg.get('type', None) == 'nat':
            set_vpp_nat(vpp_if, ifcfg)
        elif ifcfg.get('type', None) == 'wifi':
            set_vpp_wifi(vpp_if, ifcfg)
    time.sleep(15)


def set_vpp_ippass(intf_cfg, sr_int):
    print('Configuring bridge-domains with IPPass.', file=vppztpStream)
    vpp_if = vppif.VppApiInterface('vpp-config')

    for _, ifcfg in intf_cfg.items():
        if_type = ifcfg.get('type', None)
        if if_type == 'ethernet' or if_type == 'lte' or if_type == 'wifi':
            if ifcfg.get('ip-passthrough', None) == True:
                if_name = ifcfg.get('name', None)
                arp_term = if_lte.lte_check_raw_if(if_name)
                if_name = sr_int.tag_interface(if_name, sr_int.get_interface_param(if_name, 'vlan'))
                intf = vpp_if.get_ifidx_by_name(if_name)
                if intf is None:
                    intf = vpp_if.get_ifidx_by_name("host-{}".format(if_name))

                if intf is None:
                    continue

                domains = vpp_if.get_bridge_domains()
                for bd in domains:
                    intf_idx = [index['sw_if_index'] for index in bd['sw_if_details']]
                    if intf in intf_idx:
                        intf_idx.remove(intf)
                        if ENABLE_RT_CTL == False and ENABLE_DRCTL == False:
                            host_id = intf_idx[0]
                            loop_id = 0xffffffff
                        else:
                            host_id = intf_idx[1]
                            loop_id = intf_idx[0]
                        vpp_if.enable_ippass(host_idx=host_id, wan_idx=intf, rt_idx=loop_id)
                        if arp_term == True:
                            vpp_if.enable_ippass_arp_termination(host_idx=host_id)
                        print('Enable IPPass on HOST {} with WAN {}, ARP termination {}'.format(host_id,intf,arp_term), file=vppztpStream)
                        vpp_if.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='icmp')
                        vpp_if.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='udp', dst_port=53)
                        vpp_if.add_del_ippass_permit_host_entry(host_idx=host_id, protocol='udp', dst_port=123)


def set_vpp_vlan(vpp_if, ifcfg, ifid):
    # set_vpp_vlan creates sub-interface adds it to a  newly created bridge-domain
    # returns it's index
    vlan = ifcfg.get('vlan', None)
    if vlan is not None and vlan != 0:
        (retval, subidx) = vpp_if.create_vlan_subinterface(idx=ifid, vlan_id=vlan)
        print('Created sub-interface {}{}.{} with ID {}'.format(BR_PREFFIX, ifname, vlan, subidx), file=vppztpStream)
        if retval == 0 and vpp_if.vlan_rewrite_poptag1(subidx) == 0:
            ret = vpp_if.create_bridge_domain(subidx)
            vpp_if.add_to_bridge(subidx, subidx)
            vpp_if.ifup(subidx)
            return subidx
    return ifid


if __name__ == "__main__":

    if 'yes' == os.environ.get('RTCTL', ''):
        ENABLE_DRCTL=False
        ENABLE_RT_CTL=True

    intf = sri.SysrepoInterfaces(app_name='vpp-pre-ztp')
    intf.set_active_datastore("startup")
    intf_cfg = intf.get_interfaces(startup=True)
    validate_lte_dongles()

    for ifname, ifcfg in intf_cfg.items():
        if if_lte.lte_check_raw_if(ifname):
            mac = if_lte.lte_get_raw_mac(ifname)
        else:
            mac = get_if_mac(ifname)
        if mac:
            if_mac[ifname] = mac
    gen_vpp_cfg(intf_cfg)
    if False == start_vpp():
        print('Failed to launch VPP', file=vppztpStream)
        exit(1)
    else:
        print('VPP launched successfully', file=vppztpStream)

    set_vpp_bridges(intf_cfg)
    #set_vpp_ippass(intf_cfg, intf)
