#! /usr/bin/env python3
import ipaddress
import os
import string
import subprocess
import sys
import time
import random
import urllib.request
from platform import machine

import netifaces as ni
import serial
import simplejson as json
import ucpeif.pppoeclientif as pppoeif
from netaddr import IPNetwork, IPAddress
from systemd import journal
from tveif import if_lte
from ucpeif import ipsecif as ips
import re


tmp_cfg_file = "/tmp/in_tve_config.json"
net_cfg_dir = "/etc/network/interfaces.d"
serial_cfg_file = "/config/.serial-number"
ztp_auth_key = "/config/_hb/ztp_auth.key"
ovpn_client_file = "/config/etc/openvpn/client.conf"

REBOOTS_FILE = '/config/reboots'
orch_url = 'https://{}:{}/config'
ztp_port = 30636

hw_type = 'intel'
with open('/config/.hwtype', 'r') as hw_type_file:
    hw_type = hw_type_file.readline().splitlines()[0]

device_init_passed = False

if 'eventlet.os' in sys.modules:
    ztpStream = sys.stdout
else:
    ztpStream = journal.stream(identifier='ztp-pre')

orch_ip = None
hc_enabled = False
vpp_proc = None
pppoe_interfaces = []
vxlan_interfaces = []


def is_arm_arch():
    if 'aarch64' in machine():
        return True
    return False


def get_interface_vtag(interface):
    vlan = interface.get('vlan', 0)
    if 'int' == type(vlan).__name__:
        return vlan

    if 0 == len(vlan):
        return 0

    vtag = int(vlan)
    return vtag


def setup_eth_ip_if(if_cfg):
    from sysrepo_system import sysrepo_system as sys_api
    sys_client = sys_api.SysrepoSystem(app_name="ZTP read config")
    sys_client.set_active_datastore("startup")
    net_mode = sys_client.get_network_mode()

    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config interfaces")
    client.set_active_datastore("startup")

    if not 'bootproto' in if_cfg and not 'ipv4-address' in if_cfg:
        print('Interface \'%s\' configuration is not full' % if_cfg['name'], file=ztpStream)
        return

    vlan = get_interface_vtag(if_cfg)
    ifname = str("%s%s" % (if_cfg['name'], '' if (0 == vlan or 'inband' in if_cfg['name']) else '.%d' % vlan))

    if net_mode == 'vpp':
        ifname = client.get_if_sysname(ifname, vlan)
        ifname = ifname.split('.')[0]

    print('Setup ethernet IP {} interfaces'.format(ifname), file=ztpStream)

    net_cfg_file = "%s/ifcfg-%s" % (net_cfg_dir, ifname)
    options = ['/scripts/apply_networking.py', "-n%s" % ifname]
    options += ['-oyes']
    if 'dhcp' == if_cfg['bootproto'] or 'dhcp6' == if_cfg['bootproto'] or 'dhcp46' == if_cfg['bootproto']:
        options += ["-b%s" % if_cfg['bootproto']]

    elif '' != if_cfg.get('ipv4-address', '') or '' != if_cfg.get('ipv6-address', ''):  # obsolete
        options += ['-bstatic']

        # IPv4
        if '' != if_cfg.get('ipv4-address', ''):
            options += ["-a%s" % if_cfg['ipv4-address']]

        if 'netmask' in if_cfg and if_cfg['netmask']:
            options += ["-m%s" % if_cfg['netmask']]

        if 'gateway' in if_cfg and if_cfg['gateway']:
            options += ["-g%s" % if_cfg['gateway']]

        if 'dns' in if_cfg and if_cfg['dns']:
            options += ["-d%s" % if_cfg['dns']]

        # IPv6
        if '' != if_cfg.get('ipv6-address', ''):
            options += ["-f%s" % if_cfg['ipv6-address']]

        if 'netmask6' in if_cfg and if_cfg['netmask6']:
            options += ["--ipv6-netmask %s" % if_cfg['netmask6']]

        if 'gateway6' in if_cfg and if_cfg['gateway6']:
            options += ["--ipv6-gateway %s" % if_cfg['gateway6']]

        if 'dns6' in if_cfg and if_cfg['dns6']:
            options += ["--ipv6-dns %s" % if_cfg['dns6']]

    elif '' != if_cfg.get('ipv4-prefix', '') or '' != if_cfg.get('ipv6-prefix', ''):
        options += ['-bstatic']

        # IPv4
        if '' != if_cfg.get('ipv4-prefix', ''):
            options += [str("--ipv4-prefix=%s" % if_cfg['ipv4-prefix'])]

        if 'gateway' in if_cfg and if_cfg['gateway']:
            options += [str("-g%s" % if_cfg['gateway'])]

        if 'dns' in if_cfg and if_cfg['dns']:
            options += [str("-d%s" % if_cfg['dns'])]

        # IPv6
        if '' != if_cfg.get('ipv6-prefix', ''):
            options += ["-c%s" % if_cfg.get('metric', 100)]

        if '' != if_cfg.get('ipv6-prefix', ''):
            options += [str("--ipv6-prefix=%s" % if_cfg['ipv6-prefix'])]

        if 'gateway6' in if_cfg and if_cfg['gateway6']:
            options += ["--ipv6-gateway=%s" % if_cfg['gateway6']]

        if 'dns6' in if_cfg and if_cfg['dns6']:
            options += ["--ipv6-dns=%s" % if_cfg['dns6']]

    else:
        options += ['-bmanual']

    if vlan != 0:
        options += ['-v%s' % vlan]

    try:
        subprocess.check_call(options)
    except subprocess.CalledProcessError:
        print('options are: %s' % options, file=ztpStream)
    except OSError:
        print('/scripts/apply_networking.py not found', file=ztpStream)

    try:
        subprocess.check_call(['/sbin/ifup', '--force', ifname])
    except subprocess.CalledProcessError:
        print('Cannot /sbin/ifup interface %s' % ifname, file=ztpStream)
    except OSError:
        print('/sbin/ifup not found', file=ztpStream)


def get_mcc_codes_tty_dev(dev):
    mcc = "0"
    mnc = "0"
    try:
        modem = serial.Serial(dev, 115200, timeout=3)
        if_lte.lte_putcmd(modem, 'AT+COPS=3,2')
        if_lte.lte_getresponse(modem)
        if_lte.lte_putcmd(modem, 'AT+COPS?')
        lines = if_lte.lte_getresponse(modem, maxlines=5, timeout=3)
        cops = "".join(line.strip() for line in lines)
        fields = cops.split('"')
        plmn = fields[1]
        if len(plmn) == 6 or len(plmn) == 5:
            mcc = plmn[:3]
            mnc = plmn[3:]

        modem.close()
        return int(mcc), int(mnc)
    except:
        return int(mcc), int(mnc)


def setup_lte_tty_dev(if_cfg, drv):
    if 'apn' not in if_cfg or not if_cfg['apn']:
        mcc, mnc = get_mcc_codes_tty_dev(drv)
        apn, user, password = if_lte.get_lte_apn(if_lte.apn_dbase, if_lte.apn_mcc_file, mcc, mnc)
    else:
        apn=""
        user=""
        password=""
        if 'apn' in if_cfg:
            apn = if_cfg['apn']
        if 'user' in if_cfg:
            user = if_cfg['user']
        if 'pass' in if_cfg:
            password = if_cfg['pass']

    if not apn:
        apn = "internet"

    try:
        modem = serial.Serial(drv, 115200, timeout=3)
        if_lte.lte_putcmd(modem, "ATZ")
        time.sleep(1)
        if_lte.lte_getresponse(modem)
        if_lte.lte_putcmd(modem, "AT&d0")
        if_lte.lte_getresponse(modem)
        if_lte.lte_putcmd(modem, "AT&c0")
        if_lte.lte_getresponse(modem)

        if_lte.lte_putcmd(modem, 'AT+CFUN=1')
        if_lte.lte_getresponse(modem)

        if 'vz' in apn.lower():
            # Verizon requires pdp 3
            pdp = 3
        else:
            pdp = 1

        if_lte.lte_putcmd(modem, 'AT+CGDCONT=%d,"IP","%s"' % (pdp, apn))
        if_lte.lte_getresponse(modem)

        # Set authentication mode 1,password,username
        if user and password:
            if_lte.lte_putcmd(modem, 'AT$QCPDPP=1,1,%s,%s' % (password, user))
        else:
            if_lte.lte_putcmd(modem, 'AT$QCPDPP=1')

        if_lte.lte_getresponse(modem)

        # LTE only
        if_lte.lte_putcmd(modem, 'AT!SELRAT=06')
        if_lte.lte_getresponse(modem)
        time.sleep(3)

        # activate data connection
        if_lte.lte_putcmd(modem, 'AT!SCACT=1,1')
        if_lte.lte_getresponse(modem)
        retry = 10
        while True:
            time.sleep(3)
            if_lte.lte_putcmd(modem, 'AT!SCACT?')
            resp = if_lte.lte_getresponse(modem, maxlines=5, timeout=3)

            if '1,1' in resp:
                break

            if retry <= 0:
                break

            retry = retry - 1
            if_lte.lte_putcmd(modem, 'AT!SCACT=1,1')
            if_lte.lte_getresponse(modem)

        modem.close()
    except:
        print('Can\'t initialize modem.No device %s found' % drv, file=ztpStream)


def order_external_ltes():
    print('Order external lte interfaces', file=ztpStream)
    lte1_dir='/sys/class/net/lte1'
    lte2_dir='/sys/class/net/lte2'
    if os.path.isdir(lte1_dir) and os.path.isdir(lte2_dir):
        lte1_usb_port = subprocess.check_output(f"udevadm info {lte1_dir} | grep ID_PATH_TAG | " r"awk -F 'usb-' '{print $2}'", shell=True, encoding='utf-8')
        lte2_usb_port = subprocess.check_output(f"udevadm info {lte2_dir} | grep ID_PATH_TAG | " r"awk -F 'usb-' '{print $2}'", shell=True, encoding='utf-8')
        if (lte1_usb_port > lte2_usb_port):
            subprocess.check_output('ip link set lte1 name temp_lte', shell=True, encoding='utf-8')
            subprocess.check_output('ip link set lte2 name lte1', shell=True, encoding='utf-8')
            subprocess.check_output('ip link set temp_lte name lte2', shell=True, encoding='utf-8')


def generate_mac(oui):
    return("%s%02x:%02x:%02x" % (
        oui,
        random.randint(0, 255),
        random.randint(0, 255),
        random.randint(0, 255),
    ))


def validate_lte_mac():
    print('Validating MAC address of external LTE dongles', file=ztpStream)
    lte1_mac = subprocess.check_output('ip a show lte1 | grep ether | ' r"awk '{print $2}'", shell=True, encoding='utf-8')
    lte2_mac = subprocess.check_output('ip a show lte2 | grep ether | ' r"awk '{print $2}'", shell=True, encoding='utf-8')
    retry_count=0
    while True:
        print('Duplicate MAC found on external LTE dongles. Changing MAC on lte2...', file=ztpStream)
        retry_count += 1
        lte2_mac = generate_mac(oui = lte2_mac[0:9])
        if lte1_mac != lte2_mac:
            subprocess.run(['ifconfig', 'lte2', 'hw', 'ether', lte2_mac])
            break
        if retry_count > 5:
            print('Could not generate new MAC address for lte2 (external lte dongle)', file=ztpStream)
            break


def kernel_detects_external_ltes() -> int:
    # Return: (int) Number of external interfaces
    return len([iface for iface in os.listdir('/sys/class/net') if 'lte'==iface[0:3] and 'lte0'!=iface])


def kernel_detects_lte():
    detected_interfaces = os.listdir("/sys/class/net")
    ltes = ("lte", "lte0", "lte1", "lte2")
    if any(iface in detected_interfaces for iface in ltes):
        print('LTE in /sys/class/net', file=ztpStream)
        return True

    print('No LTE in /sys/class/net', file=ztpStream)
    return False


def setup_lte_ip_if(if_cfg):
    print('Setup LTE IP interface - %s,%s' % (if_cfg['name'], if_cfg['enable']), file=ztpStream)

    # check if qmi driver is there,if not,try AT-interface to program LTE
    if if_lte.lte_is_qmi(if_lte.modem_default):
        if_lte.setup_lte_qmi_dev(if_cfg, if_lte.modem_default)
    elif if_lte.lte_is_mbim(if_lte.mbim_modem_default):
        if_lte.setup_lte_mbim_dev(if_cfg, if_lte.mbim_modem_default)
    else:
        setup_lte_tty_dev(if_cfg, if_lte.modem_tty_default)

    # MBIMCLI setup
    print('Generate configuration file for \'%s\'' % if_cfg['name'], file=ztpStream)
    setup_eth_ip_if(if_cfg)


def setup_wpa_supplicant(if_cfg):
    try:
        ssid = if_cfg['ssid']
        keymgmt = if_cfg['security']['key-mgmt']
        passphrase = if_cfg['security']['passphrase']
        country = if_cfg['radio-settings']['country']
    except:
        print('Insufficient config={} !'.format(if_cfg), file=ztpStream)
        print('SSID, key management, passphrase and country are required!', file=ztpStream)
        return

    print('wifi-config ssid={} key_mgmt={} passthrase={}'.format(ssid, keymgmt, passphrase), file=ztpStream)

    try:
        keymgmt_detected = 'NONE'
        if keymgmt == 'AUTO':
            wifi_scan_result = subprocess.run(['/sbin/iwlist', 'wifi0', 'scan'],
                                              stdout=subprocess.PIPE, encoding='UTF-8')
            if 0 == wifi_scan_result.returncode:
                available_AP = wifi_scan_result.stdout.split('Cell ')
                for cell in available_AP:
                    if 0 <= cell.find('ESSID:"{}"'.format(ssid)):
                        if 0 <= cell.find('IE: IEEE 802.11i/WPA2 Version 1'):
                            keymgmt_detected = 'WPA-PSK\n\tpairwise=CCMP'
                        elif 0 <= cell.find('IE: WPA Version 1'):
                            keymgmt_detected = 'WPA-PSK\n\tpairwise=TKIP'
                        else:
                            keymgmt_detected = 'WEP'

        else:
            keymgmt_detected = keymgmt

        if 'WEP' != keymgmt_detected:
            wpa_out = subprocess.run(['/usr/bin/wpa_passphrase', ssid, passphrase],
                                     stdout=subprocess.PIPE, encoding='UTF-8', timeout=60)
            print("wpa_out {}".format(wpa_out), file=ztpStream)
            if 0 == wpa_out.returncode:
                with open('/tmp/wpa_supplicant.conf', 'w') as f:
                    print('ctrl_interface=/run/wpa_supplicant', file=f)
                    print('update_config=1', file=f)
                    print('country={}'.format(country), file=f)
                    print('network={', file=f)
                    print('\tkey_mgmt={}'.format(keymgmt_detected), file=f)
                    print('{}'.format((wpa_out.stdout).split('\n', 1)[-1]), file=f)

        else:
            with open('/tmp/wpa_supplicant.conf', 'w') as f:
                print('ctrl_interface=/run/wpa_supplicant', file=f)
                print('update_config=1', file=f)
                print('country={}'.format(country), file=f)
                print('network={', file=f)
                print('\tkey_mgmt={}'.format('NONE'), file=f)
                print('\tssid="{}"'.format(ssid), file=f)
                print('\twep_key0={}'.format(passphrase), file=f)
                print('}', file=f)

        subprocess.run(['/sbin/wpa_supplicant',
                        '-c/tmp/wpa_supplicant.conf',
                        '-iwifi0', '-B'])
        wpa_out = subprocess.run(['/sbin/iwconfig', 'wifi0'],
                                 stdout=subprocess.PIPE, encoding='UTF-8')
        print("wpa_out {}".format(wpa_out), file=ztpStream)
    except subprocess.CalledProcessError as ce:
        print('wpa_supplicant call error {}'.format(ce), file=ztpStream)
    except subprocess.SubprocessError as se:
        print('wpa_supplicant process error {}'.format(se), file=ztpStream)


def setup_wifi_ip_if(if_cfg):
    print('Setup wifi IP interface - {},enable={}'.format(if_cfg['name'], if_cfg['enable']), file=ztpStream)
    setup_eth_ip_if(if_cfg)
    setup_wpa_supplicant(if_cfg)


def validate_config(file_name):
    if not os.path.isfile(file_name):
        return None

    cfg_f = open(file_name, 'r')
    cfg = json.load(cfg_f)
    return cfg


def reboot_loop_prevent(reset=False):
    if os.path.isfile(REBOOTS_FILE):
        r_file = open(REBOOTS_FILE, 'r+')
    else:
        r_file = open(REBOOTS_FILE, 'w+')

    line0 = r_file.readline().strip()
    line_w = line0.split(' ')

    if reset:
        reboot_cnt = 0
        result = True
    else:
        if '' == line_w[0]:
            print('1', file=ztpStream)
            reboot_cnt = 0
        else:
            print('2', file=ztpStream)
            reboot_cnt = string.atoi(line_w[0])

        print('3', file=ztpStream)
        if reboot_cnt < 3:
            reboot_cnt += 1
            result = True
        else:
            result = False

    r_file.seek(0)
    r_file.truncate()
    r_file.seek(0)
    r_file.write('%d' % reboot_cnt)
    r_file.flush()
    r_file.close()

    return result


def restore_db():
    if not os.path.isfile('/config/db_restore') or not os.path.isfile('/config/mysql.tar.bz2'):
        print('There is no flag for restore', file=ztpStream)
        return

    import fc_tools as fc
    fc.untar_bz2('/config/mysql.tar.bz2', '/var/lib/mysql')
    os.remove('/config/mysql.tar.bz2')
    os.remove('/config/db_restore')


def apply_config_to_sysrepo(infilename=None):
    print('apply_config_to_sysrepo', file=ztpStream)
    if not infilename:
        print("Wrong config from manager - empty!!! error", file=ztpStream)
        return 1

    with open(infilename, 'r') as f:
        cfg_all = json.load(f)
        f.close()  # force flush

    if not 'json' in cfg_all \
       or not 'tve' in cfg_all['json'] \
       or not 'ucpe:config' in cfg_all['json']['tve']:
        print('error missing config section!!!', file=ztpStream)
        return 1

    cfg_config = {'ucpe:config': ''}

    if 'ucpe:config' in cfg_all['json']['tve'] \
       and cfg_all['json']['tve']['ucpe:config']:
        cfg_config['ucpe:config'] = cfg_all['json']['tve']['ucpe:config']

        if hw_type.lower() in ('nanopir2s', 'nanopir4s'):
            if 'dhcprelay:dhcp-relay' in cfg_config['ucpe:config']:
                cfg_config['ucpe:config'].pop('dhcprelay:dhcp-relay')

            if 'snmp-server:snmp-server' in cfg_config['ucpe:config']:
                cfg_config['ucpe:config'].pop('snmp-server:snmp-server')

            if 'syslog:syslog' in cfg_config['ucpe:config']:
                cfg_config['ucpe:config'].pop('syslog:syslog')

            if 'system-resources:system-resources' in cfg_config['ucpe:config']:
                cfg_config['ucpe:config'].pop('system-resources:system-resources')

        #if 'system-resources:system-resources' in cfg_config['ucpe:config'] and 'huge-page-size' in cfg_config['ucpe:config']['system-resources:system-resources']:
        #    cfg_config['ucpe:config']['system-resources:system-resources'].pop('huge-page-size')

        with open('/tmp/yang_config.json', 'w+') as f:
            print(json.dumps(cfg_config, indent=4, sort_keys=True), file=f)
            f.close()  # force flush

        try:
            subprocess.check_call(['/scripts/save_nulls_from_json.sh'],
                                  stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
            subprocess.check_call(['/scripts/strip_empty_values_from_json.sh', '/tmp/yang_config.json'],
                                  stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
            res = subprocess.run(['sysrepocfg', '--import=/tmp/yang_config.json',
                                   '--datastore=startup', '--format=json', '-m', 'ucpe'],
                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if res.returncode !=0:
                print("Error applying config: %s" % res.stderr.decode(), file=ztpStream)
                the_error=res.stderr.decode()
            else:
                the_error="No error."
                
            with open("/config/_hb/last_known_err", 'w') as f:
                f.write(the_error)

        except Exception as e:
            print('Error applying ucpe:config', file=ztpStream)
            print(e, file=ztpStream)
            return 1

    netconf_config = {'ietf-netconf-server:netconf-server': ''}
    if 'ietf-netconf-server:netconf-server' in cfg_all['json']['tve'] \
       and cfg_all['json']['tve']['ietf-netconf-server:netconf-server']:
        netconf_config['ietf-netconf-server:netconf-server'] = cfg_all['json']['tve'][
            'ietf-netconf-server:netconf-server']

        if 'listen' in netconf_config['ietf-netconf-server:netconf-server'] and \
           'endpoint' in netconf_config['ietf-netconf-server:netconf-server']['listen']:
            for i in netconf_config['ietf-netconf-server:netconf-server']['listen']['endpoint']:
                if i['ssh']:
                    i['ssh'].update({'ssh-server-parameters':{'server-identity':{'host-key':[{'name':'default-key','public-key':{'keystore-reference':'genkey'}}]},'client-authentication': {'supported-authentication-methods':{'publickey':[None],'passsword':[None]}}}})

        with open('/tmp/yang_netconf.json', 'w+') as f:
            print(json.dumps(netconf_config, indent=4, sort_keys=True), file=f)
            f.close()  # force flush

        try:
            subprocess.check_call(['/scripts/strip_empty_values_from_json.sh', '/tmp/yang_netconf.json'],
                                  stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
            subprocess.check_call(['sysrepocfg', '--import=/tmp/yang_netconf.json',
                                   '--datastore=startup', '--format=json', '-m', 'ietf-netconf-server'],
                                  stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
        except Exception as e:
            print("Error applying 'ietf-netconf-server'", file=ztpStream)
            print(e, file=ztpStream)
            return 1

    return 0


def validate_sysrepo_cfg():
    global orch_ip
    global hc_enabled
    from sysrepo_system import sysrepo_system as api
    client = api.SysrepoSystem(app_name="ZTP read config")
    client.set_active_datastore("startup")
    hc_enabled = client.get_homecalling_enabled()
    managers = client.get_config_servers()
    for i in managers:
        orch_ip = i
        if orch_ip is not None:
            break


def wait_vpn_up_down(if_cfg, up=True):
    wait = False
    if if_cfg['enable'] == True:
        wait = True

    cnt = 6
    while wait:
        print("Wait VPN -- %s" % ('Up' if up else 'Down'), file=ztpStream)
        time.sleep(10)
        if up and os.path.exists("/tmp/ovpn.up"):
            time.sleep(5)
            break
        if not up and not os.path.exists("/tmp/ovpn.up"):
            time.sleep(5)
            break

        cnt = cnt - 1
        if cnt <= 0:
            break


def setup_ipsecvpn_if(if_cfg):
    print('Setup IPsecVPN interface - %s,%s' % (if_cfg['name'], if_cfg['enable']), file=ztpStream)
    ret = False
    if if_cfg['enable'] is False or if_cfg['enable'] == 'false':
        return

    ipsvpn = ips.ipsec_vpn_if(if_cfg['name'])
    for key, value in if_cfg.items():
        if key == "ike-version":
            if value == '1':
                ipsvpn.ike = ips.IKE.V1
            elif value == '2':
                ipsvpn.ike = ips.IKE.V2
        elif key == "ike-phase-1":
            if 'dh-groups' in if_cfg[key]:
                for group in if_cfg[key]['dh-groups']:
                    ipsvpn.dh_group_add(int(group['group']), ips.IKE_PHASE.PHASE1)
            if 'proposals' in if_cfg[key]:
                for prop in if_cfg[key]['proposals']:
                    if 'encryption-algorithm' in prop and 'authentication-algorithm' in prop:
                        ipsvpn.proposal_add(str(prop['encryption-algorithm']), str(prop['authentication-algorithm']),
                                            ips.IKE_PHASE.PHASE1)

            # /ike-phase-1/local/rounds[id='rnd']/authentication-method
            if 'local' in if_cfg[key]:
                if 'rounds' in if_cfg[key]['local'] and len(if_cfg[key]['local']['rounds']):
                    for auth_method in if_cfg[key]['local']['rounds']:
                        if auth_method['authentication-method'] == 'pre-shared-keys':
                            ipsvpn.auth_add(ips.AUTH_SIDE.LOCAL, auth_method['id'],
                                            ips.IPSecAuth(method=ips.AUTH_METHOD.PSK, psk=auth_method['secret']))
                        elif auth_method['authentication-method'] == 'pubkeys':
                            passphrase = None if 'key-pass' not in auth_method.keys() else str(auth_method['key-pass'])
                            ipsvpn.auth_add(ips.AUTH_SIDE.LOCAL, auth_method['id'],
                                            ips.IPSecAuth(method=ips.AUTH_METHOD.PUBKEY, ca_crt=auth_method['ca-cert'],
                                                          crt=auth_method['cert'], key=auth_method['key'],
                                                          passphrase=passphrase))
                        elif auth_method['authentication-method'] == 'xauth':
                            ipsvpn.auth_add(ips.AUTH_SIDE.LOCAL, auth_method['id'],
                                            ips.IPSecAuth(method=ips.AUTH_METHOD.XAUTH, user=auth_method['username'],
                                                          psw=auth_method['password']))
                        elif auth_method['authentication-method'] == 'eap':
                            passphrase = None if 'key-pass' not in auth_method.keys() else str(auth_method['key-pass'])
                            ipsvpn.auth_add(ips.AUTH_SIDE.LOCAL, auth_method['id'],
                                            ips.IPSecAuth(method=ips.AUTH_METHOD.EAP, user=auth_method['username'],
                                                          psw=auth_method['password'], ca_crt=auth_method['ca-cert'],
                                                          crt=auth_method['cert'], key=auth_method['key'],
                                                          passphrase=passphrase))

            # /ike-phase-1/remote/rounds[id='rnd']/authentication-method
            if 'remote' in if_cfg[key]:
                if 'rounds' in if_cfg[key]['remote'] and len(if_cfg[key]['remote']['rounds']):
                    for auth_method in if_cfg[key]['remote']['rounds']:
                        if auth_method['authentication-method'] == 'pre-shared-keys':
                            ipsvpn.auth_add(ips.AUTH_SIDE.REMOTE, auth_method['id'],
                                            ips.IPSecAuth(method=ips.AUTH_METHOD.PSK, psk=auth_method['secret']))
                        elif auth_method['authentication-method'] == 'pubkeys':
                            ipsvpn.auth_add(ips.AUTH_SIDE.REMOTE, auth_method['id'],
                                            ips.IPSecAuth(method=ips.AUTH_METHOD.PUBKEY, ca_crt=auth_method['ca-cert'],
                                                          crt=auth_method['cert']))

        elif key == "ike-phase-2":
            if 'dh-groups' in if_cfg[key]:
                for group in if_cfg[key]['dh-groups']:
                    ipsvpn.dh_group_add(int(group['group']), ips.IKE_PHASE.PHASE2)
            if 'proposals' in if_cfg[key]:
                for prop in if_cfg[key]['proposals']:
                    if 'encryption-algorithm' in prop and 'authentication-algorithm' in prop:
                        ipsvpn.proposal_add(str(prop['encryption-algorithm']), str(prop['authentication-algorithm']),
                                            ips.IKE_PHASE.PHASE2)
            if 'local-ts' in if_cfg[key] and if_cfg[key]['local-ts']:
                ipsvpn.local_ts = IPNetwork(if_cfg[key]['local-ts'])
            if 'remote-ts' in if_cfg[key] and if_cfg[key]['remote-ts']:
                ipsvpn.remote_ts = IPNetwork(if_cfg[key]['remote-ts'])

        elif key == 'remote-gw':
            ipsvpn.gateway = str(value)
        elif key == 'local-id':
            ipsvpn.local_id = str(value)
        elif key == 'remote-id':
            ipsvpn.remote_id = str(value)
        elif key == 'config-mode':
            if value == 'pull':
                ipsvpn.mode_config = ips.MODE_CONFIG.PULL
            elif value == 'push':
                ipsvpn.mode_config = ips.MODE_CONFIG.PUSH
        elif key == 'protocol-mode':
            if value == 'tunnel':
                ipsvpn.child_mode = ips.CHILD_MODE.TUNNEL
            elif value == 'transport':
                ipsvpn.child_mode = ips.CHILD_MODE.TRANSPORT
        elif key == "ike-mode":
            if value == "main":
                ipsvpn.ike1_mode = ips.IKE1_MODE.MAIN
            elif value == "aggressive":
                ipsvpn.ike1_mode = ips.IKE1_MODE.AGGRESIVE
        elif key == 'install-virtual-ip':
            if value is False or value == 'false':
                ipsvpn.vips = None
            else:
                ipsvpn.vips = IPAddress(if_cfg.get('virtual-ip', '0.0.0.0'))

    ipsvpn.load_security()
    ipsvpn.load_connection(debug=True)
    ret = ipsvpn.connect(timeout=20000)
    if ret is False:
        ipsvpn.unload_connection()
        ipsvpn.unload_security()


def setup_ovpn_if(if_cfg):
    print('Setup OpenVPN interface - %s,enable=%s' % (if_cfg['name'], if_cfg['enable']), file=ztpStream)

    try:
        with open (ovpn_client_file, "rt") as openvpnf:
            vpncfg=openvpnf.read()
            mathches=re.search("\nremote (.*) (.*)\n",vpncfg)
            if mathches:
                curr_conc=mathches.group(1)
                curr_port=mathches.group(2)
    except:
        curr_conc=None
        curr_port=None

    try:
        #use default VPN port , if not set.
        if 'ovpn-cfg' in if_cfg and 'concentrator-port' in if_cfg['ovpn-cfg']:
            vpn_ports = if_cfg['ovpn-cfg']['concentrator-port']
            vpn_portslist = sum([ list(range(int(x.split("-")[0]),int(x.split("-")[1])+1)) for x in vpn_ports.split(",") if "-" in x],
                                [ int(x) for x in vpn_ports.split(",") if "-" not in x  ])
            vpn_port = random.choice(vpn_portslist)
            port_in_list =  [ele for ele in vpn_portslist if(str(ele) == str(curr_port))]
            if bool(port_in_list):
                vpn_port = curr_port
        else:
            vpn_port = "1194"

        if if_cfg['ovpn-cfg']['concentrator-endpoint']:
            if curr_port != vpn_port  or curr_conc != if_cfg['ovpn-cfg']['concentrator-endpoint']:
                concentrator_endpoint = if_cfg['ovpn-cfg']['concentrator-endpoint']
                
                if concentrator_endpoint.startswith("tcp6:"):
                    concentrator_endpoint=concentrator_endpoint.replace("tcp6:","")
                    os.system("sed -i 's/^proto .*/proto tcp6-client/' %s" % ovpn_client_file)
                    os.system("sed -i 's/^fragment .*/#fragment 1300/' %s" % ovpn_client_file)
                elif concentrator_endpoint.startswith("tcp:"):
                    concentrator_endpoint=concentrator_endpoint.replace("tcp:","")
                    os.system("sed -i 's/^proto .*/proto tcp-client/' %s" % ovpn_client_file)
                    os.system("sed -i 's/^fragment .*/#fragment 1300/' %s" % ovpn_client_file)
                elif concentrator_endpoint.startswith("udp6:"):
                    concentrator_endpoint=concentrator_endpoint.replace("udp6:","")
                    os.system("sed -i 's/^proto .*/proto udp6/' %s" % ovpn_client_file)
                    os.system("sed -i 's/^#fragment .*/fragment 1300/' %s" % ovpn_client_file)
                else:
                    concentrator_endpoint=concentrator_endpoint.replace("udp:","")
                    concentrator_endpoint=concentrator_endpoint.replace("udp4:","")
                    os.system("sed -i 's/^proto .*/proto udp/' %s" % ovpn_client_file)
                    os.system("sed -i 's/^#fragment .*/fragment 1300/' %s" % ovpn_client_file)

                os.system('sed -i \'s/^remote.*/remote\ %s\ %s/\' ' % (
                    concentrator_endpoint, vpn_port) + ovpn_client_file)
                os.system('sed -i \'s/^lport.*/lport\ %s/\' ' % (
                    vpn_port) + ovpn_client_file)

        subprocess.check_call(
            ['/usr/sbin/openvpn', '--daemon', 'ovpn-client', '--status', '/var/run/openvpn/client.status', '10', '--cd',
             '/config/etc/openvpn', '--config', '/config/etc/openvpn/client.conf', '--writepid', '/tmp/ovpn.pid'])
        wait_vpn_up_down(if_cfg, up=True)

    except:
        pass


def sr_setup_ip_if():
    # bug 4540 - during initial homecalling,ignore old lease files,and prevent from invalid GW to be installed
    os.system('rm -f /var/lib/dhclient/dhclient*.lease')

    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config interfaces")
    interfaces = client.get_interfaces(startup=True)

    print('Setup LTE interfaces', file=ztpStream)
    if kernel_detects_external_ltes() > 1:
        subprocess.check_output('ip link set lte1 down && ip link set lte2 down', shell=True, encoding='utf-8')
        order_external_ltes()
        validate_lte_mac()
        subprocess.check_output('ip link set lte1 up && ip link set lte2 up', shell=True, encoding='utf-8')
    for ifname in interfaces:
        if 'type' in interfaces[ifname]:
            if interfaces[ifname]['type'] == 'lte':
                if 'enable' in interfaces[ifname] and interfaces[ifname]['enable']:
                    if True == kernel_detects_lte():
                        setup_lte_ip_if(interfaces[ifname])

                        #if interfaces[ifname]['management']:
                        #time.sleep(5)  # for dhcp
                        #check_speed(ifname)

    print('Setup Ethernet interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname]:
            if interfaces[ifname]['type'] == 'ethernet':
                setup_eth_ip_if(interfaces[ifname])

    print('Setup VPN interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname] and interfaces[ifname]['type'] == 'openvpn':
            if 'enable' in interfaces[ifname] and interfaces[ifname]['enable']:
                setup_ovpn_if(interfaces[ifname])
        if 'type' in interfaces[ifname] and interfaces[ifname]['type'] == 'ipsecvpn':
            if 'enable' in interfaces[ifname] and interfaces[ifname]['enable']:
                setup_ipsecvpn_if(interfaces[ifname])

    print('Setup wifi interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname] and interfaces[ifname]['type'] == 'wifi' \
           and 'enable' in interfaces[ifname] and interfaces[ifname]['enable'] \
           and 'wifi-mode' in interfaces[ifname] and 'client' == interfaces[ifname]['wifi-mode']:
            setup_wifi_ip_if(interfaces[ifname])


def sr_download_config():
    global orch_ip, ztp_port
    print('Download configuration file sr_download_config', file=ztpStream)

    headers = {'Content-Type': 'application/json',
               'Authentication': open(serial_cfg_file, 'r').readline().strip()}
    if (os.path.isfile(ztp_auth_key)):
        headers['Token'] = open(ztp_auth_key, 'r').readline().strip()

    req = urllib.request.Request(orch_url.format(orch_ip, ztp_port), headers=headers)

    import ssl
    context = ssl._create_unverified_context()
    try:
        res = urllib.request.urlopen(req, timeout=30, context=context)
        return res.read()
    except urllib.error.URLError as e:
        print('Download fail URL ' + orch_url.format(orch_ip, ztp_port) + ' {}'.format(e.reason), file=ztpStream)
        return None
    except urllib.error.HTTPError as e:
        print('Download fail HTTP -- %s' % e.reason, file=ztpStream)
        return None
    except:
        pass

    return None


def sr_remove_ip_if():
    print('Clean IP interfaces', file=ztpStream)

    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config interfaces")
    interfaces = client.get_interfaces(startup=True)

    for name, interface in sorted(interfaces.items()):
        if 'type' in interfaces[name]:
            if interfaces[name]['type'] == 'ethernet' or interfaces[name]['type'] == 'lte':
                if 'lte' == interfaces[name]['type'] and False == kernel_detects_lte():
                    continue
            elif interfaces[name]['type'] == 'openvpn':
                try:
                    if 'enable' in interfaces[name] and interfaces[name]['enable']:
                        subprocess.check_call(['/usr/bin/killall', 'openvpn'])
                        wait_vpn_up_down(interfaces[name], up=False)
                        # ensure VPN is down
                        os.system('/usr/bin/killall -0 openvpn && /usr/bin/killall -9 openvpn ')
                except:
                    pass
                continue
            elif interfaces[name]['type'] == 'ipsecvpn':
                if 'enable' in interfaces[name] and (
                        interfaces[name]['enable'] is True or interfaces[name]['enable'] == 'true'):
                    ipsvpn = ips.ipsec_vpn_if(name)
                    ipsvpn.disconnect()
                    ipsvpn.unload_connection()
                    ipsvpn.unload_security()

                continue
            elif interfaces[name]['type'] == 'wifi':
                try:
                    subprocess.run(['/usr/bin/killall', '--quiet', 'wpa_supplicant'], capture_output=False)
                    # ensure wpa_supplicant is down
                    os.system('/usr/bin/killall -0 wpa_supplicant && /usr/bin/killall -9 wpa_supplicant ')
                except:
                    pass

            print('Shutdown \'%s\' interface' % name, file=ztpStream)
            vlan = get_interface_vtag(interfaces[name])
            ifname = "%s%s" % (name, '' if (0 == vlan or 'inband' in name) else '.%d' % vlan)

            try:
                subprocess.check_call(['/sbin/ifdown', '--force', ifname])
            except subprocess.CalledProcessError:
                print('Cannot down interface %s' % ifname, file=ztpStream)
                if not is_arm_arch():
                    os.system("ip link set %s down 1>/dev/null 2>/dev/null" % ifname)
            except OSError:
                print('/sbin/ifdown not found', file=ztpStream)

            options = "-z %s " % ifname
            subprocess.check_call(['/scripts/apply_networking.py', options])
            try:
                subprocess.check_call(['/sbin/ip', '-force', 'address', 'flush', ifname])
            except subprocess.CalledProcessError:
                print('Cannot flush interface %s' % ifname, file=ztpStream)
            except OSError:
                print('/sbin/ip not found', file=ztpStream)


def get_dpdk_if_pci_id():
    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config interfaces")
    interfaces = client.get_interfaces(startup=True)
    pci_ids = list()
    for ifcfg in interfaces.values():
        try:
            if ifcfg.get('mode', 'virtio') == 'dpdk':
                out = subprocess.check_output(
                    ['udevadm', 'info', '-q', 'path', '-p', '/sys/class/net/{}'.format(ifcfg.get('name'))])
            else:
                continue
            if out.split(b'/')[2] != 'virtual':
                pci_ids.append(out.decode().split('/')[-3])
        except:
            pass
    return pci_ids


def get_dpdk_ifs_driver():
    pci_ids = get_dpdk_if_pci_id()
    drivers = dict()
    for pci_id in pci_ids:
        for root, dirs, files in os.walk('/sys/bus/pci/drivers/'):
            if pci_id in dirs:
                drivers[pci_id] = root.split('/')[-1]
    return drivers


def bind_if_to_old_driver(drivers):
    for key in drivers:
        subprocess.check_call(['dpdk-devbind', '-u', key])
        subprocess.check_call(['dpdk-devbind', '-b', drivers.get(key), key])


def setup_vxlan_ip_if(config):
    from sysrepo_api import sysrepo_api3 as api
    import ucpeif.vxlanif as vxlif
    global vxlan_interfaces
    sr_api = api.SysrepoAPI(app_name="ZTP read config interfaces")
    sr_api.set_active_datastore("startup")
    local = None
    old_local = None
    old_localv6 = None
    old_remote = []
    mcast = None
    old_mcast_ip = None
    if config:
        if config.get('local', {}).get('vtep-addr', None) != None:
            local = config.get('local', {}).get('vtep-addr', None)
            if local:
                old_local = ipaddress.ip_address(local) if isinstance(ipaddress.ip_address(local),
                                                                      ipaddress.IPv4Address) else None
                old_localv6 = ipaddress.ip_address(local) if isinstance(ipaddress.ip_address(local),
                                                                        ipaddress.IPv6Address) else None
        elif config.get('local', {}).get('vtep-if', None) != None:
            local = config.get('local', {}).get('vtep-if', None)
            if local != None:
                bootproto = None
                address = None
                addressv6 = None
                bootproto = sr_api.get_item("/ucpe:config/interfaces:interface[name='{}']/bootproto".format(local))
                if bootproto == "none" and bootproto != None:
                    addressv6 = sr_api.get_item(
                        "/ucpe:config/interfaces:interface[name='{}']/ipv6-prefix".format(local))
                    if addressv6 is not None:
                        old_localv6 = ipaddress.ip_interface(addressv6).ip

                    address = sr_api.get_item("/ucpe:config/interfaces:interface[name='{}']/ipv4-prefix".format(local))
                    if address is not None:
                        old_local = ipaddress.ip_interface(address).ip

        for rvtep_addr in config.get('remote', {}).get('vtep-addrs', []):
            old_remote.append(ipaddress.ip_address(rvtep_addr.get('vtep-addr')))

        old_type = config.get('tunnel-type', 'vxlan')

        if config.get('remote', None) != None:
            mcast = config.get('remote', {}).get('mcast-vtep-addr', None)
            if mcast != None:
                old_mcast_ip = ipaddress.ip_address(mcast)

        tun_type = vxlif.TUNNEL_TYPE.VXLAN

        vni = config.get('vni', None)
        shg = config.get('shg', None)

        if old_type == 'vxlan-gpe':
            tun_type = vxlif.TUNNEL_TYPE.VXLAN_GPE

        vxlan_if = vxlif.VxLanInterface(config.get('name'), vni=vni, lvtep=old_local, lvtepv6=old_localv6,
                                        rvteps=old_remote, tun_type=tun_type, shg=shg, mcast_if=local,
                                        mcast_ip=old_mcast_ip)

        vxlan_interfaces.append(vxlan_if)

        enable = config.get('enable', False)
        vxlan_if.enable = enable

        ipv4_pr = config.get('ipv4-prefix', None)
        if ipv4_pr:
            vxlan_if.ip4 = ipaddress.IPv4Interface(str(ipv4_pr))

        ipv6_pr = config.get('ipv6-prefix', None)
        if ipv6_pr:
            vxlan_if.ip6 = ipaddress.IPv6Interface(str(ipv6_pr))

        bootproto = config.get('bootproto', None)
        bp_set = None
        if bootproto:
            if bootproto == 'none':
                bp_set = vxlif.BOOT_PROTO.NONE
            elif bootproto == 'dhcp':
                bp_set = vxlif.BOOT_PROTO.DHCP
            elif bootproto == 'dhcp6':
                bp_set = vxlif.BOOT_PROTO.DHCP6
            elif bootproto == 'dhcp46':
                bp_set = vxlif.BOOT_PROTO.DHCP46

            if bp_set:
                vxlan_if.bootproto = bp_set

        dns = config.get('dns', None)
        if dns:
            dnses = dns.split(',')
            dnsopts = ''.join(map(lambda dns_srv: ' '+ dns_srv, dnses))
            subprocess.call('/usr/bin/resolvectl dns br-{} {}'.format(config.get('name'), dnsopts), shell=True)


def setup_pppoe_ip_if(name, config):
    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config interfaces")
    client.set_active_datastore("startup")
    global pppoe_interfaces
    c_auth_type = pppoeif.AUTH_TYPE.PAP

    if config:
        if config.get('auth-type', 'pap') == 'chap':
            c_auth_type = pppoeif.AUTH_TYPE.CHAP

        c_username = config.get('username', None)
        c_password = config.get('password', None)
        c_ethif = config.get('ethernet-interface', None)
        c_status = config.get('enable', False)
        c_idle = config.get('idle-timeout', 0)
        c_retry = config.get('retry-timeout', 0)
        c_metric = config.get('metric', 130)
        c_ac_name = config.get('ac-name', None)
        c_ipv6 = config.get('ipv6', False)

        if c_ethif:
            if 'virtio' == client.get_interface_param(c_ethif, 'mode', 'virtio'):
                c_ethif = 'host-{}'.format(c_ethif)

            c_ethif = client.tag_interface(c_ethif, client.get_interface_param(c_ethif.replace('host-', ''), 'vlan',
                                                                               None))

        print("Add PPPoE configuration for \'{}\' interface".format(config.get('name')))
        ppp_if = pppoeif.PPPoEClientInterface(config.get('name'), auth_type=c_auth_type, username=c_username,
                                              password=c_password,
                                              ethif=c_ethif, enable=c_status, metric=c_metric,
                                              idle_timeout=c_idle, retry_timeout=c_retry, ipv6=c_ipv6,
                                              ac_name=c_ac_name)
        pppoe_interfaces.append(ppp_if)

        mtu = config.get('mtu', 1492)
        if mtu:
            ppp_if.mtu = int(mtu)
    for i in range(0, 10):
        try:
            ni.ifaddresses(config.get('name'))
            ip = ni.ifaddresses(config.get('name'))[ni.AF_INET][0]['addr']
            break
        except:
            print('Wait for PPPoE interface', file=ztpStream)
            time.sleep(5)
            pass


def set_vpp_networking():
    global vpp_proc
    os.system('rm -f /var/lib/dhclient/dhclient*.lease')
    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config interfaces")
    interfaces = client.get_interfaces(startup=True)

    for ifname in interfaces:
        if interfaces[ifname].get('type') == 'lte':
            if if_lte.lte_is_qmi(if_lte.modem_default):
                if_lte.setup_lte_qmi_dev(interfaces[ifname], if_lte.modem_default)
            elif if_lte.lte_is_mbim(if_lte.mbim_modem_default):
                if_lte.setup_lte_mbim_dev(interfaces[ifname], if_lte.mbim_modem_default)

    subprocess.check_call(['/usr/bin/taskset', 'ff' ,  '/scripts/vpp-pre-ztp.py'])
    time.sleep(10)
    vpp_proc = subprocess.Popen(['/opt/bin/netlink-vpp', '-d'])

    print('Setup LTE interfaces', file=ztpStream)
    for ifname in interfaces:
        if interfaces[ifname].get('type') == 'lte':
            setup_eth_ip_if(interfaces[ifname])
            #cancel speedtest on LTE up
            #time.sleep(5)  # for dhcp
            #check_speed(ifname)

    print('Setup Ethernet interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname]:
            if interfaces[ifname]['type'] == 'ethernet':
                setup_eth_ip_if(interfaces[ifname])

    print('Setup wifi interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname] and interfaces[ifname]['type'] == 'wifi' \
           and 'enable' in interfaces[ifname] and interfaces[ifname]['enable'] \
           and 'wifi-mode' in interfaces[ifname] and 'client' == interfaces[ifname]['wifi-mode']:
            setup_wifi_ip_if(interfaces[ifname])

    print('Setup vxlan interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname]:
            if interfaces[ifname]['type'] == 'vxlan' and interfaces[ifname].get('enable', False) == True:
                setup_vxlan_ip_if(interfaces[ifname])

    # subprocess.call(['/usr/lib/frr/watchfrr', '-d', '-F', 'traditional', 'zebra', 'staticd'])

    print('Setup PPPoE interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname]:
            if interfaces[ifname]['type'] == 'pppoe' and interfaces[ifname].get('enable', False):
                setup_pppoe_ip_if(ifname, interfaces[ifname])

    print('Setup VPN interfaces', file=ztpStream)
    for ifname in interfaces:
        if 'type' in interfaces[ifname] and interfaces[ifname]['type'] == 'openvpn':
            if 'enable' in interfaces[ifname] and interfaces[ifname].get('enable', False):
                setup_ovpn_if(interfaces[ifname])
        if 'type' in interfaces[ifname] and interfaces[ifname]['type'] == 'ipsecvpn':
            if 'enable' in interfaces[ifname] and interfaces[ifname].get('enable', False):
                setup_ipsecvpn_if(interfaces[ifname])


def remove_vpp_networking():
    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config interfaces")
    interfaces = client.get_interfaces(startup=True)
    global pppoe_interfaces
    global vxlan_interfaces

    for name, interface in sorted(interfaces.items()):
        if interfaces[name].get('type') == 'pppoe':
            if interfaces[name].get('enable') == True:
                ppp_if = next((a for a in pppoe_interfaces if a.name == name), None)
                pppoe_interfaces.remove(ppp_if)
                del ppp_if

    for name, interface in sorted(interfaces.items()):
        if interfaces[name].get('type') == 'vxlan':
            if interfaces[name].get('enable') == True:
                vxlan_if = next((a for a in vxlan_interfaces if a.name == name), None)
                vxlan_interfaces.remove(vxlan_if)
                del vxlan_if

    for name, interface in sorted(interfaces.items()):
        if 'type' in interfaces[name]:
            sysname = client.get_if_sysname(name)
            if sysname.find('br-') == 0:
                if is_arm_arch():
                    if 'mode' in interfaces[name]:
                        mode = interfaces[name]['mode']
                    else:
                        mode = "virtio"
                else:
                    if os.path.exists('/sys/class/net/{}'.format(name)):
                        mode = "virtio"
                    else:
                        mode = "dpdk"
                if mode == 'dpdk':
                    cmd = ['/usr/bin/vppctl', 'set', 'interface', 'state', name, 'down']
                else:
                    cmd = ['/sbin/ifdown', '--force', sysname]

                # subprocess.check_call(['/sbin/ifdown', sysname])
                try:
                    subprocess.check_call(cmd)
                except subprocess.CalledProcessError:
                    print('Cannot down interface %s' % sysname, file=ztpStream)
                except OSError:
                    print('/sbin/ifdown not found', file=ztpStream)

                options = "-z %s " % sysname
                subprocess.check_call(['/scripts/apply_networking.py', options])
                try:
                    subprocess.check_call(['/sbin/ip', '-force', 'address', 'flush', sysname])
                except subprocess.CalledProcessError:
                    print('Cannot flush interface %s' % sysname, file=ztpStream)
                except OSError:
                    print('/sbin/ip not found', file=ztpStream)

    try:
        print('Stop Netlik-VPP!', file=ztpStream)
        global vpp_proc
        pid = int(vpp_proc.pid)
        if os.path.exists("/var/run/netlink-vpp.pid"):
            with open('/var/run/netlink-vpp.pid', 'r') as f:
                pid = f.readline().strip('\n')

        os.system("kill %d" %  int(pid))
        time.sleep(3)
        if 0 == os.system("kill -0 %d" %  int(pid)):
            os.system("kill -9 %d" %  int(pid))

        print('Stop VPP! ', file=ztpStream)
        subprocess.call(['/usr/bin/killall', '/usr/bin/vpp'])
        time.sleep(3)
        if 0 == os.system("/usr/bin/killall -0 /usr/bin/vpp"):
            os.system("/usr/bin/killall -9 /usr/bin/vpp")
    except:
        print('Failed to stop VPP services ', file=ztpStream)


def sr_get_config_file(dst_file):
    print('get_config_file', file=ztpStream)

    from sysrepo_system import sysrepo_system as sys_api
    sys_client = sys_api.SysrepoSystem(app_name="ZTP read config")
    sys_client.set_active_datastore("startup")
    net_mode = sys_client.get_network_mode()

    if net_mode == 'vpp':
        drivers = get_dpdk_ifs_driver()
        set_vpp_networking()
    else:
        sr_setup_ip_if()

    new_cfg = None
    loop = 2
    while None == new_cfg and loop > 0:
        print('Download loop %d' % loop, file=ztpStream)
        loop -= 1
        new_cfg = sr_download_config()

        if new_cfg is not None:
            # We have configuration from server
            print('Config start', file=ztpStream)
            print(new_cfg, file=ztpStream)
            print('Config End', file=ztpStream)
            cfg_file = open(dst_file, 'w')
            parsed = json.loads(new_cfg)
            new_cfg = json.dumps(parsed, indent=4, sort_keys=True)
            cfg_file.write(new_cfg)
            cfg_file.close()
            break

        time.sleep(4)
    if net_mode == 'vpp':
        remove_vpp_networking()
        bind_if_to_old_driver(drivers)

    sr_remove_ip_if()

    if None == new_cfg:
        return False

    return True


def sr_diff_config_cfg_serv(cfg_file):
    global orch_ip
    cfg_f = open(cfg_file, 'r')
    cfg = json.load(cfg_f)

    for i in cfg['json']['tve']['ucpe:config']['system:system']['config-server']:
        if i['server'] == orch_ip:
            return 0
        else:
            orch_ip = i['server']
            return 1


def sync_hwtype(tmp_cfg_file):
    print('sync_hwtype', file=ztpStream)
    hw_type_file = "/config/.hwtype"
    cfg_f = open(tmp_cfg_file, 'r')
    cfg = json.load(cfg_f)
    cfg_f.close()
    count = 0

    hwtype = cfg['json']['tve']['ucpe:status']['system:system'].get('hwtype', None)
    if hwtype is not None:
        while True:
            count += 1
            hwtype_f = open(hw_type_file, 'w')
            hwtype_f.write(hwtype)
            hwtype_f.close()

            hwtype_f = open(hw_type_file, "r")
            hwtype_check = hwtype_f.readline().strip()
            hwtype_f.close()
            if hwtype == hwtype_check:
                print('HW TYPE set to %s!' % hwtype)
                break
            if count > 3:
                print('Failed to write HW TYPE!')
                break
    else:
        print('HW TYPE is not provided!')


def get_ip_address(ifname):
    # s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    # 0x8915,  # SIOCGIFADDR
    # return socket.inet_ntoa(fcntl.ioctl( s.fileno(), 0x8915, struct.pack('256s', ifname[:15]) )[20:24])
    try:
        return os.popen('ifconfig br-%s | grep "inet" | grep -v inet6' % ifname).read().split()[1]
    except IndexError:
        return ''


def check_speed(ifname):
    # Note: Use the speedtest-cli API when ZTP is updated to python 3
    ip = get_ip_address(ifname)
    if not ip:
        print("Error -- Can't get IP of interface %s" % ifname, file=ztpStream)
        return -1

    print("Testing LTE speed of interface %s, IP %s" % (ifname, ip), file=ztpStream)
    try:
        speeds = ''
        speeds = subprocess.check_output(['/usr/bin/speedtest-cli', '--source', ip, '--csv'], stderr=ztpStream).decode()
        speeds = speeds.split(',')
    except Exception as e:
        if speeds:
            print(speeds)
            print("Error while testing %s speed!" % ifname, file=ztpStream)
            print(e, file=ztpStream)
        return -1

    if len(speeds) < 8:
        print("Error while testing %s speed!" % ifname, file=ztpStream)
        return -1

    down = speeds[6]
    up = speeds[7]
    if int(float(down)) < 1000000 or int(float(up)) < 1000000:
        print("Warning -- Low speed of interface %s! Upload speed=%sbps Download speed=%sbps" % (ifname, up, down),
              file=ztpStream)
    else:
        print("Interface %s: upload speed=%sbps, download speed=%sbps" % (ifname, up, down), file=ztpStream)


def sr_download():
    # Get new configuration file from configuration server
    loop = 2
    update_ztp_statusfile("unknown")
    new_cfg = False
    while loop > 0:
        loop -= 1
        if False == sr_get_config_file(tmp_cfg_file):
            # We do not have new configuration - continue to use current one
            print('Download new configuration failed !!!', file=ztpStream)
            os.system("touch /run/ztp-failed-flag")
            break

        if 0 != apply_config_to_sysrepo(infilename=tmp_cfg_file):
            print('Error applying config to sysrepo !', file=ztpStream)
            os.remove(tmp_cfg_file)
            break
        else:
            new_cfg = True

        sync_hwtype(tmp_cfg_file)

        update_ztp_statusfile("completed")

        # If there is no change in coniguration server ip continue boot process
        if 0 == sr_diff_config_cfg_serv(tmp_cfg_file):
            print('There is no new configuration server !', file=ztpStream)
            os.remove(tmp_cfg_file)
            break

        validate_sysrepo_cfg()

    if os.path.isfile(tmp_cfg_file):
        tmp_f = open(tmp_cfg_file, 'r+')
        tmp_cfg = json.load(tmp_f)
        if new_cfg and (hw_type.lower() in ("hk6010", "hk6010-8g", "hk6010-16g")):
            device_init_passed = True
            try:
                subprocess.check_call(['/scripts/device_init.sh', 'nics'])
            except subprocess.CalledProcessError as e:
                print('Command \'%s\' failed with code %d' % (e.cmd, e.returncode), file=ovsStream)
            except OSError as e:
                print('System error - \'%s\'' % e.strerror, file=ovsStream)
            finally:
                time.sleep(30)

    else:
        # There is no new configuration
        print('There is no new configuration.', file=ztpStream)
        return 0

    reboot_loop_prevent(reset=True)

    restore_db()


UCPE_STATUS_FILE = "/config/_hb/ucpe_status.json"
UCPE_STATUS_FILE_TEMPLATE = "{\"ucpe:status\": {\"status\": \"unknown\", \"hb-status\": \"unknown\", \"nsStatus\": \"unknown\", \"ztp-status\": \"unknown\", \"certs\": \"default\", \"license-id\": \"unknown\", \"apps\": \"false\", \"cluster\":{}, \"appslist\":{}}}"


def update_ztp_statusfile(status):
    print('updating ztp status %s.' % status, file=ztpStream)
    try:
        if not (os.path.isfile(UCPE_STATUS_FILE)):
            ns_status_f = open(UCPE_STATUS_FILE, 'w')
            data = json.loads(UCPE_STATUS_FILE_TEMPLATE)
            data['ucpe:status']["ztp-status"] = str(status)
            json.dump(data, ns_status_f)
            ns_status_f.close()
            os.chmod(UCPE_STATUS_FILE, 0o777)
        else:
            with open(UCPE_STATUS_FILE, "r+", encoding='utf-8') as json_file:
                data = json.load(json_file)
                data['ucpe:status']["ztp-status"] = str(status)
                json_file.seek(0)
                json.dump(data, json_file)
                json_file.truncate()
                json_file.close()
                print('updating ztp status done.', file=ztpStream)
    except:
        pass


def main():
    print('Start', file=ztpStream)

    validate_sysrepo_cfg()
    if not hc_enabled:
        update_ztp_statusfile("disabled")
        print('Home calling is disabled !', file=ztpStream)
        from sysrepo_interfaces import sysrepo_interfaces3 as api
        client = api.SysrepoInterfaces(app_name="ZTP read config")
        interfaces = client.get_interfaces(startup=True)
        for ifname in interfaces:
            if interfaces[ifname].get('type') == 'lte':
                if if_lte.lte_is_qmi(if_lte.modem_default):
                    if_lte.setup_lte_qmi_dev(interfaces[ifname], if_lte.modem_default)
                elif if_lte.lte_is_mbim(if_lte.mbim_modem_default):
                    if_lte.setup_lte_mbim_dev(interfaces[ifname], if_lte.mbim_modem_default)
        return 0

    sr_download()

    from sysrepo_interfaces import sysrepo_interfaces3 as api
    client = api.SysrepoInterfaces(app_name="ZTP read config")
    interfaces = client.get_interfaces(startup=True)
    for ifname in interfaces:
        if interfaces[ifname].get('type') == 'lte':
            modem_type=if_lte.lte_get_driver_type(interfaces[ifname]['name'])
            if modem_type == 1:
                if_lte.reconnect_lte_qmi_dev(interfaces[ifname], if_lte.modem_default, force=True)
            elif modem_type == 2 or modem_type == 4:
                if_lte.setup_lte_mbim_dev(interfaces[ifname], if_lte.mbim_modem_default)

    print('End', file=ztpStream)
    return 0


def clean_ifcfg_files():
    ifs = ['data', 'br-data', 'nat', 'lte', 'mgmt', 'wifi', 'br-vxlan']
    for i in ifs:
        cmd = '/bin/rm -rf /etc/network/interfaces.d/ifcfg-{}*'.format(i)
        subprocess.check_call(cmd, shell=True)


def clean_at_boot():
    RESOLV_CFG = '/etc/systemd/resolved.conf'
    RESOLV_CFG_BAK = '/etc/systemd/resolved.conf.vpn'

    import shutil
    if os.path.exists(RESOLV_CFG_BAK):
        shutil.copyfile(RESOLV_CFG_BAK, RESOLV_CFG)
        os.remove(RESOLV_CFG_BAK)
        os.system("/bin/systemctl restart --no-block systemd-resolved.service")


if __name__ == "__main__":
    # ensure ztp-pre won't restart
    if (os.path.isfile("/run/ztp-pre.lck")):
        sys.exit(0)

    os.system("touch /run/ztp-pre.lck")

    clean_ifcfg_files()
    subprocess.check_call(['/usr/bin/udevadm', 'trigger', '-s', 'net'])
    subprocess.check_call(['/usr/bin/udevadm', 'settle'])

    # HK init HW in virtio to perform ZTP, then according to config
    if (hw_type.lower() in ("hk6010", "hk6010-8g", "hk6010-16g")):
        os.system("/scripts/device_init.sh nics forcevirtio")

    if (os.path.isfile('/opt/modem-preinit.sh')):
        os.system('/opt/modem-preinit.sh')

    clean_at_boot()

    ret = main()

    if (hw_type.lower() in ("hk6010", "hk6010-8g", "hk6010-16g")) and not device_init_passed:
        os.system("/scripts/device_init.sh nics")

    subprocess.call(['/usr/bin/killall', 'zebra'])
    subprocess.call(['/usr/bin/killall', 'staticd'])
    subprocess.call(['/usr/bin/killall', 'watchfrr'])

    sys.exit(ret)
